﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    WinJS.Namespace.define("MS.Entertainment", {CloudCollectionService: MS.Entertainment.UI.Framework.define(function CloudCollectionServiceConstructor() {
            this._configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
            if (this._isSyncFeatureEnabled) {
                var signInService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                var signedInUserService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                var onSignInStateChange = this._onSignInStateChange.bind(this);
                WinJS.Binding.bind(signInService, {isSignedIn: onSignInStateChange});
                WinJS.Binding.bind(signedInUserService, {isSubscription: onSignInStateChange})
            }
        }, {
            _isAvailable: false, _isEnabled: false, _isFeatureEnabled: false, _autoDownloadEnabled: false, _cloudMatchStatus: false, _shouldDownloadOnAdd: true, _syncManagerStorage: null, _syncManager: {get: function cloudCollectionService_getSyncManager() {
                        if (!this._syncManagerStorage)
                            this._syncManagerStorage = new Microsoft.Entertainment.Sync.SyncManager;
                        return this._syncManagerStorage
                    }}, _configurationManager: null, _listeningToSyncManagerEvents: false, _displayedAnyErrors: false, startListening: function startListening() {
                    if (this._isSyncFeatureEnabled && !this._listeningToSyncManagerEvents && this._syncManager) {
                        this._syncManager.addEventListener("collectionsyncend", this._onCollectionSyncEnd.bind(this), false);
                        this._listeningToSyncManagerEvents = true
                    }
                }, isAvailable: {get: function getIsAvailable() {
                        return this._isAvailable
                    }}, isFeatureEnabled: {get: function getIsFeatureEnabled() {
                        return this._isFeatureEnabled
                    }}, isEnabled: {
                    get: function getIsEnabled() {
                        return this._isEnabled
                    }, set: function setIsEnabled(newValue) {
                            if (this._isEnabled !== newValue) {
                                this._configurationManager.sync.cloudCollectionEnabled = newValue;
                                this._isEnabled = newValue;
                                this._updateIsEnabledState();
                                this._updateAutoDownloadEnabledState()
                            }
                        }
                }, isCloudMatchOptedIn: {get: function getCloudMatchOptedIn() {
                        return this._cloudMatchStatus === MS.Entertainment.CloudCollectionService.CloudMatchStatus.OptedIn
                    }}, isCloudMatchOptedOut: {get: function getCloudMatchOptedOut() {
                        return this._cloudMatchStatus === MS.Entertainment.CloudCollectionService.CloudMatchStatus.OptedOut
                    }}, isCloudMatchNotApplicable: {get: function getCloudMatchNotApplicable() {
                        return this._cloudMatchStatus === MS.Entertainment.CloudCollectionService.CloudMatchStatus.NotApplicable
                    }}, autoDownloadEnabled: {
                    get: function getAutoDownloadEnabled() {
                        return this._autoDownloadEnabled
                    }, set: function setAutoDownloadEnabled(newValue) {
                            if (this._autoDownloadEnabled !== newValue) {
                                this._configurationManager.sync.keepCloudCollectionOffline = newValue;
                                this._autoDownloadEnabled = newValue;
                                this._updateAutoDownloadEnabledState();
                                this._requestSync()
                            }
                        }
                }, shouldDownloadOnAdd: {get: function getShouldDownloadOnAdd() {
                        var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                        var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                        if (signIn.isSignedIn && signedInUser.isSubscription) {
                            this._shouldDownloadOnAdd = this._configurationManager.music.musicPassDownloadEnabled;
                            if (this.isEnabled)
                                this._shouldDownloadOnAdd |= this.autoDownloadEnabled
                        }
                        else
                            this._shouldDownloadOnAdd = false;
                        return this._shouldDownloadOnAdd
                    }}, _updateIsAvailableState: function _updateIsAvailableState() {
                    var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                    if (featureEnablement && featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.storeSync))
                        if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.syncRequiresSubscription))
                            this._isAvailable = this._configurationManager.service.lastSignedInUserSubscription;
                        else
                            this._isAvailable = true
                }, _updateIsEnabledState: function _updateIsEnabledState(skipSync) {
                    this._isEnabled = this.isAvailable && this._isEnabled;
                    if (this._isEnabled && !skipSync) {
                        var syncDelay = MS.Entertainment.Utilities.isMusicApp2 ? 5000 : 0;
                        WinJS.Promise.timeout(syncDelay).done(this._requestSync.bind(this))
                    }
                }, _updateIsFeatureEnabledState: function _updateIsFeatureEnabledState() {
                    var isEnabled = false;
                    var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                    if (featureEnablement && featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.storeSync)) {
                        var syncRequiresSubscription = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.syncRequiresSubscription);
                        var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                        var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                        isEnabled = syncRequiresSubscription ? (signIn.isSignedIn && signedInUser.isSubscription) : true
                    }
                    this._isFeatureEnabled = isEnabled
                }, _updateAutoDownloadEnabledState: function _updateAutoDownloadEnabledState() {
                    this._autoDownloadEnabled = this.isEnabled && this._autoDownloadEnabled
                }, _isSyncFeatureEnabled: {get: function get_isSyncFeatureEnabled() {
                        var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                        return (featureEnablement && featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.storeSync))
                    }}, _requestSync: function _requestSync() {
                    if (!this._syncManager) {
                        var syncManager = new Microsoft.Entertainment.Sync.SyncManager;
                        this._syncManager = syncManager
                    }
                    this._syncManager.requestSync()
                }, _onSignInStateChange: function _signInStateChanged(newSignInValue, oldSignInValue) {
                    var shouldSkipSync = oldSignInValue === undefined;
                    if (oldSignInValue !== newSignInValue) {
                        this._isAvailable = false;
                        this._isEnabled = this._configurationManager.sync.cloudCollectionEnabled;
                        this._cloudMatchStatus = this._configurationManager.shell.cloudMatching;
                        this._autoDownloadEnabled = this._configurationManager.sync.keepCloudCollectionOffline;
                        this._updateIsAvailableState();
                        this._updateIsEnabledState(shouldSkipSync);
                        this._updateAutoDownloadEnabledState();
                        this._updateIsFeatureEnabledState()
                    }
                }, setCloudMatchOptIn: function setCloudMatchOptIn(matchStatus) {
                    if (this._cloudMatchStatus !== matchStatus) {
                        this._configurationManager.shell.cloudMatching = matchStatus;
                        this._configurationManager.sync.syncBlockedItems = (matchStatus === MS.Entertainment.CloudCollectionService.CloudMatchStatus.OptedIn);
                        if (this.isAvailable && this.isEnabled && matchStatus === MS.Entertainment.CloudCollectionService.CloudMatchStatus.OptedIn)
                            this._requestSync();
                        this._cloudMatchStatus = matchStatus
                    }
                }, clearCloudMatchOptIn: function clearCloudMatchOptIn() {
                    this._configurationManager.shell.cloudMatching = MS.Entertainment.CloudCollectionService.CloudMatchStatus.Unknown;
                    this._cloudMatchStatus = MS.Entertainment.CloudCollectionService.CloudMatchStatus.Unknown
                }, _onCollectionSyncEnd: function _onCollectionSyncEnd(errors) {
                    if (!this._displayedAnyErrors && Array.isArray(errors))
                        errors.forEach(this._sendSyncErrorNotification.bind(this));
                    if (errors.length === 0 && this._displayedAnyErrors)
                        this._getAppNotificationService().removeNotificationByCategory(this._syncErrorNotificationCategory)
                }, _sendSyncErrorNotification: function _sendSyncErrorNotification(error) {
                    this._displayedAnyErrors = true;
                    var notificationTitle;
                    var notificationSubTitle = String.load(String.id.IDS_CLOUD_SYNC_ERROR_NOTIFICATION_TEXT);
                    switch (error) {
                        case Microsoft.Entertainment.Sync.CollectionSyncError.collectionFull:
                            notificationTitle = String.load(String.id.IDS_CLOUD_SYNC_COLLECTION_FULL_NOTIFICATION_HEADER);
                            break;
                        case Microsoft.Entertainment.Sync.CollectionSyncError.maxPlaylistsReached:
                            notificationTitle = String.load(String.id.IDS_CLOUD_SYNC_PLAYLIST_LIMIT_EXCEEDED_NOTIFICATION_HEADER);
                            break;
                        default:
                            notificationTitle = String.load(String.id.IDS_CLOUD_SYNC_ERROR_NOTIFICATION_HEADER);
                            break
                    }
                    var notification = new MS.Entertainment.UI.Notification({
                            notificationType: MS.Entertainment.UI.Notification.Type.Critical, title: notificationTitle, subTitle: notificationSubTitle, icon: MS.Entertainment.UI.Icon.inlineError, action: this._showSyncError.bind(this, error), category: this._syncErrorNotificationCategory, isPersistent: true, dismissOnSignOut: true
                        });
                    this._getAppNotificationService().send(notification)
                }, _showSyncError: function _showSyncError(error) {
                    var dialogTitle;
                    var dialogMessage;
                    switch (error) {
                        case Microsoft.Entertainment.Sync.CollectionSyncError.collectionFull:
                            dialogTitle = String.load(String.id.IDS_CLOUD_SYNC_COLLECTION_FULL_DIALOG_TITLE);
                            var maxTracksInCollection = this._configurationManager.sync.maxTracksInCollection;
                            dialogMessage = String.load(String.id.IDS_CLOUD_SYNC_COLLECTION_FULL_DIALOG_MESSAGE).format(maxTracksInCollection);
                            MS.Entertainment.UI.Shell.showMessageBox(dialogTitle, dialogMessage);
                            break;
                        case Microsoft.Entertainment.Sync.CollectionSyncError.maxPlaylistsReached:
                            dialogTitle = String.load(String.id.IDS_CLOUD_SYNC_PLAYLIST_LIMIT_EXCEEDED_DIALOG_TITLE);
                            var maxPlayListsInCollection = this._configurationManager.groveler.maxPlaylistsInCollection;
                            dialogMessage = String.load(String.id.IDS_CLOUD_SYNC_PLAYLIST_LIMIT_EXCEEDED_DIALOG_MESSAGE).format(maxPlayListsInCollection);
                            MS.Entertainment.UI.Shell.showMessageBox(dialogTitle, dialogMessage);
                            break;
                        default:
                            dialogTitle = String.load(String.id.IDS_CLOUD_SYNC_ERROR_DIALOG_TITLE);
                            MS.Entertainment.UI.Shell.showError(dialogTitle, error);
                            break
                    }
                    this._requestSync()
                }, _getAppNotificationService: function _getAppNotificationService() {
                    return MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appNotification)
                }, _syncErrorNotificationCategory: "SyncErrorNotification"
        }, {
            factory: function factory() {
                return new MS.Entertainment.CloudCollectionService
            }, CloudMatchStatus: {
                    Unknown: 0, OptedIn: 1, OptedOut: 2, NotApplicable: 3
                }
        })});
    MS.Entertainment.ServiceLocator.register(MS.Entertainment.Services.cloudCollection, MS.Entertainment.CloudCollectionService.factory)
})()
