﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
var MS;
(function(MS) {
    (function(Entertainment) {
        (function(Music) {
            MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Music");
            var FreePlayLimits = (function() {
                    function FreePlayLimits() {
                        this._musicLimitsManager = null;
                        this._listeners = {};
                        this._collectionFilterListeners = [];
                        this._isMonthlyFreeLimitExceeded = false;
                        this._isFreeTrialCompleted = false;
                        this._eventBindings = null;
                        this._queuedCollectionFilterEvent = null;
                        this._refreshingLimits = null;
                        this._signInBindings = null;
                        this._signedInUserBindings = null;
                        this._initialized = false;
                        this._initializedTimeout = null;
                        this._sessionManager = null
                    }
                    FreePlayLimits.prototype.initialize = function() {
                        if (!this._initialized) {
                            var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                            if (featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlay)) {
                                this._initialized = true;
                                this._musicLimitsManager = Microsoft.Entertainment.Limits.MusicLimitsManager;
                                this._isMonthlyFreeLimitExceeded = this._musicLimitsManager.isMonthlyFreeLimitExceeded;
                                this._isFreeTrialCompleted = this._musicLimitsManager.isFreeTrialCompleted;
                                this._sessionManager = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                                var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                                var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                                var onSignInStateChanged = this._onSignInStateChanged.bind(this);
                                this._signInBindings = WinJS.Binding.bind(signIn, {isSignedIn: onSignInStateChanged});
                                this._signedInUserBindings = WinJS.Binding.bind(signedInUser, {isSubscription: onSignInStateChanged});
                                this._initializedTimeout = WinJS.Promise.timeout(3000)
                            }
                        }
                    };
                    FreePlayLimits.prototype._currentMediaChanged = function() {
                        if (!WinJS.Utilities.getMember("MS.Entertainment.Music2.UpsellDialogOverlay"))
                            return;
                        var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                        if (!signedInUser.isSubscription)
                            this.getLimits().done(function(limits) {
                                var tracksLeft = limits.totalUnauthenticatedTrackAllowance - limits.totalUnauthenticatedTracksUsed;
                                if (tracksLeft > 0) {
                                    var music2 = MS.Entertainment;
                                    music2 = music2.Music2;
                                    music2.UpsellDialogOverlay.showTracksLeft(tracksLeft)
                                }
                            })
                    };
                    FreePlayLimits.prototype.dispose = function() {
                        if (this._signInBindings) {
                            this._signInBindings.cancel();
                            this._signInBindings = null
                        }
                        if (this._signedInUserBindings) {
                            this._signedInUserBindings.cancel();
                            this._signedInUserBindings = null
                        }
                    };
                    Object.defineProperty(FreePlayLimits.prototype, "isMonthlyFreeLimitExceeded", {
                        get: function() {
                            return this._isMonthlyFreeLimitExceeded
                        }, enumerable: true, configurable: true
                    });
                    Object.defineProperty(FreePlayLimits.prototype, "isFreeTrialCompleted", {
                        get: function() {
                            return this._isFreeTrialCompleted
                        }, enumerable: true, configurable: true
                    });
                    FreePlayLimits.prototype.attachListener = function(serviceMediaId, listener) {
                        MS.Entertainment.Music.assert(!MS.Entertainment.Utilities.isEmptyGuid(serviceMediaId), "serviceMediaId param not valid");
                        MS.Entertainment.Music.assert(listener instanceof Function, "listener param is not a function");
                        var listeners = this._listeners[serviceMediaId];
                        if (!listeners)
                            this._listeners[serviceMediaId] = [listener];
                        else
                            listeners.push(listener)
                    };
                    FreePlayLimits.prototype.detachListener = function(serviceMediaId, listener) {
                        MS.Entertainment.Music.assert(!MS.Entertainment.Utilities.isEmptyGuid(serviceMediaId), "serviceMediaId param not valid");
                        MS.Entertainment.Music.assert(listener instanceof Function, "listener param is not a function");
                        var listeners = this._listeners[serviceMediaId];
                        if (listeners)
                            if (listeners.length === 1) {
                                MS.Entertainment.Music.assert(listeners[0] === listener, "listener not attached");
                                if (listeners[0] === listener)
                                    delete this._listeners[serviceMediaId]
                            }
                            else {
                                var index = listeners.indexOf(listener);
                                MS.Entertainment.Music.assert(index >= 0, "listener not attached");
                                if (index >= 0)
                                    listeners.splice(index, 1)
                            }
                    };
                    FreePlayLimits.prototype.addCollectionFilterListener = function(listener) {
                        var _this = this;
                        MS.Entertainment.Music.assert(listener instanceof Function, "listener param is not a function");
                        MS.Entertainment.Music.assert(this._collectionFilterListeners.indexOf(listener) === -1, "listener already attached");
                        this._collectionFilterListeners.push(listener);
                        if (this._queuedCollectionFilterEvent) {
                            this._sendCollectionFilterEvent(this._queuedCollectionFilterEvent);
                            this._queuedCollectionFilterEvent = null
                        }
                        return {cancel: function() {
                                    var index = _this._collectionFilterListeners.indexOf(listener);
                                    if (index >= 0)
                                        _this._collectionFilterListeners.splice(index, 1)
                                }}
                    };
                    FreePlayLimits.prototype.showFreeTrackLimitExceededDialog = function(mediaItem) {
                        var _this = this;
                        return this._hydrateMedia(mediaItem).then(function() {
                                MS.Entertainment.Music.Freeplay.sendTelemetryEvent(MS.Entertainment.Music.Freeplay.Events.freeTrackLimitExceeded);
                                var streamingTrackLimitDialogTitle = String.empty;
                                var streamingTrackLimitDialogBody = String.empty;
                                var purchasableMediaItem = null;
                                var actions = _this._getMediaItemActions(mediaItem);
                                var freeIndividualTrackAllowance = _this._musicLimitsManager.freeIndividualTrackAllowance;
                                var includeTrackAllowanceText = freeIndividualTrackAllowance > 0;
                                if (mediaItem && mediaItem.name && mediaItem.artistName && (actions.canBuyTrack || actions.canBuyAlbum)) {
                                    streamingTrackLimitDialogTitle = String.load(String.id.IDS_MUSIC_STREAMING_TRACK_LIMIT_TITLE).format(mediaItem.name, mediaItem.artistName);
                                    streamingTrackLimitDialogBody = includeTrackAllowanceText ? String.load(String.id.IDS_MUSIC_STREAMING_TRACK_LIMIT_BODY).format(freeIndividualTrackAllowance) : String.load(String.id.IDS_MUSIC_STREAMING_TRACK_LIMIT_BODY_NO_COUNT);
                                    purchasableMediaItem = actions.mediaItem
                                }
                                else {
                                    streamingTrackLimitDialogTitle = String.load(String.id.IDS_MUSIC_STREAMING_TRACK_LIMIT_MULTI_TITLE);
                                    streamingTrackLimitDialogBody = includeTrackAllowanceText ? String.load(String.id.IDS_MUSIC_STREAMING_TRACK_LIMIT_MULTI_BODY).format(freeIndividualTrackAllowance) : String.load(String.id.IDS_MUSIC_STREAMING_TRACK_LIMIT_MULTI_BODY_NO_COUNT)
                                }
                                return MS.Entertainment.Music.MusicBrandDialog.show(streamingTrackLimitDialogTitle, streamingTrackLimitDialogBody, null, actions.buttons, null, purchasableMediaItem)
                            })
                    };
                    FreePlayLimits.prototype.showMonthlyFreeLimitExceededDialog = function(mediaItem) {
                        var _this = this;
                        return this._hydrateMedia(mediaItem).then(function() {
                                MS.Entertainment.Music.Freeplay.sendTelemetryEvent(MS.Entertainment.Music.Freeplay.Events.monthlyFreeLimitExceeded);
                                var contentLink = {
                                        title: String.load(String.id.IDS_MUSIC_STREAMING_MONTHLY_LIMIT_PC_FILTER_LINK), accessabilityTitle: String.load(String.id.IDS_MUSIC_STREAMING_MONTHLY_LIMIT_PC_FILTER_LINK), isEnabled: true, canExecute: function canExecute() {
                                                return true
                                            }, execute: function() {
                                                var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                                                var currentPage = navigationService.currentPage;
                                                var event = MS.Entertainment.ViewModels.MusicCollectionAutomationIds.collectionFilterAvailableOffline;
                                                if (currentPage && currentPage.iaNode && currentPage.iaNode.moniker !== MS.Entertainment.UI.Monikers.musicCollection) {
                                                    navigationService.navigateTo(MS.Entertainment.UI.Monikers.musicCollection, MS.Entertainment.UI.Monikers.musicCollectionByAlbum, null, {selectHub: true});
                                                    _this._queueCollectionFilterEvent(event)
                                                }
                                                else
                                                    _this._sendCollectionFilterEvent(event)
                                            }
                                    };
                                var actions = _this._getMediaItemActions(mediaItem);
                                var monthlyFreeHoursAllowance = _this._getMonthlyFreeHoursAllowance();
                                var dateFormatter = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).dayMonthYear;
                                var nextMonthlyFreeLimitResetDate = String.empty;
                                try {
                                    nextMonthlyFreeLimitResetDate = dateFormatter.format(new Date(_this._musicLimitsManager.nextMonthlyFreeLimitResetDate))
                                }
                                catch(error) {
                                    MS.Entertainment.Music.fail("showMonthlyFreeLimitExceededDialog has failed because date parse failed. Error message = " + (error && error.message))
                                }
                                return MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_MONTHLY_LIMIT_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_MONTHLY_LIMIT_DESC).format(monthlyFreeHoursAllowance, nextMonthlyFreeLimitResetDate), contentLink, actions.buttons, null, actions.mediaItem)
                            })
                    };
                    FreePlayLimits.prototype.showTravelLimitExceededDialog = function() {
                        MS.Entertainment.Music.Freeplay.sendTelemetryEvent(MS.Entertainment.Music.Freeplay.Events.travelLimitReached);
                        return MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_TRAVEL_LIMIT_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_TRAVEL_LIMIT_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.subscribe, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.ok])
                    };
                    FreePlayLimits.prototype.showUnauthenticatedFreeLimitExceededDialog = function() {
                        if (WinJS.Utilities.getMember("MS.Entertainment.Music2.UpsellDialogOverlay")) {
                            var music2 = MS.Entertainment;
                            music2 = music2.Music2;
                            return music2.UpsellDialogOverlay.showTracksLeft(0)
                        }
                        else {
                            MS.Entertainment.Music.Freeplay.sendTelemetryEvent(MS.Entertainment.Music.Freeplay.Events.unauthenticatedTrackLimitExceeded);
                            var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                            var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                            if (!featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlayAnonymous))
                                return MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signUp, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel]);
                            else if (configurationManager.service.lastSignedInUserXuid)
                                return MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_EXISTING_USER_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_EXISTING_USER_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signInNowForExistingUser, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel]);
                            else {
                                var adService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.adService);
                                var showMusicBrandDialog = function showBrandedDialog() {
                                        return MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_NEW_USER_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_NEW_USER_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signInNowForNewUser, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel])
                                    };
                                if (!configurationManager.music.anonymousLimitReachedVideoShown && featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlayAnonAds)) {
                                    configurationManager.music.anonymousLimitReachedVideoShown = true;
                                    var showAdPromise;
                                    var uiStateChanged;
                                    var uiStateServiceBindings;
                                    var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                                    var signal = new MS.Entertainment.UI.Framework.Signal;
                                    uiStateChanged = function uiStateChanged() {
                                        if (uiStateService.isSnapped || !uiStateService.isAppVisible)
                                            return;
                                        if (uiStateServiceBindings) {
                                            uiStateServiceBindings.cancel();
                                            uiStateServiceBindings = null
                                        }
                                        signal.complete()
                                    };
                                    showAdPromise = function showAdPromise() {
                                        return adService.playVideoAd().then(function onAdPlayed(isSignedIn) {
                                                var returnPromise;
                                                var signinService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                                                if (!signinService.isSignedIn)
                                                    returnPromise = showMusicBrandDialog();
                                                return WinJS.Promise.as(returnPromise)
                                            })
                                    };
                                    if (uiStateService.isSnapped || !uiStateService.isAppVisible) {
                                        uiStateServiceBindings = WinJS.Binding.bind(uiStateService, {
                                            isSnapped: uiStateChanged, isAppVisible: uiStateChanged
                                        });
                                        return signal.promise.then(showAdPromise)
                                    }
                                    else
                                        return showAdPromise()
                                }
                                else
                                    return showMusicBrandDialog()
                            }
                        }
                    };
                    FreePlayLimits.prototype.getLimits = function() {
                        if (!this._initialized)
                            this.initialize();
                        if (!this._musicLimitsManager)
                            return WinJS.Promise.wrapError();
                        return this._initializedTimeout.then(function() {
                                var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                                if (!signIn.isSignedIn)
                                    return WinJS.Promise.wrapError()
                            }).then(this._musicLimitsManager.getLimitsAsync).then(function(limits) {
                                try {
                                    limits = JSON.parse(limits).result
                                }
                                catch(error) {
                                    MS.Entertainment.Music.fail("getLimitsAsync has failed because JSON.parse failed. error message = " + (error && error.message) + " json = " + limits);
                                    return WinJS.Promise.wrapError(error)
                                }
                                try {
                                    limits.nextMonthlyFreeLimitResetDate = MS.Entertainment.Data.Factory.date(limits.nextMonthlyFreeLimitResetDate)
                                }
                                catch(error) {
                                    MS.Entertainment.Music.fail("getLimitsAsync has failed because date parse failed. error message = " + (error && error.message) + " json = " + limits);
                                    return WinJS.Promise.wrapError(error)
                                }
                                return limits
                            })
                    };
                    FreePlayLimits.prototype._onSignInStateChanged = function() {
                        var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                        var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                        if (signIn.isSignedIn && !signedInUser.isSubscription) {
                            if (!this._eventBindings) {
                                this._eventBindings = [];
                                this._eventBindings.push(MS.Entertainment.Utilities.addEventHandlers(this._musicLimitsManager, {
                                    monthlyfreelimitexceeded: this._onMonthlyFreeLimitExceeded.bind(this), monthlyfreelimitreset: this._onMonthlyFreeLimitReset.bind(this), freetrialstarted: this._onFreeTrialStarted.bind(this), freetrialcompleted: this._onFreeTrialCompleted.bind(this), trackfreelimitexceeded: this._onFreeTrackLimitExceeded.bind(this), trackfreelimitsreset: this._onFreeTrackLimitReset.bind(this)
                                }));
                                this._eventBindings.push(MS.Entertainment.Utilities.addEventHandlers(this._sessionManager.primarySession, {currentMediaChanged: this._currentMediaChanged.bind(this)}))
                            }
                            this._isMonthlyFreeLimitExceeded = this._musicLimitsManager.isMonthlyFreeLimitExceeded;
                            this._isFreeTrialCompleted = this._musicLimitsManager.isFreeTrialCompleted
                        }
                        else {
                            if (this._eventBindings) {
                                this._eventBindings.forEach(function(eventBinding) {
                                    eventBinding.cancel()
                                });
                                this._eventBindings = null
                            }
                            this._isMonthlyFreeLimitExceeded = false;
                            this._isFreeTrialCompleted = false
                        }
                        if (signIn.isSignedIn) {
                            if (this._refreshingLimits) {
                                this._refreshingLimits.cancel();
                                this._refreshingLimits = null
                            }
                            this._musicLimitsManager.getLimitsAsync()
                        }
                        this._dispatchEvent(this.isMonthlyFreeLimitExceeded ? MS.Entertainment.Music.Freeplay.Events.monthlyFreeLimitExceeded : MS.Entertainment.Music.Freeplay.Events.monthlyFreeLimitReset)
                    };
                    FreePlayLimits.prototype._onMonthlyFreeLimitExceeded = function() {
                        var _this = this;
                        this._isMonthlyFreeLimitExceeded = true;
                        this._sendAppNotification(String.load(String.id.IDS_MUSIC_STREAMING_MONTHLY_LIMIT_NOTIFICATION_LINE1), String.load(String.id.IDS_MUSIC_STREAMING_MONTHLY_LIMIT_NOTIFICATION_LINE2), MS.Entertainment.UI.Icon.notification, function() {
                            _this.showMonthlyFreeLimitExceededDialog()
                        });
                        this._dispatchEvent(MS.Entertainment.Music.Freeplay.Events.monthlyFreeLimitExceeded)
                    };
                    FreePlayLimits.prototype._onMonthlyFreeLimitReset = function() {
                        this._isMonthlyFreeLimitExceeded = false;
                        var monthlyFreeHoursAllowance = this._getMonthlyFreeHoursAllowance();
                        this._sendAppNotification(String.load(String.id.IDS_MUSIC_STREAMING_MONTHLY_RESET_NOTIFICATION_LINE1), String.load(String.id.IDS_MUSIC_STREAMING_MONTHLY_RESET_NOTIFICATION_LINE2).format(monthlyFreeHoursAllowance), MS.Entertainment.UI.Icon.notification);
                        this._dispatchEvent(MS.Entertainment.Music.Freeplay.Events.monthlyFreeLimitReset)
                    };
                    FreePlayLimits.prototype._onFreeTrialStarted = function() {
                        this._isFreeTrialCompleted = false;
                        this._dispatchEvent(MS.Entertainment.Music.Freeplay.Events.freeTrialStarted)
                    };
                    FreePlayLimits.prototype._onFreeTrialCompleted = function() {
                        var _this = this;
                        this._isFreeTrialCompleted = true;
                        if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.appNotification))
                            this._sendAppNotification(String.load(String.id.IDS_MUSIC_STREAMING_TRIAL_END_NOTIFICATION_LINE1), String.load(String.id.IDS_MUSIC_STREAMING_TRIAL_END_NOTIFICATION_LINE2), MS.Entertainment.UI.Icon.notification, WinJS.Utilities.markSupportedForProcessing(function() {
                                _this._showFreeTrialCompletedDialog()
                            }.bind(this)));
                        this._dispatchEvent(MS.Entertainment.Music.Freeplay.Events.freeTrialCompleted)
                    };
                    FreePlayLimits.prototype._onFreeTrackLimitExceeded = function(serviceMediaId) {
                        this._dispatchEvent(MS.Entertainment.Music.Freeplay.Events.freeTrackLimitExceeded, serviceMediaId)
                    };
                    FreePlayLimits.prototype._onFreeTrackLimitReset = function() {
                        this._dispatchEvent(MS.Entertainment.Music.Freeplay.Events.freeTrackLimitReset)
                    };
                    FreePlayLimits.prototype._dispatchEvent = function(event, serviceMediaId) {
                        var _this = this;
                        var dispatchEvent = function(serviceMediaId) {
                                var listeners = _this._listeners[serviceMediaId];
                                if (listeners)
                                    listeners.forEach(function(listener) {
                                        listener(event)
                                    })
                            };
                        if (serviceMediaId)
                            dispatchEvent(serviceMediaId);
                        else
                            for (var serviceMediaId in this._listeners)
                                dispatchEvent(serviceMediaId)
                    };
                    FreePlayLimits.prototype._sendAppNotification = function(title, subTitle, icon, action) {
                        var appNotification = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appNotification);
                        appNotification.send(new MS.Entertainment.UI.Notification({
                            notificationType: MS.Entertainment.UI.Notification.Type.Critical, title: title, subTitle: subTitle, moreDetails: null, icon: icon, action: action, category: MS.Entertainment.Music.FreePlayLimits._notificationCategory, isPersistent: true, dismissOnSignOut: true
                        }))
                    };
                    FreePlayLimits.prototype._queueCollectionFilterEvent = function(event) {
                        this._queuedCollectionFilterEvent = event
                    };
                    FreePlayLimits.prototype._sendCollectionFilterEvent = function(event) {
                        this._collectionFilterListeners.forEach(function sendCollectionFilterEvent(listener) {
                            listener(event)
                        })
                    };
                    FreePlayLimits.prototype._showFreeTrialCompletedDialog = function() {
                        MS.Entertainment.Music.Freeplay.sendTelemetryEvent(MS.Entertainment.Music.Freeplay.Events.freeTrialCompleted);
                        var monthlyFreeHoursAllowance = this._getMonthlyFreeHoursAllowance();
                        return MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_TRIAL_END_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_TRIAL_END_BODY).format(monthlyFreeHoursAllowance), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.subscribe, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.ok])
                    };
                    FreePlayLimits.prototype._getMediaItemActions = function(mediaItem) {
                        var canBuyTrack = false;
                        var canBuyAlbum = false;
                        if (mediaItem && mediaItem.rights)
                            for (var i = 0; i < mediaItem.rights.length; i++) {
                                var right = mediaItem.rights[i];
                                if (right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Purchase) {
                                    canBuyTrack = true;
                                    break
                                }
                                else if (right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.AlbumPurchase)
                                    canBuyAlbum = true
                            }
                        var buttons = [];
                        if (canBuyTrack)
                            buttons.push(MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.buyTrack);
                        else if (canBuyAlbum)
                            buttons.push(MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.buyAlbum);
                        buttons.push(MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.subscribe);
                        buttons.push(MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.ok);
                        return {
                                canBuyTrack: canBuyTrack, canBuyAlbum: canBuyAlbum, buttons: buttons, mediaItem: (mediaItem && canBuyAlbum) ? mediaItem.album : mediaItem
                            }
                    };
                    FreePlayLimits.prototype._getMonthlyFreeHoursAllowance = function() {
                        return Math.max(Math.floor(this._musicLimitsManager.monthlyFreeMinutesAllowance / 60), 0)
                    };
                    FreePlayLimits.prototype._hydrateMedia = function(mediaItem) {
                        var promise = WinJS.Promise.as();
                        if (mediaItem) {
                            var isOnline;
                            switch (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).networkStatus) {
                                case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.unknown:
                                case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.unrestricted:
                                case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.throttled:
                                case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.onDemand:
                                    isOnline = true;
                                    break;
                                case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.localOnly:
                                case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.none:
                                    isOnline = false;
                                    break;
                                default:
                                    MS.Entertainment.Music.fail("unexpected network status value");
                                    break
                            }
                            if (isOnline && mediaItem.hydrate)
                                promise = mediaItem.hydrate({forceUpdate: true})
                        }
                        return promise
                    };
                    FreePlayLimits.isFreeStreamingTrack = function(mediaItem) {
                        return !!(mediaItem && mediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.track && (mediaItem.canFreeStream || mediaItem.isFreeStreamRestrictedByIndividualLimit || mediaItem.isFreeStreamRestrictedByMonthlyLimit) && !mediaItem.hasPurchased && !mediaItem.canPlayLocally && !MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser).isSubscription)
                    };
                    FreePlayLimits.factory = function() {
                        return new MS.Entertainment.Music.FreePlayLimits
                    };
                    FreePlayLimits._notificationCategory = "AuthenticatedFreeStreamNotification";
                    return FreePlayLimits
                })();
            Music.FreePlayLimits = FreePlayLimits
        })(Entertainment.Music || (Entertainment.Music = {}));
        var Music = Entertainment.Music
    })(MS.Entertainment || (MS.Entertainment = {}));
    var Entertainment = MS.Entertainment
})(MS || (MS = {}));
MS.Entertainment.ServiceLocator.register(MS.Entertainment.Services.freePlayLimits, MS.Entertainment.Music.FreePlayLimits.factory)
