﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.ViewModels");
    var fileScope = {
            MAX_TRACK_BUTTON_NUM: 2, MAX_POPOVER_BUTTON_NUM: 4
        };
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {MusicStateHandlers: {
            _getAddToSubActionsForAlbum: function _getAddToSubActionsForAlbum(appState, stateInfo, hasAddToMyMusicRights) {
                MS.Entertainment.UI.assert(appState, "appState is required!");
                MS.Entertainment.UI.assert(stateInfo, "stateInfo is required!");
                var addToSubActions = [];
                if (appState.canAddToMyMusic && hasAddToMyMusicRights)
                    addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToMyMusic);
                else if (appState.canAddToCloud)
                    addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToCloud);
                if (appState.canAddToNowPlaying && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                    addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                return addToSubActions
            }, _addButtonsForAlbumDownloading: function _addButtonsForAlbumDownloading(appState, stateInfo, isfromCollection, hasAddToMyMusicRights, albumButtons) {
                    MS.Entertainment.UI.assert(appState, "appState is required!");
                    MS.Entertainment.UI.assert(stateInfo, "stateInfo is required!");
                    MS.Entertainment.UI.assert(albumButtons, "albumButtons is required!");
                    var buttons = [];
                    var actions = [];
                    var preconditionPromises = [];
                    var existingStartScreenTile = null;
                    if (appState.canPinToStartScreen) {
                        var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                        preconditionPromises.push(pinToStartScreenService.getTileForMediaAsync(this.media).then(function getTileForMediaAsync_complete(startScreenTile) {
                            existingStartScreenTile = startScreenTile
                        }))
                    }
                    return WinJS.Promise.join(preconditionPromises).then(function preconditionsComplete() {
                            if (isfromCollection || hasAddToMyMusicRights)
                                if (stateInfo.canPlay && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                    buttons.push(albumButtons.play);
                                else
                                    buttons.push(albumButtons.playDisabled);
                            else
                                buttons.push(albumButtons.albumPlayPreview);
                            albumButtons.addTo.setSubActions(MS.Entertainment.ViewModels.MusicStateHandlers._getAddToSubActionsForAlbum(appState, stateInfo, hasAddToMyMusicRights));
                            buttons.push(albumButtons.addTo);
                            if (appState.showArtistDetailsButton)
                                buttons.push(albumButtons.artistDetails);
                            if (appState.canPlaySmartDJ)
                                buttons.push(albumButtons.playArtistSmartDJ);
                            if (appState.canPlayOnXbox)
                                actions.push(appState.actions.playOnXbox360);
                            if (appState.canPinToStartScreen)
                                if (!existingStartScreenTile)
                                    actions.push(appState.actions.pinToStartScreen);
                                else
                                    actions.push(appState.actions.unpinFromStartScreen);
                            return WinJS.Promise.wrap({
                                    smartButtons: buttons, appbarActions: actions
                                })
                        })
                }, _addButtonsForAlbumDownloadFailed: function _addButtonsForAlbumDownloadFailed(appState, stateInfo, isfromCollection, hasAddToMyMusicRights, albumButtons) {
                    MS.Entertainment.UI.assert(appState, "appState is required!");
                    MS.Entertainment.UI.assert(stateInfo, "stateInfo is required!");
                    MS.Entertainment.UI.assert(albumButtons, "albumButtons is required!");
                    var buttons = [];
                    var actions = [];
                    var preconditionPromises = [];
                    var existingStartScreenTile = null;
                    if (appState.canPinToStartScreen) {
                        var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                        preconditionPromises.push(pinToStartScreenService.getTileForMediaAsync(this.media).then(function getTileForMediaAsync_complete(startScreenTile) {
                            existingStartScreenTile = startScreenTile
                        }))
                    }
                    return WinJS.Promise.join(preconditionPromises).then(function preconditionsComplete() {
                            if (isfromCollection || hasAddToMyMusicRights) {
                                if (stateInfo.canPlay && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                    buttons.push(albumButtons.play)
                            }
                            else
                                buttons.push(albumButtons.albumPlayPreview);
                            albumButtons.addTo.setSubActions(MS.Entertainment.ViewModels.MusicStateHandlers._getAddToSubActionsForAlbum(appState, stateInfo, hasAddToMyMusicRights));
                            buttons.push(albumButtons.addTo);
                            if (stateInfo.download.hasRetryable)
                                buttons.push(albumButtons.tryAgain);
                            else
                                buttons.push(albumButtons.remove);
                            if (appState.showArtistDetailsButton)
                                buttons.push(albumButtons.artistDetails);
                            if (appState.canPlaySmartDJ)
                                if (buttons.length < fileScope.MAX_POPOVER_BUTTON_NUM)
                                    buttons.push(albumButtons.playArtistSmartDJ);
                                else
                                    actions.push(appState.actions.playSmartDJ);
                            if (appState.canPlayOnXbox)
                                actions.push(appState.actions.playOnXbox360);
                            if (appState.canPinToStartScreen)
                                if (!existingStartScreenTile)
                                    actions.push(appState.actions.pinToStartScreen);
                                else
                                    actions.push(appState.actions.unpinFromStartScreen);
                            if (appState.canDelete)
                                actions.push(appState.actions.deleteMedia);
                            return WinJS.Promise.wrap({
                                    smartButtons: buttons, appbarActions: actions
                                })
                        })
                }, _addButtonsForAlbumDownloadPending: function _addButtonsForAlbumDownloadPending(appState, stateInfo, isfromCollection, hasAddToMyMusicRights, albumButtons) {
                    MS.Entertainment.UI.assert(appState, "appState is required!");
                    MS.Entertainment.UI.assert(stateInfo, "stateInfo is required!");
                    MS.Entertainment.UI.assert(albumButtons, "albumButtons is required!");
                    var buttons = [];
                    var actions = [];
                    var preconditionPromises = [];
                    var existingStartScreenTile = null;
                    if (appState.canPinToStartScreen) {
                        var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                        preconditionPromises.push(pinToStartScreenService.getTileForMediaAsync(this.media).then(function getTileForMediaAsync_complete(startScreenTile) {
                            existingStartScreenTile = startScreenTile
                        }))
                    }
                    return WinJS.Promise.join(preconditionPromises).then(function preconditionsComplete() {
                            if (isfromCollection || hasAddToMyMusicRights) {
                                if (stateInfo.canPlay && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                    buttons.push(albumButtons.play)
                            }
                            else
                                buttons.push(albumButtons.albumPlayPreview);
                            albumButtons.addTo.setSubActions(MS.Entertainment.ViewModels.MusicStateHandlers._getAddToSubActionsForAlbum(appState, stateInfo, hasAddToMyMusicRights));
                            buttons.push(albumButtons.addTo);
                            buttons.push(albumButtons.download);
                            if (appState.showArtistDetailsButton)
                                buttons.push(albumButtons.artistDetails);
                            if (appState.canPlaySmartDJ)
                                if (buttons.length < fileScope.MAX_POPOVER_BUTTON_NUM)
                                    buttons.push(albumButtons.playArtistSmartDJ);
                                else
                                    actions.push(appState.actions.playSmartDJ);
                            if (appState.canPlayOnXbox)
                                actions.push(appState.actions.playOnXbox360);
                            if (appState.canPinToStartScreen)
                                if (!existingStartScreenPin)
                                    actions.push(appState.actions.pinToStartScreen);
                                else
                                    actions.push(appState.actions.unpinFromStartScreen);
                            return WinJS.Promise.wrap({
                                    smartButtons: buttons, appbarActions: actions
                                })
                        })
                }, _getAddToMyMusicRights: function _getAddToMyMusicRights(appState, media) {
                    MS.Entertainment.UI.assert(appState, "appState is required!");
                    MS.Entertainment.UI.assert(media, "media is required!");
                    return MS.Entertainment.ViewModels.MusicSmartBuyStateHandlers.getAlbumRights(media).then(function getAlbumRights_complete(rights) {
                            var allowSubscriptionActions = appState.musicSubscriptionEnabled && appState.signedInUserHasSubscription;
                            var allowFreeStreamActions = appState.canFreeStream;
                            var hasSubscriptionDownloadRights = allowSubscriptionActions && rights.subscriptionDownload;
                            var hasSubscriptionStreamingRights = allowSubscriptionActions && rights.subscriptionStream;
                            var hasFreeStreamingRights = allowFreeStreamActions && rights.freeStream;
                            return hasSubscriptionStreamingRights || hasFreeStreamingRights || hasSubscriptionDownloadRights
                        })
                }, onAlbumPopoverStateChanged: function onAlbumPopoverStateChanged(stateInfo) {
                    MS.Entertainment.UI.assert(this.media, "this.media should be defined!");
                    var buttons = [];
                    var actions = [];
                    var addToSubActions = [];
                    var preconditionPromises = [];
                    var appState = MS.Entertainment.ViewModels._MusicState.getCurrentState(this.media, stateInfo);
                    var showCollectionDetails = MS.Entertainment.ViewModels.MusicStateHandlers.shouldShowCollectionDetails(this.media, stateInfo);
                    var existingStartScreenTile = null;
                    if (appState.canPinToStartScreen) {
                        var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                        preconditionPromises.push(pinToStartScreenService.getTileForMediaAsync(this.media).then(function getTileForMediaAsync_complete(startScreenTile) {
                            existingStartScreenTile = startScreenTile
                        }))
                    }
                    return WinJS.Promise.join(preconditionPromises).then(function preconditionsComplete() {
                            if (stateInfo.download.hasActive)
                                return MS.Entertainment.ViewModels.MusicStateHandlers._getAddToMyMusicRights(appState, this.media).then(function getAddToMyMusicRights_complete(hasAddToMyMusicRights) {
                                        return MS.Entertainment.ViewModels.MusicStateHandlers._addButtonsForAlbumDownloading(appState, stateInfo, this.media && this.media.fromCollection, hasAddToMyMusicRights, this.buttons)
                                    }.bind(this));
                            else if (stateInfo.collection.canPlayLocallyOrStreamFromCloud && showCollectionDetails) {
                                if ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession)
                                    if (stateInfo.collection.canPlayLocally || stateInfo.collection.canPlayLocallyOrStreamFromCloud)
                                        buttons.push(this.buttons.play);
                                    else
                                        buttons.push(this.buttons.playDisabled);
                                else
                                    buttons.push(this.buttons.remotePlayDisabled);
                                if (appState.canAddToCloud)
                                    addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToCloud);
                                if (appState.canAddToNowPlaying && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                    addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                this.buttons.addTo.setSubActions(addToSubActions);
                                buttons.push(this.buttons.addTo);
                                if (appState.showArtistDetailsButton)
                                    buttons.push(this.buttons.artistDetails);
                                if (appState.canPlaySmartDJ)
                                    buttons.push(this.buttons.playArtistSmartDJ);
                                if (appState.canPlayOnXbox)
                                    actions.push(appState.actions.playOnXbox360);
                                if (appState.canPinToStartScreen)
                                    if (!existingStartScreenTile)
                                        actions.push(appState.actions.pinToStartScreen);
                                    else
                                        actions.push(appState.actions.unpinFromStartScreen);
                                if (appState.canDelete)
                                    actions.push(appState.actions.deleteMedia);
                                if (appState.canFindAlbumInfo)
                                    actions.push(appState.actions.findAlbumInfo);
                                if ((this.media.hasPurchasableTracks || !this.media.fromCollection) && appState.musicMarketplaceConnected && this.media.tracks && stateInfo.marketplace.hasPurchased < this.media.tracks.count)
                                    actions.push(appState.actions.purchaseAlbumFromMarketplace);
                                MS.Entertainment.ViewModels.MusicStateHelpers._getDownloadAppBarActions(appState, stateInfo, this.media).forEach(function addAction(action) {
                                    actions.push(action)
                                })
                            }
                            else if (stateInfo.download.hasFailed)
                                return MS.Entertainment.ViewModels.MusicStateHandlers._getAddToMyMusicRights(appState, this.media).then(function getAddToMyMusicRights_complete(hasAddToMyMusicRights) {
                                        return MS.Entertainment.ViewModels.MusicStateHandlers._addButtonsForAlbumDownloadFailed(appState, stateInfo, this.media && this.media.fromCollection, hasAddToMyMusicRights, this.buttons)
                                    }.bind(this));
                            else if (stateInfo.download.hasPending)
                                return MS.Entertainment.ViewModels.MusicStateHandlers._getAddToMyMusicRights(appState, this.media).then(function getAddToMyMusicRights_complete(hasAddToMyMusicRights) {
                                        return MS.Entertainment.ViewModels.MusicStateHandlers._addButtonsForAlbumDownloadPending(appState, stateInfo, this.media && this.media.fromCollection, hasAddToMyMusicRights, this.buttons)
                                    }.bind(this));
                            else if (this.media.location === MS.Entertainment.Data.ItemLocation.collection) {
                                if (appState.showArtistDetailsButton)
                                    buttons.push(this.buttons.artistDetails);
                                if (appState.canPlaySmartDJ)
                                    buttons.push(this.buttons.playArtistSmartDJ);
                                if (appState.canPinToStartScreen)
                                    if (!existingStartScreenTile)
                                        actions.push(appState.actions.pinToStartScreen);
                                    else
                                        actions.push(appState.actions.unpinFromStartScreen);
                                if (appState.canDelete)
                                    actions.push(appState.actions.deleteMedia)
                            }
                            else {
                                if (!this.media.hydrated || !this.media.tracks)
                                    return WinJS.Promise.wrap(null);
                                return MS.Entertainment.ViewModels.MusicSmartBuyStateHandlers.getAlbumRights(this.media).then(function getAlbumRights_complete(rights) {
                                        var allowSubscriptionActions = appState.musicSubscriptionEnabled && appState.signedInUserHasSubscription;
                                        var allowFreeStreamActions = appState.canFreeStream;
                                        var allowSubscriptionOrFreeStreamActions = (allowSubscriptionActions || allowFreeStreamActions);
                                        var hasSubscriptionDownloadRights = allowSubscriptionActions && rights.subscriptionDownload;
                                        var hasSubscriptionStreamingRights = allowSubscriptionActions && rights.subscriptionStream;
                                        var hasFreeStreamingRights = allowFreeStreamActions && rights.freeStream;
                                        var hasAddToMyMusicRights = hasSubscriptionStreamingRights || hasFreeStreamingRights || hasSubscriptionDownloadRights;
                                        var showMusicPassOptions = allowSubscriptionActions && (hasSubscriptionStreamingRights || hasSubscriptionDownloadRights);
                                        var showFreeStreamOptions = allowFreeStreamActions && hasFreeStreamingRights;
                                        if (showMusicPassOptions) {
                                            if (rights.subscriptionStream)
                                                if (!(Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) && appState.isRemoteSession)
                                                    buttons.push(this.buttons.remotePlayDisabled);
                                                else
                                                    buttons.push(this.buttons.play);
                                            if (appState.canAddToMyMusic && hasAddToMyMusicRights)
                                                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToMyMusic);
                                            if (appState.canAddToPlaylist && appState.isSignedIn)
                                                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                            if (appState.canAddToNowPlaying && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                                            if (addToSubActions.length > 0) {
                                                this.buttons.addTo.setSubActions(addToSubActions);
                                                buttons.push(this.buttons.addTo)
                                            }
                                            if (rights.subscriptionDownload && !appState.downloadOnAddToMyMusicCloud)
                                                actions.push(appState.actions.subscriptionDownload);
                                            if (rights.albumPurchase)
                                                buttons.push(this.buttons.buyAlbum);
                                            if (appState.showArtistDetailsButton)
                                                buttons.push(this.buttons.artistDetails);
                                            if (appState.canPlaySmartDJ && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                                actions.push(appState.actions.playSmartDJ);
                                            if (appState.canPlayOnXbox || (appState.musicMarketplaceConnected && appState.consoleRegionEnabled && rights.playOnXbox))
                                                actions.push(appState.actions.playOnXbox360)
                                        }
                                        else if (showFreeStreamOptions) {
                                            if ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession)
                                                buttons.push(this.buttons.play);
                                            if (appState.canAddToMyMusic && hasAddToMyMusicRights)
                                                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToMyMusic);
                                            if (appState.canAddToPlaylist && appState.isSignedIn)
                                                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                            if (appState.canAddToNowPlaying && appState.isSignedIn)
                                                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                                            if (addToSubActions.length > 0) {
                                                this.buttons.addTo.setSubActions(addToSubActions);
                                                buttons.push(this.buttons.addTo)
                                            }
                                            if (rights.albumPurchase)
                                                buttons.push(this.buttons.buyAlbum);
                                            if (appState.showArtistDetailsButton)
                                                buttons.push(this.buttons.artistDetails);
                                            if (appState.canPlaySmartDJ)
                                                actions.push(appState.actions.playSmartDJ);
                                            if (appState.canPlayOnXbox)
                                                actions.push(appState.actions.playOnXbox360)
                                        }
                                        else {
                                            if (rights.preview && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                                buttons.push(this.buttons.albumPlayPreview);
                                            if (stateInfo.marketplace.hasPurchased && appState.cloudServiceEnabled)
                                                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToMyMusic);
                                            if (appState.canAddToNowPlaying && rights.preview && (!appState.canFreeStream || appState.isSignedIn))
                                                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                                            if (addToSubActions.length > 0) {
                                                this.buttons.addTo.setSubActions(addToSubActions);
                                                buttons.push(this.buttons.addTo)
                                            }
                                            if (rights.albumPurchase)
                                                buttons.push(this.buttons.buyAlbum);
                                            if (appState.showArtistDetailsButton)
                                                buttons.push(this.buttons.artistDetails);
                                            if (appState.canPlaySmartDJ)
                                                if (buttons.length < 4)
                                                    buttons.push(this.buttons.playArtistSmartDJ);
                                                else
                                                    actions.push(appState.actions.playSmartDJ);
                                            if (appState.canPlayOnXbox)
                                                actions.push(appState.actions.playOnXbox360)
                                        }
                                        if (appState.canPinToStartScreen)
                                            if (!existingStartScreenTile)
                                                actions.push(appState.actions.pinToStartScreen);
                                            else
                                                actions.push(appState.actions.unpinFromStartScreen);
                                        if (!buttons.length)
                                            buttons.push(this.buttons.contentNotAvailable);
                                        return WinJS.Promise.wrap({
                                                smartButtons: buttons, appbarActions: actions
                                            })
                                    }.bind(this))
                            }
                            return WinJS.Promise.wrap({
                                    smartButtons: buttons, appbarActions: actions
                                })
                        }.bind(this))
                }, onPlaylistPopoverStateChanged: function onPlaylistPopoverStateChanged(stateInfo) {
                    var buttons = [];
                    var actions = [];
                    var addToSubActions = [];
                    var preconditionPromises = [];
                    var appState = MS.Entertainment.ViewModels._MusicState.getCurrentState(this.media, stateInfo);
                    var existingStartScreenTile = null;
                    if (appState.canPinToStartScreen) {
                        var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                        preconditionPromises.push(pinToStartScreenService.getTileForMediaAsync(this.media).then(function getTileForMediaAsync_complete(startScreenTile) {
                            existingStartScreenTile = startScreenTile
                        }))
                    }
                    return WinJS.Promise.join(preconditionPromises).then(function preconditionsComplete() {
                            if (stateInfo.collection.canPlayLocally || this.media.inCollection)
                                if (this.media.count > 0) {
                                    buttons.push(this.buttons.play);
                                    if (appState.canAddToNowPlaying)
                                        addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                                    addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist)
                                }
                                else
                                    buttons.push(this.buttons.playDisabled);
                            else
                                buttons.push(this.buttons.playDisabled);
                            if (addToSubActions.length > 0) {
                                this.buttons.addTo.setSubActions(addToSubActions);
                                buttons.push(this.buttons.addTo)
                            }
                            actions.push(appState.actions.renamePlaylist);
                            actions.push(appState.actions.deleteMedia);
                            MS.Entertainment.ViewModels.MusicStateHelpers._getDownloadAppBarActions(appState, stateInfo, this.media).forEach(function addAction(action) {
                                actions.push(action)
                            });
                            if (appState.canPinToStartScreen)
                                if (!existingStartScreenTile)
                                    actions.push(appState.actions.pinToStartScreen);
                                else
                                    actions.push(appState.actions.unpinFromStartScreen);
                            return WinJS.Promise.wrap({
                                    smartButtons: buttons, appbarActions: actions
                                })
                        }.bind(this))
                }, onLocalArtistPopoverStateChanged: function onLocalArtistPopoverStateChanged(stateInfo) {
                    var buttons = [];
                    var actions = [];
                    var addToSubActions = [];
                    var preconditionPromises = [];
                    var appState = MS.Entertainment.ViewModels._MusicState.getCurrentState(this.media, stateInfo);
                    var existingStartScreenTile = null;
                    if (appState.canPinToStartScreen) {
                        var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                        preconditionPromises.push(pinToStartScreenService.getTileForMediaAsync(this.media).then(function getTileForMediaAsync_complete(startScreenTile) {
                            existingStartScreenTile = startScreenTile
                        }))
                    }
                    return WinJS.Promise.join(preconditionPromises).then(function preconditionsComplete() {
                            if (stateInfo.collection.canPlayLocallyOrStreamFromCloud) {
                                if ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally)
                                    buttons.push(this.buttons.playAllLocalArtist);
                                else
                                    buttons.push(this.buttons.remotePlayDisabled);
                                if (appState.canAddToNowPlaying && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                    addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addLocalArtistToNowPlaying);
                                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                this.buttons.addTo.setSubActions(addToSubActions);
                                buttons.push(this.buttons.addTo);
                                if (appState.canDelete)
                                    actions.push(appState.actions.deleteMedia)
                            }
                            else if (stateInfo.download.hasActive)
                                if (appState.signedInUserHasSubscription && appState.musicSubscriptionEnabled && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                    buttons.push(this.buttons.playAllMarketplaceArtist);
                                else
                                    buttons.push(this.buttons.playDisabled);
                            else if (stateInfo.download.hasFailed) {
                                if (appState.signedInUserHasSubscription && appState.musicSubscriptionEnabled && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                    buttons.push(this.buttons.playAllMarketplaceArtist);
                                else
                                    buttons.push(this.buttons.playDisabled);
                                if (stateInfo.download.hasRetryable)
                                    buttons.push(this.buttons.tryAgain);
                                else
                                    buttons.push(this.buttons.remove);
                                if (appState.canDelete)
                                    actions.push(appState.actions.deleteMedia)
                            }
                            else if (stateInfo.download.hasPending) {
                                if (appState.signedInUserHasSubscription && appState.musicSubscriptionEnabled && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                    buttons.push(this.buttons.playAllMarketplaceArtist);
                                else
                                    buttons.push(this.buttons.playDisabled);
                                buttons.push(this.buttons.download)
                            }
                            if (appState.showArtistDetailsButton)
                                buttons.push(this.buttons.artistDetails);
                            if (appState.canPlaySmartDJ)
                                buttons.push(this.buttons.playArtistSmartDJ);
                            if (appState.canPlayOnXbox)
                                actions.push(appState.actions.playOnXbox360);
                            if (appState.canPinToStartScreen)
                                if (!existingStartScreenTile)
                                    actions.push(appState.actions.pinToStartScreen);
                                else
                                    actions.push(appState.actions.unpinFromStartScreen);
                            if (!buttons.length)
                                buttons.push(this.buttons.contentNotAvailable);
                            return WinJS.Promise.wrap({
                                    smartButtons: buttons, appbarActions: actions
                                })
                        }.bind(this))
                }, onMarketplaceArtistPopoverStateChanged: function onMarketplaceArtistPopoverStateChanged(stateInfo, marketplaceTracks) {
                    var buttons = [];
                    var actions = [];
                    var addToSubActions = [];
                    var preconditionPromises = [];
                    var appState = MS.Entertainment.ViewModels._MusicState.getCurrentState(this.media, stateInfo);
                    var existingStartScreenTile = null;
                    if (appState.canPinToStartScreen) {
                        var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                        preconditionPromises.push(pinToStartScreenService.getTileForMediaAsync(this.media).then(function getTileForMediaAsync_complete(startScreenTile) {
                            existingStartScreenTile = startScreenTile
                        }))
                    }
                    return WinJS.Promise.join(preconditionPromises).then(function preconditionsComplete() {
                            if (((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession) && (!marketplaceTracks || marketplaceTracks.count > 0)) {
                                var canFreeStream = appState.canFreeStream;
                                var canSubscriptionStream = (appState.signedInUserHasSubscription && appState.musicSubscriptionEnabled);
                                var canAddToNowPlaying = appState.canAddToNowPlaying;
                                if (canFreeStream || canSubscriptionStream)
                                    buttons.push(this.buttons.playAllMarketplaceArtist);
                                else
                                    buttons.push(this.buttons.previewArtist);
                                if (appState.canAddToNowPlaying)
                                    addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addMarketplaceArtistToNowPlaying);
                                if (appState.canAddToPlaylist && appState.isSignedIn)
                                    addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist)
                            }
                            if (addToSubActions.length) {
                                this.buttons.addTo.setSubActions(addToSubActions);
                                buttons.push(this.buttons.addTo)
                            }
                            if (appState.showArtistDetailsButton)
                                buttons.push(this.buttons.artistDetails);
                            if (appState.canPlaySmartDJ)
                                buttons.push(this.buttons.playArtistSmartDJ);
                            if (appState.canPlayOnXbox)
                                actions.push(appState.actions.playOnXbox360);
                            if (appState.canPinToStartScreen)
                                if (!existingStartScreenTile)
                                    actions.push(appState.actions.pinToStartScreen);
                                else
                                    actions.push(appState.actions.unpinFromStartScreen);
                            if (!buttons.length)
                                buttons.push(this.buttons.contentNotAvailable);
                            return WinJS.Promise.wrap({
                                    smartButtons: buttons, appbarActions: actions
                                })
                        }.bind(this))
                }, onTrackInlineDetailsStateChanged: function onTrackInlineDetailsStateChanged(stateInfo) {
                    var buttons = [];
                    var actions = new MS.Entertainment.UI.Actions.ActionArray;
                    var addToSubActions = [];
                    var promiseArray = [];
                    var preconditionPromises = [];
                    var appState = MS.Entertainment.ViewModels._MusicState.getCurrentState(this.media, stateInfo);
                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                    var inNowPlaying = false;
                    if (mediaContext && mediaContext.options && mediaContext.options.containingMedia)
                        inNowPlaying = MS.Entertainment.Platform.Playback.PlaybackSession.isPlaybackSession(mediaContext.options.containingMedia.playbackItemSource);
                    var existingStartScreenTile = null;
                    if (appState.canPinToStartScreen) {
                        var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                        preconditionPromises.push(pinToStartScreenService.getTileForMediaAsync(this.media).then(function getTileForMediaAsync_complete(startScreenTile) {
                            existingStartScreenTile = startScreenTile
                        }))
                    }
                    return WinJS.Promise.join(preconditionPromises).then(function preconditionsComplete() {
                            if (appState.isPlaylistTrack && !inNowPlaying) {
                                if (appState.freeStreamBlockedByFreeLimits) {
                                    buttons.push(this.media.canPreview ? this.buttons.trackPlayPreview : this.buttons.playDisabled);
                                    if (MS.Entertainment.Utilities.hasPropertyPath(mediaContext, "options.containingMedia.editableItemSource")) {
                                        actions.push(appState.actions.moveListItemUp);
                                        actions.push(appState.actions.moveListItemDown);
                                        buttons.push(this.buttons.removeFromList)
                                    }
                                    else
                                        buttons.push(this.buttons.removeFromPlaylist);
                                    if (!appState.isSnapped)
                                        if (this.media.canPurchase)
                                            actions.push(appState.actions.purchaseTrackFromMarketplace);
                                        else if (this.media.canPurchaseAlbum)
                                            actions.push(appState.actions.purchaseAlbumFromMarketplace);
                                    if (appState.showArtistDetailsButton)
                                        actions.push(appState.actions.artistDetails);
                                    actions.push(appState.actions.subscriptionSignup)
                                }
                                else {
                                    if (stateInfo.canPlay && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                        buttons.push(this.buttons.play);
                                    else if ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || appState.isRemoteSession)
                                        buttons.push(this.buttons.remotePlayDisabled);
                                    else
                                        buttons.push(this.buttons.showPlaylistItemInformation);
                                    if (MS.Entertainment.Utilities.hasPropertyPath(mediaContext, "options.containingMedia.editableItemSource")) {
                                        actions.push(appState.actions.moveListItemUp);
                                        actions.push(appState.actions.moveListItemDown);
                                        buttons.push(this.buttons.removeFromList)
                                    }
                                    else
                                        buttons.push(this.buttons.removeFromPlaylist);
                                    if (appState.canAddToMyMusic && (this.media.canFreeStream || (appState.signedInUserHasSubscription && this.media.canSubscriptionStream)))
                                        actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToMyMusic);
                                    else if (appState.canAddToCloud)
                                        actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToCloud);
                                    if (appState.canAddToNowPlaying && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                        actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                                    if (appState.canPlaySmartDJ)
                                        actions.push(appState.actions.playSmartDJ);
                                    actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                    actions.push(appState.actions.addTo);
                                    MS.Entertainment.ViewModels.MusicStateHelpers._getDownloadAppBarActions(appState, stateInfo, this.media).forEach(function addAction(action) {
                                        actions.push(action)
                                    })
                                }
                                if (appState.showArtistDetailsAction())
                                    actions.push(appState.actions.artistDetails);
                                else if (appState.showAlbumDetailsAction())
                                    actions.push(appState.actions.exploreAlbum)
                            }
                            else if (appState.freeStreamBlockedByFreeLimits) {
                                buttons.push(this.media.canPreview ? this.buttons.trackPlayPreview : this.buttons.playDisabled);
                                if (!appState.isSnapped)
                                    if (this.media.canPurchase)
                                        buttons.push(this.buttons.buyTrack);
                                    else if (this.media.canPurchaseAlbum)
                                        buttons.push(this.buttons.buyAlbum);
                                if (appState.showArtistDetailsAction())
                                    actions.push(appState.actions.artistDetails);
                                else if (appState.showAlbumDetailsAction())
                                    actions.push(appState.actions.exploreAlbum);
                                if (appState.canDelete && !inNowPlaying)
                                    actions.push(appState.actions.deleteMedia);
                                actions.push(appState.actions.subscriptionSignup)
                            }
                            else if (stateInfo.collection.canPlayLocallyOrStreamFromCloud) {
                                if ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally)
                                    if (inNowPlaying)
                                        buttons.push(this.buttons.playNowPlayingTrack);
                                    else
                                        buttons.push(this.buttons.play);
                                else
                                    buttons.push(this.buttons.remotePlayDisabled);
                                if (appState.canAddToCloud)
                                    addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToCloud);
                                if (appState.canAddToNowPlaying && !inNowPlaying && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                    addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                                if (appState.showArtistDetailsAction())
                                    actions.push(appState.actions.artistDetails);
                                else if (appState.showAlbumDetailsAction())
                                    actions.push(appState.actions.exploreAlbum);
                                addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                this.buttons.addTo.setSubActions(addToSubActions);
                                buttons.push(this.buttons.addTo);
                                if (appState.canPlaySmartDJ)
                                    actions.push(appState.actions.playSmartDJ);
                                if (appState.canPinToStartScreen)
                                    if (!existingStartScreenTile)
                                        actions.push(appState.actions.pinToStartScreen);
                                    else
                                        actions.push(appState.actions.unpinFromStartScreen);
                                if (appState.canPlayOnXbox)
                                    actions.push(appState.actions.playOnXbox360);
                                if (appState.canDelete && !inNowPlaying)
                                    actions.push(appState.actions.deleteMedia);
                                if (appState.canShowMediaProperties)
                                    actions.push(appState.actions.mediaProperties);
                                if (appState.musicMarketplaceConnected && !appState.isSnapped && !appState.canDownloadPurchasedTrack && !appState.isDownloadingPurchasedTrack && stateInfo.download.items.length === 0) {
                                    var isLocalTrackPurchasablePromise = MS.Entertainment.ViewModels.MusicSmartBuyStateHandlers.isLocalTrackPurchasable(this.media).then(function isLocalTrackPurchasable_complete(canPurchase) {
                                            if (canPurchase)
                                                actions.push(appState.actions.purchaseTrackFromMarketplace)
                                        });
                                    promiseArray.push(isLocalTrackPurchasablePromise)
                                }
                                MS.Entertainment.ViewModels.MusicStateHelpers._getDownloadAppBarActions(appState, stateInfo, this.media).forEach(function addAction(action) {
                                    actions.push(action)
                                })
                            }
                            else if (stateInfo.download.hasFailed) {
                                if (stateInfo.canPlay && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                    if (inNowPlaying)
                                        buttons.push(this.buttons.playNowPlayingTrack);
                                    else
                                        buttons.push(this.buttons.play);
                                else
                                    buttons.push(this.buttons.playDisabled);
                                if (stateInfo.download.hasRetryable)
                                    buttons.push(this.buttons.tryAgain);
                                else
                                    buttons.push(this.buttons.remove);
                                if (appState.canDelete)
                                    actions.push(appState.actions.deleteMedia)
                            }
                            else if (stateInfo.download.hasPending) {
                                if (stateInfo.canPlay && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                    if (inNowPlaying)
                                        buttons.push(this.buttons.playNowPlayingTrack);
                                    else
                                        buttons.push(this.buttons.play);
                                else
                                    buttons.push(this.buttons.playDisabled);
                                buttons.push(this.buttons.download)
                            }
                            else if (stateInfo.download.hasActive)
                                if (stateInfo.canPlay && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                    if (inNowPlaying)
                                        buttons.push(this.buttons.playNowPlayingTrack);
                                    else
                                        buttons.push(this.buttons.play);
                                else
                                    buttons.push(this.buttons.playDisabled);
                            else if ((this.media.location || this.media.album.location) === MS.Entertainment.Data.ItemLocation.collection) {
                                buttons.push(this.buttons.playDisabled);
                                if (appState.canDelete)
                                    actions.push(appState.actions.deleteMedia)
                            }
                            else {
                                var canBuyTrack = !appState.isSnapped && !appState.canDownloadPurchasedTrack && !appState.isDownloadingPurchasedTrack && MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Purchase]);
                                var albumPurchase = !appState.isSnapped && MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.AlbumPurchase]);
                                if (canBuyTrack && this.media.album && this.media.album.isPremium)
                                    canBuyTrack = false;
                                var hasSubscriptionDownloadRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Subscription]);
                                var hasSubscriptionStreamingRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Stream]);
                                var hasFreeStreamingRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.FreeStream]);
                                var canPreview = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Preview]);
                                var contentHasSubscriptionRights = hasSubscriptionStreamingRights || hasSubscriptionDownloadRights;
                                var allowSubscriptionActions = appState.musicSubscriptionEnabled && appState.signedInUserHasSubscription;
                                var showFreeStreamActions = hasFreeStreamingRights && appState.canFreeStream;
                                var hasAddToMyMusicRights = (allowSubscriptionActions && hasSubscriptionDownloadRights) || (allowSubscriptionActions && hasSubscriptionStreamingRights) || (showFreeStreamActions && hasFreeStreamingRights);
                                if (allowSubscriptionActions && contentHasSubscriptionRights) {
                                    if (!(Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) && appState.isRemoteSession)
                                        buttons.push(this.buttons.remotePlayDisabled);
                                    else if (hasSubscriptionStreamingRights) {
                                        if (inNowPlaying)
                                            buttons.push(this.buttons.playNowPlayingTrack);
                                        else
                                            buttons.push(this.buttons.play);
                                        if (appState.canPlaySmartDJ)
                                            actions.push(appState.actions.playSmartDJ);
                                        if (appState.canPinToStartScreen)
                                            if (!existingStartScreenTile)
                                                actions.push(appState.actions.pinToStartScreen);
                                            else
                                                actions.push(appState.actions.unpinFromStartScreen)
                                    }
                                    else if (canPreview && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                        buttons.push(this.buttons.trackPlayPreview);
                                    if (appState.canAddToMyMusic && hasAddToMyMusicRights)
                                        addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToMyMusic);
                                    if (appState.canAddToPlaylist && appState.isSignedIn)
                                        addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                    if (appState.canAddToNowPlaying && !inNowPlaying && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                        addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                                    if (addToSubActions.length > 0) {
                                        this.buttons.addTo.setSubActions(addToSubActions);
                                        buttons.push(this.buttons.addTo)
                                    }
                                    if (hasSubscriptionDownloadRights && !appState.downloadOnAddToMyMusicCloud)
                                        actions.push(appState.actions.subscriptionDownload);
                                    else if (appState.canDownloadPurchasedTrack)
                                        actions.push(appState.actions.downloadFromMarketplace);
                                    if (canBuyTrack)
                                        actions.push(appState.actions.purchaseTrackFromMarketplace)
                                }
                                else if (showFreeStreamActions) {
                                    if (!(Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) && appState.isRemoteSession)
                                        buttons.push(this.buttons.remotePlayDisabled);
                                    else if (inNowPlaying)
                                        buttons.push(this.buttons.playNowPlayingTrack);
                                    else
                                        buttons.push(this.buttons.play);
                                    if (appState.canAddToMyMusic && hasAddToMyMusicRights)
                                        addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToMyMusic);
                                    if (appState.canAddToPlaylist && appState.isSignedIn)
                                        addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                    if (appState.canAddToNowPlaying && !inNowPlaying && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                        addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                                    if (addToSubActions.length > 0) {
                                        this.buttons.addTo.setSubActions(addToSubActions);
                                        buttons.push(this.buttons.addTo)
                                    }
                                    if (canBuyTrack)
                                        actions.push(appState.actions.purchaseTrackFromMarketplace);
                                    if (appState.canPlaySmartDJ)
                                        actions.push(appState.actions.playSmartDJ);
                                    if (appState.canPinToStartScreen)
                                        if (!existingStartScreenTile)
                                            actions.push(appState.actions.pinToStartScreen);
                                        else
                                            actions.push(appState.actions.unpinFromStartScreen)
                                }
                                else {
                                    if (canPreview && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                        buttons.push(this.buttons.trackPlayPreview);
                                    if (stateInfo.marketplace.hasPurchased && appState.cloudServiceEnabled) {
                                        addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToMyMusic);
                                        if (!inNowPlaying && appState.canAddToNowPlaying && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                            addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying)
                                    }
                                    if (addToSubActions.length > 0) {
                                        this.buttons.addTo.setSubActions(addToSubActions);
                                        buttons.push(this.buttons.addTo)
                                    }
                                    if (canBuyTrack)
                                        if (buttons.length < fileScope.MAX_TRACK_BUTTON_NUM)
                                            buttons.push(this.buttons.buyTrack);
                                        else
                                            actions.push(appState.actions.purchaseTrackFromMarketplace);
                                    else if (albumPurchase)
                                        if (buttons.length < fileScope.MAX_TRACK_BUTTON_NUM)
                                            buttons.push(this.buttons.buyAlbum);
                                        else
                                            actions.push(appState.actions.purchaseAlbumFromMarketplace)
                                }
                                if (appState.canPlayOnXbox)
                                    actions.push(appState.actions.playOnXbox360);
                                if (appState.showArtistDetailsAction())
                                    actions.push(appState.actions.artistDetails);
                                else if (appState.showAlbumDetailsAction())
                                    actions.push(appState.actions.exploreAlbum)
                            }
                            if (!buttons.length)
                                buttons.push(this.buttons.contentNotAvailable);
                            if (inNowPlaying && mediaContext.options.containingMedia && mediaContext.options.containingMedia.playbackItemSource && mediaContext.options.containingMedia.playbackItemSource.mediaCollection)
                                promiseArray.push(mediaContext.options.containingMedia.playbackItemSource.mediaCollection.getCount().done(function getItemCount(count) {
                                    if (count > 1)
                                        actions.push(appState.actions.removeFromNowPlaying)
                                }, function getItemCountError(error) {
                                    MS.Entertainment.Pages.fail("Failed to get gallery count. Error: " + (error && error.message))
                                }));
                            return WinJS.Promise.join(promiseArray)
                        }.bind(this)).then(function joinComplete() {
                            return WinJS.Promise.wrap({
                                    smartButtons: buttons, appbarActions: actions
                                })
                        })
                }, onMusicGallerySelectionStateChanged: function onMusicGallerySelectionStateChanged(stateInfo) {
                    var buttons = [];
                    var actions = new MS.Entertainment.UI.Actions.ActionArray;
                    var preconditionPromises = [];
                    var promiseArray = [];
                    var appState = MS.Entertainment.ViewModels._MusicState.getCurrentState(this.media, stateInfo);
                    var showCollectionDetails = MS.Entertainment.ViewModels.MusicStateHandlers.shouldShowCollectionDetails(this.media, stateInfo);
                    var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                    var inNowPlaying = false;
                    if (mediaContext && mediaContext.options && mediaContext.options.containingMedia)
                        inNowPlaying = MS.Entertainment.Platform.Playback.PlaybackSession.isPlaybackSession(mediaContext.options.containingMedia.playbackItemSource);
                    var playSelectionAction = inNowPlaying ? appState.actions.playSelectionContinuous : appState.actions.playSelection;
                    var existingStartScreenTile = null;
                    if (appState.canPinToStartScreen) {
                        var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                        preconditionPromises.push(pinToStartScreenService.getTileForMediaAsync(this.media).then(function getTileForMediaAsync_complete(startScreenTile) {
                            existingStartScreenTile = startScreenTile
                        }))
                    }
                    return WinJS.Promise.join(preconditionPromises).then(function preconditionsComplete() {
                            if (this.media.smartDJ) {
                                if ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession)
                                    actions.push(appState.actions.playSmartDJ);
                                if (appState.musicMarketplaceConnected)
                                    actions.push(appState.actions.deleteSmartDJ)
                            }
                            else if (appState.isPlaylistTrack && !inNowPlaying) {
                                if (appState.freeStreamBlockedByFreeLimits) {
                                    actions.push(this.media.canPreview ? appState.actions.previewTrack : appState.actions.playDisabled);
                                    if (!appState.isSnapped)
                                        if (this.media.canPurchase)
                                            actions.push(appState.actions.purchaseTrackFromMarketplace);
                                        else if (this.media.canPurchaseAlbum)
                                            actions.push(appState.actions.purchaseAlbumFromMarketplace);
                                    if (appState.showArtistDetailsButton)
                                        actions.push(appState.actions.artistDetails);
                                    else if (appState.showAlbumDetailsAction())
                                        actions.push(appState.actions.exploreAlbum);
                                    if (MS.Entertainment.Utilities.hasPropertyPath(mediaContext, "options.containingMedia.editableItemSource")) {
                                        actions.push(appState.actions.removeFromList);
                                        actions.push(appState.actions.moveListItemUp);
                                        actions.push(appState.actions.moveListItemDown)
                                    }
                                    else
                                        actions.push(appState.actions.removeFromPlaylist);
                                    actions.push(appState.actions.subscriptionSignup)
                                }
                                else {
                                    if (stateInfo.canPlay && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                        actions.push(playSelectionAction);
                                    else if ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession)
                                        actions.push(appState.actions.showPlaylistItemInformation);
                                    if (!inNowPlaying && appState.canAddToNowPlaying && stateInfo.canPlay && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                        actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                                    if (appState.canAddToMyMusic && (this.media.canFreeStream || (appState.signedInUserHasSubscription && this.media.canSubscriptionStream)))
                                        actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToMyMusic);
                                    else if (appState.canAddToCloud)
                                        actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToCloud);
                                    actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                    actions.push(appState.actions.addTo);
                                    if (MS.Entertainment.Utilities.hasPropertyPath(mediaContext, "options.containingMedia.editableItemSource")) {
                                        actions.push(appState.actions.removeFromList);
                                        actions.push(appState.actions.moveListItemUp);
                                        actions.push(appState.actions.moveListItemDown)
                                    }
                                    else
                                        actions.push(appState.actions.removeFromPlaylist);
                                    MS.Entertainment.ViewModels.MusicStateHelpers._getDownloadAppBarActions(appState, stateInfo, this.media).forEach(function addAction(action) {
                                        actions.push(action)
                                    });
                                    if (appState.showArtistDetailsButton)
                                        actions.push(appState.actions.artistDetails);
                                    else if (appState.showAlbumDetailsAction())
                                        actions.push(appState.actions.exploreAlbum);
                                    if (appState.canPlaySmartDJ)
                                        actions.push(appState.actions.playSmartDJ)
                                }
                                if (appState.canShowMediaProperties)
                                    actions.push(appState.actions.mediaProperties)
                            }
                            else if (appState.freeStreamBlockedByFreeLimits) {
                                actions.push(this.media.canPreview ? appState.actions.previewTrack : appState.actions.playDisabled);
                                if (!appState.isSnapped)
                                    if (this.media.canPurchase)
                                        actions.push(appState.actions.purchaseTrackFromMarketplace);
                                    else if (this.media.canPurchaseAlbum)
                                        actions.push(appState.actions.purchaseAlbumFromMarketplace);
                                if (appState.showArtistDetailsAction())
                                    actions.push(appState.actions.artistDetails);
                                else if (appState.showAlbumDetailsAction())
                                    actions.push(appState.actions.exploreAlbum);
                                if (appState.canDelete && !inNowPlaying)
                                    actions.push(appState.actions.deleteMedia);
                                actions.push(appState.actions.subscriptionSignup);
                                if (appState.canShowMediaProperties)
                                    actions.push(appState.actions.mediaProperties)
                            }
                            else if (stateInfo.collection.canPlayLocallyOrStreamFromCloud && showCollectionDetails) {
                                if ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally)
                                    actions.push(playSelectionAction);
                                if (appState.canAddToCloud)
                                    actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToCloud);
                                if (!inNowPlaying && appState.canAddToNowPlaying && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                                    actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                                actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                actions.push(appState.actions.addTo);
                                if (appState.showArtistDetailsAction())
                                    actions.push(appState.actions.artistDetails);
                                else if (appState.showAlbumDetailsAction())
                                    actions.push(appState.actions.exploreAlbum);
                                if (appState.canPlaySmartDJ)
                                    actions.push(appState.actions.playSmartDJ);
                                if (appState.canPinToStartScreen)
                                    if (!existingStartScreenTile)
                                        actions.push(appState.actions.pinToStartScreen);
                                    else
                                        actions.push(appState.actions.unpinFromStartScreen);
                                if (appState.canPlayOnXbox)
                                    actions.push(appState.actions.playOnXbox360);
                                if (appState.canDelete && !inNowPlaying)
                                    actions.push(appState.actions.deleteMedia);
                                if (this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.track && (this.media.hydrated || this.media.fromCollection)) {
                                    if (appState.canShowMediaProperties)
                                        actions.push(appState.actions.mediaProperties);
                                    if (appState.musicMarketplaceConnected && !appState.isSnapped && (!stateInfo.marketplace.hasPurchased || !stateInfo.marketplace.canDownload) && stateInfo.download.items.length === 0) {
                                        var isLocalTrackPurchasablePromise = MS.Entertainment.ViewModels.MusicSmartBuyStateHandlers.isLocalTrackPurchasable(this.media).then(function isLocalTrackPurchasable_complete(canPurchase) {
                                                if (canPurchase)
                                                    actions.push(appState.actions.purchaseTrackFromMarketplace)
                                            });
                                        promiseArray.push(isLocalTrackPurchasablePromise)
                                    }
                                }
                                else if (this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.album && appState.musicMarketplaceConnected && this.media.hasPurchasableTracks && this.media.tracks && stateInfo.marketplace.hasPurchased < this.media.tracks.count)
                                    actions.push(appState.actions.purchaseAlbumFromMarketplace);
                                if (appState.canFindAlbumInfo)
                                    actions.push(appState.actions.findAlbumInfo);
                                MS.Entertainment.ViewModels.MusicStateHelpers._getDownloadAppBarActions(appState, stateInfo, this.media).forEach(function addAction(action) {
                                    actions.push(action)
                                })
                            }
                            else if (stateInfo.download.hasFailed) {
                                if (((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession) && stateInfo.canPlay)
                                    actions.push(playSelectionAction);
                                if (appState.showArtistDetailsAction())
                                    actions.push(appState.actions.artistDetails);
                                else if (appState.showAlbumDetailsAction())
                                    actions.push(appState.actions.exploreAlbum);
                                if (stateInfo.download.hasRetryable)
                                    actions.push(appState.actions.tryDownloadAgain);
                                else
                                    actions.push(appState.actions.removeFailedDownload);
                                if (appState.canDelete)
                                    actions.push(appState.actions.deleteMedia)
                            }
                            else if (stateInfo.download.hasPending) {
                                if (((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession) && stateInfo.canPlay)
                                    actions.push(playSelectionAction);
                                actions.push(appState.actions.downloadAction)
                            }
                            else if (stateInfo.download.hasActive && showCollectionDetails) {
                                if (((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession) && stateInfo.canPlay)
                                    actions.push(playSelectionAction)
                            }
                            else if (appState.canDelete && showCollectionDetails)
                                actions.push(appState.actions.deleteMedia);
                            else {
                                if (this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.album && (!this.media.hydrated || !this.media.tracks))
                                    return WinJS.Promise.wrap(null);
                                promiseArray.push(MS.Entertainment.ViewModels.MusicSmartBuyStateHandlers.getMusicRights(this.media).then(function addMarketplaceActions(musicRights) {
                                    musicRights = musicRights || {};
                                    var canBuyTrack = !appState.isSnapped && !appState.canDownloadPurchasedTrack && !appState.isDownloadingPurchasedTrack && musicRights.trackPurchase && this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.track;
                                    var contentFullyOwned = (this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.track && stateInfo.marketplace.hasPurchased) || (this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.album && this.media.tracks && stateInfo.marketplace.hasPurchased === this.media.tracks.count);
                                    var albumPurchase = !appState.isSnapped && musicRights.albumPurchase && !contentFullyOwned && (this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.track || this.media.mediaType === Microsoft.Entertainment.Queries.ObjectType.album);
                                    if (albumPurchase && this.media.album && this.media.album.isPremium)
                                        albumPurchase = false;
                                    var allowSubscriptionActions = appState.musicSubscriptionEnabled && appState.signedInUserHasSubscription;
                                    var allowFreeStreamActions = appState.canFreeStream;
                                    var allowSubscriptionOrFreeStreamActions = (allowSubscriptionActions || allowFreeStreamActions);
                                    var hasSubscriptionDownloadRights = allowSubscriptionActions && musicRights.subscriptionDownload;
                                    var hasSubscriptionStreamingRights = allowSubscriptionActions && musicRights.subscriptionStream;
                                    var hasFreeStreamingRights = allowFreeStreamActions && musicRights.freeStream;
                                    var hasAddToMyMusicRights = hasSubscriptionStreamingRights || hasFreeStreamingRights || hasSubscriptionDownloadRights;
                                    var showSubscriptionActions = allowSubscriptionActions && (hasSubscriptionStreamingRights || hasSubscriptionDownloadRights);
                                    var showFreeStreamActions = allowFreeStreamActions && (hasFreeStreamingRights);
                                    var canPlay = hasFreeStreamingRights || hasSubscriptionStreamingRights || appState.canPlayLocallyOrStreamFromCloud;
                                    var canPreview = musicRights.preview;
                                    if ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession) {
                                        if (appState.canPlayTopSongs)
                                            actions.push(appState.actions.playTopSongs);
                                        else if (canPlay)
                                            actions.push(playSelectionAction);
                                        else if (canPreview)
                                            actions.push(appState.actions.previewSelection);
                                        if (allowSubscriptionOrFreeStreamActions && appState.canPlaySmartDJ)
                                            actions.push(appState.actions.playSmartDJ);
                                        if (!inNowPlaying && appState.canAddToNowPlaying && (!appState.canFreeStream || appState.isSignedIn) && (canPlay || appState.canPlayTopSongs))
                                            actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying)
                                    }
                                    if (appState.canAddToMyMusic && hasAddToMyMusicRights)
                                        actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToMyMusic);
                                    if (showSubscriptionActions) {
                                        if (appState.canAddToPlaylist && appState.isSignedIn)
                                            actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                        if (hasSubscriptionDownloadRights && !appState.downloadOnAddToMyMusicCloud)
                                            actions.push(appState.actions.subscriptionDownload);
                                        else if (appState.canDownloadPurchasedTrack)
                                            actions.push(appState.actions.downloadFromMarketplace)
                                    }
                                    else {
                                        if ((showFreeStreamActions || appState.canPlayTopSongs) && appState.canAddToPlaylist && appState.isSignedIn)
                                            actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                                        if (appState.canDownloadPurchasedTrack)
                                            actions.push(appState.actions.downloadFromMarketplace)
                                    }
                                    if (canBuyTrack)
                                        actions.push(appState.actions.purchaseTrackFromMarketplace);
                                    else if (albumPurchase)
                                        actions.push(appState.actions.purchaseAlbumFromMarketplace);
                                    if (appState.showArtistDetailsAction())
                                        actions.push(appState.actions.artistDetails);
                                    else if (appState.showAlbumDetailsAction())
                                        actions.push(appState.actions.exploreAlbum);
                                    if (appState.canPinToStartScreen)
                                        if (!existingStartScreenTile)
                                            actions.push(appState.actions.pinToStartScreen);
                                        else
                                            actions.push(appState.actions.unpinFromStartScreen);
                                    if (appState.canPlayOnXbox)
                                        actions.push(appState.actions.playOnXbox360);
                                    if (appState.canDelete)
                                        actions.push(appState.actions.deleteMedia);
                                    if (actions.hasSubActions(appState.actions.addTo))
                                        actions.push(appState.actions.addTo)
                                }.bind(this), function handleGetRightsFailed(error) {
                                    MS.Entertainment.ViewModels.fail("Failed to get marketplace rights during single selection. Error message " + error && error.message)
                                }))
                            }
                            if (inNowPlaying && mediaContext.options.containingMedia && mediaContext.options.containingMedia.playbackItemSource && mediaContext.options.containingMedia.playbackItemSource.mediaCollection)
                                promiseArray.push(mediaContext.options.containingMedia.playbackItemSource.mediaCollection.getCount().done(function getItemCount(count) {
                                    if (count > 1)
                                        actions.push(appState.actions.removeFromNowPlaying)
                                }, function getItemCountError(error) {
                                    MS.Entertainment.Pages.fail("Failed to get gallery count. Error: " + (error && error.message))
                                }));
                            return WinJS.Promise.join(promiseArray)
                        }.bind(this)).then(function joinComplete() {
                            return WinJS.Promise.wrap({
                                    smartButtons: buttons, appbarActions: actions
                                })
                        })
                }, onMusicGalleryPlaylistSelectionStateChanged: function onMusicGalleryPlaylistSelectionStateChanged(stateInfo) {
                    var buttons = [];
                    var actions = new MS.Entertainment.UI.Actions.ActionArray;
                    var appState = MS.Entertainment.ViewModels._MusicState.getCurrentState(this.media, stateInfo);
                    actions.push(appState.actions.playSelection);
                    if (appState.canAddToNowPlaying && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession || stateInfo.collection.canPlayLocally))
                        actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToNowPlaying);
                    if (appState.canAddToCloud)
                        actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToCloud);
                    actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                    actions.push(appState.actions.addTo);
                    actions.push(appState.actions.renamePlaylist);
                    actions.push(appState.actions.deletePlaylist);
                    MS.Entertainment.ViewModels.MusicStateHelpers._getDownloadAppBarActions(appState, stateInfo, this.media).forEach(function addAction(action) {
                        actions.push(action)
                    });
                    return WinJS.Promise.wrap({
                            smartButtons: buttons, appbarActions: actions
                        })
                }, onMusicGallerySmartDJSelectionStateChanged: function onMusicGallerySmartDJSelectionStateChanged(stateInfo) {
                    var actions = [];
                    var appState = MS.Entertainment.ViewModels._MusicState.getCurrentState(this.media, stateInfo);
                    if ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession)
                        actions.push(appState.actions.playSmartDJ);
                    if (appState.musicMarketplaceConnected)
                        actions.push(appState.actions.deleteSmartDJ);
                    return WinJS.Promise.wrap({
                            smartButtons: [], appbarActions: actions
                        })
                }, onMusicOpportunitiesChanged: function onMusicOpportunitiesChanged(stateInfo) {
                    var buttons = [];
                    var actions = new MS.Entertainment.UI.Actions.ActionArray;
                    var appState = MS.Entertainment.ViewModels._MusicState.getCurrentState(this.media, stateInfo);
                    if (this.media.isAudioAd)
                        return WinJS.Promise.wrap({
                                smartButtons: buttons, appbarActions: actions
                            });
                    if (!this.media.mediaType) {
                        if (this.options && this.options.executeLocation)
                            this.buttons = MS.Entertainment.ViewModels.SmartBuyButtons.getEngagePanelButtons(this.options.executeLocation);
                        if (((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession) && (appState.freeStreamEnabled || appState.cachedSubscriptionStatus)) {
                            buttons.push(this.buttons.playAnyArtist);
                            if (appState.smartDJEnabled)
                                buttons.push(this.buttons.newSmartDJ)
                        }
                        else if (appState.musicSubscriptionEnabled && !appState.signedInUserHasSubscription)
                            buttons.push(this.buttons.getFreeMusicPassTrial);
                        else
                            buttons.push(this.buttons.browseAllMusic);
                        buttons.push(this.buttons.playPlaylist)
                    }
                    else {
                        if (appState.canSaveNowPlaying) {
                            actions.push(appState.actions.saveNowPlayingPlaylist);
                            buttons.push(this.buttons.saveNowPlayingPlaylist)
                        }
                        if ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession) {
                            var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                            if (appState.canPlaySmartDJ && (!sessionMgr.primarySession.smartDJSeed || sessionMgr.primarySession.smartDJSeed && sessionMgr.primarySession.smartDJSeed.canonicalId !== this.media.artistServiceId) && ((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession))
                                buttons.push(this.buttons.playArtistSmartDJ)
                        }
                        var hasSubscriptionDownloadRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Subscription]);
                        var hasSubscriptionStreamingRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Stream]);
                        var hasFreeStreamingRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(this.media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.FreeStream]);
                        var contentHasSubscriptionRights = hasSubscriptionStreamingRights || hasSubscriptionDownloadRights;
                        var allowSubscriptionActions = appState.musicSubscriptionEnabled && appState.signedInUserHasSubscription;
                        var showFreeStreamActions = hasFreeStreamingRights && appState.canFreeStream;
                        var hasFreeOrSubscriptionRights = ((allowSubscriptionActions && hasSubscriptionDownloadRights) || showFreeStreamActions);
                        var hasAddToMyMusicRights = (allowSubscriptionActions && hasSubscriptionDownloadRights) || (allowSubscriptionActions && hasSubscriptionStreamingRights) || (showFreeStreamActions && hasFreeStreamingRights);
                        if (appState.canAddToMyMusic && hasAddToMyMusicRights && !stateInfo.collection.playability.counts.downloading && stateInfo.collection.playability.counts.localOrCloud <= 0) {
                            buttons.push(this.buttons.addToMyMusic);
                            buttons.push(this.buttons.addAlbumToMyMusic);
                            actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToMyMusic)
                        }
                        if (((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession) && (allowSubscriptionActions || showFreeStreamActions) && MS.Entertainment.Utilities.isValidServiceId(this.media.artistServiceId))
                            buttons.push(this.buttons.playAllMarketplaceArtist);
                        if (((Microsoft.Entertainment.Configuration.ConfigurationManager().playback.enablePlayToByRef && MS.Entertainment.Utilities.isWindowsBlue) || !appState.isRemoteSession) && appState.canPlayOnXbox) {
                            buttons.push(this.buttons.playOnXbox);
                            actions.push(appState.actions.playTrackOnXbox360)
                        }
                        if (((hasFreeOrSubscriptionRights && appState.canAddToPlaylist && appState.isSignedIn) || (this.media.inCollection && stateInfo.collection.playability.counts.cloudOnly === 0)) && !this.media.activationFilePath)
                            actions.pushSubAction(appState.actions.addTo, MS.Entertainment.UI.Actions.AddTo.subMenuIds.addTrackToPlaylist);
                        if (actions.hasSubActions(appState.actions.addTo))
                            actions.push(appState.actions.addTo);
                        if (appState.showRelatedArtists)
                            buttons.push(this.buttons.playRelatedArtist)
                    }
                    return WinJS.Promise.wrap({
                            smartButtons: buttons, appbarActions: actions
                        })
                }, onArtistImmersiveStateChanged: function onArtistImmersiveStateChanged(stateInfo) {
                    if (!this.media.hydrated && !(this.media.artist && this.media.artist.hydrated))
                        return WinJS.Promise.wrap(null);
                    var buttons = [];
                    var actions = [];
                    var preconditionPromises = [];
                    var appState = MS.Entertainment.ViewModels._MusicState.getCurrentState(this.media, stateInfo);
                    var existingStartScreenTile = null;
                    if (appState.canPinToStartScreen) {
                        var pinToStartScreenService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreen);
                        preconditionPromises.push(pinToStartScreenService.getTileForMediaAsync(this.media).then(function getTileForMediaAsync_complete(startScreenTile) {
                            existingStartScreenTile = startScreenTile
                        }))
                    }
                    return WinJS.Promise.join(preconditionPromises).then(function preconditionsComplete() {
                            if (appState.canPlaySmartDJ)
                                buttons.push(this.buttons.playArtistSmartDJ);
                            if (appState.canPinToStartScreen)
                                if (!existingStartScreenTile)
                                    actions.push(appState.actions.pinToStartScreen);
                                else
                                    actions.push(appState.actions.unpinFromStartScreen);
                            return WinJS.Promise.wrap({
                                    smartButtons: buttons, appbarActions: actions
                                })
                        }.bind(this))
                }, shouldShowCollectionDetails: function shouldShowCollectionDetails(media, stateInfo) {
                    var showCollectionDetails = false;
                    if (media)
                        showCollectionDetails = media.fromCollection || (media.location === MS.Entertainment.Data.ItemLocation.collection) || (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.track && stateInfo.collection.playability.counts.localOrCloud) || (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.album && media.tracks && stateInfo.collection.playability.counts.localOrCloud === media.tracks.count);
                    return showCollectionDetails
                }
        }})
})()
