﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    scriptValidator("/Framework/corefx.js", "/Framework/utilities.js");
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Music");
    WinJS.Namespace.define("MS.Entertainment.Music", {PlayPlaylistOverlay: MS.Entertainment.UI.Framework.defineUserControl("Components/Music/PlayPlaylist.html#playPlaylistOverlayTemplate", function playPlaylistOverlayConstructor(element, options) {
            this._queryPlaylists();
            this.dialogTitle = options.dialogTitle || String.id.IDS_PLAY_PLAYLIST_OVERLAY_TITLE;
            this.showManageLink = options.hideManageLink ? !options.hideManageLink : this.showManageLink;
            MS.Entertainment.Music.assert(options.invokeTelemetryMoniker, "invokeTelemetryMoniker parameter missing from options bucket");
            this.invokeTelemetryMoniker = options.invokeTelemetryMoniker
        }, {
            invokeTelemetryMoniker: String.empty, _container: null, _showPlaylist: false, _showNoPlaylists: false, _eventHandlers: null, initialize: function initialize() {
                    this._playlistList.selectionHelperFactory = function selectionHelperFactory() {
                        return new MS.Entertainment.UI.Controls.DoNothingSelectionHelper
                    };
                    this.importPlaylistsAction = this._getImportPlaylistsAction();
                    WinJS.Utilities.addClass(this.domElement, "playPlaylistOverlayContainer");
                    this._updateState();
                    this._eventHandlers = MS.Entertainment.Utilities.addEventHandlers(this.domElement, {
                        keypress: function onKeyPress(event) {
                            if (event.keyCode === WinJS.Utilities.Key.escape)
                                this._container.hide()
                        }.bind(this), keydown: function onKeyDown(event) {
                                if (event.keyCode === WinJS.Utilities.Key.dismissButton) {
                                    event.stopPropagation();
                                    this._container.hide()
                                }
                            }.bind(this)
                    })
                }, setOverlay: function setOverlay(container) {
                    this._container = container
                }, unload: function unload() {
                    if (this._eventHandlers) {
                        this._eventHandlers.cancel();
                        this._eventHandlers = null
                    }
                    MS.Entertainment.UI.Framework.UserControl.prototype.unload.call(this)
                }, onManagePlaylists: function onManagePlaylists(event) {
                    this._sendTelemetryEvent("PlayPlaylistOverlay_ManagePlaylists");
                    if (MS.Entertainment.Utilities.isMusicApp1)
                        this._navigateAndClose(MS.Entertainment.UI.Monikers.musicCollection, MS.Entertainment.UI.Monikers.musicCollectionPlaylists, {selectHub: true});
                    else
                        this._navigateAndClose(MS.Entertainment.UI.Monikers.musicPlaylistCollection, null)
                }, onBrowseAlbums: function onBrowseAlbums(event) {
                    this._sendTelemetryEvent("PlayPlaylistOverlay_BrowseAlbums");
                    this._navigateAndClose(MS.Entertainment.UI.Monikers.musicCollection, MS.Entertainment.UI.Monikers.musicCollectionByAlbum, {selectHub: true})
                }, onBrowseAllMusic: function onBrowseAllMusic(event) {
                    this._sendTelemetryEvent("PlayPlaylistOverlay_BrowseAllMusic");
                    this._navigateAndClose(MS.Entertainment.UI.Monikers.musicMarketplace, MS.Entertainment.UI.Monikers.musicMarketplaceAlbums)
                }, _navigateAndClose: function _navigateAndClose(page, hub, navigationArguments) {
                    var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                    navigationService.navigateTo(page, hub, null, navigationArguments);
                    this._container.hide()
                }, onPlaylistInvoked: function onPlaylistInvoked(event) {
                    if (event.detail.itemPromise)
                        event.detail.itemPromise.done(function(mediaItem) {
                            this._container.result = mediaItem && mediaItem.data;
                            this._sendTelemetryEvent("PlayPlaylistOverlay_" + this.invokeTelemetryMoniker);
                            this._container.hide()
                        }.bind(this))
                }, _queryPlaylists: function _queryPlaylists() {
                    var query = new MS.Entertainment.Data.Query.libraryPlaylists;
                    query.sort = Microsoft.Entertainment.Queries.PlaylistsSortBy.dateEditedDescending;
                    query.isLive = false;
                    query.execute().done(function querySuccess(q) {
                        if (q.result && q.result.items && q.result.items.count) {
                            this.playlists = q.result.items;
                            this._showPlaylists = true
                        }
                        else
                            this._showNoPlaylists = true;
                        this._updateState()
                    }.bind(this))
                }, _updateState: function _updateState() {
                    if (this._showPlaylists && this._playlists) {
                        this._sendTelemetryEvent("PlayPlaylistOverlay_DisplayedWithPlaylists");
                        WinJS.Utilities.removeClass(this._playlists, "hideFromDisplay");
                        this._showPlaylists = false
                    }
                    if (this._showNoPlaylists && this._noPlaylists) {
                        this._sendTelemetryEvent("PlayPlaylistOverlay_DisplayedWithoutPlaylists");
                        var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                        var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                        if (signedInUser.isSubscription || featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlay))
                            WinJS.Utilities.removeClass(this._allMusicLink, "hideFromDisplay");
                        WinJS.Utilities.removeClass(this._noPlaylists, "hideFromDisplay");
                        this._showNoPlaylists = false
                    }
                }, _sendTelemetryEvent: function _sendTelemetryEvent(event) {
                    MS.Entertainment.Utilities.Telemetry.logTelemetryEvent(event)
                }, _getImportPlaylistsAction: function _getImportPlaylistsAction() {
                    var importPlaylistsAction = MS.Entertainment.UI.Actions.ImportPlaylists.getDoImportPlaylistsAction();
                    var executed = importPlaylistsAction.executed;
                    importPlaylistsAction.executed = function executedOverride() {
                        return executed().then(function executedCompleted() {
                                this._container.hide()
                            }.bind(this))
                    }.bind(this);
                    return importPlaylistsAction
                }
        }, {
            showManageLink: true, dialogTitle: String.empty, playlists: null, importPlaylistsAction: null
        })});
    WinJS.Namespace.define("MS.Entertainment.Music", {PlayPlaylistAction: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Actions.Action", function playPlaylistActionConstructor() {
            MS.Entertainment.UI.Actions.Action.prototype.constructor.apply(this)
        }, {
            canExecute: function canExecute() {
                return true
            }, executed: function(parameter) {
                    var options = this._getOverlayOptions();
                    var overlay = new MS.Entertainment.UI.Controls.Overlay(document.createElement("div"), options);
                    return overlay.show().then(function onOverlayClosed() {
                            var mediaItem = overlay.result;
                            if (!mediaItem)
                                return;
                            var query = new MS.Entertainment.Data.Query.libraryPlaylistMediaItems;
                            query.playlistId = mediaItem.libraryId;
                            query.isLive = false;
                            MS.Entertainment.Platform.PlaybackHelpers.playMedia2(query, {
                                sessionId: MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying, autoPlay: true, saveNowPlaying: false, showImmersive: false, showAppBar: true, shuffle: false, playContext: {playlist: mediaItem}
                            })
                        })
                }, _getOverlayOptions: function() {
                    var halfPopoverWidth = 335;
                    var halfPopoverHeight = 325;
                    var centeredStyle = "calc(50% - {0}px)";
                    var horizontalMargin = centeredStyle.format(halfPopoverWidth);
                    var verticalMargin = centeredStyle.format(halfPopoverHeight);
                    var options = {
                            userControl: "MS.Entertainment.Music.PlayPlaylistOverlay", userControlOptions: {invokeTelemetryMoniker: "PlaylistPlayed"}, top: verticalMargin, left: horizontalMargin, bottom: verticalMargin, right: horizontalMargin, persistOnNavigate: false, dontWaitForContent: true, enableKeyboardLightDismiss: true, showAnimation: function showAnimation() {
                                    return WinJS.Promise.as()
                                }
                        };
                    return options
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.Music", {PickPlaylistAction: MS.Entertainment.UI.Framework.derive("MS.Entertainment.Music.PlayPlaylistAction", function pickPlaylistActionConstructor() {
            MS.Entertainment.Music.PlayPlaylistAction.prototype.constructor.apply(this)
        }, {executed: function(parameter) {
                var options = this._getOverlayOptions();
                var mediaItem = this.parameter.mediaItem;
                var automationId = this.automationId;
                options.userControlOptions.dialogTitle = String.id.IDS_PICK_PLAYLIST_OVERLAY_TITLE;
                options.userControlOptions.hideManageLink = true;
                options.userControlOptions.invokeTelemetryMoniker = "PlaylistChosen";
                return MS.Entertainment.Data.List.listToArray(mediaItem).then(function gotItems(mediaItemArray) {
                        var appView = Windows.UI.ViewManagement.ApplicationView;
                        var uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                        if (!uiStateService.isSnapped || appView.tryUnsnap()) {
                            var appBarControl = MS.Entertainment.globalControls.getControl(MS.Entertainment.globalControls.GlobalControl.appBar);
                            if (appBarControl)
                                appBarControl.hide();
                            var overlay = new MS.Entertainment.UI.Controls.Overlay(document.createElement("div"), options);
                            return overlay.show().then(function onOverlayDismissed() {
                                    var playlistItem = overlay.result;
                                    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                    var action = actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.addToPlaylist);
                                    action.automationId = automationId;
                                    action.parameter = {
                                        mediaItem: playlistItem, itemToAdd: mediaItemArray
                                    };
                                    action.execute()
                                })
                        }
                    }, function onListToArrayError(error) {
                        MS.Entertainment.Music.fail("listToArray failed with error: " + (error && error.message))
                    })
            }})});
    WinJS.Namespace.define("MS.Entertainment.Music", {AddToPlaylistAction: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Actions.Action", function AddToPlaylistActionConstructor() {
            MS.Entertainment.Music.PlayPlaylistAction.prototype.constructor.apply(this)
        }, {
            executed: function(parameter) {
                var playlistItem = parameter.mediaItem;
                var itemToAdd = parameter.itemToAdd || this.itemToAdd;
                var mediaContext;
                var collectionFilter;
                if (!playlistItem)
                    return;
                mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
                collectionFilter = mediaContext.collectionFilter;
                MS.Entertainment.Data.List.listToArray(itemToAdd).done(function gotArray(mediaItemsArray) {
                    var finalItems = mediaItemsArray.filter(function removePlaylist(item) {
                            return item.libraryId !== playlistItem.libraryId
                        });
                    if (finalItems.length)
                        MS.Entertainment.UI.Actions.Playlists._appendToPlaylist(playlistItem.libraryId, playlistItem.name, finalItems, collectionFilter);
                    if (this.callback)
                        this.callback()
                }.bind(this))
            }, canExecute: function canExecute() {
                    return true
                }
        })});
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.pickPlaylist, function() {
        return new MS.Entertainment.Music.PickPlaylistAction
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.playPlaylist, function() {
        return new MS.Entertainment.Music.PlayPlaylistAction
    });
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.addToPlaylist, function() {
        return new MS.Entertainment.Music.AddToPlaylistAction
    });
    WinJS.Namespace.define("MS.Entertainment.Music", {SelectPlaylistAction: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Actions.Action", function selectPlaylistConstructor() {
            MS.Entertainment.Music.PlayPlaylistAction.prototype.constructor.apply(this)
        }, {
            executed: function executed(param) {
                var galleryClass = param.galleryClass || this.galleryClass;
                var action;
                if (param.action)
                    action = param.action;
                else if (param.actionId) {
                    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                    action = actionService.getAction(param.actionId)
                }
                else
                    action = this.action;
                MS.Entertainment.Framework.assert(action, "No valid action given to selectPlaylist");
                action.itemToAdd = param.mediaItem;
                action.automationId = this.automationId;
                var options = {
                        galleryClass: galleryClass, action: action, media: param.mediaItem
                    };
                var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                navigationService.navigateTo(MS.Entertainment.UI.Monikers.selectPlaylist, null, null, options)
            }, canExecute: function canExecute() {
                    return true
                }
        })});
    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).register(MS.Entertainment.UI.Actions.ActionIdentifiers.selectPlaylist, function() {
        return new MS.Entertainment.Music.SelectPlaylistAction
    })
})()
