﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
var __extends = this.__extends || function(d, b) {
        for (var p in b)
            if (b.hasOwnProperty(p))
                d[p] = b[p];
        function __() {
            this.constructor = d
        }
        __.prototype = b.prototype;
        d.prototype = new __
    };
var MS;
(function(MS) {
    (function(Entertainment) {
        (function(UI) {
            (function(Actions) {
                (function(MusicPurchaseActions) {
                    var MusicPurchaseActionBase = (function(_super) {
                            __extends(MusicPurchaseActionBase, _super);
                            function MusicPurchaseActionBase() {
                                _super.call(this)
                            }
                            MusicPurchaseActionBase.prototype.canExecute = function(param) {
                                return param != null && param != undefined && param.mediaItem !== null
                            };
                            MusicPurchaseActionBase.prototype.executed = function(param) {
                                var _this = this;
                                var listOrMediaItem = MS.Entertainment.UI.Actions.extractMediaItemFromParam(param);
                                MS.Entertainment.Data.List.getData(listOrMediaItem, 0).then(function(mediaItem) {
                                    if (mediaItem.rights)
                                        return mediaItem;
                                    else {
                                        var query = _this._getMediaItemDetailsQuery();
                                        query.id = mediaItem.serviceId;
                                        query.idType = mediaItem.serviceIdType;
                                        query.impressionGuid = mediaItem.impressionGuid;
                                        return query.execute().then(function queryCompleted(query) {
                                                query.dispose();
                                                return query.result.item
                                            })
                                    }
                                }).done(function(mediaToPurchase) {
                                    var action = _this._getPurchaseAction(mediaToPurchase, param);
                                    if (action.parameter && action.parameter.offer)
                                        action.execute();
                                    else
                                        MS.Entertainment.UI.Shell.showMessageBox(String.load(String.id.IDS_MUSIC_CONTENT_NOT_AVAILABLE_BUTTON), String.load(String.id.IDS_FAI_NO_STREAMS_MATCHED_ERROR))
                                }, function(error) {
                                    MS.Entertainment.UI.Actions.fail("Get media details failed: " + (error && error.message))
                                })
                            };
                            MusicPurchaseActionBase.prototype._getMediaItemDetailsQuery = function() {
                                MS.Entertainment.UI.Actions.fail("_getMediaItemDetailsQuery() should be overrided by subclass");
                                return null
                            };
                            MusicPurchaseActionBase.prototype._getPurchaseAction = function(mediaToPurchase, param) {
                                MS.Entertainment.UI.Actions.fail("_getPurchaseAction() should be overrided by subclass");
                                return null
                            };
                            return MusicPurchaseActionBase
                        })(Actions.Action);
                    MusicPurchaseActions.MusicPurchaseActionBase = MusicPurchaseActionBase;
                    var AlbumPurchaseAction = (function(_super) {
                            __extends(AlbumPurchaseAction, _super);
                            function AlbumPurchaseAction() {
                                _super.apply(this, arguments);
                                this.automationId = MS.Entertainment.UI.AutomationIds.albumPurchase
                            }
                            AlbumPurchaseAction.prototype._getMediaItemDetailsQuery = function() {
                                return new MS.Entertainment.Data.Query.Music.AlbumDetails
                            };
                            AlbumPurchaseAction.prototype._getPurchaseAction = function(albumToPurchase, param) {
                                var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                var purchaseAction = actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase);
                                purchaseAction.automationId = MS.Entertainment.UI.AutomationIds.albumPurchase;
                                var offer;
                                var matchingRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._getMatchingRights(albumToPurchase, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.AlbumPurchase], false, false);
                                if (matchingRights && matchingRights[0])
                                    offer = matchingRights[0];
                                purchaseAction.parameter = {
                                    mediaItem: albumToPurchase, offer: offer, actionType: param.actionType, executeLocation: param.executeLocation
                                };
                                return purchaseAction
                            };
                            return AlbumPurchaseAction
                        })(MusicPurchaseActionBase);
                    MusicPurchaseActions.AlbumPurchaseAction = AlbumPurchaseAction;
                    var TrackPurchaseAction = (function(_super) {
                            __extends(TrackPurchaseAction, _super);
                            function TrackPurchaseAction() {
                                _super.apply(this, arguments);
                                this.automationId = MS.Entertainment.UI.AutomationIds.trackPurchase
                            }
                            TrackPurchaseAction.prototype._getMediaItemDetailsQuery = function() {
                                return new MS.Entertainment.Data.Query.Music.SongDetails
                            };
                            TrackPurchaseAction.prototype._getPurchaseAction = function(trackToPurchase, param) {
                                var hasTrackPurchaseRights;
                                var hasAlbumPurchaseRights;
                                var mediaItemToPurchase;
                                if (trackToPurchase.rights) {
                                    hasTrackPurchaseRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(trackToPurchase, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Purchase]);
                                    hasAlbumPurchaseRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(trackToPurchase, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.AlbumPurchase])
                                }
                                mediaItemToPurchase = (!hasTrackPurchaseRights && hasAlbumPurchaseRights) ? trackToPurchase.album : trackToPurchase;
                                var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                var purchaseAction = actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.purchase);
                                purchaseAction.automationId = MS.Entertainment.UI.AutomationIds.trackPurchase;
                                var offer;
                                var matchingRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._getMatchingRights(mediaItemToPurchase, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Purchase], false, false);
                                if (matchingRights && matchingRights[0])
                                    offer = matchingRights[0];
                                purchaseAction.parameter = {
                                    mediaItem: mediaItemToPurchase, offer: offer, actionType: param.actionType, executeLocation: param.executeLocation
                                };
                                return purchaseAction
                            };
                            return TrackPurchaseAction
                        })(MusicPurchaseActionBase);
                    MusicPurchaseActions.TrackPurchaseAction = TrackPurchaseAction
                })(Actions.MusicPurchaseActions || (Actions.MusicPurchaseActions = {}));
                var MusicPurchaseActions = Actions.MusicPurchaseActions
            })(UI.Actions || (UI.Actions = {}));
            var Actions = UI.Actions
        })(Entertainment.UI || (Entertainment.UI = {}));
        var UI = Entertainment.UI
    })(MS.Entertainment || (MS.Entertainment = {}));
    var Entertainment = MS.Entertainment
})(MS || (MS = {}));
(function() {
    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.albumPurchase, function() {
        return new MS.Entertainment.UI.Actions.MusicPurchaseActions.AlbumPurchaseAction
    });
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.trackPurchase, function() {
        return new MS.Entertainment.UI.Actions.MusicPurchaseActions.TrackPurchaseAction
    })
})()
