﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    var MSEPlatform = WinJS.Namespace.define("MS.Entertainment.Platform", null);
    WinJS.Namespace.defineWithParent(MS.Entertainment.Platform, "PlayTo", {
        _playToReceiver: null, initPlayToReceiver: function initPlayToReceiver() {
                MSEPlatform.Playback.Etw.traceString("playTo::initPlayToReceiver()");
                if (!MS.Entertainment.Platform.PlayTo._playToReceiver)
                    MS.Entertainment.Platform.PlayTo._playToReceiver = new MS.Entertainment.Platform.PlayTo.PlayToReceiver;
                return MS.Entertainment.Platform.PlayTo._playToReceiver.start()
            }, stopPlayToReceiver: function stopPlayToReceiver(deferral) {
                MSEPlatform.Playback.Etw.traceString("playTo::stopPlayToReceiver()");
                if (MS.Entertainment.Platform.PlayTo._playToReceiver)
                    return MS.Entertainment.Platform.PlayTo._playToReceiver.stop(deferral);
                return WinJS.Promise.wrap()
            }, isPlayToReceiverRunning: function isPlayToReceiverRunning() {
                MSEPlatform.Playback.Etw.traceString("playTo::isPlayToReceiverRunning()");
                if (MS.Entertainment.Platform.PlayTo._playToReceiver)
                    return MS.Entertainment.Platform.PlayTo._playToReceiver.isRunning;
                return false
            }, PlayToReceiver: MS.Entertainment.UI.Framework.define(function PlayToReceiver_constructor(){}, {
                _receiver: null, _hasStarted: false, _Pause: true, _Stopping: false, _hasSource: false, _sourceSet: false, _playbackSession: null, _delayFirstPlayPromise: null, _playbackSessionEventHandlers: null, _playToReceiverEventHandlers: null, unload: function unload() {
                        if (this._playbackSessionEventHandlers) {
                            this._playbackSessionEventHandlers.cancel();
                            this._playbackSessionEventHandlers = null
                        }
                        if (this._playToReceiverEventHandlers) {
                            this._playToReceiverEventHandlers.cancel();
                            this._playToReceiverEventHandlers = null
                        }
                        this._receiver = null
                    }, start: function PlayToReceiver_start() {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::start()");
                        if (this._hasStarted)
                            return WinJS.Promise.wrap(null);
                        var isVideoPlayback = MS.Entertainment.Utilities.isVideoApp;
                        if (!this._receiver)
                            try {
                                if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.playToReceiverNativeService))
                                    this._receiver = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.playToReceiverNativeService)
                            }
                            catch(ex) {
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::start() creating playToReceiverNative exception, errorCode= " + ex.number + ", message=" + ex.message);
                                MS.Entertainment.Utilities.Telemetry.logPlayToReceiverError(ex, "Failed to create PlayTo receiver: " + ex.message);
                                return WinJS.Promise.wrapError(ex)
                            }
                        if (!this._playbackSession) {
                            var sessionMgr = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                            this._playbackSession = sessionMgr.primarySession
                        }
                        if (!this._playbackSessionEventHandlers)
                            this._playbackSessionEventHandlers = MS.Entertainment.Utilities.addEventHandlers(this._playbackSession, {
                                currentTransportStateChanged: this._onPlaybackSessionEventTransportStateChange.bind(this), playerStateChanged: this._onPlaybackSessionEventPlayerStateChange.bind(this), playbackRateChanged: this._onPlaybackSessionEventRateChange.bind(this), currentMediaChanged: this._onPlaybackSessionEventCurrentMediaChange.bind(this), durationChanged: this._onPlaybackSessionEventDurationChange.bind(this), seekedPositionChanged: this._onPlaybackSessionEventSeeked.bind(this), errorDescriptorChanged: this._onPlaybackSessionEventErrorDescriptor.bind(this)
                            });
                        try {
                            if (!this._playToReceiverEventHandlers)
                                this._playToReceiverEventHandlers = MS.Entertainment.Utilities.addEventHandlers(this._receiver, {
                                    playrequested: this._onReceiverEventPlayRequested.bind(this), pauserequested: this._onReceiverEventPauseRequested.bind(this), sourcechangerequested: this._onReceiverEventSourceChangeRequested.bind(this), playbackratechangerequested: this._onReceiverEventPlaybackRateChangeRequested.bind(this), currenttimechangerequested: this._onReceiverEventCurrentTimeChangeRequested.bind(this), mutechangerequested: this._onReceiverEventMuteChangeRequested.bind(this), volumechangerequested: this._onReceiverEventVolumeChangeRequested.bind(this), timeupdaterequested: this._onReceiverEventTimeUpdateRequested.bind(this), stoprequested: this._onReceiverEventStopRequested.bind(this)
                                })
                        }
                        catch(ex) {
                            this._receiver = null;
                            this._hasStarted = false;
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::start() adding event listener exception, errorCode= " + ex.number + ", message=" + ex.message);
                            MS.Entertainment.Utilities.Telemetry.logPlayToReceiverError(ex, "Failed to add event listeners to the receiver: " + ex.message);
                            return WinJS.Promise.wrapError(ex)
                        }
                        try {
                            if (MS.Entertainment.Utilities.isMusicApp) {
                                this._receiver.supportsAudio = true;
                                this._receiver.friendlyName = String.load(String.id.IDS_XBOX_MUSIC_APP_TITLE)
                            }
                            else if (MS.Entertainment.Utilities.isVideoApp) {
                                this._receiver.supportsVideo = true;
                                this._receiver.friendlyName = String.load(String.id.IDS_XBOX_VIDEO_APP_TITLE)
                            }
                        }
                        catch(ex) {
                            this._receiver = null;
                            this._hasStarted = false;
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::start() setting receiver properties exception, errorCode= " + ex.number + ", message=" + ex.message);
                            MS.Entertainment.Utilities.Telemetry.logPlayToReceiverError(ex, "Failed to set receiver properties: " + ex.message);
                            return WinJS.Promise.wrapError(ex)
                        }
                        try {
                            return this._receiver.startAsync().then(function onStartSuccess() {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::start() startAync succeed");
                                    this._hasStarted = true;
                                    var timeout = isVideoPlayback ? 8000 : 0;
                                    this._delayFirstPlayPromise = WinJS.Promise.timeout(timeout)
                                }.bind(this), function onStartError(e) {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::start() startAync error, errorCode= " + e.number + ", message=" + e.message);
                                    this._hasStarted = false;
                                    MS.Entertainment.Utilities.Telemetry.logPlayToReceiverError(e, "Failed to start PlayTo receiver: " + e.message);
                                    return WinJS.Promise.wrapError(e)
                                }.bind(this))
                        }
                        catch(ex) {
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::start() startAync exception, errorCode= " + ex.number + ", message=" + ex.message);
                            this._receiver = null;
                            this._hasStarted = false;
                            MS.Entertainment.Utilities.Telemetry.logPlayToReceiverError(ex, "Failed to initialize PlayTo receiver: " + ex.message);
                            return WinJS.Promise.wrapError(ex)
                        }
                    }, stop: function PlayToReceiver_stop(deferral) {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::stop()");
                        if (!this._hasStarted) {
                            if (deferral)
                                deferral.complete();
                            return WinJS.Promise.wrap(null)
                        }
                        try {
                            return this._receiver.stopAsync().then(function onStopSuccess() {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::stop() stopAync call succeeded");
                                    this._hasStarted = false;
                                    if (deferral)
                                        deferral.complete()
                                }.bind(this), function onStopError(e) {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::stop() stopAync error, errorCode= " + e.number + ", message=" + e.message);
                                    MS.Entertainment.Utilities.Telemetry.logPlayToReceiverError(e, "Failed to stop PlayTo receiver: " + e.message);
                                    if (deferral)
                                        deferral.complete();
                                    return WinJS.Promise.wrapError(e)
                                }.bind(this))
                        }
                        catch(ex) {
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::stop() startAync exception, errorCode= " + ex.number + ", message=" + ex.message);
                            MS.Entertainment.Utilities.Telemetry.logPlayToReceiverError(ex, "Failed to stop PlayTo receiver: " + ex.message);
                            this._receiver = null;
                            if (deferral)
                                deferral.complete();
                            return WinJS.Promise.wrapError(ex)
                        }
                    }, isRunning: {get: function isRunning_Getter() {
                            return this._hasStarted
                        }}, _playUnProtectedMedia: function PlayToReceiver_playUnProtectedMedia(event, thumbnailStream) {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playUnProtectedMedia()");
                        if (!event)
                            return;
                        var blob = MSApp.createBlobFromRandomAccessStream(event.stream.contentType, event.stream);
                        var isVideoPlayback = MS.Entertainment.Utilities.isVideoApp;
                        var dataToAugment = {};
                        dataToAugment.FilePath = URL.createObjectURL(blob, {oneTimeOnly: true});
                        var mediaItem = null;
                        if (event.stream.contentType.substring(0, 5) === "audio") {
                            dataToAugment.musicProps = {
                                title: event.title, album: event.album, genre: event.genre, rating: event.rating
                            };
                            mediaItem = MS.Entertainment.Data.augment(dataToAugment, MS.Entertainment.Data.Augmenter.FileActivation.Track);
                            mediaItem.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(Microsoft.Entertainment.Platform.AcquisitionContext.playTo, Microsoft.Entertainment.Platform.AcquisitionContextType.track)
                        }
                        else {
                            dataToAugment.fileItem = {name: event.title};
                            dataToAugment.videoProps = {
                                title: event.title, rating: event.rating
                            };
                            mediaItem = MS.Entertainment.Data.augment(dataToAugment, MS.Entertainment.Data.Augmenter.FileActivation.Video)
                        }
                        if (this._delayFirstPlayPromise)
                            this._delayFirstPlayPromise.done(function _onDelayFirstPlayPromise() {
                                if (this._playbackSession) {
                                    this._playbackSession.repeat = false;
                                    this._playbackSession.shuffle = false
                                }
                                MS.Entertainment.Platform.PlaybackHelpers.playMedia2(mediaItem, {
                                    sessionId: MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying, autoPlay: false, showAppBar: false, showImmersive: isVideoPlayback, immersiveOptions: {
                                            sessionId: MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying, startFullScreen: true
                                        }, playContext: {activationFilePath: dataToAugment.FilePath}, saveNowPlaying: false
                                })
                            }.bind(this))
                    }, _playSongByServiceId: function PlayToReceiver_playSongByServiceId(serviceId) {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playSongByServiceId(), serviceId = " + serviceId);
                        var mediaPromise = null;
                        mediaPromise = MS.Entertainment.Platform.PlaybackHelpers.getMusicMediaByServiceId(serviceId, "ZuneCatalog", "track");
                        mediaPromise.done(function getMediaByServiceIdSuccess(media) {
                            if (media.mediaType !== Microsoft.Entertainment.Queries.ObjectType.track) {
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playSongByServiceId() mediaType is not track");
                                this._receiver.notifyError();
                                return
                            }
                            var appSignIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                            var signedInBindings = null;
                            var timeoutPromise = WinJS.Promise.timeout(30000);
                            timeoutPromise.done(function onTimeOutForTryingToPlay() {
                                if (signedInBindings)
                                    signedInBindings.cancel();
                                if (!appSignIn.isSignedIn) {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playSongByServiceId() timed out without signin");
                                    this._receiver.notifyError()
                                }
                            }.bind(this));
                            checkSignInStatus();
                            function checkSignInStatus() {
                                if (appSignIn.isSignedIn) {
                                    tryToPlay();
                                    timeoutPromise.cancel()
                                }
                                else
                                    signedInBindings = WinJS.Binding.bind(appSignIn, {isSignedIn: function onIsSignedInChanged() {
                                            if (appSignIn.isSignedIn) {
                                                tryToPlay();
                                                timeoutPromise.cancel();
                                                signedInBindings.cancel()
                                            }
                                        }})
                            }
                            function tryToPlay() {
                                MS.Entertainment.UI.Components.Shell.assert(appSignIn.isSignedIn || appSignIn.signInError, "signing in process should be finished!");
                                if (appSignIn.isSignedIn)
                                    MS.Entertainment.Platform.PlaybackHelpers.playMedia2(media, {
                                        sessionId: MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying, autoPlay: false, showAppBar: false, saveNowPlaying: false, showImmersive: false, immersiveOptions: {
                                                sessionId: MS.Entertainment.Platform.Playback.WellKnownPlaybackSessionId.nowPlaying, startFullScreen: true
                                            }
                                    });
                                else {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playSongByServiceId() user is not signed in");
                                    this._receiver.notifyError()
                                }
                            }
                        }.bind(this), function getMediaByServiceIdError(errorCode) {
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playSongByServiceId() get mediaByServiceIdError, errorcode = " + errorCode);
                            this._receiver.notifyError()
                        }.bind(this))
                    }, _playVideoByServiceId: function PlayToReceiver_playVideoByServiceId(serviceId) {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playVideoByServiceId(), serviceId = " + serviceId);
                        var item = {ZuneId: serviceId};
                        item.location = MS.Entertainment.Pages.BaseMediaInlineDetails.Location.marketplace;
                        var mediaPromise = null;
                        mediaPromise = MS.Entertainment.Platform.PlaybackHelpers.getMediaByServiceId(serviceId, "ZuneCatalog");
                        mediaPromise.done(function getMediaByServiceIdSuccess(media) {
                            var isMovie = MS.Entertainment.Platform.PlaybackHelpers.isMovie(media);
                            var isEpisode = MS.Entertainment.Platform.PlaybackHelpers.isTVEpisode(media);
                            if (!isMovie && !isEpisode) {
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playVideoByServiceId() mediaType is not movie or tvEpisode");
                                this._receiver.notifyError();
                                return
                            }
                            var appSignIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                            var signedInBindings = null;
                            var timeoutPromise = WinJS.Promise.timeout(30000);
                            timeoutPromise.done(function onTimeOutForTryingToPlay() {
                                if (signedInBindings)
                                    signedInBindings.cancel();
                                if (!appSignIn.isSignedIn) {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playVideoByServiceId() timed out without signed-in");
                                    this._receiver.notifyError()
                                }
                            }.bind(this));
                            checkSignInStatus();
                            function checkSignInStatus() {
                                if (appSignIn.isSignedIn) {
                                    tryToPlay();
                                    timeoutPromise.cancel()
                                }
                                else
                                    signedInBindings = WinJS.Binding.bind(appSignIn, {isSignedIn: function onIsSignedInChanged() {
                                            if (appSignIn.isSignedIn) {
                                                tryToPlay();
                                                timeoutPromise.cancel();
                                                signedInBindings.cancel()
                                            }
                                        }})
                            }
                            function tryToPlay() {
                                var isMovie = MS.Entertainment.Platform.PlaybackHelpers.isMovie(media);
                                var isEpisode = MS.Entertainment.Platform.PlaybackHelpers.isTVEpisode(media);
                                var mediaItem = null;
                                if (isMovie)
                                    mediaItem = MS.Entertainment.Data.augment(media, MS.Entertainment.Data.Augmenter.Marketplace.EdsMovie);
                                else if (isEpisode)
                                    mediaItem = MS.Entertainment.Data.augment(media, MS.Entertainment.Data.Augmenter.Marketplace.EdsTVEpisode);
                                if (mediaItem) {
                                    MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem);
                                    mediaItem.hydrate().then(function _onMediaItemHydrated() {
                                        if (mediaItem.hasServiceId && mediaItem.isValid)
                                            showVideoImmersive(mediaItem);
                                        else {
                                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playVideoByServiceId() hydrating mediaItem failed");
                                            this._receiver.notifyError()
                                        }
                                    }.bind(this))
                                }
                                else {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playVideoByServiceId() mediaItem is not movie or tvEpisode");
                                    this._receiver.notifyError()
                                }
                            }
                        }.bind(this), function getMediaByServiceIdError(errorCode) {
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playVideoByServiceId() getMediaByServiceIdError, errorcode = " + errorCode);
                            this._receiver.notifyError()
                        }.bind(this));
                        function showVideoImmersive(media) {
                            MS.Entertainment.Platform.PlaybackHelpers.playMedia2(media, {
                                autoPlay: false, showImmersive: true, immersiveOptions: {startFullScreen: true}
                            });
                            if (!media.inCollection)
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_playVideoByServiceId() item to play is not in collection.")
                        }
                    }, _onPlaybackSessionEventTransportStateChange: function _onPlaybackSessionEventTransportStateChange() {
                        if (this._hasSource)
                            if (this._playbackSession.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.playing) {
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onPlaybackSessionEventTransportStateChange() playing, calling notifyPlaying()");
                                this._receiver.notifyPlaying()
                            }
                            else if (this._playbackSession.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.paused) {
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onPlaybackSessionEventTransportStateChange() paused");
                                if (this._Stopping || (!this._Pause && this._playbackSession.currentPosition === 0)) {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onPlaybackSessionEventTransportStateChange() calling notifyStopped()");
                                    this._receiver.notifyStopped()
                                }
                                else {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onPlaybackSessionEventTransportStateChange() calling notifyPaused()");
                                    this._receiver.notifyPaused();
                                    this._Pause = false
                                }
                            }
                            else if (this._playbackSession.currentTransportState === MS.Entertainment.Platform.Playback.TransportState.stopped) {
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onPlaybackSessionEventTransportStateChange() stopped");
                                if (this._playbackSession.playerState === MS.Entertainment.Platform.Playback.PlayerState.ready) {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onPlaybackSessionEventTransportStateChange() calling notifyEnded()");
                                    this._receiver.notifyEnded()
                                }
                            }
                    }, _onPlaybackSessionEventPlayerStateChange: function _onPlaybackSessionEventPlayerStateChange() {
                        if (this._hasSource)
                            if (MS.Entertainment.Platform.Playback.PlayerState.ready === this._playbackSession.playerState) {
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onPlaybackSessionEventPlayerStateChange() playerState is ready. calling notifyLoadedMetadata()");
                                this._receiver.notifyLoadedMetadata()
                            }
                    }, _onPlaybackSessionEventErrorDescriptor: function _onPlaybackSessionEventErrorDescriptor() {
                        if (this._hasSource && this._playbackSession.errorDescriptor) {
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onPlaybackSessionEventErrorDescriptor() playback error happened. calling notifyError() and notifyStopped()");
                            MS.Entertainment.Utilities.Telemetry.logTelemetryEvent(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlayToReceiverPlaybackError);
                            this._receiver.notifyError();
                            this._receiver.notifyStopped()
                        }
                    }, _onPlaybackSessionEventCurrentMediaChange: function _onPlaybackSessionEventCurrentMediaChange(){}, _onPlaybackSessionEventDurationChange: function _onPlaybackSessionEventDurationChange() {
                        if (this._hasSource) {
                            var duration = (this._playbackSession.duration > 0 && isFinite(this._playbackSession.duration)) ? this._playbackSession.duration : 0;
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onPlaybackSessionEventDurationChange(). calling notifyDurationChange()");
                            this._receiver.notifyDurationChange(duration)
                        }
                    }, _onPlaybackSessionEventSeeked: function _onPlaybackSessionEventSeeked() {
                        if (this._hasSource)
                            if (this._Stopping)
                                this._Stopping = false;
                            else {
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onPlaybackSessionEventSeeked(). calling notifySeeked()");
                                this._receiver.notifySeeked()
                            }
                    }, _onPlaybackSessionEventRateChange: function _onPlaybackSessionEventRateChange() {
                        if (this._hasSource)
                            this._receiver.notifyRateChange(this._playbackSession.playbackRate)
                    }, _onVolumeControlEventVolumeChange: function _onVolumeControlEventVolumeChange() {
                        if (this._hasSource && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.volumeService)) {
                            var volumeControllerService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.volumeService);
                            this._receiver.notifyVolumeChange(volumeControllerService.volume, volumeControllerService.mute)
                        }
                    }, _onReceiverEventPlayRequested: function _onReceiverEventPlayRequested() {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventPlayRequested");
                        if (this._hasSource)
                            this._playbackSession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.playing;
                        else {
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventPlayRequested(). calling notifyError()");
                            this._receiver.notifyError()
                        }
                    }, _onReceiverEventPauseRequested: function _onReceiverEventPauseRequested() {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventPauseRequested");
                        if (this._hasSource) {
                            this._Pause = true;
                            this._Stopping = false;
                            this._playbackSession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.paused
                        }
                        else {
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventPauseRequested(). calling notifyError()");
                            this._receiver.notifyError()
                        }
                    }, _onReceiverEventSourceChangeRequested: function _onReceiverEventSourceChangeRequested(event) {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventSourceChangeRequested");
                        try {
                            if (event.stream) {
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventSourceChangeRequested() event.stream is not null");
                                this._hasSource = true;
                                this._sourceSet = false;
                                MS.Entertainment.Utilities.Telemetry.logTelemetryEvent(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlayToReceiverSessionRequested);
                                var serviceId = String.empty;
                                var itemUrl = event.properties["System.ItemUrl"];
                                if (itemUrl)
                                    if (itemUrl.indexOf("ms-playtoapp-xboxmusic://media/?X_id") > -1) {
                                        serviceId = itemUrl.toString().replace("ms-playtoapp-xboxmusic://media/?X_id={", "");
                                        serviceId = serviceId.toString().replace("}", "");
                                        if (serviceId) {
                                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_makeReceiverHandler() sourcechangerequested playTo By Ref music");
                                            this._playSongByServiceId(serviceId)
                                        }
                                    }
                                    else if (itemUrl.indexOf("ms-playtoapp-xboxvideo://media/?X_id") > -1) {
                                        serviceId = itemUrl.toString().replace("ms-playtoapp-xboxvideo://media/?X_id={", "");
                                        serviceId = serviceId.toString().replace("}", "");
                                        if (serviceId) {
                                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_makeReceiverHandler() sourcechangerequested playTo By Ref video");
                                            this._playVideoByServiceId(serviceId)
                                        }
                                    }
                                if (!serviceId) {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_makeReceiverHandler() sourcechangerequested, unprotected: Not by ref");
                                    if (!event.thumbnail) {
                                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventSourceChangeRequested() thumbnail is null");
                                        this._playUnProtectedMedia(event, null)
                                    }
                                    else
                                        event.thumbnail.openReadAsync().then(function onOpenReadSuccess(stream) {
                                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventSourceChangeRequested() thumbnail openReadAsync succeess");
                                            this._playUnProtectedMedia(event, stream)
                                        }.bind(this), function onOpenReadError(e) {
                                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventSourceChangeRequested() thumbnail openReadAsync error, errorCode= " + e.number + ", message=" + e.message);
                                            MS.Entertainment.Utilities.Telemetry.logPlayToReceiverError(e, "Failed to open thumbnail from the source change: " + e.message);
                                            this._playUnProtectedMedia(event, null)
                                        }.bind(this))
                                }
                            }
                            else {
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventSourceChangeRequested(). source is null; DMC is ending the session");
                                MS.Entertainment.Utilities.Telemetry.logTelemetryEvent(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.PlayToReceiverSessionCompleted);
                                if (this._hasSource && MS.Entertainment.Utilities.isVideoApp) {
                                    var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                                    if (navigationService && navigationService.currentPage.iaNode.moniker === MS.Entertainment.UI.Monikers.fullScreenNowPlaying)
                                        navigationService.navigateBack()
                                }
                                this._hasSource = false;
                                this._sourceSet = false
                            }
                        }
                        catch(ex) {
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventSourceChangeRequested() thumbnail openReadAsync exception, errorCode= " + ex.number + ", message=" + ex.message);
                            MS.Entertainment.Utilities.Telemetry.logPlayToReceiverError(ex, "Failed to switch source: " + ex.message)
                        }
                    }, _onReceiverEventPlaybackRateChangeRequested: function _onReceiverEventPlaybackRateChangeRequested(event) {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventPlaybackRateChangeRequested()");
                        if (this._hasSource)
                            this._playbackSession.playbackRate = event.rate
                    }, _onReceiverEventCurrentTimeChangeRequested: function _onReceiverEventCurrentTimeChangeRequested(event) {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventCurrentTimeChangeRequested()");
                        if (this._hasSource)
                            this._playbackSession.seekToPosition(event.time);
                        else {
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventCurrentTimeChangeRequested(). error. calling notifyError()");
                            this._receiver.notifyError()
                        }
                    }, _onReceiverEventMuteChangeRequested: function _onReceiverEventMuteChangeRequested(event) {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventMuteChangeRequested()");
                        if (this._hasSource)
                            this._playbackSession.muted = event.mute
                    }, _onReceiverEventVolumeChangeRequested: function _onReceiverEventVolumeChangeRequested(event) {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventVolumeChangeRequested()");
                        if (this._hasSource && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.volumeService)) {
                            var volumeControllerService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.volumeService);
                            volumeControllerService.volume = event.volume
                        }
                    }, _onReceiverEventTimeUpdateRequested: function _onReceiverEventTimeUpdateRequested(event) {
                        if (!this._receiver)
                            return;
                        try {
                            if (this._hasSource) {
                                var currentTime = this._playbackSession.forceTimeUpdate() > 0 ? this._playbackSession.forceTimeUpdate() : 0;
                                this._receiver.notifyTimeUpdate(currentTime)
                            }
                            else
                                this._receiver.notifyTimeUpdate(0)
                        }
                        catch(ex) {
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventTimeUpdateRequested() exception, errorCode= " + ex.number + ", message=" + ex.message);
                            MS.Entertainment.Utilities.Telemetry.logPlayToReceiverError(ex, "Failed time update: " + ex.message)
                        }
                    }, _onReceiverEventStopRequested: function _onReceiverEventStopRequested(event) {
                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventStopRequested()");
                        if (!this._receiver)
                            return;
                        try {
                            if (this._hasSource)
                                if (this._playbackSession.playerState !== MS.Entertainment.Platform.Playback.PlayerState.error) {
                                    this._Pause = false;
                                    this._Stopping = true;
                                    if (this._playbackSession.targetTransportState === MS.Entertainment.Platform.Playback.TransportState.paused) {
                                        if (this._playbackSession.currentPosition !== 0) {
                                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventStopRequested() calling seekToPosition(0)");
                                            this._playbackSession.seekToPosition(0)
                                        }
                                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventStopRequested() calling notifyStopped() 1");
                                        this._receiver.notifyStopped()
                                    }
                                    else {
                                        this._playbackSession.targetTransportState = MS.Entertainment.Platform.Playback.TransportState.paused;
                                        MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventStopRequested() calling seekToPosition(0)");
                                        this._playbackSession.seekToPosition(0)
                                    }
                                }
                                else {
                                    MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventStopRequested() calling notifyStopped() 2");
                                    this._receiver.notifyStopped()
                                }
                            else {
                                MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onReceiverEventStopRequested() calling notifyStopped() 3");
                                this._receiver.notifyStopped()
                            }
                        }
                        catch(ex) {
                            MSEPlatform.Playback.Etw.traceString("PlayToReceiver::_onRecevierEventStopRequested() exception, errorCode= " + ex.number + ", message=" + ex.message);
                            MS.Entertainment.Utilities.Telemetry.logPlayToReceiverError(ex, "PlayTo receiver failed to stop: " + ex.message)
                        }
                    }
            })
    });
    MS.Entertainment.ServiceLocator.register(MS.Entertainment.Services.playToReceiverNativeService, function getPlayToReceiverNativeService() {
        return new Windows.Media.PlayTo.PlayToReceiver
    })
})()
