﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    WinJS.Namespace.define("MS.Entertainment.Utilities.Settings", {
        onShow: function onShow(sender, id) {
            MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.typeToSearch).disableTypeToSearch();
            var eventProvider = new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell;
            switch (id) {
                case"SettingsAccount":
                    eventProvider.traceSettingsAccount_Launch(id);
                    break;
                case"SettingsAbout":
                    eventProvider.traceSettingsAbout_Launch(id);
                    break;
                case"SettingsPreferences":
                    eventProvider.traceSettingsPreferences_Launch(id);
                    break;
                case"SettingsFeedback":
                    eventProvider.traceSettingsFeedback_Launch(id);
                    break;
                case"SettingsCaptions":
                    eventProvider.traceSettingsCaptions_Launch(id);
                    break
            }
        }, onShowComplete: function onShowComplete(sender, id) {
                var uiState = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                uiState.isSettingsCharmVisible = true;
                var eventProvider = new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell;
                switch (sender.currentTarget.id) {
                    case"SettingsAccount":
                        eventProvider.traceSettingsAccount_LaunchComplete(sender.currentTarget.id);
                        break;
                    case"SettingsAbout":
                        eventProvider.traceSettingsAbout_LaunchComplete(sender.currentTarget.id);
                        break;
                    case"SettingsPreferences":
                        eventProvider.traceSettingsPreferences_LaunchComplete(sender.currentTarget.id);
                        break;
                    case"SettingsFeedback":
                        eventProvider.traceSettingsFeedback_LaunchComplete(sender.currentTarget.id);
                        break;
                    case"SettingsCaptions":
                        eventProvider.traceSettingsCaptions_LaunchComplete(sender.currentTarget.id);
                        break
                }
            }
    });
    WinJS.Namespace.define("MS.Entertainment.Utilities", {SettingsWrapper: MS.Entertainment.UI.Framework.defineUserControl(null, function(element, options) {
            MS.Entertainment.Utilities.Settings.onShow(null, element.parentElement.id)
        }, {
            processChildren: true, deferInitialization: true, ignoreChildrenInitialization: true, controlName: "SettingsWrapper", initialize: function initialize() {
                    this.domElement.parentElement.addEventListener("aftershow", MS.Entertainment.Utilities.Settings.onShowComplete, false);
                    this.domElement.parentElement.addEventListener("afterhide", this.onHideComplete, false);
                    if (this._backButton)
                        this._backButton.setAttribute("aria-label", String.load(String.id.IDS_ACC_BACK_BUTTON))
                }, backToSettings: function backToSettings() {
                    try {
                        WinJS.UI.SettingsFlyout.show()
                    }
                    catch(ex) {
                        MS.Entertainment.Utilities.fail(false, "WinJS.UI.SettingsFlyout.show() fails: " + ex.toString())
                    }
                    var eventProvider = new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell;
                    eventProvider.traceSettings_BackToHome("")
                }, onHideComplete: function onHideComplete(sender, id) {
                    MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.typeToSearch).enableTypeToSearch();
                    var uiState = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                    uiState.isSettingsCharmVisible = false
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.Utilities", {PreferenceSettingsWrapper: MS.Entertainment.UI.Framework.deriveUserControl("MS.Entertainment.Utilities.SettingsWrapper", null, function(element, options){}, {
            _onIsGrovelingChangedHandler: null, _displayingCredUI: false, initialize: function initialize() {
                    MS.Entertainment.Utilities.SettingsWrapper.prototype.initialize.call(this);
                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    this._initializeSignedInSetting(signIn);
                    this._initializeMediaLibrarySetting();
                    this._initializeLaunchToCollectionSetting();
                    this._initializeGoOnlineForMetadataSetting();
                    this._initializePromptOnPurchaseSetting(signIn);
                    this._initializeMusicPassDownloadEnabledSetting(signIn);
                    this._initializeCloudCollectionEnabledSetting();
                    this._initializeCloudMatchEnabledSetting();
                    this._initializeCloudCollectionOfflineSetting();
                    this._initializePurchasesSetting();
                    this._initializeFreePlayLimitsSetting()
                }, forgetMe: function forgetMe() {
                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    signIn.signOut();
                    this.backToSettings()
                }, restorePurchases: function restorePurchases() {
                    var service = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.purchaseHistory);
                    service.grovel(true, true)
                }, showLocalGrovelDialog: function showLocalGrovelDialog() {
                    if (WinJS.Utilities.getMember("Windows.Storage.StorageLibrary"))
                        MS.Entertainment.UI.Controls.ManageFoldersDialog.show();
                    else if (MS.Entertainment.Utilities.isMusicApp)
                        MS.Entertainment.Music.localGrovelInfoDialog.show()
                }, handleKeyDownForLocalGrovelDialog: function handleKeyDownForLocalGrovelDialog(e) {
                    if (e.keyCode !== WinJS.Utilities.Key.enter)
                        return;
                    this.showLocalGrovelDialog();
                    e.stopPropagation();
                    e.preventDefault()
                }, showCloudCollectionIconInfoDialog: function showCloudCollectionIconInfoDialog() {
                    if (MS.Entertainment.Utilities.isMusicApp)
                        MS.Entertainment.Music.cloudMatchIconDialog.show()
                }, handleKeyDownForCloudCollectionIconInfoDialog: function handleKeyDownForCloudCollectionIconInfoDialog(e) {
                    if (e.keyCode !== WinJS.Utilities.Key.enter)
                        return;
                    this.showCloudCollectionIconInfoDialog();
                    e.stopPropagation();
                    e.preventDefault()
                }, showCloudCollectionOptInDialog: function showCloudCollectionOptInDialog() {
                    if (MS.Entertainment.Utilities.isMusicApp)
                        MS.Entertainment.Music.cloudMatchOptInDialog.show()
                }, handleKeyDownForCloudCollectionOptInDialog: function handleKeyDownForCloudCollectionOptInDialog(e) {
                    if (e.keyCode !== WinJS.Utilities.Key.enter)
                        return;
                    this.showCloudCollectionOptInDialog();
                    e.stopPropagation();
                    e.preventDefault()
                }, promptToggleChange: function promptToggleChange(event) {
                    if (this.promptOnPurchase.checked)
                        (new Microsoft.Entertainment.Configuration.ConfigurationManager).generalSettings.alwaysPromptOnPurchase = this.promptOnPurchase.checked;
                    else if (this._displayingCredUI)
                        return;
                    else {
                        this._displayingCredUI = true;
                        var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                        signIn.getPassportTicket(MS.Entertainment.Utilities.SignIn.TicketType.MBI_SSL, MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_PassportTicket), false, Microsoft.Entertainment.Util.SignInPromptType.retypeCredentials).then(function success(t) {
                            (new Microsoft.Entertainment.Configuration.ConfigurationManager).generalSettings.alwaysPromptOnPurchase = false;
                            WinJS.UI.SettingsFlyout.showSettings("SettingsPreferences", "/Components/Settings/SettingsPreferences.html")
                        }.bind(this), function getPassportTicketError(errTicket) {
                            MS.Entertainment.Utilities.assert(false, "Toggling prompt preferences fails with error code: " + errTicket.number)
                        })
                    }
                }, musicPassDownloadEnabledToggleChange: function musicPassDownloadEnabledToggleChange() {
                    var checkSettingAllowedPromise = WinJS.Promise.wrap();
                    if (this.musicPassDownloadEnabled.checked)
                        checkSettingAllowedPromise = MS.Entertainment.UI.SubscriptionDownload.verifyMachineActivationIsNotAtLimit();
                    checkSettingAllowedPromise.then(function settingIsAllowed() {
                        (Microsoft.Entertainment.Configuration.ConfigurationManager()).music.musicPassDownloadEnabled = this.musicPassDownloadEnabled.checked;
                        MS.Entertainment.Utilities.Telemetry.logTelemetryEvent(MS.Entertainment.Utilities.PreferenceSettingsWrapper.TelemetryEvents.MusicPassDownloadEnabledStateChanged, MS.Entertainment.Utilities.PreferenceSettingsWrapper.States.MusicPassDownloadEnabledState, this.musicPassDownloadEnabled.checked)
                    }.bind(this), function settingIsNotAllowed(errorCode) {
                        if (errorCode === MS.Entertainment.UI.SubscriptionDownload.deviceActivationLimitExceededErrorCode) {
                            MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_DOWNLOAD_ERROR_CAPTION), errorCode);
                            this.musicPassDownloadEnabled.checked = false
                        }
                    }.bind(this))
                }, cloudCollectionEnabledToggleChange: function cloudCollectionEnabledToggleChange(event) {
                    var cloudCollectionService = this._getCloudCollectionService();
                    if (cloudCollectionService) {
                        cloudCollectionService.isEnabled = this.cloudCollectionEnabled.checked;
                        this._updateCloudCollectionDescription(cloudCollectionService.isEnabled);
                        this._initializeCloudCollectionOfflineSetting();
                        this._initializeCloudMatchEnabledSetting();
                        MS.Entertainment.Utilities.Telemetry.logTelemetryEvent(MS.Entertainment.Utilities.PreferenceSettingsWrapper.TelemetryEvents.CloudCollectionEnabledStateChanged, MS.Entertainment.Utilities.PreferenceSettingsWrapper.States.CloudCollectionEnabledState, this.cloudCollectionEnabled.checked)
                    }
                }, cloudMatchEnabledToggleChange: function cloudMatchEnabledToggleChange(event) {
                    var cloudCollectionService = this._getCloudCollectionService();
                    if (cloudCollectionService) {
                        var optIn = this.cloudMatchEnabled.checked ? MS.Entertainment.CloudCollectionService.CloudMatchStatus.OptedIn : MS.Entertainment.CloudCollectionService.CloudMatchStatus.OptedOut;
                        cloudCollectionService.setCloudMatchOptIn(optIn);
                        MS.Entertainment.Utilities.Telemetry.logTelemetryEvent(MS.Entertainment.Utilities.PreferenceSettingsWrapper.TelemetryEvents.CloudMatchEnabledStateChanged, MS.Entertainment.Utilities.PreferenceSettingsWrapper.States.CloudMatchEnabledState, this.cloudMatchEnabled.checked, MS.Entertainment.Utilities.Telemetry.Events.CloudMatchStateChangeMethod, MS.Entertainment.Utilities.Telemetry.StateChangeMethodValues.toggle)
                    }
                }, cloudCollectionOfflineToggleChange: function cloudCollectionOfflineToggleChange(event) {
                    var checkSettingAllowedPromise = WinJS.Promise.wrap();
                    if (this.cloudCollectionOffline.checked)
                        checkSettingAllowedPromise = MS.Entertainment.UI.SubscriptionDownload.verifyMachineActivationIsNotAtLimit();
                    checkSettingAllowedPromise.then(function settingIsAllowed() {
                        var cloudCollectionService = this._getCloudCollectionService();
                        if (cloudCollectionService)
                            cloudCollectionService.autoDownloadEnabled = this.cloudCollectionOffline.checked;
                        MS.Entertainment.Utilities.Telemetry.logTelemetryEvent(MS.Entertainment.Utilities.PreferenceSettingsWrapper.TelemetryEvents.CloudCollectionAutoDownloadStateChanged, MS.Entertainment.Utilities.PreferenceSettingsWrapper.States.CloudCollectionAutoDownloadState, this.cloudCollectionOffline.checked)
                    }.bind(this), function settingIsNotAllowed(errorCode) {
                        if (errorCode === MS.Entertainment.UI.SubscriptionDownload.deviceActivationLimitExceededErrorCode) {
                            MS.Entertainment.UI.Shell.showError(String.load(String.id.IDS_DOWNLOAD_ERROR_CAPTION), errorCode);
                            this.cloudCollectionOffline.checked = false
                        }
                    }.bind(this))
                }, goOnlineForMetadataChange: function goOnlineForMetadataChange(event) {
                    (new Microsoft.Entertainment.Configuration.ConfigurationManager).mdlc.connectToInternetForAlbumMetadata = this.goOnlineForMetadata.checked
                }, launchToCollectionChange: function launchToCollectionChange(event) {
                    var launchLocation;
                    if (this.launchToCollection.checked)
                        if (MS.Entertainment.Utilities.isMusicApp)
                            launchLocation = MS.Entertainment.UI.Monikers.musicCollection;
                        else if (MS.Entertainment.Utilities.isVideoApp)
                            launchLocation = MS.Entertainment.UI.Monikers.videoCollection;
                    var settingsStorage = null;
                    if (MS.Entertainment.Utilities.isApp1)
                        settingsStorage = Windows.Storage.ApplicationData.current.roamingSettings;
                    else
                        settingsStorage = Windows.Storage.ApplicationData.current.localSettings;
                    if (launchLocation)
                        settingsStorage.values["launchLocation"] = launchLocation;
                    else
                        settingsStorage.values.remove("launchLocation")
                }, _getCloudCollectionService: function _getCloudCollectionService() {
                    if (MS.Entertainment.Utilities.isMusicApp && MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.cloudCollection))
                        return MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.cloudCollection);
                    return null
                }, _initializeMediaLibrarySetting: function _initializeMediaLibrarySetting() {
                    if (MS.Entertainment.Utilities.isMusicApp) {
                        this.libraryPreferencesTitle.text = String.load(String.id.IDS_MY_MUSIC_LIBRARY_SETTINGS_TITLE);
                        if (WinJS.Utilities.getMember("Windows.Storage.StorageLibrary"))
                            this.libraryPreferencesWebLink.stringId = String.id.IDS_COLLECTION_MUSIC_MANAGE_FOLDERS_LINK;
                        else {
                            this.libraryPreferencesText.text = String.load(String.id.IDS_COLLECTION_MUSIC_LIBRARY_SETTINGS_TEXT);
                            this.libraryPreferencesWebLink.stringId = String.id.IDS_COLLECTION_MUSIC_MORE_LIBRARIES_LINK
                        }
                    }
                    else if (MS.Entertainment.Utilities.isVideoApp) {
                        this.libraryPreferencesTitle.text = String.load(String.id.IDS_COLLECTION_VIDEO_LIBRARY_SETTINGS_TITLE);
                        if (WinJS.Utilities.getMember("Windows.Storage.StorageLibrary"))
                            this.libraryPreferencesWebLink.stringId = String.id.IDS_COLLECTION_VIDEO_MANAGE_FOLDERS_LINK;
                        else {
                            this.libraryPreferencesText.text = String.load(String.id.IDS_COLLECTION_VIDEO_LIBRARY_SETTINGS_TEXT);
                            this.libraryPreferencesWebLink.stringId = String.id.IDS_COLLECTION_VIDEO_MORE_LIBRARIES_LINK;
                            this.libraryPreferencesWebLink.domElement.href = MS.Entertainment.UI.FWLink.videoLibraries
                        }
                    }
                    else
                        this._hideElement(this.libraryPreferences)
                }, _initializeSignedInSetting: function _initializeSignedInSetting(signIn) {
                    var config = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    if (signIn.isSignedIn && config.shell.showRemoveInPreferences) {
                        var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                        this.currentUserId.text = String.load(String.id.IDS_SETTINGS_ACCOUNT_CURRENT_USER).format(signedInUser.signInName);
                        if (!signedInUser.canSignOut) {
                            this._hideElement(this.forgetMeButton);
                            this._hideElement(this.forgetMeDesc)
                        }
                    }
                    else
                        this._hideElement(this.switchUserContainer)
                }, _initializePromptOnPurchaseSetting: function _initializePromptOnPurchaseSetting(signIn) {
                    if (signIn.isSignedIn && (new Microsoft.Entertainment.Configuration.ConfigurationManager).generalSettings.showPromptOnPurchaseSetting) {
                        this.promptOnPurchase.title = String.load(String.id.IDS_SETTINGS_PROMPT_TOGGLE_TITLE);
                        this.promptOnPurchase.checked = (new Microsoft.Entertainment.Configuration.ConfigurationManager).generalSettings.alwaysPromptOnPurchase
                    }
                    else
                        this._hideElement(this.promptOnPurchaseContainer)
                }, _updateMusicDownloadSettingsContainer: function _updateMusicDownloadSettingsContainer() {
                    if (this._isCloudCollectionOfflineAvailable() || this._isMusicPassDownloadAvailable())
                        this._showElement(this.downloadSettingsContainer);
                    else
                        this._hideElement(this.downloadSettingsContainer)
                }, _isMusicPassDownloadAvailable: function _isMusicPassDownloadAvailable() {
                    return MS.Entertainment.Utilities.isMusicApp && MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).isSignedIn && MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser).isSubscription
                }, _initializeMusicPassDownloadEnabledSetting: function _initializeMusicPassDownloadEnabledSetting(signIn) {
                    if (this._isMusicPassDownloadAvailable()) {
                        this.musicPassDownloadEnabled.title = String.load(String.id.IDS_MUSIC_SETTINGS_MUSIC_PASS_DOWNLOAD_DESCRIPTION);
                        this.musicPassDownloadEnabled.checked = (new Microsoft.Entertainment.Configuration.ConfigurationManager).music.musicPassDownloadEnabled
                    }
                    else
                        this._hideElement(this.musicPassDownloadEnabledSettingsContainer);
                    this._updateMusicDownloadSettingsContainer()
                }, _initializeCloudCollectionEnabledSetting: function _initializeCloudCollectionEnabledSetting() {
                    var cloudCollectionService = this._getCloudCollectionService();
                    if (MS.Entertainment.Utilities.isMusicApp && cloudCollectionService && cloudCollectionService.isAvailable) {
                        var isCloudCollectionEnabled = cloudCollectionService.isEnabled;
                        this.cloudCollectionEnabled.checked = isCloudCollectionEnabled;
                        this._updateCloudCollectionDescription(isCloudCollectionEnabled)
                    }
                    else {
                        this._hideElement(this.cloudCollectionEnabledSettingsContainer);
                        this._hideElement(this.cloudCollectionOfflineSettingsContainer);
                        this._hideElement(this.cloudIconInfoWebLink)
                    }
                    this._updateMusicDownloadSettingsContainer()
                }, _updateCloudCollectionDescription: function _updateCloudCollectionDescription(isCloudCollectionEnabled) {
                    this.cloudCollectionEnabled.title = isCloudCollectionEnabled ? String.load(String.id.IDS_MUSIC_SETTINGS_CLOUD_COLLECTION_ON_DESCRIPTION) : String.load(String.id.IDS_MUSIC_SETTINGS_CLOUD_COLLECTION_OFF_DESCRIPTION)
                }, _initializeCloudMatchEnabledSetting: function _initializeCloudMatchEnabledSetting() {
                    var cloudCollectionService = this._getCloudCollectionService();
                    if (MS.Entertainment.Utilities.isMusicApp && cloudCollectionService && cloudCollectionService.isEnabled) {
                        this.cloudMatchEnabled.checked = cloudCollectionService.isCloudMatchOptedIn;
                        this.cloudMatchEnabled.title = String.load(String.id.IDS_MUSIC_CLOUD_SETTINGS_MATCH_TOGGLE_DESC);
                        this._showElement(this.cloudMatchSettingContainer)
                    }
                    else
                        this._hideElement(this.cloudMatchSettingContainer)
                }, _isCloudCollectionOfflineAvailable: function _isCloudCollectionOfflineAvailable() {
                    var cloudCollectionService = this._getCloudCollectionService();
                    return MS.Entertainment.Utilities.isMusicApp && cloudCollectionService && cloudCollectionService.isEnabled && (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn)).isSignedIn
                }, _initializeCloudCollectionOfflineSetting: function _initializeCloudCollectionOfflineSetting() {
                    if (this._isCloudCollectionOfflineAvailable()) {
                        var isSubscriptionUser = (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser)).isSubscription;
                        this.cloudCollectionOffline.title = isSubscriptionUser ? String.load(String.id.IDS_SETTINGS_CLOUD_COLLECTION_OFFLINE_TOGGLE_TITLE) : String.load(String.id.IDS_SETTINGS_CLOUD_COLLECTION_OFFLINE_TOGGLE_TITLE_FREE);
                        this.cloudCollectionOffline.checked = this._getCloudCollectionService().autoDownloadEnabled;
                        this._showElement(this.cloudCollectionOfflineSettingsContainer)
                    }
                    else
                        this._hideElement(this.cloudCollectionOfflineSettingsContainer);
                    this._updateMusicDownloadSettingsContainer()
                }, _initializeLaunchToCollectionSetting: function _initializeLaunchToCollectionSetting() {
                    var enableMusicSetting;
                    var enableVideoSetting;
                    var appMode = MS.Entertainment.appMode;
                    var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                    if (MS.Entertainment.Utilities.isMusicApp) {
                        var musicMarketplaceEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace);
                        enableMusicSetting = musicMarketplaceEnabled && !window.onNewMusicPage
                    }
                    else if (MS.Entertainment.Utilities.isVideoApp) {
                        var moviesMarketplaceEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.moviesMarketplace);
                        var tvMarketplaceEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.tvMarketplace);
                        enableVideoSetting = moviesMarketplaceEnabled || tvMarketplaceEnabled
                    }
                    var settingsStorage = null;
                    if (MS.Entertainment.Utilities.isApp1)
                        settingsStorage = Windows.Storage.ApplicationData.current.roamingSettings;
                    else
                        settingsStorage = Windows.Storage.ApplicationData.current.localSettings;
                    if (enableMusicSetting) {
                        this.launchToCollection.title = String.load(String.id.IDS_MUSIC_SETTINGS_STARTUP_VIEW_DESC);
                        this.launchToCollection.checked = !!(settingsStorage.values["launchLocation"])
                    }
                    else if (enableVideoSetting) {
                        this.launchToCollection.title = String.load(String.id.IDS_VIDEO_SETTINGS_STARTUP_VIEW_DESC);
                        this.launchToCollection.checked = !!(settingsStorage.values["launchLocation"])
                    }
                    else
                        this._hideElement(this.launchToCollectionContainer)
                }, _initializeGoOnlineForMetadataSetting: function _initializeGoOnlineForMetadataSetting() {
                    var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                    if (MS.Entertainment.Utilities.isMusicApp) {
                        this.goOnlineForMetadata.title = String.load(String.id.IDS_MUSIC_SETTINGS_MEDIA_INFO_DESC);
                        this.goOnlineForMetadata.checked = (new Microsoft.Entertainment.Configuration.ConfigurationManager).mdlc.connectToInternetForAlbumMetadata
                    }
                    else
                        this._hideElement(this.goOnlineForMetadataContainer)
                }, _onIsGrovelingChanged: function _onIsGrovelingChanged(isGroveling) {
                    if (isGroveling) {
                        this.restorePurchaseButtonLabel.stringId = String.id.IDS_VIDEO_SETTINGS_PURCHASES_RESTORING_BUTTON;
                        this.restorePurchaseButton.disabled = true
                    }
                    else {
                        this.restorePurchaseButtonLabel.stringId = String.id.IDS_VIDEO_SETTINGS_PURCHASES_RESTORE_BUTTON;
                        this.restorePurchaseButton.disabled = false
                    }
                }, _initializePurchasesSetting: function _initializePurchasesSetting() {
                    var service = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.purchaseHistory);
                    var hideSetting = true;
                    if (service && MS.Entertainment.UI.PurchaseHistoryService.isFeatureEnabled && MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).isSignedIn && MS.Entertainment.Utilities.isVideoApp) {
                        var networkStatus = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).networkStatus;
                        switch (networkStatus) {
                            case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.localOnly:
                            case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.none:
                                break;
                            default:
                                hideSetting = false;
                                this._onIsGrovelingChangedHandler = this._onIsGrovelingChanged.bind(this);
                                service.bind("isGroveling", this._onIsGrovelingChangedHandler);
                                break
                        }
                    }
                    if (hideSetting)
                        this._hideElement(this.purchasesContainer)
                }, _initializeFreePlayLimitsSetting: function _initializeFreePlayLimitsSetting() {
                    this._hideElement(this.freePlayLimitsContainer);
                    if (!MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.freePlayLimits))
                        return;
                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                    if (!signIn.isSignedIn || signedInUser.isSubscription)
                        return;
                    var freePlayLimits = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.freePlayLimits);
                    freePlayLimits.getLimits().done(function getLimitsComplete(limits) {
                        if (limits && limits.freeTrialCompleted) {
                            if (!limits.monthlyFreeLimitExceeded)
                                this._hideElement(this.freePlayLimitsMonthlyLimitExceeded);
                            this.preferenceFreePlayLimitMeter.max = limits.monthlyFreeMinutesAllowance;
                            this.preferenceFreePlayLimitMeter.value = limits.monthlyFreeMinutesUsed;
                            var percentageUsed = 0;
                            if (limits.monthlyFreeMinutesUsed >= limits.monthlyFreeMinutesAllowance)
                                percentageUsed = 100;
                            else if (limits.monthlyFreeMinutesAllowance > 0)
                                percentageUsed = (limits.monthlyFreeMinutesUsed / limits.monthlyFreeMinutesAllowance) * 100;
                            percentageUsed = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).decimalNumber.format(Math.round(percentageUsed));
                            this.freePlayLimitsMeterPercentageLabel.textContent = String.load(String.id.IDS_SETTINGS_FREE_PLAY_LIMITS_METER_PERCENTAGE).format(percentageUsed);
                            var monthlyFreeHoursAllowance = Math.max(Math.floor(limits.monthlyFreeMinutesAllowance / 60), 0);
                            var dateFormatter = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.dateTimeFormatters).dayMonthYear;
                            var nextMonthlyFreeLimitResetDate = dateFormatter.format(limits.nextMonthlyFreeLimitResetDate);
                            this.freePlayLimitsMonthlyUsageLabel.text = String.load(String.id.IDS_MUSIC_SETTINGS_MONTHLY_LIMIT_DESC).format(monthlyFreeHoursAllowance, nextMonthlyFreeLimitResetDate);
                            this._showElement(this.freePlayLimitsContainer)
                        }
                    }.bind(this), function getLimitsError(){})
                }, onSubscribe: function onSubscribe() {
                    MS.Entertainment.Music.Freeplay.sendTelemetryEvent(MS.Entertainment.Music.Freeplay.Events.limitSettingsSubscribe);
                    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                    var action = actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.subscriptionSignup);
                    action.automationId = MS.Entertainment.UI.AutomationIds.settingsSubscriptionSignup;
                    action.execute()
                }, handleOnSubscribeKeyDown: function handleOnSubscribeKeyDown(e) {
                    if (e.keyCode !== WinJS.Utilities.Key.enter)
                        return;
                    this.onSubscribe()
                }, unload: function unload() {
                    var service = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.purchaseHistory);
                    if (service && this._onIsGrovelingChangedHandler) {
                        service.unbind("isGroveling", this._onIsGrovelingChangedHandler);
                        this._onIsGrovelingChangedHandler = null
                    }
                    MS.Entertainment.Utilities.SettingsWrapper.prototype.unload.call(this)
                }, _showElement: function _showElement(element) {
                    if (element.domElement)
                        element = element.domElement;
                    WinJS.Utilities.removeClass(element, "removeFromDisplay")
                }, _hideElement: function _hideElement(element) {
                    if (element.domElement)
                        element = element.domElement;
                    WinJS.Utilities.addClass(element, "removeFromDisplay")
                }
        }, null, {
            TelemetryEvents: {
                MusicPassDownloadEnabledStateChanged: "MusicPassDownloadEnabledStateChanged", CloudCollectionEnabledStateChanged: "CloudCollectionEnabledStateChanged", CloudCollectionAutoDownloadStateChanged: "CloudCollectionAutoDownloadStateChanged", CloudMatchEnabledStateChanged: "CloudMatchEnabledStateChanged"
            }, States: {
                    MusicPassDownloadEnabledState: "MusicPassDownloadEnabledState", CloudCollectionEnabledState: "CloudCollectionEnabledState", CloudCollectionAutoDownloadState: "CloudCollectionAutoDownloadState", CloudMatchEnabledState: "CloudMatchEnabledState"
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.Utilities", {CaptionSettingsWrapper: MS.Entertainment.UI.Framework.deriveUserControl("MS.Entertainment.Utilities.SettingsWrapper", null, function(element, options) {
            this.possibleColors = [{
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_DEFAULT), value: ""
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_COLOR_WHITE), value: "255,255,255"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_COLOR_BLACK), value: "0,0,0"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_COLOR_RED), value: "255,0,0"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_COLOR_GREEN), value: "0,255,0"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_COLOR_BLUE), value: "0,0,255"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_COLOR_YELLOW), value: "255,255,0"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_COLOR_MAGENTA), value: "255,0,255"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_COLOR_CYAN), value: "0,255,255"
                }];
            this.possibleCharacterOpacity = [{
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_DEFAULT), value: ""
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_OPACITY_OPAQUE), value: "1.0"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_OPACITY_SEMI_TRANSPARENT), value: "0.5"
                }];
            this.possibleOpacity = [{
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_DEFAULT), value: ""
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_OPACITY_OPAQUE), value: "1.0"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_OPACITY_SEMI_TRANSPARENT), value: "0.5"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_OPACITY_TRANSPARENT), value: "0"
                }];
            this.possibleSizes = [{
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_SIZE_50), value: "50%"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_SIZE_100), value: "100%"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_SIZE_150), value: "150%"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_SIZE_200), value: "200%"
                }];
            this.possibleFonts = [{
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_DEFAULT), value: ""
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_FONT_MONOSPACED_SERIF), value: "courier"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_FONT_PROPORTIONAL_SERIF), value: "times roman"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_FONT_MONOSPACED_SANS), value: "consolas"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_FONT_PROPORTIONAL_SANS), value: "arial"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_FONT_CASUAL), value: "comic sans ms"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_FONT_CURSIVE), value: "segoe script"
                }, {
                    name: String.load(String.id.IDS_SETTINGS_CAPTIONS_FONT_SMALL_CAPS), value: "trebuchet ms"
                }]
        }, {
            possibleColors: null, possibleCharacterOpacity: null, possibleOpacity: null, possibleSizes: null, possibleFonts: null, initialize: function initialize() {
                    MS.Entertainment.Utilities.SettingsWrapper.prototype.initialize.call(this);
                    this.domElement.parentElement.addEventListener("beforehide", this.onSettingsDismiss.bind(this), false);
                    this.previewText.innerText = String.load(String.id.IDS_SETTINGS_CAPTIONS_PREVIEW);
                    this.possibleColors.forEach(function loadCharacterColors(color) {
                        var opt = document.createElement("option");
                        this.characterColors.options.add(opt);
                        opt.label = color.name;
                        opt.value = color.value
                    }.bind(this));
                    this.possibleCharacterOpacity.forEach(function loadCharacterOpacity(opacity) {
                        var opt = document.createElement("option");
                        this.characterOpacity.options.add(opt);
                        opt.label = opacity.name;
                        opt.value = opacity.value
                    }.bind(this));
                    this.possibleSizes.forEach(function loadCharacterSizes(color) {
                        var opt = document.createElement("option");
                        this.characterSizes.options.add(opt);
                        opt.label = color.name;
                        opt.value = color.value
                    }.bind(this));
                    this.possibleFonts.forEach(function loadCharacterFonts(color) {
                        var opt = document.createElement("option");
                        this.characterFonts.options.add(opt);
                        opt.label = color.name;
                        opt.value = color.value
                    }.bind(this));
                    this.possibleColors.forEach(function loadBackgroundColors(color) {
                        var opt = document.createElement("option");
                        this.backgroundColors.options.add(opt);
                        opt.label = color.name;
                        opt.value = color.value
                    }.bind(this));
                    this.possibleOpacity.forEach(function loadBackgroundOpacity(opacity) {
                        var opt = document.createElement("option");
                        this.backgroundOpacity.options.add(opt);
                        opt.label = opacity.name;
                        opt.value = opacity.value
                    }.bind(this));
                    this._initializeCaptionSettings()
                }, onReset: function onReset() {
                    this.characterColors.options.selectedIndex = 0;
                    this.characterOpacity.options.selectedIndex = 0;
                    this.characterSizes.options.selectedIndex = 1;
                    this.characterFonts.options.selectedIndex = 0;
                    this.backgroundColors.options.selectedIndex = 0;
                    this.backgroundOpacity.options.selectedIndex = 0;
                    this._updatePreviewWindow()
                }, captionSettingsChange: function captionSettingsChange() {
                    this._updatePreviewWindow()
                }, onSettingsDismiss: function onSettingsDismiss() {
                    var settings = this._dumpCurrentSettings();
                    MS.Entertainment.Platform.Playback.ClosedCaptions.Renderer.saveClosedCaptionStyleSettings(settings);
                    MS.Entertainment.Platform.Playback.ClosedCaptions.Renderer.updateClosedCaptionStyleSettings(settings)
                }, _initializeCaptionSettings: function _initializeCaptionSettings() {
                    var ccSettings = MS.Entertainment.Platform.Playback.ClosedCaptions.Renderer.loadClosedCaptionStyleSettings();
                    var currIndex = 0;
                    var textBackgroundColorIndex = 0;
                    var textColorIndex = 0;
                    this.possibleColors.forEach(function searchColors(color) {
                        if (color.value === ccSettings["textBackgroundColor"])
                            textBackgroundColorIndex = currIndex;
                        if (color.value === ccSettings["textColor"])
                            textColorIndex = currIndex;
                        currIndex++
                    });
                    this.backgroundColors.options.selectedIndex = textBackgroundColorIndex;
                    this.characterColors.options.selectedIndex = textColorIndex;
                    var currIndex = 0;
                    var textOpacityIndex = 0;
                    this.possibleCharacterOpacity.forEach(function searchOpacity(opacity) {
                        if (opacity.value === ccSettings["textOpacity"])
                            textOpacityIndex = currIndex;
                        currIndex++
                    });
                    this.characterOpacity.options.selectedIndex = textOpacityIndex;
                    var currIndex = 0;
                    var textBackgroundOpacityIndex = 0;
                    this.possibleOpacity.forEach(function searchOpacity(opacity) {
                        if (opacity.value === ccSettings["textBackgroundOpacity"])
                            textBackgroundOpacityIndex = currIndex;
                        currIndex++
                    });
                    this.backgroundOpacity.options.selectedIndex = textBackgroundOpacityIndex;
                    var currIndex = 0;
                    var textFontFamilyIndex = 0;
                    this.possibleFonts.forEach(function searchFonts(font) {
                        if (font.value === ccSettings["textFontFamily"])
                            textFontFamilyIndex = currIndex;
                        currIndex++
                    });
                    this.characterFonts.options.selectedIndex = textFontFamilyIndex;
                    var currIndex = 0;
                    var textFontSizeIndex = 1;
                    this.possibleSizes.forEach(function searchSizes(size) {
                        if (size.value === ccSettings["textFontSize"])
                            textFontSizeIndex = currIndex;
                        currIndex++
                    });
                    this.characterSizes.options.selectedIndex = textFontSizeIndex;
                    this._updatePreviewWindow()
                }, _updatePreviewWindow: function _updatePreviewWindow() {
                    var settings = this._dumpCurrentSettings();
                    MS.Entertainment.Platform.Playback.ClosedCaptions.Renderer.updateClosedCaptionStyleSettings(settings, true)
                }, _dumpCurrentSettings: function _dumpCurrentSettings() {
                    var settings = {
                            textFontFamily: this.characterFonts.options[this.characterFonts.options.selectedIndex].value, textFontSize: this.characterSizes.options[this.characterSizes.options.selectedIndex].value, textColor: this.characterColors.options[this.characterColors.options.selectedIndex].value, textOpacity: this.characterOpacity.options[this.characterOpacity.options.selectedIndex].value, textBackgroundColor: this.backgroundColors.options[this.backgroundColors.options.selectedIndex].value, textBackgroundOpacity: this.backgroundOpacity.options[this.backgroundOpacity.options.selectedIndex].value
                        };
                    return settings
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.Utilities", {AboutSettingsWrapper: MS.Entertainment.UI.Framework.deriveUserControl("MS.Entertainment.Utilities.SettingsWrapper", null, function aboutSettingsWrapperConstructor(element, options){}, {
            initialize: function initialize() {
                MS.Entertainment.Utilities.SettingsWrapper.prototype.initialize.call(this);
                var helpLink;
                if (MS.Entertainment.Utilities.isMusicApp)
                    helpLink = "http://go.microsoft.com/fwlink/p/?LinkId=255543";
                else if (MS.Entertainment.Utilities.isVideoApp)
                    helpLink = "http://go.microsoft.com/fwlink/p/?LinkId=255544";
                if (helpLink)
                    this.helpLink.domElement.setAttribute("href", helpLink);
                this._showOrHideCopyrights();
                var imprintLinkUrl = this._getImprintLink();
                if (imprintLinkUrl && WinJS.Utilities.getMember("imprintLink.domElement", this)) {
                    this.imprintLink.stringId = String.id.IDS_SETTINGS_IMPRINT_LEGAL_LINK_TITLE;
                    this.imprintLink.domElement.setAttribute("href", imprintLinkUrl);
                    WinJS.Utilities.removeClass(this.imprintLink.domElement, "removeFromDisplay")
                }
                var config = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                if (config.generalSettings.showPerfTrackLog) {
                    var perfTrackLog = "";
                    MS.Entertainment.Instrumentation.PerfTrack.triggerLog.forEach(function perfTrackLogEntry(logEntry) {
                        perfTrackLog += logEntry + "\n"
                    });
                    this.perfTrackLog.value = perfTrackLog;
                    WinJS.Utilities.removeClass(this.perfTrackLog, "removeFromDisplay")
                }
                else if (WinJS.Utilities.getMember("MS.Entertainment.UI.Application.Helpers.perfMessages.length") > 0) {
                    var perfTrackLog = "";
                    for (var i = -0; i < MS.Entertainment.UI.Application.Helpers.perfMessages.length; i++)
                        perfTrackLog += MS.Entertainment.UI.Application.Helpers.perfMessages[i] + "\n";
                    this.perfTrackLog.value = perfTrackLog;
                    WinJS.Utilities.removeClass(this.perfTrackLog, "removeFromDisplay")
                }
            }, _showOrHideCopyrights: function _showOrHideCopyrights() {
                    if (MS.Entertainment.Utilities.isMusicApp) {
                        WinJS.Utilities.removeClass(this.copyrights.domElement, "removeFromDisplay");
                        var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                        var showGracenoteCopyright = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.audioFingerprinter) || featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.automaticMusicMatch) || featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMatchService);
                        if (showGracenoteCopyright)
                            WinJS.Utilities.removeClass(this.gracenoteCopyright.domElement, "removeFromDisplay");
                        else
                            WinJS.Utilities.addClass(this.gracenoteCopyright.domElement, "removeFromDisplay")
                    }
                    else
                        WinJS.Utilities.addClass(this.copyrights.domElement, "removeFromDisplay")
                }, _getImprintLink: function _getImprintLink() {
                    var globalizationManager = new Microsoft.Entertainment.Util.GlobalizationManager;
                    var currentRegion = globalizationManager.getRegion().toLowerCase();
                    var link = String.empty;
                    switch (currentRegion) {
                        case"at":
                            link = "http://go.microsoft.com/fwlink/?LinkId=317759&clcid=0xc07";
                            break;
                        case"de":
                            link = "http://go.microsoft.com/fwlink/?LinkId=317760&clcid=0x407";
                            break;
                        case"ch":
                            link = "http://go.microsoft.com/fwlink/?LinkId=317761&clcid=0x807";
                            break
                    }
                    return link
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.Utilities", {SettingsVersion: MS.Entertainment.UI.Framework.defineUserControl(null, function SettingsVersionConstructor(element) {
            this.versionPromise = WinJS.xhr({url: "version.json"}).then(function(data) {
                var versionInformation = JSON.parse(data.responseText);
                return versionInformation.version
            })
        }, {
            controlName: "SettingsVersion", versionPromise: null, initialize: function initialize() {
                    MS.Entertainment.Utilities.assert(this.versionPromise, "Promise to load the version string hasn't been set");
                    this.versionPromise.then(function(v) {
                        this.domElement.textContent = v
                    }.bind(this))
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.Utilities", {
        AccountSettingsWrapper: MS.Entertainment.UI.Framework.deriveUserControl("MS.Entertainment.Utilities.SettingsWrapper", null, function AccountSettingsWrapper_constructor(){}, {
            _signInHandler: null, _signIn: null, _signedInUser: null, _actionService: null, initialize: function initialize() {
                    MS.Entertainment.Utilities.SettingsWrapper.prototype.initialize.call(this);
                    this.domElement.parentElement.addEventListener("aftershow", MS.Entertainment.Utilities.Settings.onShowComplete, false);
                    this.domElement.parentElement.addEventListener("beforehide", this.onAccountPanelHide, false);
                    this._signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    this._signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                    this._actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                    this.onAccountShow();
                    if (this._backButton)
                        this._backButton.setAttribute("aria-label", String.load(String.id.IDS_ACC_BACK_BUTTON))
                }, unload: function unload() {
                    if (this._signInHandler)
                        this._signIn.unbind("isSignedIn", this._signInHandler);
                    MS.Entertainment.Utilities.SettingsWrapper.prototype.unload.call(this)
                }, onAccountShow: function onAccountShow() {
                    if (!this._signIn.isSignedIn) {
                        this._onlineAccountPanel.style.visibility = "collapse";
                        this._offlineAccountPanel.style.visibility = "visible";
                        this._signInHandler = this._onSignInChange.bind(this);
                        this._signIn.bind("isSignedIn", this._signInHandler)
                    }
                    else
                        this._loadXboxAccountSummaryPage()
                }, onAccountPanelHide: function onAccountPanelHide(sender, id) {
                    var uiState = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                    uiState.isSettingsCharmVisible = false;
                    var accountPanel = sender.currentTarget;
                    if (accountPanel) {
                        MS.Entertainment.UI.assert(accountPanel.winControl._dismiss, "accountPanel.winControl._dismiss missing");
                        accountPanel.winControl._dismiss()
                    }
                    if (MS.Entertainment.Utilities.Settings._signInHandler) {
                        this._signIn.unbind("isSignedIn", this._signInHandler);
                        this._signInHandler = null
                    }
                }, _loadXboxAccountSummaryPage: function _loadXboxAccountSummaryPage() {
                    if (this._onlineAccountPanel && this._signIn.isSignedIn) {
                        this._offlineAccountPanel.style.visibility = "collapse";
                        if (MS.Entertainment.Utilities.isMusicApp) {
                            if ((new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement).isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicSubscription)) {
                                WinJS.Utilities.removeClass(this._musicPassButton, "removeFromDisplay");
                                this._musicPassLevel.stringId = this._getMusicPassInformationStringId()
                            }
                            if (this._signedInUser.isSubscription)
                                WinJS.Utilities.removeClass(this._musicDevicesButton, "removeFromDisplay")
                        }
                        this._accountEmail.text = this._signedInUser.signInName;
                        if (this._signedInUser.canSignOut)
                            this._accountHeader.stringId = String.id.IDS_SETTINGS_ACCOUNT_SIGN_OUT_TC;
                        else
                            this._accountHeader.stringId = String.id.IDS_SETTINGS_ACCOUNT_YOUR_ACCOUNT_TC;
                        this._onlineAccountPanel.style.visibility = "visible"
                    }
                }, _getMusicPassInformationStringId: function _getMusicPassInformationStringId() {
                    if (!this._signedInUser.isSubscription)
                        return String.id.IDS_SETTINGS_ACCOUNT_MUSIC_PASS_DESC_NONE_TC;
                    var stringIdToReturn = MS.Entertainment.Utilities.AccountSettingsWrapper._musicPassSubscriptionStringMap[this._signedInUser.subscriptionOfferID];
                    if (!stringIdToReturn)
                        stringIdToReturn = String.id.IDS_SETTINGS_ACCOUNT_MANAGE_DEVICES_DESC_TC;
                    return stringIdToReturn
                }, _redeemCodeClick: function _redeemCodeClick(e) {
                    if (e && e.keyCode && !(e.keyCode === WinJS.Utilities.Key.enter || e.keyCode === WinJS.Utilities.Key.space))
                        return;
                    var url = MS.Entertainment.Utilities.isModernMPEnabled ? MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_ModernPurchase) + "/redeem?client=x13" : MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_XBoxLiveSSL) + "/RedeemCode";
                    this._openWebblendDialog(url)
                }, _musicPassSubscribeClick: function _musicPassSubscribeClick(e) {
                    if (e && e.keyCode && !(e.keyCode === WinJS.Utilities.Key.enter || e.keyCode === WinJS.Utilities.Key.space))
                        return;
                    if (MS.Entertainment.Utilities.isModernMPEnabled) {
                        var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                        var action = actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.subscriptionSignup);
                        action.automationId = MS.Entertainment.UI.AutomationIds.settingsSubscriptionSignup;
                        action.execute()
                    }
                    else
                        this._openWebblendDialog(MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_XBoxLiveSSL) + "/Subscriptions?titleId=4294838135")
                }, _manageMusicDevices: function _manageMusicDevices(e) {
                    if (e && e.keyCode && !(e.keyCode === WinJS.Utilities.Key.enter || e.keyCode === WinJS.Utilities.Key.space))
                        return;
                    var externalNavigationAction = this._actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.externalNavigate);
                    externalNavigationAction.parameter = MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_XBoxLiveSSL) + "/devices/manage";
                    externalNavigationAction.automationId = MS.Entertainment.UI.AutomationIds.settingsAccountManageDevices;
                    externalNavigationAction.execute()
                }, _managePaymentOptionsClick: function _managePaymentOptionsClick(e) {
                    if (e && e.keyCode && !(e.keyCode === WinJS.Utilities.Key.enter || e.keyCode === WinJS.Utilities.Key.space))
                        return;
                    var externalNavigationAction = this._actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.externalNavigate);
                    externalNavigationAction.parameter = MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_XBoxLiveSSL) + "/ManagePaymentOptions";
                    externalNavigationAction.automationId = MS.Entertainment.UI.AutomationIds.settingsAccountManagePayment;
                    externalNavigationAction.execute()
                }, _billingContactInfoClick: function _billingContactInfoClick(e) {
                    if (e && e.keyCode && !(e.keyCode === WinJS.Utilities.Key.enter || e.keyCode === WinJS.Utilities.Key.space))
                        return;
                    var externalNavigationAction = this._actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.externalNavigate);
                    externalNavigationAction.parameter = MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_XBoxLiveSSL) + "/Account/BillingAccountInformation";
                    externalNavigationAction.automationId = MS.Entertainment.UI.AutomationIds.settingsAccountBillingContact;
                    externalNavigationAction.execute()
                }, _billingHistoryClick: function _billingHistoryClick(e) {
                    if (e && e.keyCode && !(e.keyCode === WinJS.Utilities.Key.enter || e.keyCode === WinJS.Utilities.Key.space))
                        return;
                    var externalNavigationAction = this._actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.externalNavigate);
                    externalNavigationAction.parameter = MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_MicrosoftBilling);
                    externalNavigationAction.automationId = MS.Entertainment.UI.AutomationIds.settingsAccountBillingHistory;
                    externalNavigationAction.execute()
                }, _privacyClick: function _privacyClick(e) {
                    if (e && e.keyCode && !(e.keyCode === WinJS.Utilities.Key.enter || e.keyCode === WinJS.Utilities.Key.space))
                        return;
                    var externalNavigationAction = this._actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.externalNavigate);
                    externalNavigationAction.parameter = MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_XBoxLiveSSL) + "/Account/Settings";
                    externalNavigationAction.automationId = MS.Entertainment.UI.AutomationIds.settingsAccountPrivacy;
                    externalNavigationAction.execute()
                }, _contactPreferencesClick: function _contactPreferencesClick(e) {
                    if (e && e.keyCode && !(e.keyCode === WinJS.Utilities.Key.enter || e.keyCode === WinJS.Utilities.Key.space))
                        return;
                    var externalNavigationAction = this._actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.externalNavigate);
                    externalNavigationAction.parameter = MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_XBoxLiveSSL) + "/MyXbox/ContactPreferences";
                    externalNavigationAction.automationId = MS.Entertainment.UI.AutomationIds.settingsAccountContactPreferences;
                    externalNavigationAction.execute()
                }, _userAccountLinkClicked: function _userAccountLinkClicked(e) {
                    if (e && e.keyCode && !(e.keyCode === WinJS.Utilities.Key.enter || e.keyCode === WinJS.Utilities.Key.space))
                        return;
                    if (this._signedInUser.canSignOut) {
                        this._signIn.signOut();
                        try {
                            WinJS.UI.SettingsFlyout.show()
                        }
                        catch(ex) {
                            MS.Entertainment.Utilities.fail(false, "WinJS.UI.SettingsFlyout.show() fails: " + ex.toString())
                        }
                    }
                    else
                        window.open(MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_XBox) + "/Live/ChangeUser")
                }, _openWebblendDialog: function _openWebblendDialog(url) {
                    MS.Entertainment.UI.Shell.showWebHostDialog(null, {
                        desiredLeft: "0%", desiredTop: MS.Entertainment.Utilities.isModernMPEnabled ? null : "10%", desiredHeight: MS.Entertainment.Utilities.isModernMPEnabled ? (new Microsoft.Entertainment.Configuration.ConfigurationManager).service.modernWebBlendHeight : null, showBackButton: false, showCancelButton: false
                    }, {
                        sourceUrl: "", authenticatedSourceUrl: url, webHostExperienceFactory: MS.Entertainment.Utilities.SettingsExperience.factory, taskId: MS.Entertainment.UI.Controls.WebHost.TaskId.ACCOUNT, isDialog: true
                    })
                }, _onSignInChange: function _onSignInChange(isSignedIn) {
                    if (isSignedIn)
                        this._loadXboxAccountSummaryPage()
                }, _overlayOnLostFocus: function _overlayOnLostFocus(){}
        }, null, {_musicPassSubscriptionStringMap: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return {"18446189333720334375": String.id.IDS_SETTINGS_ACCOUNT_MUSIC_PASS_FREE_TWELVE_MONTH}
            })}), SettingsExperience: MS.Entertainment.UI.Framework.derive("MS.Entertainment.UI.Controls.WebHostExperience", function SettingsExperience_constructor() {
                MS.Entertainment.UI.Controls.WebHostExperience.prototype.constructor.call(this)
            }, {
                _onRedemptionSuccess: function _onRedemptionSuccess() {
                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                    signIn.refreshSignInState().then(function refreshPurchaseHistory() {
                        var purchaseHistoryService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.purchaseHistory);
                        if (signIn.isSignedIn)
                            purchaseHistoryService.grovel(false, false)
                    })
                }, messageReceived: function messageReceived(messageStruct, webHost, sendMessageFunc) {
                        var errorCode = String.empty;
                        if (!this.disposed)
                            switch (messageStruct.verb) {
                                case"CLOSE_DIALOG":
                                    if (messageStruct.reason === "SUCCESS")
                                        this.eventProvider.traceSettingsHome_Finish(messageStruct.taskId);
                                    else if (messageStruct.reason === "ERROR")
                                        this.eventProvider.traceSettingsAccount_Error(messageStruct.errorCode);
                                    else if (messageStruct.reason === "CANCEL") {
                                        try {
                                            WinJS.UI.SettingsFlyout.show()
                                        }
                                        catch(ex) {
                                            MS.Entertainment.Utilities.fail(false, "WinJS.UI.SettingsFlyout.show() fails: " + ex.toString())
                                        }
                                        this.eventProvider.traceSettings_BackToHome("")
                                    }
                                    break;
                                case"OPEN_DIALOG":
                                    switch (messageStruct.reason) {
                                        case"xblmembership":
                                        case"redeemcode":
                                        case"zunepass":
                                            MS.Entertainment.UI.Shell.showWebHostDialog(null, {
                                                desiredLeft: "0%", desiredTop: "10%", showBackButton: false, showCancelButton: false
                                            }, {
                                                sourceUrl: "", authenticatedSourceUrl: messageStruct.targetUrl, webHostExperienceFactory: MS.Entertainment.Utilities.SettingsExperience.factory, taskId: MS.Entertainment.UI.Controls.WebHost.TaskId.ACCOUNT, isDialog: true
                                            });
                                            break
                                    }
                                    break;
                                case"UPGRADE_MEMBERSHIP":
                                case"UPGRADE_MEMBERSHIP ":
                                    this.eventProvider.traceSubscriptionSignup_Start(String.empty);
                                    break;
                                case"MEMBERSHIP_UPGRADE_SUCESSFUL":
                                case"MEMBERSHIP_UPGRADE_SUCCESSFUL":
                                case"MEMBERSHIP_UPGRADE_SUCESSFUL ":
                                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                                    signIn.refreshSignInState();
                                    this.eventProvider.traceSubscriptionSignup_Finish(String.empty);
                                    break;
                                case"TOKEN_REDEMPTION_SUCCESSFUL":
                                    this._onRedemptionSuccess();
                                    break;
                                case"SIGNOUT_CLICKED":
                                    var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                                    signIn.signOut();
                                    try {
                                        WinJS.UI.SettingsFlyout.show()
                                    }
                                    catch(ex) {
                                        MS.Entertainment.Utilities.fail(false, "WinJS.UI.SettingsFlyout.show() fails: " + ex.toString())
                                    }
                                    break;
                                case"done":
                                    switch (messageStruct.status) {
                                        case"success":
                                            this._onRedemptionSuccess();
                                            break
                                    }
                                    break
                            }
                        MS.Entertainment.UI.Controls.WebHostExperience.prototype.messageReceived.apply(this, arguments)
                    }, errorReceived: function errorReceived(errorCode) {
                        MS.Entertainment.UI.Controls.WebHostExperience.prototype.errorReceived.apply(this, arguments)
                    }
            }, {factory: WinJS.Utilities.markSupportedForProcessing(function factory() {
                    return new MS.Entertainment.Utilities.SettingsExperience
                })})
    });
    (function() {
        WinJS.Application.onsettings = function onSettings(e) {
            var eventProvider = new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell;
            eventProvider.traceSettingsPopulate_Start("");
            if (!(new Microsoft.Entertainment.Configuration.ConfigurationManager).shell.retailExperience) {
                var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                var hasCaptions = MS.Entertainment.Platform.PlaybackHelpers.isClosedCaptionFeatureEnabled();
                e.detail.applicationcommands = {};
                if (signIn.isSignInEnabled)
                    e.detail.applicationcommands.SettingsAccount = {
                        href: "/Components/Settings/SettingsAccount.html", title: String.load(String.id.IDS_SETTINGS_ACCOUNT_TITLE)
                    };
                e.detail.applicationcommands.SettingsPreferences = {
                    href: "/Components/Settings/SettingsPreferences.html", title: String.load(String.id.IDS_SETTINGS_PREFERENCE_TITLE)
                };
                if (hasCaptions)
                    e.detail.applicationcommands.SettingsCaptions = {
                        href: "/Components/Settings/SettingsCaptions.html", title: String.load(String.id.IDS_SETTINGS_CAPTIONS_TITLE)
                    };
                e.detail.applicationcommands.SettingsAbout = {
                    href: "/Components/Settings/SettingsAbout.html", title: String.load(String.id.IDS_SETTINGS_ABOUT_TITLE)
                };
                e.detail.applicationcommands.SettingsFeedback = {
                    href: "/Components/Settings/SettingsFeedback.html", title: String.load(String.id.IDS_SETTINGS_FEEDBACK_TITLE)
                }
            }
            var appBar = MS.Entertainment.globalControls.getControl(MS.Entertainment.globalControls.GlobalControl.appBar);
            if (appBar && appBar.hide)
                appBar.hide();
            var helpSettingsCommand = new Windows.UI.ApplicationSettings.SettingsCommand("SettingsHelp", String.load(String.id.IDS_SETTINGS_HELP_TITLE), _helpSettingsCommandHandler);
            eventProvider.traceSettingsWinJSPopulate_Start("");
            WinJS.UI.SettingsFlyout.populateSettings(e);
            e.detail.e.request.applicationCommands.append(helpSettingsCommand);
            function _helpSettingsCommandHandler() {
                var helpLink;
                if (MS.Entertainment.Utilities.isMusicApp)
                    helpLink = "http://go.microsoft.com/fwlink/?LinkId=299474";
                else if (MS.Entertainment.Utilities.isVideoApp)
                    helpLink = "http://go.microsoft.com/fwlink/?LinkId=299476";
                var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                var externalNavigationAction = actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.externalNavigate);
                externalNavigationAction.parameter = helpLink;
                externalNavigationAction.automationId = MS.Entertainment.UI.AutomationIds.settingsHelp;
                externalNavigationAction.execute()
            }
            {};
            eventProvider.traceSettingsPopulate_End("")
        }
    })()
})()
