﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
var MS;
(function(MS) {
    (function(Entertainment) {
        (function(UI) {
            (function(Components) {
                (function(Shell) {
                    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Components.Shell");
                    var PinToStartScreen = (function() {
                            function PinToStartScreen(){}
                            Object.defineProperty(PinToStartScreen, "Interop", {
                                get: function() {
                                    return MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinToStartScreenInterop)
                                }, enumerable: true, configurable: true
                            });
                            PinToStartScreen.prototype.generateMediaTileIds = function(media) {
                                var tileIds = [];
                                if (media) {
                                    if (MS.Entertainment.Utilities.isValidServiceId(media.zuneId))
                                        tileIds.push(PinToStartScreen.tileName + "_" + media.mediaType + "_" + media.zuneId);
                                    if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeason && MS.Entertainment.Utilities.isValidServiceId(media.seriesZuneId))
                                        tileIds.push(PinToStartScreen.tileName + "_" + media.mediaType + "_" + media.seriesZuneId + "_" + media.seasonNumber);
                                    if (media.libraryId !== undefined && media.libraryId !== -1)
                                        tileIds.push(PinToStartScreen.tileName + "_" + media.mediaType + "_" + media.libraryId)
                                }
                                return tileIds
                            };
                            PinToStartScreen.prototype.generateMediaActivationArguments = function(media) {
                                if (MS.Entertainment.Utilities.isValidServiceId(media.zuneId)) {
                                    if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.tvSeason && MS.Entertainment.Utilities.isValidServiceId(media.seriesZuneId))
                                        return "{0}/?mediaType={1}&id={2}&seriesZuneId={3}&seasonNumber={4}&idType={5}".format(PinToStartScreen.deepLinkName, media.mediaType, media.zuneId, media.seriesZuneId, media.seasonNumber, MS.Entertainment.Data.Query.edsIdType.zuneCatalog);
                                    return "{0}/?mediaType={1}&id={2}&idType={3}".format(PinToStartScreen.deepLinkName, media.mediaType, media.zuneId, MS.Entertainment.Data.Query.edsIdType.zuneCatalog)
                                }
                                if (media.libraryId !== -1)
                                    return "{0}/?mediaType={1}&id={2}".format(PinToStartScreen.deepLinkName, media.mediaType, media.libraryId);
                                return null
                            };
                            PinToStartScreen.prototype.getFirstMatchingTileAsync = function(tileIds) {
                                return PinToStartScreen.Interop.findAllTilesAsync().then(function findAllTilesAsync_complete(allTilesArray) {
                                        for (var i = 0; i < allTilesArray.length; i++) {
                                            var currentTile = allTilesArray[i];
                                            if (tileIds.indexOf(currentTile.tileId) !== -1)
                                                return WinJS.Promise.as(currentTile)
                                        }
                                        return WinJS.Promise.as()
                                    }, function findAllAsync_error() {
                                        return WinJS.Promise.as()
                                    })
                            };
                            PinToStartScreen.prototype.getTileForMediaAsync = function(media) {
                                var tileIds = this.generateMediaTileIds(media);
                                return this.getFirstMatchingTileAsync(tileIds)
                            };
                            PinToStartScreen.prototype.pinMediaAsync = function(media, flyOverPosition, flyOverPlacement, smallLogo, wideTileImage) {
                                if (typeof flyOverPosition === "undefined")
                                    flyOverPosition = null;
                                if (typeof flyOverPlacement === "undefined")
                                    flyOverPlacement = Windows.UI.Popups.Placement.above;
                                if (typeof smallLogo === "undefined")
                                    smallLogo = null;
                                if (typeof wideTileImage === "undefined")
                                    wideTileImage = null;
                                var tileIds = this.generateMediaTileIds(media);
                                if (!tileIds || tileIds.length === 0)
                                    return WinJS.Promise.as();
                                var tileActivationArguments = this.generateMediaActivationArguments(media);
                                var tileShortName;
                                var isSeason = MS.Entertainment.Platform.PlaybackHelpers.isTVSeason(media);
                                var isEpisode = MS.Entertainment.Platform.PlaybackHelpers.isTVEpisode(media);
                                if (isEpisode || isSeason)
                                    tileShortName = "{0}, {1}".format(media.seriesTitle, media.title);
                                else
                                    tileShortName = media.title || media.name;
                                var tileDisplayName = tileShortName;
                                var tileOptions = Windows.UI.StartScreen.TileOptions.none;
                                var defaultImageUri = MS.Entertainment.Utilities.isVideoApp ? PinToStartScreen.DefaultImageUri.video : PinToStartScreen.DefaultImageUri.music;
                                var tileImage = null;
                                var tile = null;
                                return PinToStartScreen.Interop.getServiceImageUrlAsync(media, 320, 320, MS.Entertainment.ImageRequested.primaryImage, MS.Entertainment.Data.ImageIdType.image, defaultImageUri, MS.Entertainment.ImageContentType.jpeg).then(function getServiceImageUrl_complete(url) {
                                        tileImage = new Windows.Foundation.Uri(url);
                                        tile = wideTileImage ? new Windows.UI.StartScreen.SecondaryTile(tileIds[0], tileShortName, tileDisplayName, tileActivationArguments, tileOptions, tileImage, wideTileImage) : new Windows.UI.StartScreen.SecondaryTile(tileIds[0], tileShortName, tileDisplayName, tileActivationArguments, tileOptions, tileImage);
                                        if (smallLogo)
                                            tile.smallLogo = smallLogo;
                                        if (flyOverPosition && !flyOverPosition.isEmpty)
                                            return PinToStartScreen.Interop.requestCreateForSelectionAsync(tile, flyOverPosition, flyOverPlacement);
                                        else
                                            return PinToStartScreen.Interop.requestCreateAsync(tile)
                                    }).then(function requestCreateAsync_complete(isCreated) {
                                        if (!isCreated)
                                            return WinJS.Promise.as();
                                        media.isPinnedToStartScreen = true;
                                        var notifications = Windows.UI.Notifications;
                                        var tileTemplate = notifications.TileTemplateType.tileSquarePeekImageAndText04;
                                        var tileXmlDocument = notifications.TileUpdateManager.getTemplateContent(tileTemplate);
                                        var tileTextNodes = tileXmlDocument.getElementsByTagName("text");
                                        tileTextNodes[0].appendChild(tileXmlDocument.createTextNode(tileDisplayName));
                                        var tileImageNodes = tileXmlDocument.getElementsByTagName("image");
                                        tileImageNodes[0].setAttribute("src", tileImage.rawUri);
                                        PinToStartScreen.Interop.updateTileMetadata(tileIds[0], tileXmlDocument);
                                        if (media.mediaType !== Microsoft.Entertainment.Queries.ObjectType.tvSeason) {
                                            var badgeXmlDocument = notifications.BadgeUpdateManager.getTemplateContent(notifications.BadgeTemplateType.badgeNumber);
                                            var badgeAttributes = badgeXmlDocument.getElementsByTagName("badge");
                                            badgeAttributes[0].setAttribute("value", "playing");
                                            PinToStartScreen.Interop.updateTileBadge(tileIds[0], badgeXmlDocument)
                                        }
                                        return WinJS.Promise.as()
                                    })
                            };
                            PinToStartScreen.prototype.unpinAsync = function(tileIds, flyOverPosition, flyOverPlacement) {
                                if (typeof flyOverPosition === "undefined")
                                    flyOverPosition = null;
                                if (typeof flyOverPlacement === "undefined")
                                    flyOverPlacement = Windows.UI.Popups.Placement.above;
                                if (!tileIds || tileIds.length === 0)
                                    return WinJS.Promise.wrap();
                                var deleteTilePromises = [];
                                return PinToStartScreen.Interop.findAllTilesAsync().then(function findAllAsync_complete(allTilesArray) {
                                        var deleteTilePromises = allTilesArray.map(function allTilesArray_iterate(currentTile) {
                                                if (tileIds.indexOf(currentTile.tileId) !== -1)
                                                    if (flyOverPosition && !flyOverPosition.isEmpty)
                                                        return PinToStartScreen.Interop.requestDeleteForSelectionAsync(currentTile, flyOverPosition, flyOverPlacement);
                                                    else
                                                        return PinToStartScreen.Interop.requestDeleteAsync(currentTile)
                                            });
                                        return WinJS.Promise.join(deleteTilePromises)
                                    }).then(function deleteTilePromises_complete() {
                                        return WinJS.Promise.as()
                                    }, function unpinAsync_error() {
                                        return WinJS.Promise.as()
                                    })
                            };
                            PinToStartScreen.prototype.unpinMediaAsync = function(media, flyOverPosition, flyOverPlacement) {
                                if (typeof flyOverPosition === "undefined")
                                    flyOverPosition = null;
                                if (typeof flyOverPlacement === "undefined")
                                    flyOverPlacement = Windows.UI.Popups.Placement.above;
                                if (!media)
                                    return WinJS.Promise.wrap();
                                var tileIds = this.generateMediaTileIds(media);
                                return this.unpinAsync(tileIds, flyOverPosition, flyOverPlacement).then(function unpinAsync_complete() {
                                        media.isPinnedToStartScreen = false
                                    })
                            };
                            PinToStartScreen.create = function() {
                                return new MS.Entertainment.UI.Components.Shell.PinToStartScreen
                            };
                            PinToStartScreen.tileName = "XboxPin";
                            PinToStartScreen.deepLinkName = "playPin";
                            PinToStartScreen.DefaultImageUri = {
                                video: "ms-appx:///images/tiles/XBL_VIDEO_150x150_A.png", music: "ms-appx:///images/tiles/XBL_MUSIC_150x150_A.png"
                            };
                            return PinToStartScreen
                        })();
                    Shell.PinToStartScreen = PinToStartScreen
                })(Components.Shell || (Components.Shell = {}));
                var Shell = Components.Shell
            })(UI.Components || (UI.Components = {}));
            var Components = UI.Components
        })(Entertainment.UI || (Entertainment.UI = {}));
        var UI = Entertainment.UI
    })(MS.Entertainment || (MS.Entertainment = {}));
    var Entertainment = MS.Entertainment
})(MS || (MS = {}));
if ((new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement).isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.pinToStart))
    MS.Entertainment.ServiceLocator.register(MS.Entertainment.Services.pinToStartScreen, MS.Entertainment.UI.Components.Shell.PinToStartScreen.create)
