﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Hydrator");
scriptValidator();
(function() {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Hydrator");
    WinJS.Namespace.define("MS.Entertainment.Hydrator", {
        artist: function artist(hydrating, options) {
            options = options || {};
            var result;
            var queryWatcher;
            var loadedLibraryId;
            var skipHydrate = hydrating.fromCollection && !options.forceUpdate;
            if (skipHydrate || !hydrating.hasServiceId)
                result = null;
            else {
                var artistQuery = MS.Entertainment.Hydrator.prepareDetailsQuery(new MS.Entertainment.Data.Query.Music.ArtistDetails, new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.artist"), hydrating);
                result = artistQuery.execute().then(function querySuccess(q) {
                    var item = q.result.item;
                    if (!item || !item.isValid) {
                        if (item && !item.isValidType) {
                            var telemetryParameterArray = [{
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.EdsMediaType, parameterValue: item.mediaTypeString
                                    }, {
                                        parameterName: MS.Entertainment.Utilities.Telemetry.TelemetryParameterNames.ExpectedMediaType, parameterValue: item.expectedMediaTypeString
                                    }];
                            MS.Entertainment.Utilities.Telemetry.logTelemetryEventWithParameterArray(MS.Entertainment.Utilities.Telemetry.TelemetryEvents.HydratorWrongMediaType, telemetryParameterArray)
                        }
                        return WinJS.Promise.wrapError(new Error("Failed to retrieve an artist media item from the query."))
                    }
                    return MS.Entertainment.Hydrator.sanitizeIds(item, hydrating)
                }.bind(this))
            }
            return WinJS.Promise.as(result)
        }, album: function albums(hydrating, options) {
                if ((hydrating.location === MS.Entertainment.Data.ItemLocation.marketplace) || (!hydrating.fromCollection && !hydrating.location) || (!hydrating.inCollection && !hydrating.location))
                    return MS.Entertainment.Hydrator.albumMarketplace(hydrating, options);
                else
                    return MS.Entertainment.Hydrator.albumCollection(hydrating, options)
            }, albumMarketplace: function albumMarketplace(hydrating, options) {
                options = options || {};
                var result;
                var skipHydrate = hydrating.fromCollection && !options.forceUpdate;
                if (skipHydrate || !hydrating.hasServiceId)
                    result = null;
                else {
                    var albumQuery;
                    if (options.skipTracks)
                        albumQuery = MS.Entertainment.Hydrator.prepareDetailsQuery(new MS.Entertainment.Data.Query.Music.AlbumDetails, new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.albumMarketplace"), hydrating);
                    else
                        albumQuery = MS.Entertainment.Hydrator.prepareDetailsQuery(new MS.Entertainment.Data.Query.Music.AlbumWithTracks, new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.albumMarketplace"), hydrating);
                    if (albumQuery.setTrackAlbum)
                        albumQuery.setTrackAlbum(hydrating);
                    result = albumQuery.execute().then(function querySuccess(q) {
                        var item = q.result ? q.result.item : null;
                        if (!item || !item.isValid)
                            return WinJS.Promise.wrapError(new Error("Failed to retrieve an album media item from the query."));
                        var promises = [MS.Entertainment.Hydrator.sanitizeIds(item, hydrating)];
                        if (item.artist && hydrating.artist) {
                            MS.Entertainment.Hydrator.sanitizeLibraryIds(item.artist, hydrating.artist);
                            promises.push(MS.Entertainment.Hydrator.sanitizeIds(item.artist, null))
                        }
                        return WinJS.Promise.join(promises)
                    }).then(function(promiseResults) {
                        return promiseResults && promiseResults[0]
                    })
                }
                return WinJS.Promise.as(result)
            }, topSongsAlbum: function topSongsAlbum(hydrating, options) {
                options = options || {};
                var result;
                var skipHydrate = hydrating.fromCollection && !options.forceUpdate;
                if (skipHydrate || !hydrating.hasServiceId)
                    result = null;
                else {
                    var albumQuery = MS.Entertainment.Hydrator.prepareDetailsQuery(new MS.Entertainment.Data.Query.Music.TopSongsAlbumWithTracks, new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.topSongsAlbum"), hydrating);
                    result = albumQuery.execute().then(function querySuccess(q) {
                        var item = q.result.item;
                        return MS.Entertainment.Hydrator.sanitizeIds(item, hydrating)
                    })
                }
                return WinJS.Promise.as(result)
            }, albumCollection: function albumCollection(hydrating, options) {
                options = options || {};
                var result;
                var item = new MS.Entertainment.Data.Augmenter.Library.Album;
                var queryWatcher = new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.albumCollection");
                item.zuneId = hydrating.zuneId;
                result = MS.Entertainment.Hydrator.sanitizeIds(item, hydrating);
                if (!options.skipTracks)
                    result = result.then(function goLibraryId() {
                        var query = new MS.Entertainment.Data.Query.libraryTracks;
                        query.albumId = item.libraryId;
                        query.sort = Microsoft.Entertainment.Queries.TracksSortBy.numberAscending;
                        query.mediaAvailability = options.collectionFilter;
                        query.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(Microsoft.Entertainment.Platform.AcquisitionContext.collection, Microsoft.Entertainment.Platform.AcquisitionContextType.album, MS.Entertainment.Utilities.isValidServiceId(hydrating.canonicalId) ? hydrating.canonicalId : String.empty);
                        queryWatcher.registerQuery(query);
                        return query.execute()
                    }.bind(this)).then(function querySuccess(q) {
                        item.tracks = q.result && q.result.items;
                        var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                        var isMusicMarketplaceEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicMarketplace);
                        if (isMusicMarketplaceEnabled && item.tracks) {
                            var mediaStore = new Microsoft.Entertainment.Platform.MediaStore;
                            var provider = mediaStore.trackProvider;
                            return item.tracks.forEachAll(function getPurchasedInfo(args) {
                                    var track = args.item.data || {};
                                    if (!track.hasServiceId)
                                        return;
                                    if (track.inCollection && track.localFilesCount > 0)
                                        return provider.getPreferredFileUrlAsync(track.libraryId).then(function getPreferredFileUrlAsync_complete(preferredFile) {
                                                if (!preferredFile || preferredFile.type !== Microsoft.Entertainment.Platform.FileUrlType.localFileUrl || preferredFile.protection !== Microsoft.Entertainment.Platform.ProtectionState.unprotected) {
                                                    args.stop = true;
                                                    item.hasPurchasableTracks = true
                                                }
                                            }, function ignoreError(){});
                                    else {
                                        args.stop = true;
                                        item.hasPurchasableTracks = true
                                    }
                                })
                        }
                    }).then(function returnItem() {
                        return item
                    });
                return WinJS.Promise.as(result)
            }, track: function track(hydrating, options) {
                options = options || {};
                var result;
                var skipHydrate = hydrating.inCollection && hydrating.fromCollection && !options.forceUpdate;
                if (skipHydrate || !this.hasServiceId)
                    result = null;
                else {
                    var trackQuery = MS.Entertainment.Hydrator.prepareDetailsQuery(new MS.Entertainment.Data.Query.Music.SongDetails, new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.track"), hydrating);
                    result = trackQuery.execute().then(function querySuccess(q) {
                        var item = q.result.item;
                        if (!item || !item.isValid)
                            return WinJS.Promise.wrapError(new Error("Failed to retrieve a track media item from the query."));
                        var promises = [MS.Entertainment.Hydrator.sanitizeIds(item, hydrating)];
                        if (item.album && hydrating.album) {
                            MS.Entertainment.Hydrator.sanitizeLibraryIds(item.album, hydrating.album);
                            promises.push(MS.Entertainment.Hydrator.sanitizeIds(item.album, null))
                        }
                        if (item.artist && hydrating.artist) {
                            MS.Entertainment.Hydrator.sanitizeLibraryIds(item.artist, hydrating.artist);
                            promises.push(MS.Entertainment.Hydrator.sanitizeIds(item.artist, null))
                        }
                        return WinJS.Promise.join(promises)
                    }.bind(this)).then(function(promiseResults) {
                        return promiseResults && promiseResults[0]
                    })
                }
                return WinJS.Promise.as(result)
            }, playlistCollection: function playlistCollection(hydrating, options) {
                options = options || {};
                var result;
                var queryWatcher = new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.playlistCollection");
                var item = new MS.Entertainment.Data.Augmenter.Library.Playlist;
                var query = new MS.Entertainment.Data.Query.libraryPlaylistMediaItems;
                query.playlistId = hydrating.libraryId;
                queryWatcher.registerQuery(query);
                result = query.execute().then(function querySuccess(q) {
                    item.tracks = q.result.items;
                    return item
                });
                return WinJS.Promise.as(result)
            }, playlist: function playlist(hydrating, options) {
                options = options || {};
                var result;
                var queryWatcher = new MS.Entertainment.Framework.QueryWatcher("MS.Entertainment.Hydrator.playlistCollection");
                var item = new MS.Entertainment.Data.Augmenter.Library.Playlist;
                var query = new MS.Entertainment.Data.Query.libraryPlaylistMediaItems;
                query.cloudSyncItemId = hydrating.cloudSyncItemId;
                var playlistQuery = MS.Entertainment.UI.Actions.Playlists.queryPlaylistById(hydrating.cloudSyncItemId, true);
                result = playlistQuery.then(function querySuccess(playlist) {
                    return playlist
                }, function error() {
                    return null
                });
                return WinJS.Promise.as(result)
            }
    })
})()
