﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    var edsArray = MS.Entertainment.Data.Query.edsArray;
    var edsEndpointType = MS.Entertainment.Data.Query.edsEndpointType;
    var edsIdType = MS.Entertainment.Data.Query.edsIdType;
    var edsMediaGroup = MS.Entertainment.Data.Query.edsMediaGroup;
    var edsMediaType = MS.Entertainment.Data.Query.edsMediaType;
    var edsSortOrder = MS.Entertainment.Data.Query.edsSortOrder;
    var edsString = MS.Entertainment.Data.Query.edsString;
    var edsFields = MS.Entertainment.Data.Query.edsFields;
    var edsEditorialId = MS.Entertainment.Data.Factory.Marketplace && MS.Entertainment.Data.Factory.Marketplace.edsEditorialId;
    var SEARCH_CHUNK_SIZE = 20;
    var BROWSE_CHUNK_SIZE = 25;
    var MAX_LIST_DELETE = 100;
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.Data.Query.Music");
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music", {
        EDSMusicQuery: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.EDSWrapperQuery, null, {createTargetDeviceType: function createTargetDeviceType() {
                        return MS.Entertainment.Utilities.defaultEdsTargetDeviceTypeFromApp
                    }})
        }), EDSMusicFirstPartyQuery: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.EDSMusicQuery, null, {createEDSParameters: function createEDSParameters() {
                            return {firstPartyOnly: true}
                        }})
            })
    });
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music", {requestFields: {
            crossMediaSearch: [edsFields.duration, edsFields.genres, edsFields.hasTrackMediaGuide, edsFields.id, edsFields.images, edsFields.isExplicit, edsFields.label, edsFields.mediaGroup, edsFields.mediaItemType, edsFields.musicAlbumType, edsFields.name, edsFields.parentAlbum, edsFields.primaryArtist, edsFields.providers, edsFields.releaseDate, edsFields.subGenres, edsFields.trackCount, edsFields.trackNumber, edsFields.zuneId], albumDetails: [edsFields.duration, edsFields.id, edsFields.images, edsFields.isExplicit, edsFields.genres, edsFields.label, edsFields.mediaGroup, edsFields.mediaItemType, edsFields.name, edsFields.primaryArtist, edsFields.providers, edsFields.releaseDate, edsFields.subGenres, edsFields.trackCount, edsFields.zuneId], albumBrowse: [edsFields.duration, edsFields.id, edsFields.images, edsFields.isExplicit, edsFields.genres, edsFields.label, edsFields.mediaGroup, edsFields.mediaItemType, edsFields.name, edsFields.primaryArtist, edsFields.providers, edsFields.releaseDate, edsFields.subGenres, edsFields.trackCount, edsFields.zuneId], artistsDetails: [edsFields.description, edsFields.genres, edsFields.hasTrackMediaGuide, edsFields.id, edsFields.images, edsFields.mediaGroup, edsFields.mediaItemType, edsFields.name, edsFields.relatedMedia, edsFields.sortName, edsFields.subGenres, edsFields.zuneId], artistsSearch: [edsFields.genres, edsFields.hasTrackMediaGuide, edsFields.images, edsFields.id, edsFields.mediaGroup, edsFields.mediaItemType, edsFields.name, edsFields.sortName, edsFields.subGenres, edsFields.zuneId], artists: [edsFields.genres, edsFields.hasTrackMediaGuide, edsFields.images, edsFields.id, edsFields.mediaGroup, edsFields.mediaItemType, edsFields.name, edsFields.subGenres, edsFields.zuneId], artistBio: [edsFields.description], artistImages: [edsFields.images], related: [edsFields.relatedMedia], songs: [edsFields.duration, edsFields.genres, edsFields.hasTrackMediaGuide, edsFields.images, edsFields.id, edsFields.isExplicit, edsFields.mediaGroup, edsFields.mediaItemType, edsFields.name, edsFields.parentAlbum, edsFields.providers, edsFields.primaryArtist, edsFields.releaseDate, edsFields.subGenres, edsFields.subTitle, edsFields.trackNumber, edsFields.zuneId], songsNoAlbum: [edsFields.duration, edsFields.genres, edsFields.hasTrackMediaGuide, edsFields.images, edsFields.id, edsFields.isExplicit, edsFields.mediaGroup, edsFields.mediaItemType, edsFields.name, edsFields.providers, edsFields.primaryArtist, edsFields.releaseDate, edsFields.subGenres, edsFields.subTitle, edsFields.trackNumber, edsFields.zuneId], subscriptions: [edsFields.availabilities, ]
        }});
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data, "Query", {
        searchTracks: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                    search: "", chunkSize: SEARCH_CHUNK_SIZE, createResourceURI: function() {
                            return this.getResourceEndpoint() + "/music/track"
                        }, createParameters: function createParameters() {
                            return {
                                    chunkSize: this.chunkSize, q: this.search
                                }
                        }, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.entriesPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.rightsPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.clientTypesPluralizer], resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.TracksResult
                })
        }), genreTracks: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                        genreId: "", chunkSize: BROWSE_CHUNK_SIZE, createResourceURI: function() {
                                var suffix;
                                if (this.genreId)
                                    suffix = "music/genre/" + this.genreId + "/tracks";
                                else
                                    suffix = "music/track";
                                return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                            }, createParameters: function createParameters() {
                                return {chunkSize: this.chunkSize}
                            }, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.entriesPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.rightsPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.clientTypesPluralizer], resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.TracksResult, aggregateChunks: true
                    })
            })
    });
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data, "Query", {searchArtists: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                    search: "", chunkSize: SEARCH_CHUNK_SIZE, createResourceURI: function() {
                            return this.getResourceEndpoint() + "/music/artist"
                        }, createParameters: function createParameters() {
                            return {
                                    chunkSize: this.chunkSize, q: this.search, startIndex: 1, isActionable: true
                                }
                        }, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.entriesPluralizer], resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.ArtistsResult
                })
        })});
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data, "Query", {searchAlbums: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                    search: "", chunkSize: SEARCH_CHUNK_SIZE, createResourceURI: function() {
                            return this.getResourceEndpoint() + "/music/album"
                        }, createParameters: function createParameters() {
                            return {
                                    chunkSize: this.chunkSize, q: this.search, startIndex: 1, isActionable: true
                                }
                        }, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.entriesPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.rightsPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.clientTypesPluralizer], resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.AlbumsResult
                })
        })});
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data, "Query", {searchPlaylists: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                    search: "", chunkSize: SEARCH_CHUNK_SIZE, createResourceURI: function() {
                            return this.getResourceEndpoint() + "/music/playlist"
                        }, createParameters: function createParameters() {
                            return {
                                    chunkSize: this.chunkSize, q: this.search, startIndex: 1, isActionable: true
                                }
                        }, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.entriesPluralizer], resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.PlaylistsResult
                })
        })});
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data, "Query", {searchMusicVideos: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                    search: "", chunkSize: SEARCH_CHUNK_SIZE, createResourceURI: function() {
                            return this.getResourceEndpoint() + "/music/musicvideo"
                        }, createParameters: function createParameters() {
                            return {
                                    chunkSize: this.chunkSize, q: this.search, startIndex: 1, isActionable: true
                                }
                        }, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.entriesPluralizer], resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.MusicVideosResult
                })
        })});
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data, "Query", {musicSubGenre: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                    genreId: null, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.SubGenresResult, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.subGenresPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.peerGenresPluralizer], createResourceURI: function() {
                            return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), "music/genre/" + this.genreId)
                        }
                })
        })});
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data, "Query", {musicGenres: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                    createResourceURI: function() {
                        return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), "/music/genre")
                    }, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.entriesPluralizer], resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.GenresResult, aggregateChunks: true
                })
        })});
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data, "Query", {MusicHubsOverride: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.HubsOverride, null, {
                    appString: "/Override-Music", getSpotlightFeedVersionSubPath: function getSpotlightFeedVersionSubPath() {
                            var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                            var subPath = String.empty;
                            if (MS.Entertainment.Utilities.isMusicApp1)
                                subPath = configurationManager.service.musicApp1SpotlightVersion;
                            else
                                subPath = configurationManager.service.musicApp2SpotlightVersion;
                            return subPath
                        }
                })
        })});
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data, "Query", {MusicHub: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                    createResourceURI: function() {
                        return this.getResourceEndpoint() + "/hubs/music/"
                    }, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.entriesPluralizer, "editorialItems/editorialItem"], resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EditorialEntriesResult, aggregateChunks: false, supportsContentNotifications: true
                })
        })});
    WinJS.Namespace.defineWithParent(MS.Entertainment.Data, "Query", {
        browseFeaturedAlbums: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                    createResourceURI: function() {
                        return this.getResourceEndpoint() + "/music/featured/albums/"
                    }, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.entriesPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.rightsPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.clientTypesPluralizer], resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.AlbumsResult, aggregateChunks: false
                })
        }), browseFeaturedGenreAlbums: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                        genreId: String.empty, aggregateChunks: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.FeaturedGenreAlbumsResult, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.entriesPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.rightsPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.clientTypesPluralizer, MSE.Data.Query.marketplaceWrapperQuery.editorialItemsPluralizer, MSE.Data.Query.marketplaceWrapperQuery.featuresPluralizer, MSE.Data.Query.marketplaceWrapperQuery.albumsPluralizer, ], createResourceURI: function() {
                                var suffix;
                                if (this.genreId)
                                    suffix = "music/genre/" + this.genreId + "/hub";
                                else
                                    suffix = "music/featured/albums";
                                return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                            }
                    })
            })
    });
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music", {
        BrowseFeaturedAlbums: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.marketplaceWrapperQuery, null, {
                    genreId: null, aggregateChunks: false, autoEncodeUri: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSFeaturedGenreAlbumsResult, autoUpdateProperties: {}, getResourceEndpointID: function getResourceEndpointID() {
                            return MS.Entertainment.Endpoint.id.seid_RootCatalog
                        }, pluralizers: [MS.Entertainment.Data.Query.marketplaceWrapperQuery.entriesPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.rightsPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.clientTypesPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.editorialItemsPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.featuresPluralizer, MS.Entertainment.Data.Query.marketplaceWrapperQuery.albumsPluralizer, ], createResourceURI: function() {
                            var suffix;
                            if (this.genreId) {
                                var editorialGenreId = MS.Entertainment.Data.Factory.Marketplace.edsEditorialId(this.genreId);
                                suffix = ["music", "genre", editorialGenreId, "hub"]
                            }
                            else
                                suffix = "music/featured/albums";
                            return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                        }
                })
        }), ArtistDetails: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.EDSMusicQuery, null, {
                        id: null, idType: null, chunked: false, chunkSize: 0, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSArtistDetailResultItem, requestFields: MS.Entertainment.Data.Query.Music.requestFields.artistsDetails, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.details)
                            }, createParameters: function createParameters() {
                                MS.Entertainment.Data.Query.Music.assert(!!this.id, "The given artist details service id is invalid");
                                return {
                                        ids: this.id, idType: (this.idType !== edsIdType.canonical) ? this.idType : null, mediaGroup: edsMediaGroup.musicArtistType, firstPartyOnly: MS.Entertainment.Utilities.isModernMPEnabled
                                    }
                            }
                    })
            }), AlbumDetails: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.EDSMusicFirstPartyQuery, null, {
                        id: null, idType: null, chunked: false, chunkSize: 0, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSAlbumDetailsResultItem, requestFields: MS.Entertainment.Data.Query.Music.requestFields.albumDetails, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.details)
                            }, createParameters: function createParameters() {
                                MS.Entertainment.Data.Query.Music.assert(!!this.id, "The given album details service id is invalid");
                                return {
                                        ids: this.id, idType: (this.idType !== edsIdType.canonical) ? this.idType : null, mediaGroup: edsMediaGroup.musicType
                                    }
                            }
                    })
            }), SongDetails: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.EDSMusicQuery, null, {
                        id: null, idType: null, chunked: false, chunkSize: 0, inflateParentItems: true, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSTrackDetailsResultItem, requestFields: MS.Entertainment.Data.Query.Music.requestFields.songs, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.details)
                            }, createParameters: function createParameters() {
                                MS.Entertainment.Data.Query.Music.assert(!!this.id, "The given song details service id is invalid");
                                return {
                                        ids: this.id, idType: (this.idType !== edsIdType.canonical) ? this.idType : null, inflateParentItems: (this.inflateParentItems) ? true : null, mediaGroup: edsMediaGroup.musicType, firstPartyOnly: MS.Entertainment.Utilities.isModernMPEnabled
                                    }
                            }
                    })
            }), Artists: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.EDSMusicFirstPartyQuery, null, {
                        chunkSize: BROWSE_CHUNK_SIZE, aggregateChunks: true, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSArtistsResult, orderBy: null, genreId: null, subGenreId: null, hasSmartDJ: null, requestFields: MS.Entertainment.Data.Query.Music.requestFields.artists, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.browse)
                            }, createParameters: function createParameters() {
                                return {
                                        desiredMediaItemTypes: edsMediaType.musicArtist, orderBy: this.orderBy, genre: edsString(this.genreId), subgenre: edsString(this.subGenreId), hasTrackMediaGuide: this.hasSmartDJ
                                    }
                            }
                    })
            }), RelatedArtists: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.EDSMusicFirstPartyQuery, null, {
                        id: null, chunkSize: BROWSE_CHUNK_SIZE, aggregateChunks: true, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSArtistsResult, orderBy: null, requestFields: MS.Entertainment.Data.Query.Music.requestFields.artists, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.related)
                            }, createParameters: function createParameters() {
                                MS.Entertainment.Data.Query.Music.assert(!!this.id, "The given related artist service id is invalid");
                                return {
                                        id: this.id, desiredMediaItemTypes: edsMediaType.musicArtist, mediaItemType: edsMediaType.musicArtist, orderBy: this.orderBy
                                    }
                            }
                    })
            }), InfluenceArtists: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.EDSMusicFirstPartyQuery, null, {
                        id: null, chunkSize: 0, aggregateChunks: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSRelatedResult, orderBy: null, requestFields: MS.Entertainment.Data.Query.Music.requestFields.related, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.details)
                            }, createParameters: function createParameters() {
                                MS.Entertainment.Data.Query.Music.assert(!!this.id, "The given influence artist service id is invalid");
                                return {
                                        ids: this.id, mediaGroup: edsMediaGroup.musicArtistType, orderBy: this.orderBy
                                    }
                            }
                    })
            }), Albums: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.EDSMusicFirstPartyQuery, null, {
                        chunkSize: BROWSE_CHUNK_SIZE, aggregateChunks: true, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSAlbumsResult, orderBy: null, genreId: null, subGenreId: null, requestFields: MS.Entertainment.Data.Query.Music.requestFields.albumBrowse, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.browse)
                            }, createParameters: function createParameters() {
                                return {
                                        desiredMediaItemTypes: edsMediaType.album, orderBy: this.orderBy, genre: edsString(this.genreId), subgenre: edsString(this.subGenreId)
                                    }
                            }
                    })
            }), Songs: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.EDSMusicFirstPartyQuery, null, {
                        chunkSize: BROWSE_CHUNK_SIZE, aggregateChunks: true, inflateParentItems: true, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSTracksResult, orderBy: null, genreId: null, subGenreId: null, requestFields: MS.Entertainment.Data.Query.Music.requestFields.songs, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.browse)
                            }, createParameters: function createParameters() {
                                return {
                                        desiredMediaItemTypes: edsMediaType.track, orderBy: this.orderBy, genre: edsString(this.genreId), subgenre: edsString(this.subGenreId), inflateParentItems: (this.inflateParentItems) ? true : null
                                    }
                            }
                    })
            }), Genres: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.EDSWrapperQuery, null, {
                        chunkSize: 0, genreId: null, id: null, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSGenresResult, requestFields: MS.Entertainment.Data.EDSWrapperQuery.requestFieldsNotNeeded, createResourceURI: function() {
                                var endpointType;
                                if (this.genreId || this.id)
                                    endpointType = edsEndpointType.metadataMusicSubGenres;
                                else
                                    endpointType = edsEndpointType.metadataMusicGenres;
                                return this.getResourceEndpoint(endpointType)
                            }, createParameters: function createParameters() {
                                return {queryRefinerValue: this.genreId || this.id}
                            }
                    })
            }), CrossMusicSearch: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.CrossMediaSearch, null, {
                        requestFields: MS.Entertainment.Data.Query.Music.requestFields.crossMediaSearch, groupTypes: [edsMediaType.track, edsMediaType.album, edsMediaType.musicArtist], resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSMusicSearchResult
                    })
            }), AutoSuggest: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.AutoSuggest, null, {mediaType: MS.Entertainment.Data.Query.bbxMediaType.music})
            }), IntroPanel: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.ServiceWrapperQuery, null, {
                        chunked: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.IntroPanelResult, createResourceURI: function() {
                                return MS.Entertainment.Utilities.UriFactory.create(MS.Entertainment.Endpoint.id.seid_WinPhoneXboxDotCom, "x8/feeds/1.1/Intro-Music")
                            }, pluralizers: ["BodyText/p"]
                    })
            }), Subscriptions: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.EDSMusicQuery, null, {
                        chunked: false, chunkSize: 0, useCache: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSSubscriptionDetailResultItem, requestFields: MS.Entertainment.Data.Query.Music.requestFields.subscriptions, forceSecureEndpoint: true, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.details)
                            }, createParameters: function createParameters() {
                                var config = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                                return {
                                        ids: config.service.subscriptionProductId, desiredMediaItemTypes: edsMediaType.subscription
                                    }
                            }
                    })
            })
    });
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music", {
        MusicVideoDetails: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.SongDetails, null, {
                    id: null, idType: null, chunked: false, chunkSize: 0, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSMusicVideoDetailsResultItem, createResourceURI: function() {
                            return this.getResourceEndpoint(edsEndpointType.details)
                        }, createParameters: function createParameters() {
                            MS.Entertainment.Data.Query.Music.assert(!!this.id, "The given music video service id is invalid");
                            return {
                                    ids: this.id, idType: (this.idType !== edsIdType.canonical) ? this.idType : null, mediaGroup: edsMediaGroup.musicType, inflateParentItems: (this.inflateParentItems) ? true : null
                                }
                        }
                })
        }), SmartDJInnerQuery: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Songs, null, {
                        id: null, chunkSize: -1, useCache: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSSmartDJTracksResult, forceSecureEndpoint: true, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.mediaGuide)
                            }, createParameters: function createParameters() {
                                MS.Entertainment.Data.Query.Music.assert(!!this.id, "The given smart dj service id is invalid");
                                return {
                                        id: this.id, mediaItemType: edsMediaType.musicArtist, desiredMediaItemTypes: edsMediaType.track, inflateParentItems: (this.inflateParentItems) ? true : null, firstPartyOnly: true
                                    }
                            }
                    })
            }), ArtistSearch: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Artists, null, {
                        chunkSize: BROWSE_CHUNK_SIZE, aggregateChunks: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSArtistsResult, requestFields: MS.Entertainment.Data.Query.Music.requestFields.artistsSearch, search: null, autoSuggestSeed: false, enabledImpressionGuid: true, forceSecureEndpoint: true, hasSmartDJ: null, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.singleMediaGroupSearch)
                            }, createParameters: function createParameters() {
                                return {
                                        q: edsString(this.search), desiredMediaItemTypes: edsMediaType.musicArtist, hasTrackMediaGuide: this.hasSmartDJ
                                    }
                            }, createHeaders: function createHeaders() {
                                return {"x-xbl-autoSuggest-seed-text": this.autoSuggestSeed && this.search ? window.encodeURIComponent(this.search) : null}
                            }
                    })
            }), AlbumSearch: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Albums, null, {
                        chunkSize: BROWSE_CHUNK_SIZE, aggregateChunks: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSAlbumsResult, search: null, autoSuggestSeed: false, enabledImpressionGuid: true, forceSecureEndpoint: true, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.singleMediaGroupSearch)
                            }, createParameters: function createParameters() {
                                return {
                                        q: edsString(this.search), desiredMediaItemTypes: edsMediaType.album
                                    }
                            }, createHeaders: function createHeaders() {
                                return {"x-xbl-autoSuggest-seed-text": this.autoSuggestSeed && this.search ? window.encodeURIComponent(this.search) : null}
                            }
                    })
            }), ArtistAlbums: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Albums, null, {
                        id: null, artistId: null, aggregateChunks: true, chunkSize: BROWSE_CHUNK_SIZE, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSAlbumsResult, orderBy: edsSortOrder.releaseDate, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.browse)
                            }, createParameters: function createParameters() {
                                MS.Entertainment.Data.Query.Music.assert(!!this.id || !!this.artistId, "The given artist album service id is invalid");
                                return {
                                        id: this.id || this.artistId, mediaItemType: edsMediaType.musicArtist, desiredMediaItemTypes: edsMediaType.album, orderBy: this.orderBy
                                    }
                            }
                    })
            }), ArtistSongs: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Songs, null, {
                        id: null, artistId: null, aggregateChunks: true, chunkSize: BROWSE_CHUNK_SIZE, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSArtistTracksResult, orderBy: edsSortOrder.releaseDate, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.browse)
                            }, createParameters: function createParameters() {
                                MS.Entertainment.Data.Query.Music.assert(!!this.id || !!this.artistId, "The given artist songs service id is invalid");
                                return {
                                        id: this.id || this.artistId, mediaItemType: edsMediaType.musicArtist, desiredMediaItemTypes: edsMediaType.track, orderBy: this.orderBy, inflateParentItems: (this.inflateParentItems) ? true : null
                                    }
                            }
                    })
            }), AlbumSongs: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Songs, null, {
                        id: null, albumId: null, aggregateChunks: true, chunkSize: BROWSE_CHUNK_SIZE, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSAlbumTracksResult, orderBy: null, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.browse)
                            }, createParameters: function createParameters() {
                                MS.Entertainment.Data.Query.Music.assert(!!this.id || !!this.albumId, "The given album songs service id is invalid");
                                return {
                                        id: this.id || this.albumId, mediaItemType: edsMediaType.album, desiredMediaItemTypes: edsMediaType.track, orderBy: this.orderBy, inflateParentItems: (this.inflateParentItems) ? true : null
                                    }
                            }
                    })
            }), SongSearch: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Songs, null, {
                        chunkSize: BROWSE_CHUNK_SIZE, aggregateChunks: false, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSTracksResult, search: null, autoSuggestSeed: false, enabledImpressionGuid: true, forceSecureEndpoint: true, createResourceURI: function() {
                                return this.getResourceEndpoint(edsEndpointType.singleMediaGroupSearch)
                            }, createParameters: function createParameters() {
                                return {
                                        q: edsString(this.search), desiredMediaItemTypes: edsMediaType.track, inflateParentItems: (this.inflateParentItems) ? true : null
                                    }
                            }, createHeaders: function createHeaders() {
                                return {"x-xbl-autoSuggest-seed-text": this.autoSuggestSeed && this.search ? window.encodeURIComponent(this.search) : null}
                            }
                    })
            })
    });
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music", {
        SmartDJ: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return WinJS.Class.derive(MS.Entertainment.Data.PlaybackQuery, function SmartDJ() {
                    MS.Entertainment.Data.PlaybackQuery.prototype.constructor.call(this, new MS.Entertainment.Data.Query.Music.SmartDJInnerQuery)
                }, {
                    id: {
                        get: function() {
                            return this.query ? this.query.id : null
                        }, set: function(value) {
                                if (this.query)
                                    this.query.id = value
                            }
                    }, impressionGuid: {
                            get: function() {
                                return this.query ? this.query.impressionGuid : null
                            }, set: function(value) {
                                    if (this.query && "impressionGuid" in this.query)
                                        this.query.impressionGuid = value
                                }
                        }
                })
        }), ArtistBio: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.ArtistDetails, null, {requestFields: MS.Entertainment.Data.Query.Music.requestFields.artistBio})
            }), AlbumWithTracks: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return WinJS.Class.derive(MS.Entertainment.Data.BrowseDetailQuery, function AlbumWithTracks() {
                        MS.Entertainment.Data.BrowseDetailQuery.prototype.constructor.call(this);
                        this._resultAlbum = {
                            data: null, overrideData: null
                        }
                    }, {
                        _resultAlbum: null, browseConstructor: MS.Entertainment.Data.Query.Music.AlbumSongs, detailConstructor: MS.Entertainment.Data.Query.Music.AlbumDetails, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSAlbumWithTracksResult, dispose: function dispose() {
                                MS.Entertainment.Data.BrowseDetailQuery.prototype.dispose.call(this);
                                this._resultAlbum = null
                            }, idType: {
                                get: function() {
                                    if (this.detail)
                                        return this.detail.idType;
                                    else
                                        return null
                                }, set: function(value) {
                                        if (this.detail)
                                            this.detail.idType = value
                                    }
                            }, id: {
                                get: function() {
                                    if (this.browse)
                                        return this.browse.id;
                                    else
                                        return null
                                }, set: function(value) {
                                        if (this.browse)
                                            this.browse.id = value;
                                        if (this.detail)
                                            this.detail.id = value
                                    }
                            }, setTrackAlbum: function setTrackAlbum(album) {
                                if (this._resultAlbum)
                                    this._resultAlbum.overrideData = album
                            }, _preInnerExecute: function _preInnerExecute() {
                                var promise = MS.Entertainment.Data.BrowseDetailQuery.prototype._preInnerExecute.call(this);
                                this.browse.inflateParentItems = false;
                                this.browse.requestFields = MS.Entertainment.Data.Query.Music.requestFields.songsNoAlbum;
                                if (MS.Entertainment.Utilities.AcquisitionData)
                                    this.browse.acquisitionData = new MS.Entertainment.Utilities.AcquisitionData(Microsoft.Entertainment.Platform.AcquisitionContext.store, Microsoft.Entertainment.Platform.AcquisitionContextType.album);
                                this.browse.resultAugmentation = this._createTrackResultAugmentation(this.browse.resultAugmentation, this._resultAlbum);
                                return promise
                            }, _createTrackResultAugmentation: function _createTrackResultAugmentation(baseAugmentation, resultAlbum) {
                                return MS.Entertainment.Data.AugmentQuery.modifyListResultAugmentation(baseAugmentation, {
                                        album: MS.Entertainment.Data.Property.convert(String.empty, function() {
                                            return resultAlbum && (resultAlbum.overrideData || resultAlbum.data)
                                        }), albumServiceId: MS.Entertainment.Data.Property.convertNoDeflate("album.canonicalId", MS.Entertainment.Data.Factory.self), albumName: MS.Entertainment.Data.Property.convertNoDeflate("album.name", MS.Entertainment.Data.Factory.self)
                                    })
                            }, _endInnerExecute: function _endInnerExecute() {
                                MS.Entertainment.Data.BrowseDetailQuery.prototype._endInnerExecute.call(this);
                                if (this.result && this.result.item) {
                                    if (this._resultAlbum)
                                        this._resultAlbum.data = this.result.item;
                                    if (this.browse && this.browse.acquisitionData)
                                        this.browse.acquisitionData.acquisitionContextID = MS.Entertainment.Utilities.isValidServiceId(this.result.item.canonicalId) ? this.result.item.canonicalId : String.empty
                                }
                            }
                    })
            }), ArtistTopSongs: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.ArtistSongs, null, {orderBy: edsSortOrder.allTimePlayCount})
            }), ArtistNewSongs: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.ArtistSongs, null, {orderBy: edsSortOrder.releaseDate})
            }), ArtistTopAlbums: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.ArtistAlbums, null, {orderBy: edsSortOrder.mostPopular})
            }), ArtistNewAlbums: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.ArtistAlbums, null, {orderBy: edsSortOrder.releaseDate})
            }), TopArtists: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Artists, null, {orderBy: edsSortOrder.mostPopular})
            }), TopAlbums: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Albums, null, {orderBy: edsSortOrder.mostPopular})
            }), TopSongs: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Songs, null, {orderBy: edsSortOrder.mostPopular})
            }), NewAlbums: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Albums, null, {orderBy: edsSortOrder.releaseDate})
            }), SubGenres: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.Genres, null, {resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSSubGenresResult})
            }), ArtistImages: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.ArtistDetails, null, {
                        requestFields: MS.Entertainment.Data.Query.Music.requestFields.artistImages, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSArtistImagesResult
                    })
            })
    });
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music", {
        TopSongsAlbumWithTracks: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.ArtistTopSongs, null, {resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSTopSongsAlbumDetailsResultItem})
        }), SmartDJList: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.ESListServiceWrapperQuery, function SmartDJList() {
                        this.base()
                    }, {
                        clearResultOnFailure: true, useCache: true, useIfMatchCache: true, chunkSize: BROWSE_CHUNK_SIZE, aggregateChunks: true, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSSmartDJResults, requestFields: MS.Entertainment.Data.EDSWrapperQuery.requestFieldsNotNeeded, forceSecureEndpoint: true, hasTotalCount: true, xuid: String.empty, getResourceEndpointID: function getResourceEndpointID() {
                                return MS.Entertainment.Endpoint.id.seid_ESListService
                            }, createResourceURI: function() {
                                var suffix = "users/xuid(" + this.xuid + ")/lists/recn/xblsmartdj";
                                return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                            }, createEDSHeaders: function createEDSHeaders() {
                                return {
                                        "Content-Type": "application/json", Accept: "application/json", "x-xbl-contract-version": "2", "x-xbl-client-type": MS.Entertainment.Data.EDSWrapperQuery.clientType, "x-xbl-device-type": this.getHeaderDeviceType(), "x-xbl-client-version": "1.0"
                                    }
                            }, getResponseVersion: function getResponseVersion(result) {
                                return result.listVersion
                            }
                    })
            }), SmartDJAdd: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.ESListServiceWrapperQuery, function smartDJAdd(artist, featuredArtistsNames) {
                        this.base();
                        if (artist) {
                            var data = {Items: [MS.Entertainment.Data.Query.Music.SmartDJAdd.convertArtistToSmartDJ(artist, featuredArtistsNames)]};
                            this.postData = {
                                contentType: MS.Entertainment.Data.Query.Music.SmartDJAdd.contentType, data: JSON.stringify(data)
                            };
                            this.requestType = MS.Entertainment.Data.ServiceWrapperQuery.RequestTypes.post
                        }
                    }, {
                        chunkSize: 0, useCache: false, forceSecureEndpoint: true, requestFields: MS.Entertainment.Data.EDSWrapperQuery.requestFieldsNotNeeded, getResourceEndpointID: function getResourceEndpointID() {
                                return MS.Entertainment.Endpoint.id.seid_ESListService
                            }, createResourceURI: function() {
                                var suffix;
                                var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                                suffix = "users/xuid(" + signedInUser.xuid + ")/lists/recn/xblsmartdj";
                                return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                            }, createEDSHeaders: function createEDSHeaders() {
                                return {
                                        Accept: "application/json", "x-xbl-contract-version": "2", "x-xbl-client-type": MS.Entertainment.Data.EDSWrapperQuery.clientType, "x-xbl-device-type": this.getHeaderDeviceType(), "x-xbl-client-version": MS.Entertainment.Data.EDSWrapperQuery.currentVersion, "If-Match": 0
                                    }
                            }
                    }, {
                        contentType: "application/json", convertArtistToSmartDJ: function convertArtistToSmartDJ(artist, featuredArtistsNames) {
                                var smartDJ;
                                if (artist) {
                                    var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                                    var locale = signedInUser.locale;
                                    var subtitle = String.empty;
                                    if (featuredArtistsNames)
                                        if (featuredArtistsNames.length >= 2)
                                            subtitle = String.load(String.id.IDS_SMARTDJ_2_FEATURED_ARTISTS).format(featuredArtistsNames[0], featuredArtistsNames[1]);
                                        else if (featuredArtistsNames.length === 1)
                                            subtitle = String.load(String.id.IDS_SMARTDJ_1_FEATURED_ARTIST).format(featuredArtistsNames[0]);
                                    smartDJ = {
                                        ContentType: MS.Entertainment.Data.Query.edsMediaType.musicArtist, ItemId: artist.canonicalId, ImageUrl: artist.imageResizeUri || artist.imageUri || String.empty, Title: artist.name, SubTitle: subtitle, Locale: locale
                                    }
                                }
                                return smartDJ || {}
                            }
                    })
            }), SmartDJDelete: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.ESListServiceWrapperQuery, function smartDJDelete(listIndices, listVersion) {
                        this.base();
                        if (listIndices && listIndices.length > 0) {
                            var length = Math.min(listIndices.length, MAX_LIST_DELETE);
                            this._indices = [];
                            for (var i = 0; i < length; i++)
                                if (typeof listIndices[i] === "number" && listIndices[i] >= 0)
                                    this._indices.push(listIndices[i]);
                                else
                                    MS.Entertainment.Data.Query.Music.fail("Invalid index passed to SmartDJDelete. index: " + listIndices[i]);
                            this._indices = this._indices.join()
                        }
                        this._listVersion = listVersion;
                        this.requestType = MS.Entertainment.Data.ServiceWrapperQuery.RequestTypes.deleteRequest
                    }, {
                        chunkSize: 0, useCache: false, _indices: String.empty, _listVersion: 0, forceSecureEndpoint: true, requestFields: MS.Entertainment.Data.EDSWrapperQuery.requestFieldsNotNeeded, getResourceEndpointID: function getResourceEndpointID() {
                                return MS.Entertainment.Endpoint.id.seid_ESListService
                            }, createResourceURI: function() {
                                var suffix;
                                var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                                suffix = "users/xuid(" + signedInUser.xuid + ")/lists/recn/xblsmartdj";
                                return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                            }, createEDSHeaders: function createEDSHeaders() {
                                return {
                                        Accept: "application/json", "x-xbl-contract-version": "2", "x-xbl-client-type": MS.Entertainment.Data.EDSWrapperQuery.clientType, "x-xbl-device-type": this.getHeaderDeviceType(), "x-xbl-client-version": MS.Entertainment.Data.EDSWrapperQuery.currentVersion, "if-match": this._listVersion
                                    }
                            }, createParameters: function createParameters() {
                                return {indexes: this._indices}
                            }
                    })
            })
    });
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music", {
        RecentList: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.ESListServiceWrapperQuery, function RecentList() {
                    this.base();
                    var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                    this.xuid = configurationManager.service.lastSignedInUserXuid
                }, {
                    clearResultOnFailure: true, useCache: true, chunkSize: BROWSE_CHUNK_SIZE, aggregateChunks: true, resultAugmentation: MS.Entertainment.Data.Augmenter.Marketplace.EDSRecentListResults, requestFields: MS.Entertainment.Data.EDSWrapperQuery.requestFieldsNotNeeded, forceSecureEndpoint: true, hasTotalCount: true, xuid: String.empty, getResourceEndpointID: function getResourceEndpointID() {
                            return MS.Entertainment.Endpoint.id.seid_ESListService
                        }, createResourceURI: function() {
                            var suffix = MS.Entertainment.Data.Query.Music.RecentList.defaultSuffix.format(this.xuid);
                            return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                        }, createEDSHeaders: function createEDSHeaders() {
                            return {
                                    "Content-Type": "application/json", Accept: "application/json", "x-xbl-contract-version": "2", "x-xbl-client-type": MS.Entertainment.Data.EDSWrapperQuery.clientType, "x-xbl-device-type": this.getHeaderDeviceType(), "x-xbl-client-version": MS.Entertainment.Data.EDSWrapperQuery.currentVersion
                                }
                        }, getResponseVersion: function getResponseVersion(result) {
                            return result.listVersion
                        }
                }, {defaultSuffix: "users/xuid({0})/lists/RECN/MusicRecents"})
        }), RecentListAdd: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.ESListServiceWrapperQuery, function recentListAdd(mediaItem, listVersion) {
                        this.base();
                        if (mediaItem) {
                            var edsMediaItem;
                            var cloudProviderId = "X13Music";
                            var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                            var locale = signedInUser.locale;
                            switch (mediaItem.mediaType) {
                                case Microsoft.Entertainment.Queries.ObjectType.playlist:
                                    edsMediaItem = {
                                        ContentType: MS.Entertainment.Data.Query.edsMediaType.musicPlaylist, ItemId: null, ProviderId: mediaItem.cloudSyncItemId, Provider: cloudProviderId, ImageUrl: mediaItem.imageResizeUri, Title: mediaItem.name, Locale: locale
                                    };
                                    break;
                                case Microsoft.Entertainment.Queries.ObjectType.album:
                                    edsMediaItem = {
                                        ContentType: MS.Entertainment.Data.Query.edsMediaType.album, ItemId: mediaItem.canonicalId, ProviderId: null, Provider: null, ImageUrl: mediaItem.imageResizeUri, Title: mediaItem.name, Locale: locale
                                    };
                                    break;
                                case Microsoft.Entertainment.Queries.ObjectType.person:
                                case Microsoft.Entertainment.Queries.ObjectType.smartDJ:
                                    edsMediaItem = {
                                        ContentType: MS.Entertainment.Data.Query.edsMediaType.musicArtist, ItemId: mediaItem.canonicalId, ProviderId: null, Provider: null, ImageUrl: mediaItem.imageResizeUri || mediaItem.ImageUrl, Title: mediaItem.name, Locale: locale
                                    };
                                    break;
                                case Microsoft.Entertainment.Queries.ObjectType.track:
                                    var trackImage = mediaItem.imageResizeUri;
                                    mediaItem = mediaItem.album;
                                    if (mediaItem)
                                        edsMediaItem = {
                                            ContentType: MS.Entertainment.Data.Query.edsMediaType.album, ItemId: mediaItem.canonicalId, ProviderId: null, Provider: null, AltImageUrl: mediaItem.imageResizeUri || trackImage, Title: mediaItem.name, Locale: locale
                                        };
                                    break;
                                default:
                                    MS.Entertainment.ViewModels.fail("Object type " + mediaItem.mediaType + " is unable to push to the recent service");
                                    break
                            }
                            if (edsMediaItem) {
                                var data = {Items: [edsMediaItem]};
                                this.postData = {
                                    contentType: MS.Entertainment.Data.Query.Music.SmartDJAdd.contentType, data: JSON.stringify(data)
                                }
                            }
                            this._listVersion = listVersion;
                            this.requestType = MS.Entertainment.Data.ServiceWrapperQuery.RequestTypes.post
                        }
                    }, {
                        chunkSize: 0, useCache: false, forceSecureEndpoint: true, requestFields: MS.Entertainment.Data.EDSWrapperQuery.requestFieldsNotNeeded, getImagePromise: null, getResourceEndpointID: function getResourceEndpointID() {
                                return MS.Entertainment.Endpoint.id.seid_ESListService
                            }, createResourceURI: function() {
                                var suffix;
                                var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                                var suffix = MS.Entertainment.Data.Query.Music.RecentList.defaultSuffix.format(signedInUser.xuid);
                                return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                            }, createEDSHeaders: function createEDSHeaders() {
                                return {
                                        Accept: "application/json", "x-xbl-contract-version": "2", "x-xbl-client-type": MS.Entertainment.Data.EDSWrapperQuery.clientType, "x-xbl-device-type": this.getHeaderDeviceType(), "x-xbl-client-version": MS.Entertainment.Data.EDSWrapperQuery.currentVersion, "if-match": this._listVersion
                                    }
                            }, execute: function execute() {
                                if (this.postData)
                                    return MS.Entertainment.Data.Query.ESListServiceWrapperQuery.prototype.execute.apply(this, arguments);
                                else
                                    return WinJS.Promise.as()
                            }
                    }, {contentType: "application/json"})
            }), RecentListDelete: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.ESListServiceWrapperQuery, function RecentListDelete(listIndices, listVersion) {
                        this.base();
                        if (listIndices && listIndices.length > 0) {
                            var length = Math.min(listIndices.length, MAX_LIST_DELETE);
                            this._indices = [];
                            for (var i = 0; i < length; i++)
                                if (typeof listIndices[i] === "number" && listIndices[i] >= 0)
                                    this._indices.push(listIndices[i]);
                                else
                                    MS.Entertainment.Data.Query.Music.fail("Invalid index passed to RecentListDelete. index: " + listIndices[i]);
                            this._indices = this._indices.join()
                        }
                        this._listVersion = listVersion;
                        this.requestType = MS.Entertainment.Data.ServiceWrapperQuery.RequestTypes.deleteRequest
                    }, {
                        chunkSize: 0, useCache: false, _indices: String.empty, _listVersion: 0, forceSecureEndpoint: true, requestFields: MS.Entertainment.Data.EDSWrapperQuery.requestFieldsNotNeeded, getResourceEndpointID: function getResourceEndpointID() {
                                return MS.Entertainment.Endpoint.id.seid_ESListService
                            }, createResourceURI: function() {
                                var suffix;
                                var signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                                suffix = MS.Entertainment.Data.Query.Music.RecentList.defaultSuffix.format(signedInUser.xuid);
                                return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                            }, createEDSHeaders: function createEDSHeaders() {
                                return {
                                        Accept: "application/json", "x-xbl-contract-version": "2", "x-xbl-client-type": MS.Entertainment.Data.EDSWrapperQuery.clientType, "x-xbl-device-type": this.getHeaderDeviceType(), "x-xbl-client-version": MS.Entertainment.Data.EDSWrapperQuery.currentVersion, "if-match": this._listVersion
                                    }
                            }, createParameters: function createParameters() {
                                return {indexes: this._indices}
                            }
                    })
            })
    });
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music.WebPlaylist", {
        ConfigurationOperation: {
            defaultSites: "webPlaylist", filters: "webPlaylistFilters"
        }, ConfigurationQuery: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return WinJS.Class.derive(MS.Entertainment.Data.ServiceWrapperQuery, function ConfigurationQuery() {
                        MS.Entertainment.Data.ServiceWrapperQuery.prototype.constructor.apply(this, arguments)
                    }, {
                        operation: null, chunked: false, serviceType: MS.Entertainment.UI.Framework.lazyDefine(function() {
                                return MS.Entertainment.Data.ServiceWrapperQuery.ServiceTypes.json
                            }), getResourceEndpointID: function getResourceEndpointID() {
                                return MS.Entertainment.Endpoint.id.seid_WinPhoneXboxDotCom
                            }, createResourceURI: function createResourceURI() {
                                var suffix = "music/config/x13/" + this.createOperation();
                                return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                            }, createOperation: function createOperation() {
                                return WinJS.Binding.unwrap(this.operation)
                            }
                    })
            }), DefaultSites: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return WinJS.Class.derive(MS.Entertainment.Data.Query.Music.WebPlaylist.ConfigurationQuery, function ConfigurationQuery() {
                        MS.Entertainment.Data.Query.Music.WebPlaylist.ConfigurationQuery.prototype.constructor.apply(this, arguments)
                    }, {
                        operation: MS.Entertainment.Data.Query.Music.WebPlaylist.ConfigurationOperation.defaultSites, resultAugmentation: MS.Entertainment.Data.Augmenter.WebPlaylist.WebSitesResult
                    })
            }), Filters: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return WinJS.Class.derive(MS.Entertainment.Data.Query.Music.WebPlaylist.ConfigurationQuery, function ConfigurationQuery() {
                        MS.Entertainment.Data.Query.Music.WebPlaylist.ConfigurationQuery.prototype.constructor.apply(this, arguments)
                    }, {operation: MS.Entertainment.Data.Query.Music.WebPlaylist.ConfigurationOperation.filters})
            })
    });
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music.EchoNest", {
        EchoNestOperation: {artistExtract: "artist/extract"}, EchoNestBucket: {familiarity: "familiarity"}, echonestApiKey: MS.Entertainment.UI.Framework.lazyDefine(function() {
                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                return configurationManager.service.echoNestApiKey
            }), EchoNestQuery: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return WinJS.Class.derive(MS.Entertainment.Data.ServiceWrapperQuery, function echoNestQuery() {
                        MS.Entertainment.Data.ServiceWrapperQuery.prototype.constructor.apply(this, arguments)
                    }, {
                        _innerQueryChunked: false, apiKey: null, operation: null, serviceType: MS.Entertainment.UI.Framework.lazyDefine(function() {
                                return MS.Entertainment.Data.ServiceWrapperQuery.ServiceTypes.json
                            }), getResourceEndpointID: function getResourceEndpointID() {
                                return MS.Entertainment.Endpoint.id.seid_EchoNestDeveloperApi
                            }, createResourceURI: function createResourceURI() {
                                var suffix = this.createOperation();
                                return MS.Entertainment.Utilities.UriFactory.create(this.getResourceEndpoint(), suffix)
                            }, createParameters: function createParameters() {
                                var parameters = WinJS.Binding.unwrap(this.parameters) || {};
                                parameters.api_key = this.apiKey || MS.Entertainment.Data.Query.Music.EchoNest.echonestApiKey;
                                parameters.format = "json";
                                return parameters
                            }, createOperation: function createOperation() {
                                return WinJS.Binding.unwrap(this.operation)
                            }
                    })
            }), ExtractArtists: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return WinJS.Class.derive(MS.Entertainment.Data.Query.Music.EchoNest.EchoNestQuery, function echoNestQuery() {
                        MS.Entertainment.Data.Query.Music.EchoNest.EchoNestQuery.prototype.constructor.apply(this, arguments);
                        this.minPopularity = MS.Entertainment.Data.Query.Music.EchoNest.ExtractArtists.s_defaultMinPopularity;
                        var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                        this._maxTextChunkSize = configurationManager.service.echoNestTextExtractMaxSize;
                        this._maxTextSize = this._maxTextChunkSize * configurationManager.service.echoNestTextExtractMaxChunks
                    }, {
                        resultAugmentation: MS.Entertainment.Data.Augmenter.EchoNest.ArtistsResult, aggregateChunks: true, chunkSize: -1, operation: MS.Entertainment.Data.Query.Music.EchoNest.EchoNestOperation.artistExtract, text: null, minPopularity: -1, _maxTextChunkSize: -1, _maxTextSize: -1, _vistedText: 0, visitedText: {get: function get_visitedText() {
                                    return this._vistedText
                                }}, maxTextChunkSize: {get: function get_maxTextChunkSize() {
                                    return this._maxTextChunkSize
                                }}, totalChunks: {get: function get_totalChunks() {
                                    return (!this.text || !this.text.length) ? 0 : Math.ceil(this.text.length / this._maxTextChunkSize)
                                }}, createParameters: function createParameters() {
                                var parameters = MS.Entertainment.Data.Query.Music.EchoNest.EchoNestQuery.prototype.createParameters.apply(this, arguments);
                                parameters = parameters || {};
                                parameters.results = 100;
                                parameters.bucket = MS.Entertainment.Data.Query.Music.EchoNest.EchoNestBucket.familiarity;
                                parameters.min_familiarity = this.minPopularity >= 0 ? this.minPopularity : MS.Entertainment.Data.Query.Music.EchoNest.ExtractArtists.s_defaultMinPopularity;
                                var textOffset = 0;
                                if (this.currentChunk)
                                    textOffset = this.currentChunk.textOffset;
                                parameters.text = this.text && this.text.substring(textOffset, textOffset + this._maxTextChunkSize);
                                this._vistedText += parameters.text.length;
                                return parameters
                            }, _preInnerExecute: function _preInnerExecute() {
                                var returnResult = MS.Entertainment.Data.Query.Music.EchoNest.EchoNestQuery.prototype._preInnerExecute.apply(this, arguments);
                                if (!this.currentChunk)
                                    this._vistedText = 0;
                                return returnResult
                            }, _calculateNextChunkKey: function _calculateNextChunkKey(result) {
                                var newTextOffset = 0;
                                var nextChunk = null;
                                var maxTextSize = 0;
                                if (this.text)
                                    maxTextSize = this.text.length;
                                if (this._maxTextSize >= 0)
                                    maxTextSize = Math.min(maxTextSize, this._maxTextSize);
                                newTextOffset = this._vistedText;
                                if (this.text && newTextOffset > 0 && newTextOffset < maxTextSize)
                                    nextChunk = {textOffset: newTextOffset};
                                return nextChunk
                            }
                    }, {s_defaultMinPopularity: 0.50})
            })
    });
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music.WMIS", {
        queryEndpoint: null, getAlbumDetailsEndpoint: null, WMISQueryWrapper: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.ServiceWrapperQuery, null, {_preInnerExecute: function _preInnerExecute() {
                            var basePromise = MS.Entertainment.Data.ServiceWrapperQuery.prototype._preInnerExecute.apply(this, arguments);
                            var returnPromise = basePromise;
                            if (!MS.Entertainment.Data.Query.Music.WMIS.queryEndpoint || !MS.Entertainment.Data.Query.Music.WMIS.getAlbumDetailsEndpoint)
                                returnPromise = basePromise.then(function executeFieldsQuery() {
                                    var query = new MS.Entertainment.Data.Query.Music.WMIS.Endpoints;
                                    return query.execute()
                                }.bind(this)).then(function queryComplete(q) {
                                    MS.Entertainment.Data.Query.Music.WMIS.queryEndpoint = q.result.queryEndpoint;
                                    MS.Entertainment.Data.Query.Music.WMIS.getAlbumDetailsEndpoint = q.result.getAlbumDetailsEndpoint
                                }.bind(this));
                            return returnPromise
                        }})
            })
    });
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music.WMIS", {
        AlbumSearch: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.WMIS.WMISQueryWrapper, null, {
                    search: null, resultAugmentation: MS.Entertainment.Data.Augmenter.WMIS.AlbumsResult, createResourceURI: function() {
                            return MS.Entertainment.Data.Query.Music.WMIS.queryEndpoint
                        }, createParameters: function createParameters() {
                            return {
                                    SearchString: this.search, maxNumberOfResults: 100, resultTypeString: "album", countOnly: false
                                }
                        }, pluralizers: ["Result"]
                })
        }), AlbumWithTracks: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.Query.Music.WMIS.WMISQueryWrapper, null, {
                        id: null, chunked: false, chunkSize: 0, resultAugmentation: MS.Entertainment.Data.Augmenter.WMIS.AlbumWithTracksResult, createResourceURI: function() {
                                return MS.Entertainment.Data.Query.Music.WMIS.getAlbumDetailsEndpoint
                            }, createParameters: function createParameters() {
                                return {
                                        albumId: this.id, volume: 1
                                    }
                            }, pluralizers: ["track"]
                    })
            }), Endpoints: MS.Entertainment.UI.Framework.lazyDefine(function() {
                return MS.Entertainment.derive(MS.Entertainment.Data.ServiceWrapperQuery, null, {
                        id: null, resultAugmentation: MS.Entertainment.Data.Augmenter.WMIS.EndpointsResult, createResourceURI: function() {
                                return MS.Entertainment.Endpoint.load(MS.Entertainment.Endpoint.id.seid_WMISEndpoints)
                            }, createParameters: function createParameters() {
                                return {apiVersion: "1.0"}
                            }, pluralizers: ["ENDPOINT"]
                    })
            })
    });
    WinJS.Namespace.define("MS.Entertainment.Data.Query.Music.MusicMatch", {AlbumSearch: MS.Entertainment.UI.Framework.lazyDefine(function() {
            return MS.Entertainment.derive(MS.Entertainment.Data.WrapperQuery, null, {
                    albumLibraryId: null, resultAugmentation: MS.Entertainment.Data.Augmenter.MusicMatch.AlbumsResult, createInnerQuery: function createInnerQuery() {
                            return {executeAsync: function executeAsync() {
                                        return Microsoft.Entertainment.Marketplace.Marketplace.getAlbumsSuggestAsync(this.albumLibraryId)
                                    }.bind(this)}
                        }
                })
        })})
})()
