﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.DeepLink");
WinJS.Namespace.define("MS.Entertainment.UI.DeepLink", {
    _hideDeepLinkLoadingOverlayTimeoutMS: 10000, _deepLinksFactories: {}, isRegistered: function isRegistered(appMode) {
            return !!MS.Entertainment.UI.DeepLink._deepLinksFactories[appMode]
        }, registerDeepLinksFactory: function registerDeepLinksFactory(appMode, factory) {
            MS.Entertainment.UI.DeepLink._deepLinksFactories[appMode] = factory
        }, getTestHooks: function getTestHooks() {
            return {getRegisteredDeepLinksFactory: function getRegisteredDeepLinksFactory() {
                        return MS.Entertainment.UI.DeepLink._deepLinksFactories
                    }}
        }, ParamType: {
            identifier: /^[_a-zA-Z][_a-zA-Z0-9]*$/g, guid: /^(\{{0,1}([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}\}{0,1})$/g, mediaId: /^((\{{0,1}([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}\}{0,1})|[0-9]+)$/g, boolean: /^true|false$/gi, integer: /^[-+]?[0-9]+$/g, unsigned_integer: /^[0-9]+$/g, stringFormat: /.+/g, test: function test(type, value) {
                    type.lastIndex = 0;
                    return type.test(value)
                }
        }, PlayStateType: {
            play: "play", pause: "pause"
        }, processDeepLink: (function processDeepLink(command, params) {
            var _deepLinks = null;
            function convertToActionableParam(paramInput, paramType) {
                var type = MS.Entertainment.UI.DeepLink.ParamType;
                switch (paramType) {
                    case type.identifier:
                    case type.stringFormat:
                    case type.guid:
                        return paramInput;
                    case type.boolean:
                        return (paramInput.toLowerCase() === "true");
                    case type.integer:
                    case type.unsigned_integer:
                        return parseInt(paramInput);
                    case type.mediaId:
                        if (type.test(type.guid, paramInput))
                            return paramInput;
                        else if (type.test(type.unsigned_integer, paramInput))
                            return parseInt(paramInput);
                        else {
                            MS.Entertainment.UI.DeepLink.fail("The parameter input of type mediaId was invalid.  ParamInput=" + paramInput);
                            return null
                        }
                    default:
                        MS.Entertainment.UI.DeepLink.fail("The parameter type is not known.  ParamType=" + paramType);
                        return null
                }
            }
            return function processDeepLink(command, params) {
                    if (!command)
                        throw"processDeepLink: command parameter is mandatory";
                    if (!params)
                        throw"processDeepLink: command parameter is mandatory";
                    var type = MS.Entertainment.UI.DeepLink.ParamType;
                    if (!_deepLinks) {
                        var factory = MS.Entertainment.UI.DeepLink._deepLinksFactories[MS.Entertainment.appMode];
                        if (factory)
                            _deepLinks = factory();
                        if (!_deepLinks)
                            return false;
                        for (var name in _deepLinks) {
                            var deepLink = _deepLinks[name];
                            MS.Entertainment.UI.DeepLink.assert(deepLink.actionId, "DeepLink definition: 'actionId' field not specified");
                            MS.Entertainment.UI.DeepLink.assert(deepLink.params, "DeepLink definition: 'params' field not specified");
                            for (var paramName in deepLink.params) {
                                var param = deepLink.params[paramName];
                                MS.Entertainment.UI.DeepLink.assert(typeof param.required === "boolean", "DeepLink definition: 'required' field not specified");
                                MS.Entertainment.UI.DeepLink.assert(param.type, "DeepLink definition: 'type' field not specified")
                            }
                        }
                    }
                    deepLink = _deepLinks[command];
                    if (!deepLink)
                        return false;
                    for (name in deepLink.params)
                        if (deepLink.params[name].required && !params[name])
                            return false;
                    for (name in params) {
                        var parameterInfo = deepLink.params[name];
                        if (!parameterInfo)
                            return false;
                        parameterInfo.type.lastIndex = 0;
                        if (!type.test(parameterInfo.type, params[name]))
                            return false;
                        if (parameterInfo.values)
                            if (parameterInfo.caseInsensitive) {
                                var parameterValue = params[name].toLocaleLowerCase();
                                var validValue = false;
                                for (var i = 0; i < parameterInfo.values.length; i++)
                                    if (parameterInfo.values[i].toLocaleLowerCase() === parameterValue) {
                                        validValue = true;
                                        break
                                    }
                                if (!validValue)
                                    return false
                            }
                            else {
                                var actionParam = convertToActionableParam(params[name], deepLink.params[name].type);
                                if (actionParam === undefined || actionParam === null || deepLink.params[name].values.indexOf(actionParam) === -1)
                                    return false
                            }
                    }
                    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                    if (!actionService.isRegistered(deepLink.actionId))
                        return false;
                    var action = actionService.getAction(deepLink.actionId);
                    if (!action.automationId)
                        action.automationId = MS.Entertainment.UI.AutomationIds.deepLink;
                    var actionParams = {};
                    for (name in params) {
                        actionParams[name] = convertToActionableParam(params[name], deepLink.params[name].type);
                        if (actionParams[name] === undefined || actionParams[name] === null)
                            return false
                    }
                    action.parameter = actionParams;
                    action.command = command;
                    action.execute();
                    if (MS.Entertainment.Utilities.isMusicApp2 && !deepLink.hideOverlay)
                        MS.Entertainment.UI.DeepLink._showDeepLinkLoadingOverlay();
                    return true
                }
        })(), processProtocol: function processProtocol(url, dataPoint, arguments) {
            var decodedUrl = decodeURI(url);
            var match = /^[a-zA-Z0-9-]+:\/\/([a-zA-Z-]+)\/?\?(.*)$/g.exec(decodedUrl);
            if (!match && arguments)
                match = /^([a-zA-Z-]+)\/?\?(.*)$/g.exec(arguments);
            if (!match || !match[1])
                return false;
            var command = match[1];
            var isValid = true;
            var params = {};
            if (match[2]) {
                var nameValuePairs = match[2].split("&");
                for (var i = 0; i < nameValuePairs.length; i++) {
                    var nameValuePair = nameValuePairs[i];
                    match = /([_a-zA-Z][_a-zA-Z0-9-]*)=(.*)$/g.exec(nameValuePair);
                    if (match && match[1] && match[2] && !params[match[1]])
                        switch (match[1].toLowerCase()) {
                            case"source":
                                if (dataPoint)
                                    dataPoint.appendParameter("DeeplinkSource", match[2]);
                                break;
                            case"partnertoken":
                                MS.Entertainment.UI.DeepLink.assert(match[2], "No partner token received when processing deep link: " + url);
                                if (dataPoint)
                                    dataPoint.appendParameter("DeepLinkPartnerToken", match[2]);
                                this._updateSignInWithPartnerToken(match[2]);
                                break;
                            case"eventtype":
                                break;
                            default:
                                params[match[1]] = match[2];
                                break
                        }
                    else {
                        isValid = false;
                        break
                    }
                }
            }
            if (isValid)
                isValid = this.processDeepLink(command, params);
            var eventProvider = new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell;
            if (isValid)
                eventProvider.traceDeepLink_Protocol_Invoked(url);
            else
                eventProvider.traceDeepLink_Protocol_Failed(url);
            return isValid
        }, _updateSignInWithPartnerToken: function _updateSignInWithPartnerToken(partnerToken) {
            MS.Entertainment.UI.DeepLink.assert(partnerToken, "No partner token received when processing deep link");
            if (!partnerToken || !MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.signIn)) {
                MS.Entertainment.UI.DeepLink.fail("Unable to get sign in service for registering partner token");
                return
            }
            var signInService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
            signInService.updatePartnerToken(partnerToken)
        }, processTile: function processTile(tileArgs) {
            if (typeof tileArgs.command !== "string")
                return false;
            var command = tileArgs.command;
            var params = {};
            for (var name in tileArgs)
                if (name !== "command" && typeof tileArgs[name] === "string")
                    params[name] = tileArgs[name];
            var isValid = this.processDeepLink(command, params);
            var argumentsAsText = "command=" + command;
            for (name in params)
                argumentsAsText = argumentsAsText + ", " + name + "=" + params[name];
            var eventProvider = new Diagnostics.Tracing.Microsoft.Entertainment.Instrumentation.Providers.Shell;
            if (isValid)
                eventProvider.traceDeepLink_Tile_Invoked(argumentsAsText);
            else
                eventProvider.traceDeepLink_Tile_Failed(argumentsAsText);
            return isValid
        }, _showDeepLinkLoadingOverlay: function _showDeepLinkLoadingOverlay() {
            MS.Entertainment.UI.Controls.Overlay.showLoadingOverlay({
                userControl: "MS.Entertainment.UI.Controls.LoadingControl", hideOnNowPlaying: true, onInitialize: function setModel(overlayContent) {
                        var progressControl = document.createElement("progress");
                        WinJS.Utilities.addClass(progressControl, "loadingRing win-ring win-large");
                        overlayContent.appendChild(progressControl);
                        return WinJS.Promise.wrap()
                    }, userControlOptions: {
                        isBusy: true, height: null
                    }, closeTimeoutMS: MS.Entertainment.UI.DeepLink._hideDeepLinkLoadingOverlayTimeoutMS
            })
        }
});
MS.Entertainment.UI.DeepLink.registerDeepLinksFactory(Microsoft.Entertainment.Application.AppMode.music2, function() {
    var type = MS.Entertainment.UI.DeepLink.ParamType;
    return {
            location: {
                actionId: MS.Entertainment.UI.Actions.ActionIdentifiers.deepLinkLocation, params: {
                        id: {
                            type: type.identifier, required: true, values: [MS.Entertainment.UI.Monikers.homeHub, MS.Entertainment.UI.Monikers.musicMarketplaceFeatured, ]
                        }, gamerTag: {
                                type: type.stringFormat, required: false
                            }
                    }
            }, details: {
                    actionId: MS.Entertainment.UI.Actions.ActionIdentifiers.deepLinkDetails, params: {
                            id: {
                                type: type.guid, required: true
                            }, desiredMediaItemType: {
                                    type: type.identifier, required: false, values: [MS.Entertainment.Data.Query.edsMediaType.album, MS.Entertainment.Data.Query.edsMediaType.musicArtist, MS.Entertainment.Data.Query.edsMediaType.track], caseInsensitive: true
                                }, idType: {
                                    type: type.identifier, required: false, values: [MS.Entertainment.Data.Query.edsIdType.canonical, MS.Entertainment.Data.Query.edsIdType.zuneCatalog, ], caseInsensitive: true
                                }
                        }
                }, play: {
                    actionId: MS.Entertainment.UI.Actions.ActionIdentifiers.deepLinkPlay, params: {
                            id: {
                                type: type.guid, required: true
                            }, startIndex: {
                                    type: type.unsigned_integer, required: false
                                }, desiredMediaItemType: {
                                    type: type.identifier, required: false, values: [MS.Entertainment.Data.Query.edsMediaType.album, MS.Entertainment.Data.Query.edsMediaType.musicArtist, MS.Entertainment.Data.Query.edsMediaType.track], caseInsensitive: true
                                }, idType: {
                                    type: type.identifier, required: false, values: [MS.Entertainment.Data.Query.edsIdType.canonical, MS.Entertainment.Data.Query.edsIdType.zuneCatalog, ], caseInsensitive: true
                                }, gamerTag: {
                                    type: type.stringFormat, required: false
                                }, playSmartDJ: {
                                    type: type.boolean, required: false
                                }
                        }
                }, "default": {
                    actionId: MS.Entertainment.UI.Actions.ActionIdentifiers.deepLinkXBLDefault, params: {
                            PlayState: {
                                type: type.identifier, required: false, values: [MS.Entertainment.UI.DeepLink.PlayStateType.play, MS.Entertainment.UI.DeepLink.PlayStateType.pause], caseInsensitive: true
                            }, DeepLinkInfo: {
                                    type: type.stringFormat, required: false
                                }
                        }, hideOverlay: true
                }, "media-details": {
                    actionId: MS.Entertainment.UI.Actions.ActionIdentifiers.deepLinkXBLDetails, params: {
                            ContentID: {
                                type: type.guid, required: true
                            }, ContentType: {
                                    type: type.identifier, required: false, values: [MS.Entertainment.Data.Query.edsMediaType.album, MS.Entertainment.Data.Query.edsMediaType.musicArtist, MS.Entertainment.Data.Query.edsMediaType.track, MS.Entertainment.Data.Query.edsMediaType.musicPlaylist], caseInsensitive: true
                                }, DeepLinkInfo: {
                                    type: type.stringFormat, required: false
                                }
                        }
                }, "media-playback": {
                    actionId: MS.Entertainment.UI.Actions.ActionIdentifiers.deepLinkXBLPlayback, params: {
                            ContentID: {
                                type: type.guid, required: true
                            }, ContentType: {
                                    type: type.identifier, required: false, values: [MS.Entertainment.Data.Query.edsMediaType.album, MS.Entertainment.Data.Query.edsMediaType.musicArtist, MS.Entertainment.Data.Query.edsMediaType.track, MS.Entertainment.Data.Query.edsMediaType.musicPlaylist], caseInsensitive: true
                                }, PlaySmartDJ: {
                                    type: type.boolean, required: false
                                }, DeepLinkInfo: {
                                    type: type.stringFormat, required: false
                                }
                        }
                }
        }
})
