﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function(undefined) {
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Actions");
    WinJS.Namespace.define("MS.Entertainment.UI.Actions", {
        navigate: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function navigateConstructor() {
            this.base()
        }, {
            automationId: MS.Entertainment.UI.AutomationIds.navigate, offlineMessageTitle: null, executed: function executed(param, referenceElement) {
                    if (referenceElement)
                        MS.Entertainment.Utilities.Telemetry.logPageAction({domElement: referenceElement}, {
                            uri: MS.Entertainment.Utilities.getCurrentUserLocation(), pageTypeId: MS.Entertainment.Utilities.Telemetry.PageTypeId.Dash
                        });
                    if (param === MS.Entertainment.UI.Actions.navigate.NavigateLocation.back) {
                        if (!MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).overlayVisible)
                            MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateBack()
                    }
                    else if (param === MS.Entertainment.UI.Actions.navigate.NavigateLocation.home) {
                        var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                        navigationService.navigateToDefaultPage()
                    }
                    else {
                        var location = this._extractLocation(param);
                        var args = this._extractArguments(param);
                        MS.Entertainment.UI.Actions.assert((location), "the navigate command was expecting a location object, but didn't get one.");
                        MS.Entertainment.UI.Actions.assert((typeof location.page === "string"), "the navigate command was expecting a string page location, but didn't get one.");
                        if (location && location.page)
                            MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(location.page, location.hub, location.panel, args)
                    }
                }, canExecute: function canExecute(param) {
                    if (this.disableWhenOffline)
                        switch (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).networkStatus) {
                            case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.unknown:
                            case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.unrestricted:
                            case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.throttled:
                            case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.onDemand:
                                break;
                            case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.localOnly:
                            case MS.Entertainment.UI.NetworkStatusService.NetworkStatus.none:
                                if (this.offlineMessageTitle) {
                                    var errorCode = MS.Entertainment.Platform.Playback.Error.NS_E_WMPIM_USEROFFLINE.code;
                                    MS.Entertainment.UI.Shell.showError(this.offlineMessageTitle, errorCode)
                                }
                                return false
                        }
                    if (this.disableOnServicesDisabled)
                        return MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).servicesEnabled;
                    var location = this._extractLocation(param);
                    return !MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).isSnapped && (location) && ((typeof location.page === "string") || location === MS.Entertainment.UI.Actions.navigate.NavigateLocation.back || location === MS.Entertainment.UI.Actions.navigate.NavigateLocation.home)
                }, _extractLocation: function _extractLocation(param) {
                    var location;
                    if (param === MS.Entertainment.UI.Actions.navigate.NavigateLocation.back)
                        location = param;
                    else if (param === MS.Entertainment.UI.Actions.navigate.NavigateLocation.home)
                        location = param;
                    else if (typeof param === "object")
                        location = param;
                    else
                        location = {page: param};
                    return location
                }, _extractArguments: function _extractArguments(param) {
                    var result = null;
                    var args = null;
                    if (param && param.args) {
                        args = WinJS.Binding.unwrap(param.args);
                        result = {};
                        for (var key in args)
                            result[key] = args[key]
                    }
                    return result
                }
        }, {NavigateLocation: {
                back: "back", home: "home"
            }}), musicCollectionNavigate: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function musicCollectionNavigateConstructor() {
                this.base()
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.navigate, executed: function executed() {
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(MS.Entertainment.UI.Monikers.musicCollection)
                    }, canExecute: function canExecute() {
                        return true
                    }
            }), videoCollectionNavigate: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function videoCollectionNavigateConstructor() {
                this.base()
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.navigate, executed: function executed() {
                        var hub = MS.Entertainment.UI.Monikers.allVideoCollection;
                        var view = MS.Entertainment.ViewModels.VideoCollection.ViewTypes.all;
                        try {
                            view = Windows.Storage.ApplicationData.current.localSettings.values["VideoCollectionView"]
                        }
                        catch(e) {}
                        switch (view) {
                            case MS.Entertainment.ViewModels.VideoCollection.ViewTypes.all:
                                hub = MS.Entertainment.UI.Monikers.allVideoCollection;
                                break;
                            case MS.Entertainment.ViewModels.VideoCollection.ViewTypes.movies:
                                hub = MS.Entertainment.UI.Monikers.movieCollection;
                                break;
                            case MS.Entertainment.ViewModels.VideoCollection.ViewTypes.tv:
                                hub = MS.Entertainment.UI.Monikers.tvCollection;
                                break;
                            case MS.Entertainment.ViewModels.VideoCollection.ViewTypes.other:
                                hub = MS.Entertainment.UI.Monikers.otherVideoCollection;
                                break
                        }
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(MS.Entertainment.UI.Monikers.videoCollection, hub)
                    }, canExecute: function canExecute() {
                        return true
                    }
            }), videoCollectionMovieNavigate: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function videoCollectionMovieNavigateConstructor() {
                this.base()
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.navigate, executed: function executed() {
                        var args = {selectHub: true};
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(MS.Entertainment.UI.Monikers.videoCollection, MS.Entertainment.UI.Monikers.movieCollection, null, args)
                    }, canExecute: function canExecute() {
                        return true
                    }
            }), videoCollectionTvNavigate: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function videoCollectionTvNavigateConstructor() {
                this.base()
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.navigate, executed: function executed() {
                        var args = {selectHub: true};
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(MS.Entertainment.UI.Monikers.videoCollection, MS.Entertainment.UI.Monikers.tvCollection, null, args)
                    }, canExecute: function canExecute() {
                        return true
                    }
            }), videoCollectionOtherNavigate: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function videoCollectionOtherNavigateConstructor() {
                this.base()
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.navigate, executed: function executed() {
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(MS.Entertainment.UI.Monikers.videoCollection, MS.Entertainment.UI.Monikers.otherVideoCollection)
                    }, canExecute: function canExecute() {
                        return true
                    }
            }), depthGalleryNavigate: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function depthGalleryNavigateConstructor() {
                this.base()
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.navigate, executed: function executed() {
                        var parameter = this.parameter || {};
                        var page = parameter.page;
                        var hub = parameter.hub;
                        var panel = parameter.panel;
                        var navigationArguments = parameter.args || {};
                        navigationArguments.data = parameter.data;
                        MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation).navigateTo(page, hub, panel, navigationArguments)
                    }, canExecute: function canExecute() {
                        return true
                    }
            }), videoPostRollNavigate: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function videoPostRollNavigateConstructor() {
                this.base()
            }, {
                automationId: MS.Entertainment.UI.AutomationIds.navigate, executed: function executed() {
                        var parameter = this.parameter || {};
                        var page = parameter.page;
                        var hub = parameter.hub;
                        var panel = parameter.panel;
                        var abTestName = parameter.abTestName;
                        var abGroupName = parameter.abGroupName;
                        var videoPostRollAction = parameter.videoPostRollAction;
                        MS.Entertainment.Utilities.Telemetry.logVideoPostRollAction(abTestName, abGroupName, videoPostRollAction);
                        var navigationService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.navigation);
                        navigationService.clearBackStackOnNextNavigate(true);
                        navigationService.navigateTo(page, hub, panel)
                    }, canExecute: function canExecute() {
                        return true
                    }
            })
    });
    WinJS.Namespace.define("MS.Entertainment.UI.Actions", {videoMarketplaceNavigate: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.navigate", function videoMarketplaceNavigateConstructor() {
            this.base()
        }, {
            moniker: String.empty, executed: function executed() {
                    var serviceConfig = (new Microsoft.Entertainment.Configuration.ConfigurationManager).service;
                    var query;
                    var view = String.empty;
                    var movie = true;
                    switch (this.moniker) {
                        case MS.Entertainment.UI.Monikers.movieMarketplaceNewReleases:
                        case MS.Entertainment.UI.Monikers.movieMarketplaceFeatured:
                            view = (this.moniker === MS.Entertainment.UI.Monikers.movieMarketplaceNewReleases) ? "movieNewReleases" : "movieFeatured";
                            query = new MS.Entertainment.Data.Query.MovieHub;
                            break;
                        case MS.Entertainment.UI.Monikers.movieMarketplaceTopPurchased:
                            view = "movieTopPurchased";
                            query = new MS.Entertainment.Data.Query.Video.EdsBrowseMoviesByGenreStudio;
                            break;
                        case MS.Entertainment.UI.Monikers.tvMarketplaceNewReleases:
                        case MS.Entertainment.UI.Monikers.tvMarketplaceFeatured:
                            view = (this.moniker === MS.Entertainment.UI.Monikers.tvMarketplaceNewReleases) ? "tvNewReleases" : "tvFeatured";
                            movie = false;
                            query = new MS.Entertainment.Data.Query.TvHub;
                            break;
                        case MS.Entertainment.UI.Monikers.tvMarketplaceTopPurchased:
                            view = "tvTopPurchased";
                            query = new MS.Entertainment.Data.Query.Video.EdsBrowseTVByGenreNetwork;
                            movie = false;
                            break;
                        default:
                            MS.Entertainment.UI.Actions.assert(fail, "videoMarketplaceNavigate received an unknown view");
                            break
                    }
                    var queryInfo = {};
                    var notificationInfo = MS.Entertainment.ViewModels.VideoMarketplace.getNotificationInfo(view);
                    if (notificationInfo) {
                        var modQueryInfo = MS.Entertainment.ViewModels.VideoMarketplace.modifyQuery(query, notificationInfo.resultModifier, notificationInfo.taskKeyGetter, notificationInfo.notifier);
                        if (modQueryInfo) {
                            queryInfo.notifications = modQueryInfo.notifications;
                            queryInfo.sender = modQueryInfo.sender
                        }
                    }
                    queryInfo.query = query;
                    queryInfo.promise = query.execute();
                    MS.Entertainment.ViewModels.VideoMarketplace.preloadedVideoMarketplaceQueryInfo = queryInfo;
                    if (movie)
                        if (!MS.Entertainment.Utilities.isApp2) {
                            delete Windows.Storage.ApplicationData.current.roamingSettings.values["movieMarketplaceFilter"];
                            delete Windows.Storage.ApplicationData.current.roamingSettings.values["movieMarketplaceSecondaryFilter"]
                        }
                        else {
                            delete Windows.Storage.ApplicationData.current.localSettings.values["movieMarketplaceFilter"];
                            delete Windows.Storage.ApplicationData.current.localSettings.values["movieMarketplaceSecondaryFilter"]
                        }
                    else if (!MS.Entertainment.Utilities.isApp2) {
                        delete Windows.Storage.ApplicationData.current.roamingSettings.values["tvMarketplaceFilter"];
                        delete Windows.Storage.ApplicationData.current.roamingSettings.values["tvMarketplaceSecondaryFilter"]
                    }
                    else {
                        delete Windows.Storage.ApplicationData.current.localSettings.values["tvMarketplaceFilter"];
                        delete Windows.Storage.ApplicationData.current.localSettings.values["tvMarketplaceSecondaryFilter"]
                    }
                    var executed = (new MS.Entertainment.UI.Actions.navigate).executed;
                    executed.apply(this, arguments)
                }
        })});
    var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.navigate, function() {
        return new MS.Entertainment.UI.Actions.navigate
    });
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.musicCollectionNavigate, function() {
        return new MS.Entertainment.UI.Actions.musicCollectionNavigate
    });
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.videoCollectionNavigate, function() {
        return new MS.Entertainment.UI.Actions.videoCollectionNavigate
    });
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.videoCollectionMovieNavigate, function() {
        return new MS.Entertainment.UI.Actions.videoCollectionMovieNavigate
    });
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.videoCollectionOtherNavigate, function() {
        return new MS.Entertainment.UI.Actions.videoCollectionOtherNavigate
    });
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.videoMarketplaceNavigate, function() {
        return new MS.Entertainment.UI.Actions.videoMarketplaceNavigate
    });
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.videoCollectionTvNavigate, function() {
        return new MS.Entertainment.UI.Actions.videoCollectionTvNavigate
    });
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.depthGalleryNavigate, function() {
        return new MS.Entertainment.UI.Actions.depthGalleryNavigate
    });
    actionService.register(MS.Entertainment.UI.Actions.ActionIdentifiers.videoPostRollNavigate, function() {
        return new MS.Entertainment.UI.Actions.videoPostRollNavigate
    })
})()
