﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
(function() {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.ViewModels");
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {
        ArtistImmersiveViewModel: MS.Entertainment.defineOptionalObservable(function artistImmersiveViewModel() {
            this.frames = new MS.Entertainment.ObservableArray;
            MS.Entertainment.UI.Framework.loadTemplate("/Components/Immersive/Music/ArtistOverview.html", "ImmersiveOverview");
            if (MS.Entertainment.Utilities.isMusicApp1) {
                if (window.onNewMusicPage)
                    this.phases = [this._loadHeroPanel, this._loadNowPlayingPanel];
                else
                    this.phases = [this._loadHeroPanel, this._loadNowPlayingPanel, this._loadBioPanel, this._loadDiscographyPanel, this._loadRelatedArtistPanel, this._loadSecondaryFrames];
                this._heroControl = MS.Entertainment.UI.Controls.ImmersiveHero
            }
            else if (MS.Entertainment.Utilities.isMusicApp2) {
                this.phases = [this._loadAboutPanel, this._loadFullDiscographyPanel, this._loadTopSongsPanel, this._loadRelatedArtistPanel, this._loadSecondaryFrames];
                this._heroControl = MS.Entertainment.UI.Controls.Music2ImmersiveHero;
                this._heroColumns = 0;
                this.skipInitialFocus = true
            }
            else
                MS.Entertainment.ViewModels.fail("Unknown app mode.  Must be Music1 or Music2")
        }, {
            _sessionId: null, _loadedMediaItem: null, _loadingPromise: null, _hydratePromise: null, _mediaItem: null, _originalMediaItem: null, _disposed: false, _nowPlayingBindings: null, _smartDJBindings: null, _currentPhase: 0, _frameFlushCount: 1, _frozen: false, _isGenericOverview: false, _useBackgroundImage: false, _heroControl: null, _heroColumns: 2, _heroAdded: false, _heroDelayMS: 500, skipInitialFocus: false, phases: null, secondaryFrames: null, nowPlayingFrame: null, smartBuyStateEngine: null, _mediaContext: null, _appBarButtonBinding: null, deferredFrameLoadTimeout: 2500, dispose: function dispose() {
                    this._releaseSmartBuyStateEngine();
                    this._releaseAlbumViewModel();
                    this._releaseRelatedArtistsViewModel();
                    this._releaseDiscographyViewModel();
                    this._releaseNowPlayingBindings();
                    this._releaseSmartDJBindings();
                    this._disposed = true
                }, freeze: function freeze() {
                    this._frozen = true;
                    this._releaseNowPlayingBindings()
                }, thaw: function thaw() {
                    this._frozen = false;
                    if (this.phases && this.phases.indexOf(this._loadNowPlayingPanel) >= 0)
                        this._loadNowPlayingPanel()
                }, mediaItem: {
                    get: function() {
                        return this._mediaItem
                    }, set: function(value) {
                            if (value !== this._mediaItem && !this._disposed) {
                                var oldValue = this._mediaItem;
                                this._mediaItem = value;
                                if (this.heroDataContext)
                                    this.heroDataContext.mediaItem = value;
                                this.notify("mediaItem", value, oldValue)
                            }
                        }
                }, sessionId: {
                    get: function() {
                        return this._sessionId
                    }, set: function(value) {
                            if (value !== this._sessionId && !this._disposed) {
                                var oldValue = this._sessionId;
                                this._sessionId = value;
                                if (this.heroDataContext)
                                    this.heroDataContext.sessionId = value;
                                this.notify("sessionId", value, oldValue)
                            }
                        }
                }, updateMetaData: function updateMetaData(mediaItem, deferAdditionalFrames) {
                    if (this._disposed)
                        return;
                    var nowPlayingPanelIndex;
                    var firstSecondaryIndex;
                    this._originalMediaItem = mediaItem;
                    if (mediaItem.mediaType !== Microsoft.Entertainment.Queries.ObjectType.person && mediaItem.artist) {
                        mediaItem = mediaItem.artist;
                        mediaItem = MS.Entertainment.ViewModels.MediaItemModel.augment(mediaItem)
                    }
                    this.heroDataContext = new MS.Entertainment.ViewModels.ArtistHeroViewModel(mediaItem);
                    this.heroDataContext.sessionId = this.sessionId;
                    if (this._hydratePromise && mediaItem.isEqual(this.mediaItem) && !this._isGenericOverview)
                        return this._hydratePromise;
                    this.mediaItem = mediaItem;
                    if (this.frames.length > 1) {
                        nowPlayingPanelIndex = this._getIndexOfNowPlayingPanel();
                        firstSecondaryIndex = 1;
                        if (nowPlayingPanelIndex > 0) {
                            if (nowPlayingPanelIndex > firstSecondaryIndex)
                                this.frames.splice(firstSecondaryIndex, nowPlayingPanelIndex - firstSecondaryIndex);
                            firstSecondaryIndex = nowPlayingPanelIndex + 1
                        }
                        this.frames.splice(firstSecondaryIndex, this.frames.length - firstSecondaryIndex)
                    }
                    if (!this.smartBuyStateEngine || this.smartBuyStateEngine.media !== mediaItem) {
                        this._releaseSmartBuyStateEngine();
                        this.smartBuyStateEngine = new MS.Entertainment.ViewModels.SmartBuyStateEngine;
                        this.smartBuyStateEngine.initialize(this._originalMediaItem.mediaType === Microsoft.Entertainment.Queries.ObjectType.track ? this._originalMediaItem : this.mediaItem, MS.Entertainment.ViewModels.SmartBuyButtons.getArtistImmersiveButtons(this._originalMediaItem, MS.Entertainment.UI.Actions.ExecutionLocation.canvas), MS.Entertainment.ViewModels.MusicStateHandlers.onArtistImmersiveStateChanged);
                        if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.appToolbar)) {
                            var appBarService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar);
                            this._mediaContext = appBarService.pushMediaContext(mediaItem, null, this.smartBuyStateEngine.currentAppbarActions, {executeLocation: MS.Entertainment.UI.Actions.ExecutionLocation.canvas});
                            this._appBarButtonBinding = WinJS.Binding.bind(this.smartBuyStateEngine, {currentAppbarActions: function currentAppbarActionsChanged() {
                                    if (!this.disposed && this._mediaContext)
                                        this._mediaContext.setToolbarActions(this.smartBuyStateEngine.currentAppbarActions)
                                }.bind(this)})
                        }
                    }
                    var hydratePromise = null;
                    if (mediaItem.hydrate)
                        hydratePromise = mediaItem.hydrate({
                            forceUpdate: mediaItem.fromCollection, listenForDBUpdates: true
                        }).then(function mediaHydrated() {
                            this.mediaItem = mediaItem;
                            if (this._useBackgroundImage)
                                this.backgroundImageUri = mediaItem.backgroundImageUri;
                            return mediaItem
                        }.bind(this), function mediaHydrateFailed() {
                            return mediaItem
                        }.bind(this));
                    else
                        hydratePromise = WinJS.Promise.wrap(mediaItem);
                    if (this._loadingPromise)
                        this._loadingPromise.cancel();
                    this.secondaryFrames = [];
                    this._isGenericOverview = false;
                    this._loadingPromise = this._loadPhase(0, hydratePromise, deferAdditionalFrames);
                    this._loadingPromise.done(function() {
                        this._loadingPromise = null
                    }.bind(this), function() {
                        this._loadingPromise = null
                    }.bind(this));
                    this._hydratePromise = hydratePromise;
                    return this._hydratePromise
                }, _loadPhase: function _loadPhase(phase, param, deferAdditionalFrames) {
                    if (this._disposed || phase >= this.phases.length)
                        return WinJS.Promise.as();
                    var phasePromise = this.phases[phase].call(this, param);
                    return phasePromise.then(function endPhase() {
                            return this._endPhase(phase, param, deferAdditionalFrames)
                        }.bind(this), function(error) {
                            if (error.name !== "Canceled")
                                return this._endPhase(phase, param, deferAdditionalFrames);
                            else
                                return WinJS.Promise.wrapError(error)
                        }.bind(this))
                }, _endPhase: function _endPhase(phase, param, deferAdditionalFrames) {
                    return WinJS.Promise.timeout(deferAdditionalFrames ? this.deferredFrameLoadTimeout : 0).then(function _delay() {
                            if (this.secondaryFrames && phase === this._frameFlushCount) {
                                this.frames.spliceArray(this.frames.length, 0, this.secondaryFrames);
                                this.secondaryFrames = []
                            }
                            return this._loadPhase(phase + 1, param)
                        }.bind(this))
                }, _loadHeroPanel: function _loadHeroPanel(mediaHydratePromise) {
                    if (this._heroAdded)
                        return mediaHydratePromise;
                    var heroFrame = function makeHeroFrame() {
                            var frame = MS.Entertainment.UI.Controls.Immersive.makeFrame(String.load(String.id.IDS_DETAILS_OVERVIEW), this._heroColumns, this._heroControl);
                            frame.columnStyle = "immersiveHeroFrameColumn firstImmersiveTwoColumn";
                            frame.getData = function frameGetData() {
                                return WinJS.Promise.as(this.heroDataContext)
                            }.bind(this);
                            return frame
                        }.bind(this)();
                    if (!this.frames || this.frames.length === 0)
                        this.frames = new MS.Entertainment.ObservableArray([heroFrame]);
                    else
                        this.frames.push(heroFrame);
                    this._heroAdded = true;
                    return mediaHydratePromise
                }, _loadNowPlayingPanel: function _loadNowPlayingPanel() {
                    this._releaseNowPlayingBindings();
                    this._nowPlayingBindings = WinJS.Binding.bind(MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState), {nowPlayingVisible: function onNowPlayingVisible(nowPlayingVisible) {
                            if (nowPlayingVisible)
                                this._insertNowPlayingPanel();
                            else
                                this._removeNowPlayingPanel()
                        }.bind(this)});
                    return WinJS.Promise.wrap()
                }, _releaseNowPlayingBindings: function _releaseNowPlayingBindings() {
                    if (this._nowPlayingBindings) {
                        this._nowPlayingBindings.cancel();
                        this._nowPlayingBindings = null
                    }
                }, _releaseSmartDJBindings: function _releaseSmartDJBindings() {
                    if (this._smartDJBindings) {
                        this._smartDJBindings.cancel();
                        this._smartDJBindings = null
                    }
                }, _getIndexOfNowPlayingPanel: function _getIndexOfNowPlayingPanel() {
                    var nowPlayingFrameIndex = -1;
                    if (this.nowPlayingFrame)
                        nowPlayingFrameIndex = this.frames.indexOf(this.nowPlayingFrame);
                    return nowPlayingFrameIndex
                }, _insertNowPlayingPanel: function _insertNowPlayingPanel() {
                    if (this._disposed || this._frozen)
                        return;
                    var nowPlayingFrameIndex = this._getIndexOfNowPlayingPanel();
                    if (nowPlayingFrameIndex < 0) {
                        this.nowPlayingFrame = this._makeNowPlayingFrame();
                        nowPlayingFrameIndex = Math.min(1, this.frames.length);
                        if (this.nowPlayingFrame)
                            this.frames.insert(nowPlayingFrameIndex, this.nowPlayingFrame)
                    }
                    var sessionManager = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                    this._releaseSmartDJBindings();
                    if (sessionManager && sessionManager.primarySession)
                        this._smartDJBindings = WinJS.Binding.bind(sessionManager.primarySession, {smartDJSeed: this._smartDJSeedChanged.bind(this)})
                }, _removeNowPlayingPanel: function _removeNowPlayingPanel() {
                    if (this._disposed || this._frozen)
                        return;
                    var nowPlayingFrameIndex = this._getIndexOfNowPlayingPanel();
                    if (nowPlayingFrameIndex >= 0)
                        this.frames.removeAt(nowPlayingFrameIndex);
                    this._releaseSmartDJBindings()
                }, _loadActivitiesPanel: function _loadActivitiesPanel(mediaHydratePromise) {
                    return mediaHydratePromise.then(function addFrame(result) {
                            if (result && result.activities)
                                this.secondaryFrames.push(result.activities)
                        }.bind(this))
                }, _loadBioPanel: function _loadBioPanel(result) {
                    this.biographyViewModel = new MS.Entertainment.ViewModels.ArtistBiographyViewModel(this.mediaItem, false);
                    return this.biographyViewModel.execute().then(function createAboutPanel() {
                            if (this.biographyViewModel && this.biographyViewModel.description) {
                                var bioFrame = null;
                                this._populateSmartDJButton(this.biographyViewModel);
                                bioFrame = this._makeOverviewFrame(false);
                                if (bioFrame)
                                    this.secondaryFrames.push(bioFrame)
                            }
                        }.bind(this))
                }, _loadAboutPanel: function _loadAboutPanel(mediaHydratePromise) {
                    this.biographyViewModel = new MS.Entertainment.ViewModels.ArtistBiographyViewModel(this.mediaItem, true);
                    if (!this.frames || this.frames.length === 0)
                        this.frames = new MS.Entertainment.ObservableArray([function makeBioFrame() {
                                var bioFrame = null;
                                bioFrame = this._makeOverviewFrame(false);
                                return bioFrame
                            }.bind(this)()]);
                    this._loadHeroPanel(mediaHydratePromise);
                    mediaHydratePromise = mediaHydratePromise.then(function delay() {
                        return WinJS.Promise.timeout(this._heroDelayMS)
                    }.bind(this));
                    mediaHydratePromise.done(function execute() {
                        this.biographyViewModel.execute()
                    }.bind(this));
                    return mediaHydratePromise
                }, _loadDiscographyPanel: function _loadDiscographyPanel(result) {
                    this._releaseAlbumViewModel();
                    this.albumViewModel = new MS.Entertainment.ViewModels.ImmersiveDetailsArtistAlbums(this.mediaItem);
                    return this.albumViewModel.execute().then(function addFrame(result) {
                            if (this.albumViewModel && this.albumViewModel.heroItem) {
                                this.secondaryFrames.push(this._makeAlbumsFrame());
                                if (!this.biographyViewModel || !this.biographyViewModel.smartBuyStateEngine)
                                    this._populateSmartDJButton(this.albumViewModel)
                            }
                        }.bind(this))
                }, _loadFullDiscographyPanel: function _loadFullDiscographyPanel(result) {
                    this._releaseDiscographyViewModel();
                    this.discographyViewModel = new MS.Entertainment.ViewModels.ImmersiveDetailsArtistsDiscography(this.mediaItem);
                    return this.discographyViewModel.execute().then(function addFrame(result) {
                            var discographyViewModel = WinJS.Binding.unwrap(this.discographyViewModel);
                            if (discographyViewModel && ((discographyViewModel.firstPanelItems && discographyViewModel.firstPanelItems.length > 0) || (discographyViewModel.secondPanelItems && discographyViewModel.secondPanelItems.length > 0)))
                                this.secondaryFrames.push(this._makeDiscographyFrame())
                        }.bind(this))
                }, _loadTopSongsPanel: function _loadTopSongsPanel(result) {
                    if (!this.mediaItem || !this.mediaItem.canonicalId)
                        return WinJS.Promise.as();
                    var query = new MS.Entertainment.Data.Query.Music.ArtistTopSongs;
                    query.id = this.mediaItem.canonicalId;
                    query.impressionGuid = this.mediaItem.impressionGuid;
                    return query.execute().then(function addFrame(result) {
                            if (result && result.result && result.result.total > 0)
                                this.secondaryFrames.push(this._makeTopSongsFrame(result))
                        }.bind(this))
                }, _loadRelatedArtistPanel: function _loadRelatedArtistPanel(result) {
                    this._releaseRelatedArtistsViewModel();
                    this.relatedArtistsViewModel = new MS.Entertainment.ViewModels.ImmersiveDetailsRelatedArtists(this.mediaItem);
                    return this.relatedArtistsViewModel.execute().then(function addFrame(result) {
                            var relatedArtistsViewModel = WinJS.Binding.unwrap(this.relatedArtistsViewModel);
                            if (relatedArtistsViewModel && relatedArtistsViewModel.firstPanelItems && relatedArtistsViewModel.firstPanelItems.length > 0)
                                this.secondaryFrames.push(this._makeRelatedArtistsFrame())
                        }.bind(this))
                }, _loadSecondaryFrames: function _loadSecondaryFrames(result) {
                    if (this._disposed)
                        return;
                    var mediaItemToLoad;
                    var minFrames = 1;
                    var nowPlayingPanelIndex = this._getIndexOfNowPlayingPanel();
                    if (nowPlayingPanelIndex >= 0)
                        minFrames++;
                    if (this.frames.length + this.secondaryFrames.length <= minFrames) {
                        mediaItemToLoad = this._originalMediaItem;
                        this.secondaryFrames.push(this._makeOverviewFrame(true))
                    }
                    else
                        mediaItemToLoad = this.mediaItem;
                    this._loadedMediaItem = mediaItemToLoad;
                    if (this.secondaryFrames.length)
                        this.frames.spliceArray(this.frames.length, 0, this.secondaryFrames);
                    return WinJS.Promise.as()
                }, _releaseAlbumViewModel: function _releaseAlbumViewModel() {
                    if (this.albumViewModel) {
                        this.albumViewModel.dispose();
                        this.albumViewModel = null
                    }
                }, _releaseRelatedArtistsViewModel: function _releaseRelatedArtistsViewModel() {
                    if (this.relatedArtistsViewModel) {
                        this.relatedArtistsViewModel.dispose();
                        this.relatedArtistsViewModel = null
                    }
                }, _releaseDiscographyViewModel: function _releaseDiscographyViewModel() {
                    if (this.discographyViewModel) {
                        this.discographyViewModel.dispose();
                        this.discographyViewModel = null
                    }
                }, _releaseSmartBuyStateEngine: function _releaseSmartBuyStateEngine() {
                    if (this._appBarButtonBinding) {
                        this._appBarButtonBinding.cancel();
                        this._appBarButtonBinding = null
                    }
                    if (this._mediaContext) {
                        this._mediaContext.clearContext();
                        this._mediaContext = null
                    }
                    if (this.smartBuyStateEngine) {
                        this.smartBuyStateEngine.unload();
                        this.smartBuyStateEngine = null
                    }
                }, _populateSmartDJButton: function _populateSmartDJButton(smartDJViewModel) {
                    if (smartDJViewModel)
                        smartDJViewModel.smartBuyStateEngine = this.smartBuyStateEngine
                }, _makeOverviewFrame: function _makeOverviewFrame(useGenericOverview) {
                    this._isGenericOverview = useGenericOverview;
                    var frame;
                    if (useGenericOverview) {
                        frame = MS.Entertainment.UI.Controls.Immersive.makeFrame(String.load(String.id.IDS_DETAILS_OVERVIEW), 1, MS.Entertainment.UI.Controls.ArtistImmersiveOverviewGenericSummary, "Components/Immersive/Music/MoreArtistOverviewGeneric.html");
                        frame.hideViewMoreIfEnoughSpace = true;
                        frame.getData = function frameGetData() {
                            return WinJS.Promise.wrap(this._originalMediaItem)
                        }.bind(this)
                    }
                    else {
                        var title = String.load(String.id.IDS_MUSIC_ARTIST_BIO_TITLE);
                        var columns = 1;
                        var control = MS.Entertainment.UI.Controls.ArtistImmersiveOverviewSummary;
                        if (MS.Entertainment.Utilities.isMusicApp2) {
                            title = String.load(String.id.IDS_MUSIC_ABOUT_PANEL_TEXT);
                            columns = 1;
                            control = MS.Entertainment.UI.Controls.ArtistImmersiveHeroSummary
                        }
                        frame = MS.Entertainment.UI.Controls.Immersive.makeFrame(title, columns, control, "/Components/Immersive/Music/MoreArtistOverview.html");
                        frame.hideViewMoreIfEnoughSpace = true;
                        if (MS.Entertainment.Utilities.isMusicApp2)
                            frame.columnStyle = "immersiveAboutPanel";
                        frame.getData = function frameGetData() {
                            return WinJS.Promise.wrap(this.biographyViewModel)
                        }.bind(this)
                    }
                    frame.onShowMore = this._freezeVisualizations;
                    frame.onHideMore = this._thawVisualizations;
                    return frame
                }, _makeAlbumsFrame: function _makeAlbumsFrame() {
                    var frame = MS.Entertainment.UI.Controls.Immersive.makeFrame(String.load(String.id.IDS_MUSIC_DISCOGRAPHY_TITLE), 2, MS.Entertainment.UI.Controls.ImmersiveRelatedItems, "Components/Immersive/Music/MoreArtistAlbums.html", MS.Entertainment.ViewModels.MusicCollection.ViewTypes.albums);
                    frame.hideViewMoreIfEnoughSpace = false;
                    frame.viewMoreInfo.title = String.load(String.id.IDS_DETAILS_ARTIST_ALBUMS_VIEW_ALL);
                    frame.getData = function frameGetData() {
                        return WinJS.Promise.wrap(this.albumViewModel)
                    }.bind(this);
                    frame.onShowMore = this._freezeVisualizations;
                    frame.onHideMore = this._thawVisualizations;
                    return frame
                }, _makeDiscographyFrame: function _makeDiscographyFrame() {
                    var frame = MS.Entertainment.UI.Controls.Immersive.makeFrame(String.load(String.id.IDS_MUSIC_DISCOGRAPHY_TITLE), -1, MS.Entertainment.UI.Controls.ImmersiveRelatedTileItems, String.empty, MS.Entertainment.ViewModels.MusicCollection.ViewTypes.albums);
                    frame.columnStyle = "immersiveAutoSize";
                    frame.getData = function frameGetData() {
                        return WinJS.Promise.wrap(this.discographyViewModel)
                    }.bind(this);
                    frame.onShowMore = this._freezeVisualizations;
                    frame.onHideMore = this._thawVisualizations;
                    return frame
                }, _makeRelatedArtistsFrame: function _makeRelatedArtistsFrame() {
                    var columns = MS.Entertainment.Utilities.isMusicApp1 ? 2 : -1;
                    var frame = MS.Entertainment.UI.Controls.Immersive.makeFrame(String.load(String.id.IDS_MUSIC_RELATED_ARTISTS_TEXT), columns, MS.Entertainment.UI.Controls.ImmersiveRelatedTileItems, "Components/Immersive/Music/MoreArtistAlbums.html", MS.Entertainment.ViewModels.MusicCollection.ViewTypes.artists);
                    if (columns > 0) {
                        frame.hideViewMoreIfEnoughSpace = false;
                        frame.viewMoreInfo.title = String.load(String.id.IDS_MUSIC_RELATED_ARTISTS_LINK)
                    }
                    else
                        frame.columnStyle = "immersiveAutoSize";
                    frame.getData = function frameGetData() {
                        return WinJS.Promise.wrap(this.relatedArtistsViewModel)
                    }.bind(this);
                    frame.onShowMore = this._freezeVisualizations;
                    frame.onHideMore = this._thawVisualizations;
                    return frame
                }, _makeTopSongsFrame: function _makeTopSongsFrame(result) {
                    var frame = MS.Entertainment.UI.Controls.Immersive.makeFrame(String.load(String.id.IDS_MUSIC2_TOP_SONGS_PANEL_TEXT), 3, MS.Entertainment.UI.Controls.NowPlayingPlaylist, null, MS.Entertainment.ViewModels.MusicCollection.ViewTypes.songs);
                    frame.hideViewMoreIfEnoughSpace = true;
                    frame.getData = function frameGetData() {
                        return WinJS.Promise.wrap({
                                dataSource: result.result.items, galleryTemplate: MS.Entertainment.UI.Controls.NowPlayingPlaylistTemplates.topSongs, focusFirstItemOnPageLoad: false
                            })
                    }.bind(this);
                    return frame
                }, _makeNowPlayingFrame: function _makeNowPlayingFrame() {
                    var frame = null;
                    var sessionManager = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                    var title = this._getPlaylistHeading(sessionManager);
                    frame = MS.Entertainment.UI.Controls.Immersive.makeFrame(title, 1, MS.Entertainment.UI.Controls.NowPlayingPlaylist, null, MS.Entertainment.ViewModels.MusicCollection.ViewTypes.songs);
                    frame.columnStyle = "immersiveNowPlayingColumn";
                    frame.hideViewMoreIfEnoughSpace = true;
                    frame.getData = function frameGetData() {
                        return WinJS.Promise.wrap({
                                playbackSession: sessionManager.nowPlayingSession, galleryTemplate: MS.Entertainment.UI.Controls.NowPlayingPlaylistTemplates.nowPlaying
                            })
                    }.bind(this);
                    return frame
                }, _smartDJSeedChanged: function _smartDJSeedChanged() {
                    var sessionManager = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                    if (this.nowPlayingFrame)
                        this.nowPlayingFrame.heading = this._getPlaylistHeading(sessionManager)
                }, _getPlaylistHeading: function _getPlaylistHeading(sessionManager) {
                    if (sessionManager && sessionManager.nowPlayingSession)
                        return sessionManager.nowPlayingSession.smartDJSeed ? sessionManager.nowPlayingSession.smartDJSeed.name : String.load(String.id.IDS_HOME_NOW_PLAYING_LC)
                }, _freezeVisualizations: function _freezeVisualizations() {
                    if (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).nowPlayingVisible)
                        MS.Entertainment.UI.Controls.MusicVisualization.freezeShapes()
                }, _thawVisualizations: function _thawVisualizations() {
                    if (MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).nowPlayingVisible)
                        MS.Entertainment.UI.Controls.MusicVisualization.thawShapes()
                }
        }, {
            frames: null, heroDataContext: null, biographyViewModel: null, albumViewModel: null, relatedArtistsViewModel: null, discographyViewModel: null
        }, {Monikers: {
                overview: "overview", albums: "albums", activities: "activities", related: "related", relatedArtists: "relatedArtists"
            }}), ArtistBiographyViewModel: MS.Entertainment.defineOptionalObservable(function artistBiographyViewModel(mediaItem, noTags) {
                this.mediaItem = mediaItem;
                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                this._noTags = noTags || configurationManager.music.disableArtistBioLinks
            }, {
                _noTags: true, execute: function execute() {
                        var description = String.empty;
                        if (this.mediaItem)
                            if (this.mediaItem.backgroundImage)
                                this.backgroundImageUri = this.mediaItem.backgroundImage.url;
                            else
                                this.backgroundImageUri = this.mediaItem.backgroundImageUri;
                        if (this.mediaItem)
                            description = this._noTags ? this.mediaItem.descriptionNoTags : this.mediaItem.description;
                        var promise = null;
                        this._processDescription(description ? description : String.empty);
                        if (this.mediaItem && this.mediaItem.hasServiceId && !this.mediaItem.hasDescription && !this.description) {
                            var biographyQuery = new MS.Entertainment.Data.Query.Music.ArtistBio;
                            biographyQuery.id = this.mediaItem.serviceId;
                            biographyQuery.impressionGuid = this.mediaItem.impressionGuid;
                            biographyQuery.idType = this.mediaItem.serviceIdType;
                            promise = biographyQuery.execute().then(function querySuccess(q) {
                                if (q.result.item)
                                    this._processDescription(q.result.item.content);
                                return this.description
                            }.bind(this))
                        }
                        else
                            promise = WinJS.Promise.wrap(this.description);
                        return promise
                    }, _processDescription: function _processDescription(description) {
                        this.mediaItem.description = this.description = (description || String.empty)
                    }
            }, {
                mediaItem: null, backgroundImageUri: String.empty, description: String.empty, smartBuyStateEngine: null, backgroundArtAvailable: false
            }), ImmersiveDetailsArtistAlbums: MS.Entertainment.defineOptionalObservable(function immersiveDetailsArtistAlbums(source) {
                this.source = source;
                this.selectedTemplate = MS.Entertainment.ViewModels.ImmersiveDetailsArtistAlbums.defaultTemplate;
                this._fileTransferListenerId = "ImmersiveDetailsArtistAlbums_" + MS.Entertainment.Utilities.getSessionUniqueInteger();
                this._signedInUser = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser);
                this._signInBindings = WinJS.Binding.bind(this._signedInUser, {isSubscription: this._updateSubscriptionLink.bind(this)})
            }, {
                _maxItems: 11, _moreModel: null, _currentExecution: null, _signedInUser: null, _signInBindings: null, _fileTransferListenerId: null, _disposed: false, source: null, dispose: function dispose() {
                        this._cancelQuery();
                        if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.fileTransfer)) {
                            var fileTransferService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransfer);
                            fileTransferService.unregisterListener(this._fileTransferListenerId)
                        }
                        if (this._signInBindings) {
                            this._signInBindings.cancel();
                            this._signInBindings = null
                        }
                        if (this._moreModel) {
                            this._moreModel.dispose();
                            this._moreModel = null
                        }
                        this._disposed = true
                    }, execute: function execute() {
                        return this._beginQuery()
                    }, moreModel: {get: function() {
                            if (!this._moreModel && this.source && !this._disposed) {
                                this._moreModel = new MS.Entertainment.ViewModels.ArtistMarketplace;
                                this._moreModel.artist = this.source;
                                this._moreModel.view = MS.Entertainment.ViewModels.ArtistMarketplace.ViewTypes.artist
                            }
                            return this._moreModel
                        }}, _createHeroItem: function _createHeroItem(item) {
                        return MS.Entertainment.Data.augment(item || {}, MS.Entertainment.ViewModels.ImmersiveDetailsArtistAlbums.HeroAugmentation)
                    }, _cancelQuery: function _cancelQuery() {
                        if (this._currentExecution) {
                            this._currentExecution.cancel();
                            this._currentExecution = null
                        }
                    }, _beginQuery: function _beginQuery() {
                        if (this._disposed)
                            return;
                        this.heroItem = null;
                        this.heroActionItem = null;
                        this.heroMediaItem = null;
                        this.items = [];
                        this._cancelQuery();
                        if (this.source && !MS.Entertainment.Utilities.isEmptyGuid(this.source.canonicalId)) {
                            var sender;
                            var notifications;
                            var query = new MS.Entertainment.Data.Query.Music.ArtistAlbums;
                            query.orderBy = MS.Entertainment.Data.Query.convertToLegacySortIfNeeded(MS.Entertainment.Data.Query.edsSortOrder.releaseDate);
                            query.chunkSize = 10;
                            query.chunked = false;
                            query.artistId = this.source.canonicalId;
                            query.impressionGuid = this.source.impressionGuid;
                            if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.fileTransfer)) {
                                notifications = new MS.Entertainment.UI.ContentNotification.NotificationModification(MS.Entertainment.UI.ContentNotification.listResult(), MS.Entertainment.UI.FileTransferService.keyFromProperty("serviceId", false, true));
                                sender = notifications.createSender();
                                notifications.modifyQuery(query)
                            }
                            this._currentExecution = query.execute().then(function queryCompleted(q) {
                                if (sender) {
                                    var fileTransferService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransfer);
                                    fileTransferService.registerListener(this._fileTransferListenerId, MS.Entertainment.UI.FileTransferService.keyFromProperty("albumMediaId", false, true), sender, MS.Entertainment.UI.FileTransferNotifiers.trackCollection)
                                }
                                if (q.result.items)
                                    return q.result.items.toArrayAll(0, this._maxItems);
                                else
                                    return []
                            }.bind(this)).then(function setResult(array) {
                                array = Array.isArray(array) ? array : [];
                                this.items = array.length > 1 ? array.slice(1) : array;
                                this.heroItem = array[0] ? this._createHeroItem(array[0]) : null;
                                this.heroActionItem = array[0];
                                array.forEach(function hydrateItem(item) {
                                    if (item)
                                        MS.Entertainment.ViewModels.MediaItemModel.hydrateLibraryInfoAsync(item)
                                }.bind(this))
                            }.bind(this)).then(this._endQuery.bind(this), this._endQuery.bind(this))
                        }
                        else
                            this.items = [];
                        return this._currentExecution || WinJS.Promise.wrap()
                    }, _endQuery: function _endQuery() {
                        this._currentExecution = null
                    }, _updateSubscriptionLink: function _updateSubscriptionLink() {
                        var relatedAction = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.subscriptionSignup);
                        if (relatedAction.canExecute()) {
                            this.relatedAction = relatedAction;
                            this.relatedActionParam = MS.Entertainment.Music.Freeplay.Events.musicPassUpsellArtistDiscographyInvoked;
                            this.relatedActionStringId = String.id.IDS_MUSIC_GET_A_SUBSCRIPTION_TEXT;
                            this.relatedActionAccStringId = String.id.IDS_MUSIC_GET_A_SUBSCRIPTION_TEXT
                        }
                    }
            }, {
                heroItem: null, heroActionItem: null, items: null, selectedTemplate: null, relatedAction: null, relatedActionParam: null, relatedActionStringId: null, relatedActionAccStringId: null, smartBuyStateEngine: null, backgroundImageUri: null
            }, {
                HeroAugmentation: MS.Entertainment.Data.define(null, {
                    heroLabel: MS.Entertainment.Data.Property.alias("releaseYear", String.empty), heroMediaItem: MS.Entertainment.Data.Property.alias(String.empty, null)
                }), defaultTemplate: {
                        templateUrl: "Components/Music/MusicSharedTemplates.html#verticalAlbumSummaryThumbnailHost", panelTemplateUrl: "MS.Entertainment.Pages.MusicAlbumInlineDetails", panelOptions: {location: MS.Entertainment.Data.ItemLocation.marketplace}, imageWidth: "360px", imageHeight: "360px", className: "albums"
                    }
            }), ImmersiveDetailsArtistsDiscography: MS.Entertainment.defineOptionalObservable(function immersiveDetailsArtistsDiscography(mediaItem) {
                this.mediaItem = mediaItem;
                this.selectedTemplate = {
                    templateUrl: "Components/Music/MusicSharedTemplates.html#largeDiscographyAlbumTemplate", className: "flexPanel albumDiscography", panelTemplateUrl: "MS.Entertainment.Pages.MusicAlbumInlineDetails", panelOptions: {location: MS.Entertainment.Data.ItemLocation.marketplace}
                }
            }, {
                _numberOfCollectionAlbums: 6, _numberOfMarketplaceAlbums: 6, _disposed: false, dispose: function dispose() {
                        this._disposed = true
                    }, execute: function execute() {
                        if (this._disposed)
                            return;
                        var promise = null;
                        var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                        this.firstPanelItems = [];
                        this.secondPanelItems = [];
                        if (this.mediaItem) {
                            var libraryInfoPromise = WinJS.Promise.wrap();
                            if (!this.mediaItem.libraryId || this.mediaItem.libraryId < 0)
                                libraryInfoPromise = MS.Entertainment.ViewModels.MediaItemModel.hydrateLibraryInfoAsync(this.mediaItem);
                            promise = libraryInfoPromise.then(function executeLibraryQuery() {
                                if (this.mediaItem.libraryId < 0)
                                    return WinJS.Promise.as();
                                var sender;
                                var notifications;
                                var query = new MS.Entertainment.Data.Query.libraryAlbums;
                                query.aggregateChunks = false;
                                query.orderBy = MS.Entertainment.Data.Query.convertToLegacySortIfNeeded(MS.Entertainment.Data.Query.edsSortOrder.releaseDate);
                                query.chunkSize = 10;
                                query.chunked = false;
                                query.sort = Microsoft.Entertainment.Queries.AlbumsSortBy.dateAddedDescending;
                                query.artistId = this.mediaItem.libraryId;
                                if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.fileTransfer)) {
                                    notifications = new MS.Entertainment.UI.ContentNotification.NotificationModification(MS.Entertainment.UI.ContentNotification.listResult(), MS.Entertainment.UI.FileTransferService.keyFromProperty("serviceId", false, true));
                                    sender = notifications.createSender();
                                    notifications.modifyQuery(query)
                                }
                                return query.execute().then(function queryCompleted(q) {
                                        if (sender) {
                                            var fileTransferService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransfer);
                                            fileTransferService.registerListener(this._fileTransferListenerId, MS.Entertainment.UI.FileTransferService.keyFromProperty("albumMediaId", false, true), sender, MS.Entertainment.UI.FileTransferNotifiers.trackCollection)
                                        }
                                        if (q.result.items)
                                            return q.result.items.toArray(0, this._numberOfCollectionAlbums + 1);
                                        else
                                            return []
                                    }.bind(this))
                            }.bind(this)).then(function setResult(array) {
                                if (array && array.length > 0) {
                                    if (array.length <= this._numberOfCollectionAlbums)
                                        this.firstPanelItems = array;
                                    else {
                                        this.firstPanelItems = array.slice(0, this._numberOfCollectionAlbums);
                                        var collectionNavigateAction = actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.navigate);
                                        collectionNavigateAction.parameter = {
                                            page: MS.Entertainment.UI.Monikers.artistAlbums, hub: MS.Entertainment.UI.Monikers.artistAlbumsCollection, args: {
                                                    selectHub: true, artistId: this.mediaItem.canonicalId, artistLibraryId: this.mediaItem.libraryId, artistName: this.mediaItem.name
                                                }
                                        };
                                        this.firstPanelMoreAction = {
                                            action: collectionNavigateAction, voicePhrase: String.load(String.id.IDS_MUSIC2_ARTIST_DETAILS_MORE_COLLECTION_ALBUMS_VUI_ALM), voicePhoneticPhrase: String.load(String.id.IDS_MUSIC2_ARTIST_DETAILS_MORE_COLLECTION_ALBUMS_VUI_PRON), voiceConfidence: String.load(String.id.IDS_MUSIC2_ARTIST_DETAILS_MORE_COLLECTION_ALBUMS_VUI_CONF)
                                        }
                                    }
                                    if (this.firstPanelItems.length > 0) {
                                        this.showFirstPanel = true;
                                        if (MS.Entertainment.Utilities.isMusicApp2)
                                            this.firstPanelHeader = String.load(String.id.IDS_MUSIC2_DISCOGRAPHY_COLLECTION_TITLE_LC)
                                    }
                                }
                                else
                                    return null
                            }.bind(this)).then(function getMarketplaceAlbums() {
                                if (!MS.Entertainment.Utilities.isEmptyGuid(this.mediaItem.canonicalId)) {
                                    var sender;
                                    var notifications;
                                    var query = new MS.Entertainment.Data.Query.Music.ArtistAlbums;
                                    query.aggregateChunks = false;
                                    query.orderBy = MS.Entertainment.Data.Query.convertToLegacySortIfNeeded(MS.Entertainment.Data.Query.edsSortOrder.releaseDate);
                                    query.chunkSize = 10;
                                    query.chunked = false;
                                    query.artistId = this.mediaItem.canonicalId;
                                    query.impressionGuid = this.mediaItem.impressionGuid;
                                    if (MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.fileTransfer)) {
                                        notifications = new MS.Entertainment.UI.ContentNotification.NotificationModification(MS.Entertainment.UI.ContentNotification.listResult(), MS.Entertainment.UI.FileTransferService.keyFromProperty("serviceId", false, true));
                                        sender = notifications.createSender();
                                        notifications.modifyQuery(query)
                                    }
                                    return query.execute().then(function queryCompleted(q) {
                                            if (sender) {
                                                var fileTransferService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.fileTransfer);
                                                fileTransferService.registerListener(this._fileTransferListenerId, MS.Entertainment.UI.FileTransferService.keyFromProperty("albumMediaId", false, true), sender, MS.Entertainment.UI.FileTransferNotifiers.trackCollection)
                                            }
                                            if (q.result.items)
                                                return q.result.items.toArray(0, this._numberOfMarketplaceAlbums + 1);
                                            else
                                                return []
                                        }.bind(this)).then(function hydrateListLibraryInfoAsync(marketplaceAlbums) {
                                            var albums = marketplaceAlbums || [];
                                            return MS.Entertainment.ViewModels.MediaItemModel.hydrateListLibraryInfoAsync(albums).then(function onHydrateLibraryIdComplete() {
                                                    return albums
                                                }, function onHydrateLibraryIdError(error) {
                                                    var errorMessage = error && errorMessage;
                                                    MS.Entertainment.ViewModels.fail("hydrateListLibraryInfoAsync failed with error: " + errorMessage);
                                                    return albums
                                                })
                                        }).then(function setMarketplaceAlbums(marketplaceAlbums) {
                                            if (marketplaceAlbums.length <= this._numberOfMarketplaceAlbums)
                                                this.secondPanelItems = marketplaceAlbums;
                                            else {
                                                this.secondPanelItems = marketplaceAlbums.slice(0, this._numberOfMarketplaceAlbums);
                                                var actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
                                                var marketPlaceNavigateAction = actionService.getAction(MS.Entertainment.UI.Actions.ActionIdentifiers.navigate);
                                                marketPlaceNavigateAction.parameter = {
                                                    page: MS.Entertainment.UI.Monikers.artistAlbums, hub: MS.Entertainment.UI.Monikers.artistAlbumsMarketplace, args: {
                                                            selectHub: true, artistId: this.mediaItem.canonicalId, artistLibraryId: this.mediaItem.libraryId, artistName: this.mediaItem.name
                                                        }
                                                };
                                                this.secondPanelMoreAction = {
                                                    action: marketPlaceNavigateAction, voicePhrase: String.load(String.id.IDS_MUSIC2_ARTIST_DETAILS_MORE_CATALOG_ALBUMS_VUI_ALM), voicePhoneticPhrase: String.load(String.id.IDS_MUSIC2_ARTIST_DETAILS_MORE_CATALOG_ALBUMS_VUI_PRON), voiceConfidence: String.load(String.id.IDS_MUSIC2_ARTIST_DETAILS_MORE_CATALOG_ALBUMS_VUI_CONF)
                                                }
                                            }
                                            if (this.secondPanelItems.length > 0) {
                                                this.showSecondPanel = true;
                                                this.secondPanelHeader = String.load(String.id.IDS_MARKETPLACE_PIVOT)
                                            }
                                        }.bind(this))
                                }
                                else
                                    return null
                            }.bind(this))
                        }
                        else
                            promise = WinJS.Promise.wrap(this.items);
                        return promise
                    }
            }, {
                firstPanelItems: null, firstPanelHeader: String.empty, firstPanelMoreAction: null, secondPanelItems: null, secondPanelHeader: null, secondPanelMoreAction: null, showFirstPanel: false, showSecondPanel: false, selectedTemplate: null
            }), ImmersiveDetailsRelatedArtists: MS.Entertainment.defineOptionalObservable(function immersiveDetailsRelatedArtists(mediaItem) {
                this.mediaItem = mediaItem;
                this.selectedTemplate = {
                    templateUrl: MS.Entertainment.Utilities.isMusicApp1 ? "Components/Music/MusicSharedTemplates.html#relatedArtistTileTemplate" : "Components/Music/MusicSharedTemplates.html#relatedArtistTileLargeTemplate", className: "flexPanel relatedArtists", panelTemplateUrl: "MS.Entertainment.Pages.MusicArtistInlineDetails", panelOptions: {location: MS.Entertainment.Data.ItemLocation.marketplace}
                };
                this._uiStateServiceBindings = MS.Entertainment.Utilities.addEventHandlers(MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState), {windowresize: function refreshView() {
                        this.execute()
                    }.bind(this)})
            }, {
                _uiStateServiceBindings: null, _numberOfSimilarItems: 0, _numberOfInfluencingArtists: 0, _moreModel: null, _disposed: false, dispose: function dispose() {
                        if (this._moreModel) {
                            this._moreModel.dispose();
                            this._moreModel = null
                        }
                        if (this._uiStateServiceBindings) {
                            this._uiStateServiceBindings.cancel();
                            this._uiStateServiceBindings = null
                        }
                        this._disposed = true
                    }, execute: function execute() {
                        if (this._disposed)
                            return;
                        var promise = null;
                        this.items = [];
                        this.firstPanelItems = [];
                        this.secondPanelItems = [];
                        if (MS.Entertainment.Utilities.isMusicApp1) {
                            var screenHeight = MS.Entertainment.Utilities.getWindowHeight();
                            var rowCount = screenHeight >= 1080 ? 3 : 2;
                            this._numberOfSimilarItems = (2 * rowCount);
                            this._numberOfInfluencingArtists = 0
                        }
                        else {
                            this._numberOfSimilarItems = 6;
                            this._numberOfInfluencingArtists = 6
                        }
                        if (this.mediaItem && !MS.Entertainment.Utilities.isEmptyGuid(this.mediaItem.canonicalId)) {
                            var relatedArtistsQuery = new MS.Entertainment.Data.Query.Music.RelatedArtists;
                            relatedArtistsQuery.id = this.mediaItem.canonicalId;
                            relatedArtistsQuery.impressionGuid = this.mediaItem.impressionGuid;
                            relatedArtistsQuery.chunkSize = this._numberOfSimilarItems;
                            relatedArtistsQuery.chunked = false;
                            promise = relatedArtistsQuery.execute().then(function relatedArtistQuerySuccess(response) {
                                if (response.result.itemsArray)
                                    return response.result.itemsArray
                            }.bind(this)).then(function relatedArtistQuerySetItemList(itemList) {
                                if (itemList && itemList.length > 0) {
                                    if (itemList.length <= this._numberOfSimilarItems)
                                        this.firstPanelItems = itemList;
                                    else
                                        this.firstPanelItems = itemList.slice(0, this._numberOfSimilarItems);
                                    if (this.firstPanelItems.length > 0) {
                                        this.showFirstPanel = true;
                                        if (MS.Entertainment.Utilities.isMusicApp2)
                                            this.firstPanelHeader = String.load(String.id.IDS_MUSIC_SIMILAR_ARTISTS_TEXT)
                                    }
                                    var influencedByQuery = new MS.Entertainment.Data.Query.Music.InfluenceArtists;
                                    influencedByQuery.id = this.mediaItem.canonicalId;
                                    influencedByQuery.impressionGuid = this.mediaItem.impressionGuid;
                                    influencedByQuery.chunkSize = this._numberOfInfluencingArtists;
                                    influencedByQuery.chunked = false;
                                    return influencedByQuery.execute().then(function influencedByQuerySuccess(response) {
                                            if (response.result && response.result.itemsArray && response.result.itemsArray.length > 0 && response.result.itemsArray[0]) {
                                                this.influencedCount = response.result.itemsArray.length;
                                                if (MS.Entertainment.Utilities.isMusicApp2) {
                                                    var items = response.result.itemsArray;
                                                    if (items.length <= this._numberOfInfluencingArtists)
                                                        this.secondPanelItems = items;
                                                    else
                                                        this.secondPanelItems = items.slice(0, this._numberOfInfluencingArtists);
                                                    if (this.secondPanelItems.length > 0) {
                                                        this.showSecondPanel = true;
                                                        this.secondPanelHeader = String.load(String.id.IDS_MUSIC_INFLUENCING_ARTISTS_TEXT)
                                                    }
                                                }
                                            }
                                        }.bind(this), function onError(){})
                                }
                            }.bind(this))
                        }
                        else
                            promise = WinJS.Promise.wrap(this.items);
                        return promise
                    }, moreModel: {get: function() {
                            if (!this._moreModel && this.mediaItem && !this._disposed) {
                                this._moreModel = new MS.Entertainment.ViewModels.ArtistMarketplace;
                                this._moreModel.artist = this.mediaItem;
                                if (this.influencedCount && this.influencedCount > 0)
                                    this._moreModel.view = MS.Entertainment.ViewModels.ArtistMarketplace.ViewTypes.relatedArtistsAndInfluencers;
                                else
                                    this._moreModel.view = MS.Entertainment.ViewModels.ArtistMarketplace.ViewTypes.relatedArtists
                            }
                            return this._moreModel
                        }}
            }, {
                items: null, influencedCount: 0, firstPanelItems: null, firstPanelHeader: String.empty, secondPanelItems: null, secondPanelHeader: String.empty, showFirstPanel: false, showSecondPanel: false, selectedTemplate: null
            }), ArtistHeroViewModel: MS.Entertainment.deferredDerive(MS.Entertainment.ViewModels.BaseHeroViewModel, function artistHeroViewModel(mediaItem) {
                this.base(mediaItem);
                this._initialize();
                this.actionDescription = String.empty
            }, {
                isMarketplace: false, playlist: null, _eventHandler: null, dispose: function dispose() {
                        if (this._eventHandler) {
                            this._eventHandler.cancel();
                            this._eventHandler = null
                        }
                    }, _initialize: function _initialize() {
                        if (!this.mediaItem || MS.Entertainment.Utilities.isEmptyGuid(this.mediaItem.serviceId))
                            return;
                        if (this._eventHandler) {
                            this._eventHandler.cancel();
                            this._eventHandler = null
                        }
                        if (this.buttons)
                            this.buttons.clear();
                        this.actionDescription = String.empty;
                        var mediaStore = new Microsoft.Entertainment.Platform.MediaStore;
                        if (this.mediaItem.fromCollection)
                            mediaStore.artistProvider.getLibraryIdFromMediaIdAsync(this.mediaItem.serviceId).then(function loadLibraryOrServiceTracks(result) {
                                if (result && MS.Entertainment.Utilities.isValidLibraryId(result.libraryId))
                                    this._createCollectionButtons(result.libraryId);
                                else {
                                    this._createMarketplaceButtons();
                                    this.isMarketplace = true
                                }
                            }.bind(this));
                        else {
                            this._createMarketplaceButtons();
                            this.isMarketplace = true
                        }
                    }, _createCollectionButtons: function _createCollectionButtons(libraryId) {
                        var buttons = MS.Entertainment.ViewModels.SmartBuyButtons.getArtistImmersiveDetailsButtonsDefault(this.mediaItem, true);
                        if (buttons.playAll && buttons.playAll.parameter)
                            buttons.playAll.parameter.libraryId = libraryId;
                        var query = new MS.Entertainment.Data.Query.libraryTracks;
                        query.artistId = libraryId;
                        query.execute().then(this._setResult.bind(this));
                        this.actionDescription = String.load(String.id.IDS_MUSIC_SONGS_IN_COLLECTION);
                        this.playbackItemSource = query.clone();
                        this._addButtons(buttons)
                    }, _createMarketplaceButtons: function _createMarketplaceButtons() {
                        var hydratePromise = WinJS.Promise.as();
                        if (!MS.Entertainment.Utilities.isValidServiceId(this.mediaItem.canonicalId))
                            hydratePromise = this.mediaItem.hydrate();
                        hydratePromise.done(function getButtons() {
                            var query = new MS.Entertainment.Data.Query.Music.ArtistTopSongs;
                            query.id = this.mediaItem.canonicalId;
                            if (MS.Entertainment.Utilities.isMusicApp2) {
                                query.isChunked = false;
                                query.chunkSize = 1
                            }
                            query.impressionGuid = this.mediaItem.impressionGuid;
                            query.execute().then(this._setMarketplaceResult.bind(this), function showNotAvailable() {
                                this.actionDescription = String.load(String.id.IDS_DETAILS_TOP_SONGS);
                                this._addButtons({playAll: MS.Entertainment.ViewModels.SmartBuyButtons.createButtonContentNotAvailable(MS.Entertainment.UI.Actions.ExecutionLocation.canvas)})
                            }.bind(this));
                            this.playbackItemSource = this.mediaItem
                        }.bind(this))
                    }, _showPlaylist: function _showPlaylist(param) {
                        this.dispatchEvent("showplaylist", param)
                    }, _setMarketplaceResult: function _setMarketplaceResult(query) {
                        if (query.result.items !== null && query.result.items.count) {
                            this.actionDescription = String.load(String.id.IDS_DETAILS_TOP_SONGS);
                            this._addButtons(MS.Entertainment.ViewModels.SmartBuyButtons.getArtistImmersiveDetailsButtonsDefault(this.mediaItem, false));
                            this._setResult(query)
                        }
                    }, _setResult: function _setResult(query) {
                        var buttons = MS.Entertainment.ViewModels.SmartBuyButtons.getArtistImmersiveDetailsButtonsViewPlaylist(this.mediaItem);
                        if (buttons && buttons.viewPlaylist && buttons.viewPlaylist.invoker) {
                            if (this._eventHandler) {
                                this._eventHandler.cancel();
                                this._eventHandler = null
                            }
                            this._eventHandler = MS.Entertainment.Utilities.addEvents(buttons.viewPlaylist.invoker, {invoked: this._showPlaylist.bind(this)})
                        }
                        this._addButtons(buttons);
                        this.playlist = query.result.items
                    }
            })
    })
})()
