﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.ViewModels");
(function() {
    "use strict";
    var _actionService;
    var _createBasicButton = function _createBasicButton(actionId, titleId, parameter, iconInfo, voiceInfo) {
            if (!_actionService)
                _actionService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions);
            var button = _actionService.getAction(actionId);
            if (titleId)
                button.title = String.load(titleId);
            if (parameter) {
                button.parameter = parameter;
                button.parameter.actionType = MS.Entertainment.UI.Actions.ActionWrapperType.button
            }
            if (iconInfo)
                button.iconInfo = iconInfo;
            if (button.parameter) {
                if (button.parameter.automationId)
                    button.automationId = button.parameter.automationId;
                if (button.parameter.id)
                    button.id = button.parameter.id
            }
            if (voiceInfo && button.isVoiceEnabled) {
                if (voiceInfo.guiStringId)
                    button.title = String.load(voiceInfo.guiStringId);
                button.voicePhraseStringId = voiceInfo.voicePhraseStringId;
                button.voicePhoneticPhraseStringId = voiceInfo.voicePhoneticPhraseStringId;
                if (typeof(voiceInfo.voiceConfidence) === "number")
                    button.voiceConfidence = voiceInfo.voiceConfidence
            }
            return button
        };
    var _createButton = function createButton(actionId, titleId, parameter, iconInfo, voiceInfo) {
            var button = _createBasicButton(actionId, titleId, parameter, iconInfo, voiceInfo);
            if (button.parameter && button.parameter.dismissOverlayOnExecute) {
                var oldExecuted = button.executed.bind(button);
                button.executed = WinJS.Utilities.markSupportedForProcessing(function executed(actionData) {
                    var domEvent = document.createEvent("Event");
                    domEvent.initEvent("dismissoverlay", true, true);
                    var popover = document.querySelector(".overlayAnchor .popOver");
                    if (popover)
                        popover.dispatchEvent(domEvent);
                    oldExecuted(actionData)
                })
            }
            var mediaContext = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.appToolbar).currentMediaContext;
            if (mediaContext.collectionFilter) {
                var previousExecuted = button.executed.bind(button);
                button.executed = WinJS.Utilities.markSupportedForProcessing(function executed(actionData, referenceElement) {
                    actionData.collectionFilter = mediaContext.collectionFilter;
                    previousExecuted(actionData, referenceElement)
                })
            }
            if (button.parameter && button.parameter.preRollVideoAdIfNeeded) {
                var lastExecuted = button.executed.bind(button);
                button.executed = WinJS.Utilities.markSupportedForProcessing(function executed(actionData, referenceElement) {
                    var collectionFilter = mediaContext.collectionFilter;
                    if (referenceElement)
                        MS.Entertainment.Utilities.Telemetry.logPageAction({domElement: referenceElement}, {
                            uri: MS.Entertainment.Utilities.getCurrentUserLocation(), pageTypeId: MS.Entertainment.Utilities.Telemetry.PageTypeId.Dash
                        });
                    if (collectionFilter === Microsoft.Entertainment.Platform.MediaAvailability.availableOffline || !MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.adService))
                        lastExecuted(actionData);
                    else {
                        var containingMedia = mediaContext.options && mediaContext.options.containingMedia;
                        var invokedMedia = (containingMedia && actionData.mediaItem === containingMedia.playbackItemSource) ? mediaContext.mediaItem : actionData.mediaItem;
                        var adService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.adService);
                        adService.isStreamingBlocked(invokedMedia).done(function isStreamingBlocked_complete(signInNeeded) {
                            var promise;
                            var dialogDismissed;
                            promise = new WinJS.Promise(function(c, e, p) {
                                dialogDismissed = c
                            }).then(function dialogDismissed() {
                                var signIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn);
                                var collectionFilter = mediaContext && mediaContext.collectionFilter;
                                return (!signInNeeded || signIn.isSignedIn) ? adService.playVideoAdIfRequired(invokedMedia, collectionFilter) : false
                            }).done(function playVideoAdIfRequired_complete(adPlayedIfNeeded) {
                                if (adPlayedIfNeeded)
                                    lastExecuted(actionData)
                            }, function playVideoAdIfRequired_failed(error) {
                                MS.Entertainment.ViewModels.fail("playVideoAdIfRequired_failed: " + (error && error.message));
                                lastExecuted(actionData)
                            });
                            if (signInNeeded) {
                                var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                                var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
                                if (!featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlayAnonymous))
                                    MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_SIGN_IN_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signUp, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed);
                                else if (configurationManager.service.lastSignedInUserXuid)
                                    MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_EXISTING_USER_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_EXISTING_USER_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signInNowForExistingUser, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed);
                                else
                                    MS.Entertainment.Music.MusicBrandDialog.show(String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_NEW_USER_TITLE), String.load(String.id.IDS_MUSIC_STREAMING_KEEP_PLAYING_NEW_USER_DESC), null, [MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.signInNowForNewUser, MS.Entertainment.Music.MusicBrandDialog.dialogButtonIds.cancel], dialogDismissed)
                            }
                            else
                                dialogDismissed()
                        }, function isStreamingBlocked_failed(error) {
                            MS.Entertainment.ViewModels.fail("isStreamingBlocked_failed: " + (error && error.message))
                        })
                    }
                })
            }
            return button
        };
    var _isFreeUser = function _isFreeUser() {
            var configurationManager = new Microsoft.Entertainment.Configuration.ConfigurationManager;
            var isSubscription = configurationManager.service.lastSignedInUserSubscription;
            return !isSubscription
        };
    WinJS.Namespace.define("MS.Entertainment.ViewModels.Actions", {GenericPlayButton: MS.Entertainment.deferredDerive("MS.Entertainment.UI.Actions.Action", function genericPlayButtonConstructor() {
            this.base();
            this.id = MS.Entertainment.UI.Actions.ActionIdentifiers.genericPlay;
            this.title = String.load(String.id.IDS_PLAY_BUTTON);
            this.automationId = MS.Entertainment.UI.AutomationIds.genericPlay;
            this.parameter = {
                automationId: this.automationId, executeLocation: MS.Entertainment.UI.Actions.ExecutionLocation.canvas
            };
            this.iconInfo = {
                icon: MS.Entertainment.UI.Icon.play, isToggleButton: false, isChecked: false, isDisabled: false
            }
        }, {
            _button: null, _executedSignal: null, _executedParam: null, executed: function executed(param) {
                    if (this._button)
                        return this._button.executed(param);
                    else if (!this._executedSignal) {
                        this._executedSignal = new MS.Entertainment.UI.Framework.Signal;
                        this._executedParam = param
                    }
                    return this._executedSignal.promise
                }, canExecute: function canExecute(param) {
                    return this._button ? this._button.canExecute(param) : true
                }, wrap: function wrap(button) {
                    MS.Entertainment.ViewModels.assert(button, "Button param expected");
                    this._button = button;
                    this.id = button.id;
                    this.title = button.title;
                    this.parameter = button.parameter;
                    this.iconInfo = button.iconInfo;
                    this.automationId = button.automationId;
                    if (button.parameter) {
                        if (button.parameter.automationId)
                            this.automationId = button.parameter.automationId;
                        if (button.parameter.id)
                            this.id = button.parameter.id
                    }
                    if (this._executedSignal) {
                        var executedSignal = this._executedSignal;
                        var executedParam = this._executedParam;
                        this._executedSignal = null;
                        this._executedParam = null;
                        WinJS.Promise.as(this.execute(executedParam)).done(function onExecuteComplete() {
                            executedSignal.complete()
                        }, function onExecuteError(error) {
                            executedSignal.error(error)
                        })
                    }
                }
        })});
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {SmartBuyButtons: {
            createGenericPlayButton: function createGenericPlayButton() {
                return new MS.Entertainment.ViewModels.Actions.GenericPlayButton
            }, createButtonPlayDisabled: function createButtonPlayDisabled(executeLocation) {
                    var playStringId = MS.Entertainment.Utilities.isMusicApp ? String.id.IDS_PLAY_BUTTON : String.id.IDS_DETAILS_PLAY_NOW_LABEL;
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.disabled, playStringId, {
                            automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayDisabled, executeLocation: executeLocation
                        }, {
                            icon: MS.Entertainment.UI.Icon.play, isToggleButton: true, isChecked: false, isDisabled: true
                        })
                }, createButtonDeleteMedia: function createButtonDeleteMedia(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.deleteMedia, String.id.IDS_DELETE_LABEL, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonDelete
                        }, {icon: WinJS.UI.AppBarIcon["delete"]}, {
                            guiStringId: String.id.IDS_DELETE_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_DELETE_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_DELETE_BUTTON_VUI_PRON, voiceConfidence: MS.Entertainment.Utilities.isApp2 ? MS.Entertainment.UI.App2.VoiceStatics.criticalActionConfidence : 0.0
                        })
                }, createButtonTryAgain: function createButtonTryAgain(media, executeLocation, useVerboseDownloadText) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadAgain, useVerboseDownloadText ? String.id.IDS_MUSIC_DETAILS_RETRY_DOWNLOAD_ACTION : String.id.IDS_DETAILS_TRY_AGAIN_LABEL, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonTryAgain
                        }, {icon: WinJS.UI.AppBarIcon.download});
                    return button
                }, createButtonPlayOnXbox: function createButtonPlayOnXbox(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playOnXbox, String.id.IDS_DETAILS_PLAY_ON_XBOX_LABEL, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayOnXbox, dismissOverlayOnExecute: true
                        }, {
                            icon: MS.Entertainment.UI.Icon.sendToXbox, adornerMode: MS.Entertainment.UI.Controls.IconButtonMode.Custom, adornerRing: MS.Entertainment.UI.Icon.sendToXboxAdorner, className: "enablePressedState"
                        })
                }, smartBuyButtonTemplateSelector: WinJS.Utilities.markSupportedForProcessing(function smartBuyButtonTemplateSelector(item) {
                    var result;
                    function loadItemTemplate(itemTemplate) {
                        return MS.Entertainment.UI.Framework.loadTemplate(itemTemplate, null, true).then(function(templateControl) {
                                return templateControl
                            })
                    }
                    if (item.parameter && item.parameter.itemTemplate)
                        return loadItemTemplate(item.parameter.itemTemplate);
                    else
                        return loadItemTemplate(this._itemTemplate)
                }), createButtonDownload: function createButtonDownload(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.download, String.id.IDS_DETAILS_DOWNLOAD_LABEL, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonDownload
                        }, {icon: WinJS.UI.AppBarIcon.download})
                }, _createButtonPinToDashsboardInternal: function _createButtonPinToDashsboardInternal(media, executeLocation, strings) {
                    var pinnedItemsList = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.pinnedList);
                    var hasPin = pinnedItemsList.hasPin(media);
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.pinMediaItem, hasPin ? strings.unpinButtonGuiStringId : strings.pinButtonGuiStringId, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPinToDashboard, strings: strings
                        }, null, {
                            guiStringId: hasPin ? strings.unpinButtonGuiStringId : strings.pinButtonGuiStringId, voicePhraseStringId: hasPin ? strings.unpinButtonVoicePhraseStringId : strings.pinButtonVoicePhraseStringId, voicePhoneticPhraseStringId: hasPin ? strings.unpinButtonVoicePhoneticPhraseStringId : strings.pinButtonVoicePhoneticPhraseStringId, voiceConfidenceStringId: hasPin ? strings.unpinButtonVoiceConfidenceStringId : strings.pinButtonVoiceConfidenceStringId
                        });
                    button.parameter.button = button;
                    button.refresh();
                    return button
                }, createButtonPlayMedia: function createButtonPlayMedia(media, executeLocation, playlist) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia, String.id.IDS_PLAY_BUTTON, {
                            mediaItem: media, playlist: playlist, showAppBar: true, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlay
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_PLAY_MUSIC_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_PLAY_MUSIC_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_PLAY_MUSIC_BUTTON_VUI_PRON
                        })
                }, createButtonPlaylistPlay: function createButtonPlaylistPlay(media, executeLocation) {
                    return this.createButtonPlayMedia(media, executeLocation, media)
                }, createButtonMatchAlbumInfo: function createButtonMatchAlbumInfo(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.findAlbumInfo, String.id.IDS_MATCH_ALBUM_INFO_COMMAND, {
                            mediaItem: media, automationId: MS.Entertainment.UI.AutomationIds.smartButtonFindAlbumInfo, executeLocation: executeLocation
                        }, {icon: WinJS.UI.AppBarIcon.find});
                    return button
                }, createButtonTrackPlay: function createButtonTrackPlay(media, track, index, executeLocation, dontUseTrackListForPlayback) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia, String.id.IDS_PLAY_BUTTON, {
                            mediaItem: media || track, track: track, offset: index, dontUseTrackListForPlayback: dontUseTrackListForPlayback, playlist: track && MS.Entertainment.Utilities.isValidLibraryId(track.playlistId) ? track.playlistId : null, showAppBar: true, preRollVideoAdIfNeeded: true, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayTrack
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_PLAY_MUSIC_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_PLAY_MUSIC_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_PLAY_MUSIC_BUTTON_VUI_PRON
                        })
                }, createButtonPlayNowPlayingTrack: function createButtonPlayNowPlayingTrack(media, track, index, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia, String.id.IDS_PLAY_BUTTON, {
                            mediaItem: media || track, track: track, offset: index, showAppBar: true, preRollVideoAdIfNeeded: false, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayNowPlayingTrack
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_PLAY_MUSIC_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_PLAY_MUSIC_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_PLAY_MUSIC_BUTTON_VUI_PRON
                        })
                }, createButtonRemotePlayDisabled: function createButtonRemotePlayDisabled(executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.disabled, String.id.IDS_MUSIC_PLAY_TO_DISABLED_TEXT, {
                            executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayDisabled
                        }, {
                            icon: MS.Entertainment.UI.Icon.play, isToggleButton: true, isChecked: false, isDisabled: true
                        })
                }, createButtonMore: function createButtonMore(executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.more, String.id.IDS_DETAILS_MORE_ACTION, {
                            executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonOverflowAction
                        }, {icon: WinJS.UI.AppBarIcon.more})
                }, createButtonAddTo: function createButtonAddTo(media, executeLocation, automationId) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.addTo, this._getAddToString(media), {
                            mediaItem: media, executeLocation: executeLocation, automationId: automationId
                        }, {icon: WinJS.UI.AppBarIcon.add}, this._getAddToVoiceStrings(media))
                }, _getAddToString: function _getAddToString(media) {
                    if (MS.Entertainment.Utilities.isMusicApp1)
                        return String.id.IDS_MUSIC_ADD_TO_LIST_BUTTON;
                    switch (media.mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                            return String.id.IDS_MUSIC2_ADD_ALBUM_TO_LIST_BUTTON;
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            return String.id.IDS_MUSIC2_ADD_SONG_TO_LIST_BUTTON;
                        case Microsoft.Entertainment.Queries.ObjectType.playlist:
                            return String.id.IDS_MUSIC2_ADD_PLAYLIST_TO_LIST_BUTTON;
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                            return String.id.IDS_MUSIC2_ADD_ARTIST_TOP_SONGS_TO_LIST_BUTTON;
                        default:
                            return String.id.IDS_MUSIC_ADD_TO_LIST_BUTTON
                    }
                }, _getAddToVoiceStrings: function _getAddToVoiceStrings(media) {
                    var voiceGuiDisplayStringId;
                    var voicePhraseStringId;
                    var voicePhoneticPhraseStringId;
                    if (MS.Entertainment.Utilities.isMusicApp1)
                        return null;
                    switch (media.mediaType) {
                        case Microsoft.Entertainment.Queries.ObjectType.album:
                            voiceGuiDisplayStringId = String.id.IDS_MUSIC2_ADD_ALBUM_TO_LIST_BUTTON_VUI_GUI;
                            voicePhraseStringId = String.id.IDS_MUSIC2_ADD_ALBUM_TO_LIST_BUTTON_VUI_ALM;
                            voicePhoneticPhraseStringId = String.id.IDS_MUSIC2_ADD_ALBUM_TO_LIST_BUTTON_VUI_PRON;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.track:
                            voiceGuiDisplayStringId = String.id.IDS_MUSIC2_ADD_SONG_TO_LIST_BUTTON_VUI_GUI;
                            voicePhraseStringId = String.id.IDS_MUSIC2_ADD_SONG_TO_LIST_BUTTON_VUI_ALM;
                            voicePhoneticPhraseStringId = String.id.IDS_MUSIC2_ADD_SONG_TO_LIST_BUTTON_VUI_PRON;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.playlist:
                            voiceGuiDisplayStringId = String.id.IDS_MUSIC2_ADD_PLAYLIST_TO_LIST_BUTTON_VUI_GUI;
                            voicePhraseStringId = String.id.IDS_MUSIC2_ADD_PLAYLIST_TO_LIST_BUTTON_VUI_ALM;
                            voicePhoneticPhraseStringId = String.id.IDS_MUSIC2_ADD_PLAYLIST_TO_LIST_BUTTON_VUI_PRON;
                            break;
                        case Microsoft.Entertainment.Queries.ObjectType.person:
                            voiceGuiDisplayStringId = String.id.IDS_MUSIC2_ADD_ARTIST_TOP_SONGS_TO_LIST_BUTTON_VUI_GUI;
                            voicePhraseStringId = String.id.IDS_MUSIC2_ADD_ARTIST_TOP_SONGS_TO_LIST_BUTTON_VUI_ALM;
                            voicePhoneticPhraseStringId = String.id.IDS_MUSIC2_ADD_ARTIST_TOP_SONGS_TO_LIST_BUTTON_VUI_PRON;
                            break;
                        default:
                            voiceGuiDisplayStringId = String.id.IDS_MUSIC_ADD_TO_LIST_BUTTON_VUI_GUI;
                            voicePhraseStringId = String.id.IDS_MUSIC_ADD_TO_LIST_BUTTON_VUI_ALM;
                            voicePhoneticPhraseStringId = String.id.IDS_MUSIC_ADD_TO_LIST_BUTTON_VUI_PRON;
                            break
                    }
                    return {
                            guiStringId: voiceGuiDisplayStringId, voicePhraseStringId: voicePhraseStringId, voicePhoneticPhraseStringId: voicePhoneticPhraseStringId
                        }
                }, createButtonRemoveFromNowPlaying: function createButtonRemoveFromNowPlaying(container, key, offset, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.removeFromPlaybackSession, String.id.IDS_NOW_PLAYING_REMOVE_TRACK_ACTION, {
                            mediaItem: container, key: key, offset: offset, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonRemoveFromNowPlaying
                        }, {icon: WinJS.UI.AppBarIcon.remove})
                }, createButtonContentNotAvailable: function createButtonContentNotAvailable(executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.disabled, String.id.IDS_MUSIC_CONTENT_NOT_AVAILABLE_BUTTON, {
                            executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonContentNotAvailable
                        }, {icon: MS.Entertainment.UI.Icon.play})
                }, createButtonPlaylistItemInformation: function createButtonPlaylistItemInformation() {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.showPlaylistItemInformation, String.id.IDS_PLAYLIST_ITEM_BUTTON_NOT_AVAILABLE, {automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlaylistItemInformation}, {icon: MS.Entertainment.UI.Icon.info})
                }, createButtonAddMarketplaceArtistToNowPlaying: function createButtonAddMarketplaceArtistToNowPlaying(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.addArtistToNowPlaying, String.id.IDS_DETAILS_ADD_TO_NOW_PLAYING, {
                            mediaItem: (media && media.mediaType === Microsoft.Entertainment.Queries.ObjectType.track) ? media.artist : media, playCollectionItemsOnly: false, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonAddMarketplaceArtistToNowPlaying, subTitle: media ? media.artistName : String.empty
                        }, {icon: MS.Entertainment.UI.Icon.addToNowPlaying})
                }, createButtonPlayArtistSmartDJ: function createButtonPlayArtistSmartDJ(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playSmartDJ, String.id.IDS_DETAILS_PLAY_SMART_DJ_MIX_NOW_LABEL, {
                            mediaItem: media, showImmersive: false, showAppBar: true, playCollectionItemsOnly: false, executeLocation: executeLocation, preRollVideoAdIfNeeded: false, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayArtistSmartDj, subTitle: media ? media.artistName : String.empty
                        }, {
                            icon: MS.Entertainment.UI.Icon.smartDj, iconPressed: MS.Entertainment.UI.Icon.smartDjPressed, hideDefaultRing: true, className: "smartDj"
                        }, {
                            guiStringId: String.id.IDS_DETAILS_PLAY_RADIO_MIX_NOW_LABEL_VUI_GUI, voicePhraseStringId: String.id.IDS_DETAILS_PLAY_RADIO_MIX_NOW_LABEL_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_DETAILS_PLAY_RADIO_MIX_NOW_LABEL_VUI_PRON
                        });
                    return button
                }, createMusicButtonPinToDashboard: function createMusicButtonPinToDashboard(media, executeLocation) {
                    if (!MS.Entertainment.ServiceLocator.isServiceRegistered(MS.Entertainment.Services.pinnedList))
                        return null;
                    var strings = {
                            pinningTitleStringId: String.id.IDS_MUSIC2_PIN_BUTTON_PINNING, unpinningTitleStringId: String.id.IDS_MUSIC2_PIN_BUTTON_UNPINNING, pinButtonGuiStringId: String.id.IDS_MUSIC2_PIN_BUTTON_PIN_VUI_GUI, pinButtonVoicePhraseStringId: String.id.IDS_MUSIC2_PIN_BUTTON_PIN_VUI_ALM, pinButtonVoicePhoneticPhraseStringId: String.id.IDS_MUSIC2_PIN_BUTTON_PIN_VUI_PRON, pinButtonVoiceConfidenceStringId: String.id.IDS_MUSIC2_PIN_BUTTON_PIN_VUI_CONF, unpinButtonGuiStringId: String.id.IDS_MUSIC2_PIN_BUTTON_UNPIN_VUI_GUI, unpinButtonVoicePhraseStringId: String.id.IDS_MUSIC2_PIN_BUTTON_UNPIN_VUI_ALM, unpinButtonVoicePhoneticPhraseStringId: String.id.IDS_MUSIC2_PIN_BUTTON_UNPIN_VUI_PRON, unpinButtonVoiceConfidenceStringId: String.id.IDS_MUSIC2_PIN_BUTTON_UNPIN_VUI_CONF
                        };
                    return MS.Entertainment.ViewModels.SmartBuyButtons._createButtonPinToDashsboardInternal(media, executeLocation, strings)
                }, createButtonAddToMyMusic: function createButtonAddToMyMusic(media, containerItem, executeLocation, stringId, automationId) {
                    var subTitle;
                    if (containerItem)
                        subTitle = containerItem.name;
                    else
                        subTitle = media ? media.name : String.empty;
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.subscriptionAddToMyMusic, stringId, {
                            mediaItem: media, containerItem: containerItem, executeLocation: executeLocation, automationId: automationId, subTitle: subTitle
                        }, {icon: MS.Entertainment.UI.Icon.addToCollection})
                }, createButtonShuffleAllMusic: function createButtonShuffleAllMusic(executeLocation) {
                    return _createBasicButton(MS.Entertainment.UI.Actions.ActionIdentifiers.shuffleAllMusic, String.id.IDS_MUSIC_ENGAGE_SHUFFLE_MUSIC_ACTION, {
                            engageTitle: String.id.IDS_MUSIC_ENGAGE_SHUFFLE_MUSIC_ACTION, subTitle: String.load(String.id.IDS_MUSIC_ENGAGE_SHUFFLE_MUSIC_DESC), executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonShuffleAll
                        }, {
                            icon: MS.Entertainment.UI.Icon.shuffleOn, className: "icon-shuffle"
                        })
                }, createButtonPlayAnyArtist: function createButtonPlayAnyArtist(executeLocation) {
                    var showFreeStrings = _isFreeUser();
                    return _createBasicButton(MS.Entertainment.UI.Actions.ActionIdentifiers.selectArtist, showFreeStrings ? String.id.IDS_MUSIC_ENGAGE_PLAY_ARTIST_FREE_TITLE : String.id.IDS_MUSIC_ENGAGE_PLAY_ARTIST_ACTION, {
                            engageTitle: showFreeStrings ? String.id.IDS_MUSIC_ENGAGE_PLAY_ARTIST_ACTION_FREE : String.id.IDS_MUSIC_ENGAGE_PLAY_ARTIST_ACTION, subTitle: showFreeStrings ? String.load(String.id.IDS_MUSIC_ENGAGE_PLAY_ARTIST_FREE_DESC) : String.load(String.id.IDS_MUSIC_ENGAGE_PLAY_ARTIST_DESC), executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayAnyArtist, isSmartDJ: false
                        }, {
                            icon: MS.Entertainment.UI.Icon.play, className: "icon-play"
                        })
                }, createButtonNewSmartDJ: function createButtonNewSmartDJ(executeLocation) {
                    var showFreeStrings = _isFreeUser();
                    return _createBasicButton(MS.Entertainment.UI.Actions.ActionIdentifiers.selectArtist, showFreeStrings ? String.id.IDS_MUSIC_ENGAGE_NEW_SMARTDJ_FREE_TITLE : String.id.IDS_MUSIC_ENGAGE_NEW_SMARTDJ_ACTION, {
                            engageTitle: String.id.IDS_MUSIC_ENGAGE_NEW_SMARTDJ_ACTION, subTitle: showFreeStrings ? String.load(String.id.IDS_MUSIC_ENGAGE_NEW_SMARTDJ_FREE_DESC) : String.load(String.id.IDS_MUSIC_ENGAGE_NEW_SMARTDJ_DESC), executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonNewSmartDj, isSmartDJ: true
                        }, {
                            icon: (executeLocation == MS.Entertainment.UI.Actions.ExecutionLocation.opportunity) ? MS.Entertainment.UI.Icon.smartDjNoRing : MS.Entertainment.UI.Icon.smartDj, iconPressed: (executeLocation === MS.Entertainment.UI.Actions.ExecutionLocation.opportunity) ? MS.Entertainment.UI.Icon.smartDjNoRing : MS.Entertainment.UI.Icon.smartDjPressed, hideDefaultRing: true, className: "icon-smartDJ"
                        })
                }, createButtonPlayPlaylist: function createButtonPlayPlaylist(executeLocation) {
                    var showFreeStrings = false;
                    if (_isFreeUser()) {
                        var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                        var freeStreamEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlay);
                        showFreeStrings = freeStreamEnabled
                    }
                    return _createBasicButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playPlaylist, showFreeStrings ? String.id.IDS_MUSIC_ENGAGE_PLAY_PLAYLIST_FREE_TITLE : String.id.IDS_MUSIC_ENGAGE_PLAY_PLAYLIST_ACTION, {
                            engageTitle: String.id.IDS_MUSIC_ENGAGE_PLAY_PLAYLIST_ACTION, subTitle: showFreeStrings ? String.load(String.id.IDS_MUSIC_ENGAGE_PLAY_PLAYLIST_FREE_DESC) : String.load(String.id.IDS_MUSIC_ENGAGE_PLAY_PLAYLIST_DESC), executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayPlaylist
                        }, {
                            icon: MS.Entertainment.UI.Icon.playlist, className: "icon-playlist"
                        })
                }, createButtonPlayAnyArtistApp2: function createButtonPlayAnyArtist(executeLocation) {
                    return _createBasicButton(MS.Entertainment.UI.Actions.ActionIdentifiers.searchArtist, String.id.IDS_MUSIC_ENGAGE_PLAY_ARTIST_ACTION, {
                            subTitle: String.load(String.id.IDS_MUSIC_ENGAGE_PLAY_ARTIST_DESC), executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayAnyArtist, galleryClass: "playActionGallery"
                        }, {
                            icon: MS.Entertainment.UI.Icon.play, className: "icon-play"
                        })
                }, createButtonNewSmartDJApp2: function createButtonNewSmartDJ(executeLocation) {
                    return _createBasicButton(MS.Entertainment.UI.Actions.ActionIdentifiers.searchArtist, String.id.IDS_MUSIC_ENGAGE_NEW_SMARTDJ_ACTION, {
                            subTitle: String.load(String.id.IDS_MUSIC_ENGAGE_NEW_SMARTDJ_DESC), executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonNewSmartDj, hasSmartDJ: true, galleryClass: "smartDjActionGallery"
                        }, {
                            icon: (executeLocation == "opportunity") ? MS.Entertainment.UI.Icon.smartDjNoRing : MS.Entertainment.UI.Icon.smartDj, iconPressed: MS.Entertainment.UI.Icon.smartDjPressed, hideDefaultRing: true, className: "icon-smartDJ"
                        })
                }, createButtonPlayPlaylistApp2: function createButtonPlayPlaylistApp2(executeLocation) {
                    return _createBasicButton(MS.Entertainment.UI.Actions.ActionIdentifiers.selectPlaylist, String.id.IDS_MUSIC_ENGAGE_PLAY_PLAYLIST_ACTION, {
                            subTitle: String.load(String.id.IDS_MUSIC_ENGAGE_PLAY_PLAYLIST_DESC), executeLocation: executeLocation, actionId: MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayPlaylist, galleryClass: "playActionGallery"
                        }, {
                            icon: MS.Entertainment.UI.Icon.playlist, className: "icon-playlist"
                        })
                }, createButtonWelcomeApp2: function createButtonWelcomeApp2() {
                    var button = _createBasicButton(MS.Entertainment.UI.Actions.ActionIdentifiers.disabled, String.id.IDS_MUSIC2_WELCOME_TILE_TITLE_NOTAG, {subTitle: String.load(String.id.IDS_MUSIC2_WELCOME_TILE_PRIMARY_LOADING)}, {className: String.empty});
                    button.canExecute = function canExecute() {
                        return true
                    };
                    button.isEnabled = true;
                    return button
                }, createButtonBrowseAllMusic: function createButtonBrowseAllMusic(executeLocation) {
                    return _createBasicButton(MS.Entertainment.UI.Actions.ActionIdentifiers.navigate, String.id.IDS_COLLECTION_MUSIC_EMPTY_LINK_PREMIUM, {
                            engageTitle: String.id.IDS_COLLECTION_MUSIC_EMPTY_LINK_PREMIUM, page: MS.Entertainment.UI.Monikers.musicMarketplace, hub: MS.Entertainment.UI.Monikers.musicMarketplaceFeatured, subTitle: String.load(String.id.IDS_MUSIC_COLLECTION_EMPTY_DESC), executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBrowseAllMusic
                        }, {
                            icon: WinJS.UI.AppBarIcon.shop, className: "icon-shop"
                        })
                }, createButtonGetFreeMusicPassTrial: function createButtonGetFreeMusicPassTrial(executeLocation) {
                    return _createBasicButton(MS.Entertainment.UI.Actions.ActionIdentifiers.subscriptionWithSignIn, String.id.IDS_MUSIC_ENGAGE_PREMIUM_TRIAL_ACTION, {
                            engageTitle: String.id.IDS_MUSIC_ENGAGE_PREMIUM_TRIAL_ACTION, subTitle: String.load(String.id.IDS_MUSIC_ENGAGE_PREMIUM_TRIAL_DESC), executeLocation: executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.engage, automationId: MS.Entertainment.UI.AutomationIds.smartButtonGetFreeMusicPassTrial
                        }, {
                            icon: WinJS.UI.AppBarIcon.shop, className: "icon-shop"
                        })
                }, createButtonSubscriptionDownload: function createButtonSubscriptionDownload(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.subscriptionDownload, String.id.IDS_DETAILS_DOWNLOAD_LABEL, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonDownload
                        }, {icon: WinJS.UI.AppBarIcon.download})
                }, createButtonDownloadRemove: function createButtonDownloadRemove(media, executeLocation, useVerboseDownloadText) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.downloadRemove, useVerboseDownloadText ? String.id.IDS_MUSIC_DETAILS_CLEAR_DOWNLOAD_ERROR_ACTION : String.id.IDS_CLEAR_DOWNLOAD_ERROR_BUTTON, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonRemove
                        }, {icon: WinJS.UI.AppBarIcon.remove});
                    return button
                }, createButtonArtistDetails: function createButtonArtistDetails(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.showImmersiveDetails, String.id.IDS_DETAILS_VIEW_FULL_ARTIST, {
                            mediaItem: media, showDetails: true, autoPlay: false, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonArtistDetails, dismissOverlayOnExecute: true
                        }, {icon: MS.Entertainment.Utilities.isMusicApp2 ? WinJS.UI.AppBarIcon.contact : MS.Entertainment.UI.Icon.details}, {
                            guiStringId: String.id.IDS_DETAILS_VIEW_FULL_ARTIST_VUI_GUI, voicePhraseStringId: String.id.IDS_DETAILS_VIEW_FULL_ARTIST_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_DETAILS_VIEW_FULL_ARTIST_VUI_PRON
                        })
                }, createButtonArtistDetailsLX: function createButtonArtistDetailsLX(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.artistDetailsNavigate, String.id.IDS_DETAILS_VIEW_FULL_ARTIST, media.artist, {
                            icon: WinJS.UI.AppBarIcon.contact, className: "icon-contact"
                        }, {
                            guiStringId: String.id.IDS_DETAILS_VIEW_FULL_ARTIST_VUI_GUI, voicePhraseStringId: String.id.IDS_DETAILS_VIEW_FULL_ARTIST_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_DETAILS_VIEW_FULL_ARTIST_VUI_PRON
                        })
                }, createButtonAlbumPlayPreview: function createButtonAlbumPlayPreview(media, executeLocation, dontUseTrackListForPlayback) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia, String.id.IDS_DETAILS_PREVIEW_ALBUM_LABEL, {
                            mediaItem: media, showAppBar: true, playPreviewOnly: true, dontUseTrackListForPlayback: dontUseTrackListForPlayback, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayAlbumPreview, id: "playAlbumPreview"
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_DETAILS_PREVIEW_ALBUM_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_DETAILS_PREVIEW_ALBUM_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_DETAILS_PREVIEW_ALBUM_BUTTON_VUI_PRON
                        })
                }, createButtonAlbumPlay: function createButtonAlbumPlay(media, executeLocation, dontUseTrackListForPlayback) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia, String.id.IDS_MUSIC_PLAY_ALBUM_LABEL, {
                            mediaItem: media, showAppBar: true, dontUseTrackListForPlayback: dontUseTrackListForPlayback, preRollVideoAdIfNeeded: true, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayAlbum, id: "playAlbum"
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_MUSIC_PLAY_ALBUM_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_MUSIC_PLAY_ALBUM_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_MUSIC_PLAY_ALBUM_BUTTON_VUI_PRON
                        })
                }, createButtonTrackPlayPreview: function createButtonTrackPlayPreview(media, executeLocation, dontUseTrackListForPlayback) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playMedia, String.id.IDS_DETAILS_PREVIEW_SONG_LABEL, {
                            mediaItem: media, showAppBar: true, dontUseTrackListForPlayback: dontUseTrackListForPlayback, playPreviewOnly: true, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayTrackPreview
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_DETAILS_PREVIEW_SONG_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_DETAILS_PREVIEW_SONG_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_DETAILS_PREVIEW_SONG_BUTTON_VUI_PRON
                        })
                }, createButtonPreviewArtistTopSongs: function createButtonPreviewArtistTopSongs(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playArtist, String.id.IDS_MUSIC_PREVIEW_TOP_SONGS_BUTTON, {
                            mediaItem: media, showAppBar: true, playPreviewOnly: true, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayArtistPreview
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_MUSIC_PREVIEW_TOP_SONGS_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_MUSIC_PREVIEW_TOP_SONGS_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_MUSIC_PREVIEW_TOP_SONGS_BUTTON_VUI_PRON
                        });
                    return button
                }, createButtonPlayAllLocalArtist: function createButtonPlayAllLocalArtist(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playArtist, MS.Entertainment.Utilities.isMusicApp1 ? String.id.IDS_MUSIC_PLAY_ALL_BUTTON : String.id.IDS_MUSIC_PLAY_ALL_BUTTON_VUI_GUI, {
                            mediaItem: media, showImmersive: false, showAppBar: true, playCollectionItemsOnly: true, preRollVideoAdIfNeeded: true, executeLocation: executeLocation, librarySort: Microsoft.Entertainment.Queries.TracksSortBy.albumTitleDiscNumberNumberAscending, automationId: MS.Entertainment.UI.AutomationIds.smartButtonArtistLocalPlayAll
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_MUSIC_PLAY_ALL_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_MUSIC_PLAY_ALL_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_MUSIC_PLAY_ALL_BUTTON_VUI_PRON
                        });
                    return button
                }, createButtonPlayRelatedArtist: (function createButtonPlayRelatedArtist(media, executeLocation) {
                    var buttonPlayRelatedArtists = null;
                    var updateButtonPlayRelatedArtist = function updateButtonPlayRelatedArtist() {
                            if (buttonPlayRelatedArtists && buttonPlayRelatedArtists.items) {
                                var relatedArtists = buttonPlayRelatedArtists.items;
                                var relatedArtistIndex = Math.floor(Math.random() * relatedArtists.length);
                                var relatedArtist = relatedArtists[relatedArtistIndex];
                                buttonPlayRelatedArtists.parameter.mediaItem = relatedArtist;
                                buttonPlayRelatedArtists.parameter.subTitle = relatedArtist.name;
                                buttonPlayRelatedArtists.isEnabled = true
                            }
                        };
                    return function createButtonPlayRelatedArtist(media, executeLocation) {
                            if (!buttonPlayRelatedArtists) {
                                buttonPlayRelatedArtists = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playArtist, String.id.IDS_NOW_PLAYING_OP_TOP_SONGS_RELATED_ACTION, {
                                    showImmersive: false, showAppBar: true, playCollectionItemsOnly: false, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonPlayRelatedArtist
                                }, {icon: MS.Entertainment.UI.Icon.play});
                                buttonPlayRelatedArtists.isEnabled = false
                            }
                            if (media && media.artist && media.artist.hasCanonicalId && (!buttonPlayRelatedArtists.items || media.artist.canonicalId !== buttonPlayRelatedArtists.artistId)) {
                                buttonPlayRelatedArtists.artistId = null;
                                buttonPlayRelatedArtists.items = null;
                                var query = new MS.Entertainment.Data.Query.Music.RelatedArtists;
                                query.id = media.artist.canonicalId;
                                query.impressionGuid = media.impressionGuid;
                                query.execute().done(function querySuccess(response) {
                                    if (response && response.result.itemsArray) {
                                        buttonPlayRelatedArtists.artistId = media.artist.canonicalId;
                                        buttonPlayRelatedArtists.items = response.result.itemsArray;
                                        updateButtonPlayRelatedArtist()
                                    }
                                }.bind(this), function onError(error) {
                                    MS.Entertainment.ViewModels.assert(error.message, "Failed to get related artists for the play top songs button. Error message: " + error.message)
                                })
                            }
                            else
                                updateButtonPlayRelatedArtist();
                            return buttonPlayRelatedArtists
                        }
                })(), createButtonPlayAllMarketplaceArtist: function createButtonPlayAllMarketplaceArtist(media, executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playArtist, String.id.IDS_MUSIC_PLAY_TOP_SONGS_BUTTON, {
                            mediaItem: (media && media.mediaType === Microsoft.Entertainment.Queries.ObjectType.track) ? media.artist : media, showImmersive: false, showAppBar: true, playCollectionItemsOnly: false, preRollVideoAdIfNeeded: true, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonArtistMarketplacePlayAll, subTitle: media ? media.artistName : String.empty
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: String.id.IDS_MUSIC_PLAY_TOP_SONGS_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_MUSIC_PLAY_TOP_SONGS_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_MUSIC_PLAY_TOP_SONGS_BUTTON_VUI_PRON
                        });
                    return button
                }, createButtonSaveNowPlayingPlaylist: function createButtonSaveNowPlayingPlaylist(executeLocation) {
                    var button = _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.saveNowPlayingPlaylist, String.id.IDS_HOME_NOW_PLAYING, {
                            executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonSaveNowPlayingPlaylist, subTitle: String.load(String.id.IDS_NOW_PLAYING_OP_SAVE_NOW_PLAYING_ACTION)
                        }, {icon: WinJS.UI.AppBarIcon.save});
                    return button
                }, createButtonBuyAlbum: function createButtonBuyAlbum(media, executeLocation) {
                    return _createButton(MS.Entertainment.Utilities.isMusicApp1 ? MS.Entertainment.UI.Actions.ActionIdentifiers.albumPurchase : MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_DETAILS_BUY_ALBUM_LABEL, {
                            mediaItem: media, offer: null, target: MS.Entertainment.Platform.PurchaseHelpers.PC_TARGET, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBuyAlbum
                        }, {icon: MS.Entertainment.UI.Icon.shop})
                }, createButtonBuyTrack: function createButtonBuyTrack(media, executeLocation) {
                    return _createButton(MS.Entertainment.Utilities.isMusicApp1 ? MS.Entertainment.UI.Actions.ActionIdentifiers.trackPurchase : MS.Entertainment.UI.Actions.ActionIdentifiers.purchase, String.id.IDS_DETAILS_BUY_SONG_LABEL, {
                            mediaItem: media, offer: null, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonBuyTrack
                        }, {icon: MS.Entertainment.UI.Icon.shop})
                }, createButtonDisabledBuyTrack: function createButtonBuyTrack(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.disabled, String.id.IDS_DETAILS_BUY_SONG_LABEL, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonDisabledBuyTrack
                        }, {icon: MS.Entertainment.UI.Icon.shop})
                }, createImmersiveArtistPlayAllButton: function createImmersiveArtistPlayAllButton(media, playCollectionItemsOnly, previewOnly, showImmersive, overridePageChange) {
                    var guiStringId;
                    var voicePhraseStringId;
                    var voicePhoneticPhraseStringId;
                    var buttonTextStringId;
                    var automationId;
                    if (playCollectionItemsOnly) {
                        buttonTextStringId = String.id.IDS_MUSIC_PLAY_ALL_BUTTON;
                        automationId = MS.Entertainment.UI.AutomationIds.smartButtonArtistLocalPlayAll;
                        guiStringId = String.id.IDS_MUSIC_PLAY_ALL_BUTTON_VUI_GUI;
                        voicePhraseStringId = String.id.IDS_MUSIC_PLAY_ALL_BUTTON_VUI_ALM;
                        voicePhoneticPhraseStringId = String.id.IDS_MUSIC_PLAY_ALL_BUTTON_VUI_PRON
                    }
                    else if (previewOnly) {
                        buttonTextStringId = String.id.IDS_MUSIC_PREVIEW_TOP_SONGS_BUTTON;
                        automationId = MS.Entertainment.UI.AutomationIds.smartButtonPlayArtistPreview;
                        guiStringId = String.id.IDS_MUSIC_PREVIEW_TOP_SONGS_BUTTON_VUI_GUI;
                        voicePhraseStringId = String.id.IDS_MUSIC_PREVIEW_TOP_SONGS_BUTTON_VUI_ALM;
                        voicePhoneticPhraseStringId = String.id.IDS_MUSIC_PREVIEW_TOP_SONGS_BUTTON_VUI_PRON
                    }
                    else {
                        buttonTextStringId = String.id.IDS_MUSIC_PLAY_TOP_SONGS_BUTTON;
                        automationId = MS.Entertainment.UI.AutomationIds.smartButtonArtistMarketplacePlayAll;
                        guiStringId = String.id.IDS_MUSIC_PLAY_TOP_SONGS_BUTTON_VUI_GUI;
                        voicePhraseStringId = String.id.IDS_MUSIC_PLAY_TOP_SONGS_BUTTON_VUI_ALM;
                        voicePhoneticPhraseStringId = String.id.IDS_MUSIC_PLAY_TOP_SONGS_BUTTON_VUI_PRON
                    }
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.playArtist, buttonTextStringId, {
                            mediaItem: media, showAppBar: true, showImmersive: showImmersive, showDetails: true, playPreviewOnly: !!previewOnly, overridePageChange: overridePageChange, playCollectionItemsOnly: playCollectionItemsOnly, executeLocation: MS.Entertainment.UI.Actions.ExecutionLocation.canvas, preRollVideoAdIfNeeded: true, automationId: automationId
                        }, {icon: MS.Entertainment.UI.Icon.play}, {
                            guiStringId: guiStringId, voicePhraseStringId: voicePhraseStringId, voicePhoneticPhraseStringId: voicePhoneticPhraseStringId
                        })
                }, createImmersiveArtistAddToButton: function createImmersiveArtistAdToButtion(media, playCollectionItemsOnly, executeLocation, automationId) {
                    var buttonTextStringId = String.id.IDS_MUSIC_ADD_TO_LIST_BUTTON;
                    var guiStringId;
                    var voicePhraseStringId;
                    var voicePhoneticPhraseStringId;
                    if (MS.Entertainment.Utilities.isMusicApp2)
                        if (playCollectionItemsOnly) {
                            buttonTextStringId = String.id.IDS_MUSIC2_ADD_ARTIST_COLLECTION_TO_LIST_BUTTON;
                            guiStringId = String.id.IDS_MUSIC2_ADD_ARTIST_COLLECTION_TO_LIST_BUTTON_VUI_GUI;
                            voicePhraseStringId = String.id.IDS_MUSIC2_ADD_ARTIST_COLLECTION_TO_LIST_BUTTON_VUI_ALM;
                            voicePhoneticPhraseStringId = String.id.IDS_MUSIC2_ADD_ARTIST_COLLECTION_TO_LIST_BUTTON_VUI_PRON
                        }
                        else {
                            buttonTextStringId = String.id.IDS_MUSIC2_ADD_ARTIST_TOP_SONGS_TO_LIST_BUTTON;
                            guiStringId = String.id.IDS_MUSIC2_ADD_ARTIST_TOP_SONGS_TO_LIST_BUTTON_VUI_GUI;
                            voicePhraseStringId = String.id.IDS_MUSIC2_ADD_ARTIST_TOP_SONGS_TO_LIST_BUTTON_VUI_ALM;
                            voicePhoneticPhraseStringId = String.id.IDS_MUSIC2_ADD_ARTIST_TOP_SONGS_TO_LIST_BUTTON_VUI_PRON
                        }
                    var button = this.createButtonAddTo(media, executeLocation, automationId);
                    if (button.isVoiceEnabled) {
                        button.title = String.load(guiStringId);
                        button.voicePhraseStringId = voicePhraseStringId;
                        button.voicePhoneticPhraseStringId = voicePhoneticPhraseStringId
                    }
                    else
                        button.title = String.load(buttonTextStringId);
                    return button
                }, createShowPlaylistButton: function createShowPlaylistButton(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.invoke, String.id.IDS_MUSIC_VIEW_PLAYLIST, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonShowPlaylist
                        }, {icon: MS.Entertainment.UI.Icon.playlist})
                }, createButtonRemoveFromList: function createButtonRemoveFromList() {
                    var actionAvailable = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.actions).isRegistered(MS.Entertainment.UI.Actions.ActionIdentifiers.removeFromList);
                    var action = null;
                    if (actionAvailable)
                        action = _createBasicButton(MS.Entertainment.UI.Actions.ActionIdentifiers.removeFromList);
                    return action
                }, createButtonRemoveFromPlaylist: function createRemoveFromPlaylistButton(media, index, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.removeFromPlaylist, String.id.IDS_PLAYLIST_REMOVE, {
                            mediaItem: media, index: index, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonRemoveFromPlaylist
                        }, {icon: WinJS.UI.AppBarIcon.remove}, {
                            guiStringId: String.id.IDS_PLAYLIST_REMOVE_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_PLAYLIST_REMOVE_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_PLAYLIST_REMOVE_BUTTON_VUI_PRON
                        })
                }, createButtonRenamePlaylist: function createButtonRenamePlaylist(media, executeLocation) {
                    return _createButton(MS.Entertainment.UI.Actions.ActionIdentifiers.renamePlaylist, String.id.IDS_MUSIC_RENAME_PLAYLIST, {
                            mediaItem: media, executeLocation: executeLocation, automationId: MS.Entertainment.UI.AutomationIds.smartButtonRenamePlaylist
                        }, {icon: WinJS.UI.AppBarIcon.edit}, {
                            guiStringId: String.id.IDS_MUSIC_RENAME_PLAYLIST_BUTTON_VUI_GUI, voicePhraseStringId: String.id.IDS_MUSIC_RENAME_PLAYLIST_BUTTON_VUI_ALM, voicePhoneticPhraseStringId: String.id.IDS_MUSIC_RENAME_PLAYLIST_BUTTON_VUI_PRON
                        })
                }, getAlbumInlineDetailsButtons: function getAlbumInlineDetailsButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    return {
                            play: this.createButtonAlbumPlay(media, location), playArtistSmartDJ: this.createButtonPlayArtistSmartDJ(media, location), playOnXbox: this.createButtonPlayOnXbox(media, location), addTo: this.createButtonAddTo(media, location, MS.Entertainment.UI.AutomationIds.smartButtonAddToFromPopover), albumPlayPreview: this.createButtonAlbumPlayPreview(media, location), buyAlbum: this.createButtonBuyAlbum(media, location), artistDetails: this.createButtonArtistDetails(media, location), download: this.createButtonDownload(media, location), playDisabled: this.createButtonPlayDisabled(location), remotePlayDisabled: this.createButtonRemotePlayDisabled(location), tryAgain: this.createButtonTryAgain(media, location), remove: this.createButtonDownloadRemove(media, location), contentNotAvailable: this.createButtonContentNotAvailable(media, location), deleteMedia: this.createButtonDeleteMedia(media, location), pinToDashboard: this.createMusicButtonPinToDashboard(media, location)
                        }
                }, getAlbumDetailsButtonsLX: function getAlbumDetailsButtonsLX(media, executeLocation, genericPlayButton) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    var buttons = {
                            genericPlay: genericPlayButton, artistDetails: this.createButtonArtistDetailsLX(media, location), play: this.createButtonAlbumPlay(media, location, true), playArtistSmartDJ: this.createButtonPlayArtistSmartDJ(media, location), playOnXbox: this.createButtonPlayOnXbox(media, location), addTo: this.createButtonAddTo(media, location, MS.Entertainment.UI.AutomationIds.smartButtonAddToFromPopover), albumPlayPreview: this.createButtonAlbumPlayPreview(media, location, true), buyAlbum: this.createButtonBuyAlbum(media, location), playDisabled: this.createButtonPlayDisabled(location), remotePlayDisabled: this.createButtonRemotePlayDisabled(location), download: this.createButtonDownload(media, location), subscriptionDownload: this.createButtonSubscriptionDownload(media, location), tryAgain: this.createButtonTryAgain(media, location, true), remove: this.createButtonDownloadRemove(media, location, true), contentNotAvailable: this.createButtonContentNotAvailable(media, location), deleteMedia: this.createButtonDeleteMedia(media, location), pinToDashboard: this.createMusicButtonPinToDashboard(media, location), findAlbumInfo: this.createButtonMatchAlbumInfo(media, location), more: this.createButtonMore(location)
                        };
                    return buttons
                }, getArtistInlineDetailsButtons: function getArtistInlineDetailsButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    return {
                            playOnXbox: this.createButtonPlayOnXbox(media, location), addTo: this.createButtonAddTo(media, location, MS.Entertainment.UI.AutomationIds.smartButtonAddToFromPopover), previewArtist: this.createButtonPreviewArtistTopSongs(media, location), playAllLocalArtist: this.createButtonPlayAllLocalArtist(media, location), playAllMarketplaceArtist: this.createButtonPlayAllMarketplaceArtist(media, location), playArtistSmartDJ: this.createButtonPlayArtistSmartDJ(media, location), artistDetails: this.createButtonArtistDetails(media, location), playDisabled: this.createButtonPlayDisabled(location), remotePlayDisabled: this.createButtonRemotePlayDisabled(location), download: this.createButtonDownload(media, location), tryAgain: this.createButtonTryAgain(media, location), remove: this.createButtonDownloadRemove(media, location), contentNotAvailable: this.createButtonContentNotAvailable(media, location), deleteMedia: this.createButtonDeleteMedia(media, location), pinToDashboard: this.createMusicButtonPinToDashboard(media, location)
                        }
                }, getArtistDetailsHeaderButtons: function getArtistDetailsHeaderButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    return {playArtistSmartDJ: this.createButtonPlayArtistSmartDJ(media, location)}
                }, getArtistDetailsLocalAlbumsButtons: function getArtistDetailsLocalAlbumsButtons(localAlbumsQuery, localTracksQuery, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    return {
                            playLocalAlbums: this.createButtonPlayMedia(localTracksQuery, location), addLocalAlbumsTo: this.createButtonAddTo(localAlbumsQuery, location, MS.Entertainment.UI.AutomationIds.smartButtonAddToFromPopover)
                        }
                }, getPlaylistInlineDetailsButtons: function getPlaylistInlineDetailsButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    return {
                            play: this.createButtonPlaylistPlay(media, location), addTo: this.createButtonAddTo(media, location, MS.Entertainment.UI.AutomationIds.smartButtonAddToFromPopover), playDisabled: this.createButtonPlayDisabled(location), contentNotAvailable: this.createButtonContentNotAvailable(media, location), deleteMedia: this.createButtonDeleteMedia(media, location), renamePlaylist: this.createButtonRenamePlaylist(media, location), pinToDashboard: this.createMusicButtonPinToDashboard(media, location)
                        }
                }, getPlaylistDetailsButtonsLX: function getPlaylistDetailsButtonsLX(media, executeLocation, genericPlayButton) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.popover;
                    var buttons = this.getPlaylistInlineDetailsButtons(media, executeLocation);
                    buttons.genericPlay = genericPlayButton;
                    buttons.more = this.createButtonMore(location);
                    buttons.download = this.createButtonDownload(media, location);
                    buttons.subscriptionDownload = this.createButtonSubscriptionDownload(media, location);
                    buttons.tryAgain = this.createButtonTryAgain(media, location, true);
                    buttons.remove = this.createButtonDownloadRemove(media, location, true);
                    return buttons
                }, getTrackInlineDetailsButtons: function getTrackInlineDetailsButtons(mediaContext) {
                    var track = mediaContext.mediaItem;
                    var location = MS.Entertainment.UI.Actions.ExecutionLocation.invokeInline;
                    var playbackItemSource = null;
                    var trackIndex = null;
                    var trackKey = null;
                    var dontUseTrackListForPlayback = false;
                    if (mediaContext.options) {
                        if (mediaContext.options.containingMedia) {
                            playbackItemSource = mediaContext.options.containingMedia.playbackItemSource;
                            trackIndex = mediaContext.options.containingMedia.playbackOffset;
                            trackKey = mediaContext.options.containingMedia.playbackKey;
                            dontUseTrackListForPlayback = !!mediaContext.options.containingMedia.dontUseTrackListForPlayback
                        }
                        if (mediaContext.options.executeLocation)
                            location = mediaContext.options.executeLocation
                    }
                    var actions = {
                            play: this.createButtonTrackPlay(playbackItemSource, track, trackIndex, location, dontUseTrackListForPlayback), playOnXbox: this.createButtonPlayOnXbox(track, location), addTo: this.createButtonAddTo(track, location, MS.Entertainment.UI.AutomationIds.smartButtonAddToFromTrackInline), trackPlayPreview: this.createButtonTrackPlayPreview(track, location, dontUseTrackListForPlayback), buyTrack: this.createButtonBuyTrack(track, location), buyAlbum: this.createButtonBuyAlbum(track, location), download: this.createButtonDownload(track, location), playDisabled: this.createButtonPlayDisabled(location), remotePlayDisabled: this.createButtonRemotePlayDisabled(location), tryAgain: this.createButtonTryAgain(track, location), remove: this.createButtonDownloadRemove(track, location), contentNotAvailable: this.createButtonContentNotAvailable(track, location), removeFromPlaylist: this.createButtonRemoveFromPlaylist(track, trackIndex, location), removeFromNowPlaying: this.createButtonRemoveFromNowPlaying(playbackItemSource, trackKey, trackIndex, location), showPlaylistItemInformation: this.createButtonPlaylistItemInformation(), deleteMedia: this.createButtonDeleteMedia(mediaContext.mediaItem, location), playNowPlayingTrack: this.createButtonPlayNowPlayingTrack(playbackItemSource, track, trackIndex, location)
                        };
                    var removeFromList = this.createButtonRemoveFromList();
                    if (removeFromList)
                        actions.removeFromList = removeFromList;
                    return actions
                }, getTrackOpportunityButtons: function getTrackOpportunityButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.opportunity;
                    var opportunityButtons = this.getEngagePanelButtons(location);
                    opportunityButtons.playArtistSmartDJ = this.createButtonPlayArtistSmartDJ(media, location);
                    opportunityButtons.addMarketplaceArtistToNowPlaying = this.createButtonAddMarketplaceArtistToNowPlaying(media, location);
                    opportunityButtons.playOnXbox = this.createButtonPlayOnXbox(media, location);
                    opportunityButtons.addToMyMusic = this.createButtonAddToMyMusic(media, null, location, String.id.IDS_NOW_PLAYING_OP_ADD_SONG_ACTION, MS.Entertainment.UI.AutomationIds.smartButtonAddToMyMusicTrackOp);
                    opportunityButtons.addAlbumToMyMusic = this.createButtonAddToMyMusic(media, media ? media.album : null, location, String.id.IDS_NOW_PLAYING_OP_ADD_ALBUM_ACTION, MS.Entertainment.UI.AutomationIds.smartButtonAddToMyMusicAlbumOp);
                    opportunityButtons.playAllMarketplaceArtist = this.createButtonPlayAllMarketplaceArtist(media, location);
                    opportunityButtons.saveNowPlayingPlaylist = this.createButtonSaveNowPlayingPlaylist(location);
                    opportunityButtons.playRelatedArtist = this.createButtonPlayRelatedArtist(media, location);
                    return opportunityButtons
                }, getEngagePanelButtons: function getEngagePanelButtons(executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.opportunity;
                    var buttons = {
                            browseAllMusic: this.createButtonBrowseAllMusic(location), shuffleAllMusic: this.createButtonShuffleAllMusic(location)
                        };
                    if (MS.Entertainment.Utilities.isMusicApp1) {
                        buttons.getFreeMusicPassTrial = this.createButtonGetFreeMusicPassTrial(location);
                        buttons.playAnyArtist = this.createButtonPlayAnyArtist(location);
                        buttons.newSmartDJ = this.createButtonNewSmartDJ(location);
                        buttons.playPlaylist = this.createButtonPlayPlaylist(location)
                    }
                    else
                        buttons.welcomeText = this.createButtonWelcomeApp2();
                    return buttons
                }, getArtistImmersiveButtons: function getArtistImmersiveButtons(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.canvas;
                    return {
                            playArtistSmartDJ: this.createButtonPlayArtistSmartDJ(media, location), pinToDashboard: this.createMusicButtonPinToDashboard(media, location)
                        }
                }, getArtistImmersiveDetailsButtonsDefault: function getArtistImmersiveDetailsButtonsInCollection(media, playCollectionItemsOnly, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.canvas;
                    var featureEnablement = new Microsoft.Entertainment.FeatureEnablement.FeatureEnablement;
                    var freeStreamEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlay);
                    var anonFreeStreamEnabled = featureEnablement.isEnabled(Microsoft.Entertainment.FeatureEnablement.FeatureItem.musicFreePlayAnonymous);
                    var isSignedIn = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signIn).isSignedIn;
                    var isSubscription = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.signedInUser).isSubscription;
                    var hasFreeStreamsLeft = Microsoft.Entertainment.Limits.MusicLimitsManager.totalUnauthenticatedTracksUsed < Microsoft.Entertainment.Limits.MusicLimitsManager.totalUnauthenticatedTrackAllowance;
                    var previewOnly = isSignedIn ? !isSubscription && !freeStreamEnabled : !anonFreeStreamEnabled;
                    var addToSubActions = [];
                    var buttons = {playAll: this.createImmersiveArtistPlayAllButton(media, playCollectionItemsOnly, previewOnly, true, false, location)};
                    var sessionManager = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.sessionManager);
                    if (MS.Entertainment.Utilities.isMusicApp2) {
                        if (media && media.hasSmartDJ)
                            buttons.playArtistSmartDJ = this.createButtonPlayArtistSmartDJ(media, location);
                        addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addToPlaylist);
                        if (!sessionManager || !sessionManager.primarySession || !sessionManager.primarySession.smartDJSeed)
                            addToSubActions.push(MS.Entertainment.UI.Actions.AddTo.subMenuIds.addMarketplaceArtistToNowPlaying);
                        if (addToSubActions.length && (hasFreeStreamsLeft || isSubscription)) {
                            var addToButton = this.createImmersiveArtistAddToButton(media, playCollectionItemsOnly, location, MS.Entertainment.UI.AutomationIds.smartButtonAddToFromArtistImmersive);
                            addToButton.setSubActions(addToSubActions);
                            buttons.addTo = addToButton
                        }
                        buttons.pinToDashboard = this.createMusicButtonPinToDashboard(media, location)
                    }
                    return buttons
                }, getArtistImmersiveDetailsButtonsViewPlaylist: function getArtistImmersiveDetailsButtonsViewPlaylist(media, executeLocation) {
                    var location = executeLocation || MS.Entertainment.UI.Actions.ExecutionLocation.canvas;
                    var result;
                    if (MS.Entertainment.Utilities.isMusicApp1)
                        result = {viewPlaylist: this.createShowPlaylistButton(media, location)};
                    return result
                }
        }});
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {SmartBuyStateHandlers: {
            _canXboxAccessMedia: function _canXboxAccessMedia(media, rights) {
                if (!MS.Entertainment.Platform.PlaybackHelpers.isPlayToXboxFeatureEnabled(media))
                    return false;
                else if (!media || !media.hasServiceId)
                    return false;
                else if ((!media.rights || media.rights.length === 0) && media.hasCanonicalId)
                    return true;
                return MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(media, MS.Entertainment.ViewModels.SmartBuyStateHandlers.Tuner.Xbox360, rights)
            }, _mediaHasAnyRight: function _mediaHasAnyRight(media, tuner, rights, freeOnly, excludeFree) {
                    var possibleRights = MS.Entertainment.ViewModels.SmartBuyStateHandlers._getMatchingRights(media, tuner, rights, freeOnly, excludeFree);
                    return (possibleRights && possibleRights.length > 0)
                }, mediaHasAnyRight: function mediaHasAnyRight(media, tuner, rights, freeOnly, excludeFree) {
                    return MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(media, tuner, rights, freeOnly, excludeFree)
                }, _getMatchingRights: function _getMatchingRights(media, tuner, rights, freeOnly, excludeFree) {
                    if (!media || !media.rights)
                        return null;
                    var possibleRights = [];
                    var nowDate = Date.now();
                    if (!MS.Entertainment.Utilities.clientSupportsFreeJustPlays)
                        excludeFree = false;
                    for (var i = 0; i < media.rights.length; i++) {
                        var right = media.rights[i];
                        if (!right)
                            continue;
                        if (!MS.Entertainment.Utilities.rightSupportsTuner(right, tuner))
                            continue;
                        if (!rights || rights.length === 0) {
                            possibleRights.push(right);
                            continue
                        }
                        var foundRight = false;
                        for (var j = 0; j < rights.length; j++)
                            if (right.licenseRight === rights[j] && (MS.Entertainment.ViewModels.SmartBuyStateHandlers._isValidOffer(right, media.mediaType) || right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Preview || right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.PreviewStream || right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.FreeStream || right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Subscription || right.licenseRight === MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Stream)) {
                                foundRight = true;
                                break
                            }
                        if (!foundRight)
                            continue;
                        if (right.hasOffer && right.startDate && right.endDate)
                            try {
                                var startDate = Date.parse(right.startDate);
                                if (isNaN(startDate))
                                    startDate = parseInt(right.startDate.split("(")[1].split(")")[0]);
                                var endDate = Date.parse(right.endDate);
                                if (isNaN(endDate))
                                    endDate = parseInt(right.endDate.split("(")[1].split(")")[0]);
                                if (startDate > nowDate || nowDate > endDate)
                                    continue
                            }
                            catch(e) {
                                MS.Entertainment.Data.fail("Invalid start or end date specified on media, service id = " + media.serviceId, null, MS.Entertainment.UI.Debug.errorLevel.low)
                            }
                        if (freeOnly && (!right.hasOffer || !MS.Entertainment.Utilities.isRightFree(right) || right.priceCurrencyCode === "MTK"))
                            continue;
                        if (excludeFree && (!right.hasOffer || MS.Entertainment.Utilities.isRightFree(right)))
                            continue;
                        if (MS.Entertainment.ViewModels.SmartBuyStateHandlers._filterRight(right, media))
                            continue;
                        possibleRights.push(right)
                    }
                    return possibleRights
                }, _filterRight: function _filterRight(right, media) {
                    return (MS.Entertainment.Utilities.isVideoApp2 && right.deliveryFormat === MS.Entertainment.Data.Augmenter.Marketplace.edsDeliveryFormat.download && media.mediaType !== Microsoft.Entertainment.Queries.ObjectType.tvSeason) || (MS.Entertainment.Utilities.isVideoApp1 && MS.Entertainment.Utilities.isModernMPEnabled && !right.signedOffer) || right.videoEncoding === MS.Entertainment.Data.Augmenter.Marketplace.edsVideoEncoding.h264
                }, _isValidOffer: function _isValidOffer(right, mediaType) {
                    var isValidOffer = right && right.offerId;
                    if (isValidOffer && (mediaType === Microsoft.Entertainment.Queries.ObjectType.track || mediaType === Microsoft.Entertainment.Queries.ObjectType.album)) {
                        var paymentTypes = right.paymentInstruments;
                        if (paymentTypes)
                            paymentTypes = Array.isArray(paymentTypes) ? paymentTypes : [paymentTypes];
                        isValidOffer = paymentTypes && paymentTypes.length && paymentTypes.some(function doesOfferSupportPurchaseType(paymentType) {
                            return (paymentType === MS.Entertainment.Data.Augmenter.Marketplace.edsPaymentType.creditCard)
                        })
                    }
                    return isValidOffer
                }, Tuner: {
                    All: MS.Entertainment.Data.Augmenter.Marketplace.edsDeviceType.all, Xbox360: MS.Entertainment.Data.Augmenter.Marketplace.edsDeviceType.xbox360, Windows: MS.Entertainment.Data.Augmenter.Marketplace.edsDeviceType.pc, Zune30: MS.Entertainment.Data.Augmenter.Marketplace.edsDeviceType.zuneDevice, ZuneMobile: MS.Entertainment.Data.Augmenter.Marketplace.edsDeviceType.zuneMobile
                }, MarketplaceRight: {
                    Rent: "Rent", RentStream: "RentStream", Purchase: "Purchase", PurchaseStream: "PurchaseStream", AlbumPurchase: "AlbumPurchase", Preview: "Preview", PreviewStream: "PreviewStream", Download: "Download", FreeStream: "FreeStream", SeasonPurchase: "SeasonPurchase", SeasonPurchaseStream: "SeasonPurchaseStream", Stream: "Stream", Subscription: "Subscription", SubscriptionFree: "SubscriptionFree", TransferToPortableDevice: "TransferToPortableDevice", Trial: "Trial"
                }, PaymentTypes: {
                    creditCard: "Credit Card", points: "Microsoft Points", mobileOperator: "Mobile Operator"
                }
        }})
})()
