﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
(function() {
    "use strict";
    MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.ViewModels");
    WinJS.Namespace.define("MS.Entertainment.ViewModels", {MusicSmartBuyStateHandlers: {
            isLocalTrackPurchasable: function isLocalTrackPurchasable(track) {
                var provider;
                if (!track || !track.hasServiceId)
                    return WinJS.Promise.wrap(false);
                provider = (new Microsoft.Entertainment.Platform.MediaStore).trackProvider;
                if (track.inCollection && track.localFilesCount > 0)
                    return provider.getPreferredFileUrlAsync(track.libraryId).then(function getPreferredFileUrlAsync_complete(preferredFile) {
                            var canPurchase = (!preferredFile || preferredFile.type !== Microsoft.Entertainment.Platform.FileUrlType.localFileUrl || preferredFile.protection !== Microsoft.Entertainment.Platform.ProtectionState.unprotected);
                            return WinJS.Promise.wrap(canPurchase)
                        }, function getPreferredFileUrlAsync_error() {
                            return WinJS.Promise.wrap(false)
                        });
                else
                    return WinJS.Promise.wrap(true)
            }, getMusicRights: function getMusicRights(media) {
                    var rightsPromise;
                    if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.album)
                        rightsPromise = MS.Entertainment.ViewModels.MusicSmartBuyStateHandlers.getAlbumRights(media);
                    else if (media.mediaType === Microsoft.Entertainment.Queries.ObjectType.track)
                        rightsPromise = MS.Entertainment.ViewModels.MusicSmartBuyStateHandlers.getTrackRights(media);
                    return WinJS.Promise.as(rightsPromise || {})
                }, getAlbumRights: function getAlbumRights(media) {
                    var rights = {
                            albumPurchase: false, freeStream: false, playOnXbox: false, preview: false, subscriptionDownload: false, subscriptionStream: false, trackPurchase: false
                        };
                    rights.albumPurchase = !media.isPremium && MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(media, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.AlbumPurchase]);
                    if (media.tracks)
                        return MS.Entertainment.ViewModels.MusicSmartBuyStateHandlers.getRightsFromTrackList(media.tracks, rights, media);
                    else
                        return WinJS.Promise.wrap(rights)
                }, getRightsFromTrackList: function getRightsFromTrackList(mediaList, previousRights, albumMedia) {
                    var rights = previousRights || {
                            albumPurchase: false, freeStream: false, playOnXbox: false, preview: false, subscriptionDownload: false, subscriptionStream: false, trackPurchase: false
                        };
                    return mediaList.itemsFromIndex(0, 0, mediaList.count).then(function getRightsFromTrackList_itemsFromIndex_complete(args) {
                            for (var i = 0; i < args.items.length; i++) {
                                var item = args.items[i].data;
                                var trackRights = MS.Entertainment.ViewModels.MusicSmartBuyStateHandlers.getTrackRights(item);
                                rights.albumPurchase = rights.albumPurchase || trackRights.albumPurchase;
                                rights.trackPurchase = rights.trackPurchase || trackRights.trackPurchase;
                                rights.subscriptionDownload = rights.subscriptionDownload || trackRights.subscriptionDownload;
                                rights.subscriptionStream = rights.subscriptionStream || trackRights.subscriptionStream;
                                rights.playOnXbox = rights.playOnXbox || trackRights.playOnXbox;
                                rights.freeStream = rights.freeStream || trackRights.freeStream;
                                rights.preview = rights.preview || trackRights.preview;
                                if (rights.albumPurchase && rights.trackPurchase && rights.subscriptionDownload && rights.subscriptionStream && rights.freeStream && rights.playOnXbox && rights.preview)
                                    break
                            }
                            rights.albumPurchase = rights.albumPurchase && (!albumMedia || !albumMedia.isPremium);
                            return rights
                        }.bind(this))
                }, getTrackRights: function getTrackRights(item) {
                    var rights = {
                            albumPurchase: false, freeStream: false, playOnXbox: false, preview: false, subscriptionDownload: false, subscriptionStream: false, trackPurchase: false
                        };
                    rights.albumPurchase = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(item, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.AlbumPurchase]);
                    rights.trackPurchase = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(item, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Purchase]);
                    rights.subscriptionDownload = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(item, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Subscription]);
                    rights.subscriptionStream = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(item, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Stream]);
                    rights.playOnXbox = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(item, MS.Entertainment.ViewModels.SmartBuyStateHandlers.Tuner.Xbox360, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Stream]);
                    rights.freeStream = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(item, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.FreeStream]);
                    rights.preview = MS.Entertainment.ViewModels.SmartBuyStateHandlers._mediaHasAnyRight(item, MS.Entertainment.Utilities.defaultClientTypeFromApp, [MS.Entertainment.ViewModels.SmartBuyStateHandlers.MarketplaceRight.Preview]);
                    return rights
                }
        }})
})()
