﻿/* Copyright (C) Microsoft Corporation. All rights reserved. */
scriptValidator();
WinJS.Namespace.define("MS.Entertainment.UI.Controls.ShapeVisualization", {ShapeType: {
        Square: "square", Circle: "circle", Rectangle: "rectangle", Triangle: "triangle", Trapezoid: "trapezoid", Parallelogram: "parallelogram", Diamond: "diamond"
    }});
MS.Entertainment.UI.Debug.defineAssert("MS.Entertainment.UI.Controls");
WinJS.Namespace.define("MS.Entertainment.UI.Controls", {ShapeVisualizationEngine: MS.Entertainment.UI.Framework.defineUserControl(null, function(element, options) {
        this._shapeContainer = element;
        this._currentColorPalette = options.currentColorPalette;
        this._updateSize = this._updateSize.bind(this)
    }, {
        _shapeContainer: null, _currentColorPalette: null, _animationEnabled: true, _renderLoopQueued: false, _initialized: false, _isFrozen: false, _currentShapeIndex: -1, _currentAnimationIndex: -1, _currentShape: null, _shapes: [], _localFolder: null, _spawnPoints: [], _defaultAnimation: null, _queuedSprites: [], _savedAnims: [], _scaleSettings: 0, _maxShapes: 0, _uiStateService: null, _thawPromise: null, _lastWidth: 0, _BASE_WIDTH: 1366, _BASE_HEIGHT: 768, _resizeTimeoutPromise: null, _STORAGE_FILE_NAME: "savedAnims.txt", _SCENE_SPAWN_OFFSET: 20000, _PROCESS_SPRITE_QUEUE_DELAY: 50, _LAYOUT_RESIZE_DELAY: 500, _MAX_RANDOM_TRY_COUNT: 10, _allocatedPixels: 0, _MAX_PIXEL_ALLOCATION: 0, _activeSprites: 0, _SPAWN_RATE: 2500, _ITEMS_ON_SCREEN: 5, _spawnInterval: null, controlName: "ShapeVisualizationEngine", initialize: function initialize() {
                MS.Entertainment.UI.Framework.UserControl.prototype.initialize.apply(this, arguments);
                this._uiStateService = MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState);
                this._MAX_PIXEL_ALLOCATION = MS.Entertainment.Utilities.getWindowWidth() * MS.Entertainment.Utilities.getWindowHeight() * 6;
                this._initShapeLibrary();
                this._updateSize();
                MS.Entertainment.Utilities.attachResizeEvent(this._shapeContainer, this._updateSize);
                this._loadAnims();
                if (this._currentColorPalette)
                    this.notifyColorChanged(this._currentColorPalette);
                else
                    this._createScene();
                this._initialized = true
            }, unload: function unload() {
                this._stopAllAnims();
                MS.Entertainment.Utilities.detachResizeEvent(this._shapeContainer, this._updateSize);
                MS.Entertainment.UI.Framework.UserControl.prototype.unload.call(this)
            }, thaw: function thaw() {
                MS.Entertainment.UI.Framework.UserControl.prototype.thaw.call(this);
                this._isFrozen = false;
                this._animationEnabled = true;
                this._thawPromise = WinJS.Promise.timeout(500).then(function delayedStartAnims() {
                    this._startAllAnims();
                    this._thawPromise = null
                }.bind(this))
            }, freeze: function freeze() {
                this._clearCanvas();
                this._stopAllAnims();
                MS.Entertainment.UI.Framework.UserControl.prototype.freeze.call(this)
            }, _updateSize: function _updateSize(sourceElement) {
                if (!this._shapeContainer)
                    return;
                if (this._thawPromise) {
                    this._thawPromise.cancel();
                    this._thawPromise = null
                }
                this._MAX_PIXEL_ALLOCATION = MS.Entertainment.Utilities.getWindowWidth() * MS.Entertainment.Utilities.getWindowHeight() * 6;
                var currentWidth = this._shapeContainer.clientWidth;
                var currentHeight = this._shapeContainer.clientHeight;
                if (currentWidth > 0 && this._lastWidth === currentWidth)
                    return;
                this._lastWidth = currentWidth;
                if (this._resizeTimeoutPromise)
                    this._resizeTimeoutPromise.cancel();
                var scaleSettings = this._calculateScaleFactor();
                if (scaleSettings.scaleFactor !== this._scaleSettings.scaleFactor) {
                    this._scaleSettings = scaleSettings;
                    this._clearCanvas();
                    this._generateSpawnPoints();
                    this._updateShapeSizes();
                    if (sourceElement)
                        this._createScene()
                }
            }, notifyMediaChanged: function notifyMediaChanged(newValue, oldValue) {
                if (oldValue) {
                    this._clearCanvas();
                    this._updateCurrentShape()
                }
            }, notifyColorChanged: function notifyColorChanged(colorPalette) {
                this._currentColorPalette = colorPalette;
                this._updateCurrentAnimation();
                this._createScene()
            }, clearScene: function clearScene() {
                this._clearCanvas()
            }, startScene: function startScene() {
                this._createScene()
            }, _stopAllAnims: function _stopAllAnims() {
                this._animationEnabled = false;
                this._clearCanvas();
                this._clearTimer()
            }, _startAllAnims: function _startAllAnims() {
                this._animationEnabled = true;
                this._createScene();
                this._startTimer()
            }, _startTimer: function _startTimer() {
                if (this._frozen || this._unloaded || !this._animationEnabled)
                    return;
                if (!this._spawnInterval)
                    this._spawnInterval = window.setInterval(this._onSpawnInterval.bind(this), this._SPAWN_RATE)
            }, _clearTimer: function _clearTimer() {
                if (this._spawnInterval) {
                    window.clearInterval(this._spawnInterval);
                    this._spawnInterval = null
                }
            }, _refreshLayout: function _refreshLayout() {
                this._stopAllAnims();
                this._startAllAnims()
            }, _calculateScaleFactor: function _calculateScaleFactor() {
                var scaleSettings = {};
                var currentWidth = this._shapeContainer.clientWidth;
                var currentHeight = this._shapeContainer.clientHeight;
                arC = currentWidth / currentHeight;
                arB = this._BASE_WIDTH / this._BASE_HEIGHT;
                scaleSettings.baseAspectRatio = arB;
                scaleSettings.currentAspectRatio = arC;
                if (this._uiStateService.isSnapped)
                    scaleSettings.scaleFactor = currentWidth / this._BASE_WIDTH;
                else if (arC > arB)
                    scaleSettings.scaleFactor = currentHeight / this._BASE_HEIGHT;
                else
                    scaleSettings.scaleFactor = currentWidth / this._BASE_WIDTH;
                return scaleSettings
            }, _createScene: function createScene() {
                if (!this.currentAnimation || !this._animationEnabled || this._frozen || !MS.Entertainment.ServiceLocator || !MS.Entertainment.ServiceLocator.getService(MS.Entertainment.Services.uiState).animationsEnabled)
                    return;
                var spawnIndexes = [];
                switch (this.currentAnimation.SpawnOriginType) {
                    case"Fixed":
                        if (this.currentAnimation.FixedSpawnNodes)
                            spawnIndexes = this.currentAnimation.FixedSpawnNodes.split(",");
                        break;
                    case"Random":
                    default:
                        var newSpawnIndex = Math.floor((Math.random() * this._spawnPoints.length) + 0);
                        spawnIndexes.push(newSpawnIndex);
                        var tryCount;
                        while (spawnIndexes.length < 3) {
                            newSpawnIndex = Math.floor((Math.random() * this._spawnPoints.length) + 0);
                            tryCount = 0;
                            while (this._hasCollision(newSpawnIndex, spawnIndexes) && tryCount < this._MAX_RANDOM_TRY_COUNT) {
                                newSpawnIndex = Math.floor((Math.random() * this._spawnPoints.length) + 0);
                                tryCount++
                            }
                            spawnIndexes.push(newSpawnIndex)
                        }
                        break
                }
                this.currentAnimation.TransformSettings.RandomHelper.Values = null;
                var nodeIndex = -1;
                var shapeObj = this._shapes[this.currentAnimation.Shape];
                var transformOrigin;
                var maxShapes = this._maxShapes ? Math.min(this._maxShapes, this.currentAnimation.ItemCount) : this.currentAnimation.ItemCount;
                for (var i = 0; i < maxShapes; i++) {
                    switch (this.currentAnimation.SpawnOrder) {
                        case"Sequential":
                            if (this.currentAnimation.ItemCount < spawnIndexes.length)
                                nodeIndex++;
                            else if (i % Math.floor(this.currentAnimation.ItemCount / spawnIndexes.length) === 0)
                                nodeIndex = (nodeIndex + 1 < spawnIndexes.length) ? nodeIndex + 1 : nodeIndex;
                            break;
                        case"Random":
                        default:
                            nodeIndex = this._generateRandom(0, spawnIndexes.length);
                            break
                    }
                    if (nodeIndex === -1)
                        nodeIndex++;
                    var spriteDetails = this._createSprite(this._shapes[this.currentAnimation.Shape], this._currentColorPalette[this._generateRandom(0, 2)], this.currentAnimation, i);
                    if (spriteDetails.sprite) {
                        var sprite = spriteDetails.sprite;
                        transformOrigin = this.currentAnimation.TransformOrigins;
                        if (transformOrigin.indexOf(",") !== -1) {
                            var origins = transformOrigin.split(",");
                            var originIndex = this._generateRandom(0, origins.length);
                            transformOrigin = origins[originIndex].trim(" ")
                        }
                        sprite.style.transformOrigin = transformOrigin;
                        var transformPoint = transformOrigin.trim(" ").split(" ");
                        transformPoint[0] = transformPoint[0].replace("%", "");
                        transformPoint[1] = transformPoint[1].replace("%", "");
                        sprite.style.left = (this._spawnPoints[spawnIndexes[nodeIndex]].x - ((transformPoint[0] / 100) * shapeObj.width)) + "px";
                        sprite.style.top = (this._spawnPoints[spawnIndexes[nodeIndex]].y - ((transformPoint[1] / 100) * shapeObj.height)) + "px";
                        sprite.addEventListener("animationend", this._spriteAnimationEnd.bind(this), false);
                        this._queuedSprites.splice(0, 0, spriteDetails)
                    }
                }
                this._startTimer()
            }, _spriteAnimationEnd: function _spriteAnimationEnd(event) {
                try {
                    event.srcElement.removeEventListener("animationend", this._spriteAnimationEnd, false);
                    if (this._shapeContainer.children.length)
                        this._shapeContainer.removeChild(event.srcElement)
                }
                catch(error) {
                    MS.Entertainment.UI.Controls.assert(error.message === "NotFoundError", "Error occurred while stopping sprite animation. Error message: " + error.message)
                }
                var bitmapSize = ((event.srcElement.style.width.replace("px", "")) * (event.srcElement.style.height.replace("px", "")));
                this._allocatedPixels -= bitmapSize;
                this._activeSprites--
            }, _loadAnims: function _loadAnims() {
                try {
                    this._shapes[3].Animations = JSON.parse('[{"Name":"Dia1_All_Ran_20_6cnt","Shape":"3","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,2,5,3,3,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Dia1_Ran3_Ran_20_6cnt","Shape":"3","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"4","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[2,2,4,4,2,3,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Dia1_RanOut_Ran_20_6cnt","Shape":"3","TransformOrigins":"50% 50%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,3,4,4,4,3,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Dia1_LinHor_Seq_20_mix_6cnt","Shape":"3","TransformOrigins":"0% 50%, 50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"6,7,8,16,17,18","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,4,3,3,2,3,3,2,4],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Dia1_LinVer_Seq_20_5cnt","Shape":"3","TransformOrigins":"0% 50%","SpriteSpawnOffset":"2500","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"2,7,12,17,22","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,4,3,2,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Dia1_Cor4_Seq_20_5cnt","Shape":"3","TransformOrigins":"50% 50%, 0% 50%, 50% 0%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,4,20,24,0","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,4,5,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[2,2,1,1,4,4,1,3]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Dia1_Cor2_Rdm_20_6cnt","Shape":"3","TransformOrigins":"50% 100%, 50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,4,20,4,20,4","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[6,3,3,4,3,3],"Min":"3","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"0","Max":"1","Values":[0,0,0,0,0,0]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"5","Values":[2,4,4,4,4,1,4,1]}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[1,1,1,4,3,3,2,4]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Dia1_Mig_Seq_20_5cnt","Shape":"3","TransformOrigins":"50% 0%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"4,8,12,16,20","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,5,4,2,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Dia1_Pnt2_Seq_20_6cnt","Shape":"3","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"10,14","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,2,3,3,5,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Dia1_Pnt2_Seq_20b_4cnt","Shape":"3","TransformOrigins":"50% 50%","SpriteSpawnOffset":"5000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"4","SpawnOriginType":"Fixed","FixedSpawnNodes":"1,23","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[6,4,5,3,3],"Min":"3","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Dia1_LinVer_Ran_20_6cnt","Shape":"3","TransformOrigins":"0% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"1,11,21,3,13,23","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,3,5,2,2,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Dia1_All_Ran_7.5_6cnt","Shape":"3","TransformOrigins":"50% 50%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(0, 0.5, 0.4, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,3,4,4,2,2,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Dia1_Ran3_Ran_10_4cnt","Shape":"3","TransformOrigins":"50% 50%","SpriteSpawnOffset":"5000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"4","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"10000","TransformEasing":"cubic-bezier(0.32, 1, 0.69, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,4,6,4],"Min":"3","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Dia1_RanOut_Ran_7.5_6cnt","Shape":"3","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(0, 0.5, 0.4, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,1,1,3,1,1,1],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Dia1_All_Pnt3_20_6cnt","Shape":"3","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"16,16,12,12,18,18","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,3,3,3,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}}]');
                    this._shapes[4].Animations = JSON.parse('[{"Name":"Hcir1_All_Ran_20_6cnt","Shape":"4","TransformOrigins":"50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,4,3,3,4,3,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"4","Values":[4,2,2,2,4,4,4,1]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_Ran3_Ran_20_6cnt_mi","Shape":"4","TransformOrigins":"50% 100%, 0% 100%, 100% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,5,2,5,2,2,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_RanOut_Ran_20_6cnt","Shape":"4","TransformOrigins":"50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,4,4,2,3,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_LinHor_Seq_20_6cnt","Shape":"4","TransformOrigins":"50% 100%, 0% 100%, 100% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"5,6,7,8,9,15,16,17,18,19","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,1,3,3,2,1,1,3,2,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_LinVer_Seq_20_6cnt","Shape":"4","TransformOrigins":"50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"7,12,17","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.15","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,2,4,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"2"}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_Cor2_Seq_20_6cnt","Shape":"4","TransformOrigins":"50% 100%, 0% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,4,20,4,20,4,20","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.05,.64,.61,.99)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,1,1,1,3,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"0","Max":"1","Values":[0,0,0,0,0,0]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"5","Values":[2,4,4,4,4,1,4,1]}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[1,1,1,4,3,3,2,4]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Hcir1_Mig_Seq_20_6cnt","Shape":"4","TransformOrigins":"50% 100%, 0% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"8,12,16","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.05,.64,.61,.99)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.15","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,4,6,6,3],"Min":"2","Max":"5"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Hcir1_Pnt2_Seq_20_6cnt","Shape":"4","TransformOrigins":"0% 100%, 100% 100%, 50% 100%, 50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"6,18","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,3,1,1,1,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"4","Values":[3,1,4,2,2,1]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_Pnt2b_Seq_20_6cnt","Shape":"4","TransformOrigins":"0% 100%, 100% 100%, 50% 100%, 50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"11,13","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,3,2,2,2,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"4","Values":[3,2,4,2,1,3]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_LinHor2_Seq_20_6cnt","Shape":"4","TransformOrigins":"50% 100%, 0% 100%, 100% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"9,8,8,6,6,5","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,1,2,2,3,3,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_LinHor3_Seq_20_6cnt","Shape":"4","TransformOrigins":"50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"13,12,11","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,5,3,2,4,2],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"4"}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_LinHor4b_Seq_7.5_6cnt","Shape":"4","TransformOrigins":"50% 100%, 100% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"10,11,11,13,13,14","SpawnOrder":"Sequential","TransformDuration":"7500","TransformEasing":"cubic-bezier(0,.39,.27,.94)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.1","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,1,1,2,2,2,2],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_LinHor5_Seq_20_6cnt","Shape":"4","TransformOrigins":"50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"15,16,16,18,18,19","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,1,1,2,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_LinHor6_Seq_7.5_6cnt","Shape":"4","TransformOrigins":"50% 100%, 100% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"14,13,13,11,11,10","SpawnOrder":"Sequential","TransformDuration":"7500","TransformEasing":"cubic-bezier(0,.53,.29,1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.1","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,1,2,1,2,1],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_Pnt2c_Seq_20_7cnt","Shape":"4","TransformOrigins":"50% 100%","SpriteSpawnOffset":"2500","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Fixed","FixedSpawnNodes":"6,18,6,18,6,18,6","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,3,2,1,1,3,1],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"4","Values":[2,3,4,2,4,3,4,4]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_Pnt2d_Seq_20_7cnt","Shape":"4","TransformOrigins":"50% 100%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Fixed","FixedSpawnNodes":"21,3,21,3,21,3,21","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,4,2,3,2,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[2,4,3,1,3,2,1,2]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_Pnt3_Echo_Seq_20_6cnt","Shape":"4","TransformOrigins":"50% 100%, 50% 100%, 50% 100%, 50% 100%, 50% 100%, 50% 100%, 0% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"16,16,7,7,18,18","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.09,.18,.27,.34)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,1,1,1,2,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"4","Values":[3,1,4,2,2,1]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Hcir1_Pnt3_Echo_Seq_10_6cnt","Shape":"4","TransformOrigins":"50% 100%, 50% 100%, 50% 100%, 50% 100%, 50% 100%, 50% 100%, 0% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"6,6,12,12,18,18","SpawnOrder":"Sequential","TransformDuration":"10000","TransformEasing":"cubic-bezier(.09,.18,.27,.34)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,1,1,1,2,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"4","Values":[3,1,4,2,2,1]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}}]');
                    this._shapes[5].Animations = JSON.parse('[{"Name":"Par1_All_Seq_20_6cnt","Shape":"5","TransformOrigins":"0% 100%, 50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"10,14","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,3,4,3,5,4,3,5],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par1_Ran3_Ran_20_6cnt","Shape":"5","TransformOrigins":"50% 100%, 0% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,4,3,2,2,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par1_RanOut_Ran_20_6cnt","Shape":"5","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,6,3,6,5,3],"Min":"3","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par1_LinHor_Seq_20_6cnt","Shape":"5","TransformOrigins":"0% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"10,10,11,11,13,13","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,5,3,3,3,3,6],"Min":"3","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par1_LinVer_Seq_20_6cnt","Shape":"5","TransformOrigins":"50% 0%,100% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"2,7,12,17,12,2","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,2,4,4,2,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par1_Cor4_Seq_20_6cnt","Shape":"5","TransformOrigins":"50% 50%, 50% 0%, 0% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,4,20,24,4,20","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,5,4,3,2,4,5],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[2,2,1,1,4,4,1,3]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Par1_Cor2_Rdm_20_6cnt","Shape":"5","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,4,20,4,20,4","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.35","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,3,2,4,3,2,3,2],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"0","Max":"1","Values":[0,0,0,0,0,0,0,0]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"5","Values":[2,4,4,4,4,1,4,1]}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[1,1,1,4,3,3,2,4]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Par1_Mig_Seq_20_6cnt","Shape":"5","TransformOrigins":"75% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,11,7,21,17","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,1,2,2,1,2,1,3,1],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"180","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Par1_Pnt2_Seq_20_6cnt","Shape":"5","TransformOrigins":"50% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"5,19,5,19,5,19","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,4,4,2,4,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"2","Values":[2,1,1,2,2,1]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par1_Pnt2b_Seq_20b_6cnt","Shape":"5","TransformOrigins":"0% 100%, 50% 100%, 50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"16,8","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,4,3,5,2,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par1_LinVer_Ran_20_6cnt","Shape":"5","TransformOrigins":"0% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"1,6,11,16,21,3,8,13,23","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,3,4,2,2,4,4,3,4],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par1_All_Ran_7.5_7cnt","Shape":"5","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(0, 0.5, 0.4, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,1,1,2,2,1,1],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par1_Ran3_Ran_7.5_6cnt","Shape":"5","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(0.32, 1, 0.69, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,3,3,1,3,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par1_RanOut_Ran_7.5_6cnt","Shape":"5","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(0, 0.5, 0.4, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,1,2,1,1,1,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}}]');
                    this._shapes[7].Animations = JSON.parse('[{"Name":"Par3_All_Ran_20_6cnt","Shape":"7","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,4,4,3,3,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par3_RanOut_Ran_20_6cnt","Shape":"7","TransformOrigins":"50% 50%, 50% 50%, 33.33% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,3,5,2,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par3_LinHor_Seq_20_mix_6cnt","Shape":"7","TransformOrigins":"33.33% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"5,6,7,19,18,17","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,4,2,3,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par3_LinVer_Seq_20_6cnt","Shape":"7","TransformOrigins":"0% 100%, 33.33% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"2,7,12,17,22,12","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,4,2,5,2,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par3_Cor4_Seq_20_6cnt","Shape":"7","TransformOrigins":"50% 50%, 33.33% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,4,20,24,4,20","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.05,.64,.61,.99)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,3,2,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"4","Values":[3,2,1,2,1,3]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Par3_Cor2_Rdm_20_6cnt","Shape":"7","TransformOrigins":"50% 50%, 50% 50%, 50% 50%, 33.33% 0%, 0% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,4","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(.05,.64,.61,.99)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,4,2,2,2,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"0","Max":"1","Values":[0,0,0,0,0,0,0]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"5","Values":[2,4,4,4,4,1,4,1]}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[1,1,1,4,3,3,2,4]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Par3_Mig_Seq_20_6cnt","Shape":"7","TransformOrigins":"33.33% 100%, 33.33% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"8,12,16","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.05,.64,.61,.99)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,1,3,3,2,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Pra3_Pnt2_Seq_20_5cnt","Shape":"7","TransformOrigins":"33.33% 0%, 33.33% 100%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"10,14,10,14,10","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,4,2,3,2,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Par3_Pnt2b_Seq_20_6cnt","Shape":"7","TransformOrigins":"33.33% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"11,13","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,3,4,2,2,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}}]');
                    this._shapes[15].Animations = JSON.parse('[{"Name":"Tra2_All_Ran_7.5_6cnt","Shape":"15","TransformOrigins":"0% 33.33%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(.09,.18,.27,.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,3,2,2,2,3,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true}}]}},{"Name":"Tra2_Ran3_Ran_20_6cnt","Shape":"15","TransformOrigins":"100% 100%, 100% 33.333%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(.12,.69,.25,.71)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,3,3,3,2,2,2,4],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra2_RanOut_Ran_7.5_6cnt","Shape":"15","TransformOrigins":"100% 100%, 100% 33.333%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(0,.53,.29,1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,4,2,4,4,4,4,2,3,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra2_LinHor_Seq_7.5_6cnt","Shape":"15","TransformOrigins":"100% 50%, 100% 100%, 100% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"10,11,11,12,13,13,14","SpawnOrder":"Sequential","TransformDuration":"7500","TransformEasing":"cubic-bezier(0,.53,.29,1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,1,2,3,1,2,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true}}]}},{"Name":"Tra2_LinVer_Seq_20_5cnt","Shape":"15","TransformOrigins":"100% 100%, 100% 50%","SpriteSpawnOffset":"2000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"3,6,13,16,23","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,4,3,5,2,4,3,3,5,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra2_Mig_Seq_20_6cnt","Shape":"15","TransformOrigins":"100% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,16,16,8,8,4","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.12,.69,.25,.71)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,1,1,1,1,2,2,1,1,2],"Min":"1","Max":"2"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true}}]}},{"Name":"Tra2_Pnt2_Seq_20_6cnt","Shape":"15","TransformOrigins":"100% 50%, 100% 100%, 100% 0%","SpriteSpawnOffset":"2500","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"11,13,11,13,11,13","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.15","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,4,4,2,2,2,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"180","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"2","Values":[1,2,1,2,2,2,1,2]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra2_Pnt2b_Seq_20_6cnt","Shape":"15","TransformOrigins":"100% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"16,8,16,8,16,8","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.15","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,2,3,5,2,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"180","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"2","Values":[1,1,1,2,2,2]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra2_LinVer2_Seq_7.5_5cnt","Shape":"15","TransformOrigins":"100% 100%, 100% 50%","SpriteSpawnOffset":"2500","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"3,6,13,16,23","SpawnOrder":"Sequential","TransformDuration":"7500","TransformEasing":"cubic-bezier(0,.53,.29,1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,2,1,2,3,3,2,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra2_Mig2_Seq_7.5_6cnt","Shape":"15","TransformOrigins":"100% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,16,16,12,8,4","SpawnOrder":"Sequential","TransformDuration":"7500","TransformEasing":"cubic-bezier(0,.53,.29,1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,2,1,1,1,2,1,2,1,2],"Min":"1","Max":"2"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true}}]}},{"Name":"Tra2_Mig3_Seq_20_7cnt","Shape":"15","TransformOrigins":"100% 50%","SpriteSpawnOffset":"2000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,6,6,12,18,18,24","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.09,.18,.27,.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,3,3,2,4,3,4,3,4],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true}}]}}]');
                    this._shapes[16].Animations = JSON.parse('[{"Name":"Tra3_RanAll_Ran_20_6cnt","Shape":"16","TransformOrigins":"0% 100%, 50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,4,4,2,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"45","ApplyRandom":true,"ApplyIteration":true},"To":{"Value":"45","ApplyRandom":true,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"45","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"45","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra3_Ran3_Ran_20_6cnt","Shape":"16","TransformOrigins":"33% 100%, 33% 0%, 50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,2,2,3,2,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"45","ApplyRandom":true,"ApplyIteration":true},"To":{"Value":"45","ApplyRandom":true,"ApplyIteration":true}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra3_RanOut_Ran_20_6cnt","Shape":"16","TransformOrigins":"33% 50%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,2,4,2,4,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra3_RanOut_Ran_20_6cnt","Shape":"16","TransformOrigins":"33% 50%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,4,2,3,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra3_LinVer_Seq_20_6cnt","Shape":"16","TransformOrigins":"0% 100%, 33.33% 100%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"2,7,12,17,22,17,7","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,6,4,4,5,3,6,6,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra3_Cor4_Seq_20_6cnt","Shape":"16","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,0,24,4,20,4","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,5,2,5,2,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[2,2,1,1,4,4,1,3]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tra3_Cor2_Seq_20_6cnt","Shape":"16","TransformOrigins":"0% 100%, 50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,4,20,4,20,4","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,2,2,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"0","Max":"1","Values":[0,0,0,0,0,0]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"5","Values":[2,4,4,4,4,1,4,1]}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[1,1,1,4,3,3,2,4]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tra3_Mig_Seq_20_5cnt","Shape":"16","TransformOrigins":"33.33% 0%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,6,12,18,24","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,4,4,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[1,2,2,2,4]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tra3_Pnt2_Ran_20_6cnt","Shape":"16","TransformOrigins":"33.33% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"15,19","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,4,4,2,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra3_Pnt2b_Ran_20_6cnt","Shape":"16","TransformOrigins":"50% 100%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"5,9","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,4,4,3,4,4,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra3_Rot_Ran_20_6cnt","Shape":"16","TransformOrigins":"33.33% 0%, 66.66% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"16,8,16,8,16,8","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,2,5,2,2,4],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"45","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"45","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra3_RanAll_Ran_20_6cnt","Shape":"16","TransformOrigins":"0% 100%, 50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,3,3,1,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"45","ApplyRandom":true,"ApplyIteration":true},"To":{"Value":"45","ApplyRandom":true,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"45","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"45","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra3_RanOut_Ran_20_6cnt","Shape":"16","TransformOrigins":"33% 50%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,2,4,2,4,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra3_Pnt3_Echo_Ran_20_6cnt","Shape":"16","TransformOrigins":"50% 100%,50% 100%,50% 100%,50% 100%,0% 100%,50% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"6,6,13,13,16,16","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,1,1,1,2,1],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"45","ApplyRandom":true,"ApplyIteration":true},"To":{"Value":"45","ApplyRandom":true,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"45","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"45","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tra3_Pnt3_Echo_Ran_10_6cnt","Shape":"16","TransformOrigins":"50% 100%,50% 100%,50% 100%,50% 100%,0% 100%,50% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"16,16,12,12,18,18","SpawnOrder":"Sequential","TransformDuration":"10000","TransformEasing":"cubic-bezier(0,.53,.29,1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".5","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,2,1,1,1],"Min":"1","Max":"2"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"45","ApplyRandom":true,"ApplyIteration":true},"To":{"Value":"45","ApplyRandom":true,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"45","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"45","ApplyRandom":true,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}}]');
                    this._shapes[17].Animations = JSON.parse('[{"Name":"Tri1_Pnt3Echo_B_Seq_7_6cnt","Shape":"17","TransformOrigins":"50% 100%, 50% 100%,50% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"11,11,12,12,13,13","SpawnOrder":"Sequential","TransformDuration":"7000","TransformEasing":"cubic-bezier(0,.53,.29,1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,3,3,2,3,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_Ran3_Ran_20_6cnt","Shape":"17","TransformOrigins":"50% 100%, 50% 0%, 50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"1","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"3","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[2,2,4,4,2,3,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_RanOut_Ran_20_6cnt","Shape":"17","TransformOrigins":"50% 50%, 50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,4,4,3,5],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_LinHor_Seq_20_5cnt","Shape":"17","TransformOrigins":"50% 100%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"5,6,7,8,9","SpawnOrder":"Sequential","TransformDuration":"7500","TransformEasing":"cubic-bezier(0,.53,.29,1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,2,3,3,3,3,1,3,1,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_LinVer_Ran_20_6cnt","Shape":"17","TransformOrigins":"100% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"1,6,11,16,21,3,8,13,18,23","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,2,3,2,4,3,2,4],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_Cor4_Seq_20_6cnt","Shape":"17","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,4,20,24,0,4,20","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,4,5,5,3,5,4,5],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[2,2,1,1,4,4,1,3]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri1_Cor2_Rdm_20_6cnt","Shape":"17","TransformOrigins":"0% 100%, 50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,4,20,4,20,4,20","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,4,2,4,4,3,4,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"0","Max":"1","Values":[0,0,0,0,0,0,0,0]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"5","Values":[2,4,4,4,4,1,4,1]}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[1,1,1,4,3,3,2,4]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri1_Mig_Seq_20_5cnt","Shape":"17","TransformOrigins":"0% 100%, 50% 50%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"4,8,12,16,20","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,5,2,2,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri1_Pnt2_Seq_20_6cnt","Shape":"17","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"10,14","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[5,5,4,5,4,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_Pnt2_Seq_20b_6cnt","Shape":"17","TransformOrigins":"50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"23,1,23,1,23,1,23","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,3,2,2,3,1,3,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_All_Ran_7.5_6cnt","Shape":"17","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(0, 0.5, 0.4, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,2,2,3,3,1,1,1],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_Ran3_Ran_10_6cnt","Shape":"17","TransformOrigins":"50% 50%, 50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"10000","TransformEasing":"cubic-bezier(0.32, 1, 0.69, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,3,2,1,3,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_RanOut_Ran_7.5_6cnt","Shape":"17","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(0, 0.5, 0.4, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,3,3,1,3,1,2,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_Mig2_Seq_20_7cnt","Shape":"17","TransformOrigins":"50% 0%, 0% 100%","SpriteSpawnOffset":"2500","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"8","SpawnOriginType":"Fixed","FixedSpawnNodes":"5,16,16,7,7,18,18,9","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,4,2,2,3,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri1_LinHorb_Seq_20_5cnt","Shape":"17","TransformOrigins":"50% 100%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"19,18,17,16,15","SpawnOrder":"Sequential","TransformDuration":"7500","TransformEasing":"cubic-bezier(0,.53,.29,1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,2,3,3,3,3,1,3,1,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_Pnt3Echo_Seq_20_6cnt","Shape":"17","TransformOrigins":"50% 100%, 50% 100%,50% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"6,6,12,12,18,18","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,3,2,2,3,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri1_Pnt3Echo_B_Seq_10_6cnt","Shape":"17","TransformOrigins":"50% 100%, 50% 100%,50% 100%, 50% 100%,50% 100%, 50% 100%,50% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"11,11,12,12,13,13","SpawnOrder":"Sequential","TransformDuration":"10000","TransformEasing":"cubic-bezier(0,.53,.29,1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":".75","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,1,2,2,1,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}}]');
                    this._shapes[18].Animations = JSON.parse('[{"Name":"Tri2_All_Ran_20_6cnt","Shape":"18","TransformOrigins":"50% 50%, 100% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"10,14","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,3,5,3,5,4,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri2_Ran3_Ran_20_7cnt","Shape":"18","TransformOrigins":"50% 100%, 0% 100%, 50% 50%, 100% 100%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,4,4,4,3,2,2,2],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri2_RanOut_Ran_20_6cnt","Shape":"18","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,3,2,2,4,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri2_LinHor_Ran_20_6cnt","Shape":"18","TransformOrigins":"0% 100%, 100% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"5,6,7,8,9,15,16,17,18,19","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,3,2,3,2,3,3,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri2_LinVer_Seq_20_6cnt","Shape":"18","TransformOrigins":"100% 0%, 100% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"1,6,11,16,21,16,6","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,3,4,4,4,2,2,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri2_Cor4_Seq_20_6cnt","Shape":"18","TransformOrigins":"0% 100%, 50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,4,24,0,20,24","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,4,4,2,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[2,2,1,1,4,4,1,3]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri2_Cor2_Rdm_20_6cnt","Shape":"18","TransformOrigins":"0% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,4,20,4,20,4,20","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,2,2,4,2,2,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"0","Max":"1","Values":[0,0,0,0,0,0,0]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"5","Values":[2,4,4,4,4,1,4,1]}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[1,1,1,4,3,3,2,4]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri2_Mig_Seq_20_5cnt","Shape":"18","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"4,8,12,16,20","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,3,3,4,2,4],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri2_Pnt2_Seq_20_6cnt","Shape":"18","TransformOrigins":"100% 100%","SpriteSpawnOffset":"2500","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"14,10,14,10,14,10","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,3,3,2,2,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"2","Values":[2,2,1,1,1,1]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri2_Pnt2b_Ran_20b_6cnt","Shape":"18","TransformOrigins":"100% 100%, 50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"16,8","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,3,3,2,3,3,2,4],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri2_All_Ran_7.5_6cnt","Shape":"18","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(0, 0.5, 0.4, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,3,1,3,1,3,1,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri2_Ran3_Ran_7.5_7cnt","Shape":"18","TransformOrigins":"50% 100%, 0% 100%, 50% 50%, 100% 100%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(0.32, 1, 0.69, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,1,3,1,1,1,1,1],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri2_RanOut_Ran_7.5_6cnt","Shape":"18","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"7500","TransformEasing":"cubic-bezier(0, 0.5, 0.4, 1)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,3,4,4,3,2,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}}]');
                    this._shapes[19].Animations = JSON.parse('[{"Name":"Tri3_All_Ran_20_6cnt","Shape":"19","TransformOrigins":"0% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,5,2,3,3,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri3_Ran3_Ran_20_6cnt","Shape":"19","TransformOrigins":"0% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"4","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[2,2,4,4,2,3,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri3_RanOut_Ran_20_6cnt","Shape":"19","TransformOrigins":"50% 50%, 0% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,4,4,3,5],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri3_LinHorb_Seq_20_5cnt","Shape":"19","TransformOrigins":"0% 50%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"19,18,17,16,15","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,1,1,1,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri3_LinVer_Ran_20_6cnt","Shape":"19","TransformOrigins":"0% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"21,16,11,23,18,13","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,2,2,2,2],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"270","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"270","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri3_Cor4_Seq_20_6cnt","Shape":"19","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,4,20,24,0,4","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,3,3,4,5],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[2,2,1,1,4,4,1,3]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri3_Cor2_Rdm_20_6cnt","Shape":"19","TransformOrigins":"0% 50%, 50% 50%","SpriteSpawnOffset":"2500","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"8","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,4,20,4,20,4,20,4","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,5,4,5,5,4,6,3],"Min":"3","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"0","Max":"1","Values":[0,0,0,0,0,0,0,0]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"5","Values":[2,4,4,4,4,1,4,1]}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[1,1,1,4,3,3,2,4]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri3_Mig_Seq_20_5cnt","Shape":"19","TransformOrigins":"50% 50%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,1,22,3,24","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,4,2,2,5],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":true}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri3_Pnt2_Seq_20_6cnt","Shape":"19","TransformOrigins":"50% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"10,14","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[6,4,6,5,3,5],"Min":"3","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri3_Pnt2b_Seq_20_6cnt","Shape":"19","TransformOrigins":"0% 100%, 0% 50%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"1,3","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,4,4,2,3,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri3_Migb_Seq_20_6cnt","Shape":"19","TransformOrigins":"50% 25%, 0% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"11,12,13","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,4,5,4,3,2],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri3_LinHor_Seq_20_5cnt","Shape":"19","TransformOrigins":"0% 50%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"5,6,7,8,9","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0.09, 0.18, 0.27, 0.34)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,1,1,1,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}}]');
                    this._shapes[22].Animations = JSON.parse('[{"Name":"Tri6_All_Ran_20_7cnt","Shape":"22","TransformOrigins":"50% 0%, 0% 100%, 50% 100%, 50% 50%, 50% 50%, 50% 50%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":".25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,5,2,5,5,4,4,5],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri6_Ran3_Ran_20_7cnt","Shape":"22","TransformOrigins":"50% 0%, 0% 100%, 50% 100%, 50% 50%","SpriteSpawnOffset":2850,"InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":7,"SpawnOriginType":"Random","FixedSpawnNodes":"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,4,3,3,3,4],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri6_RanOut_Ran_20_7cnt","Shape":"22","TransformOrigins":"50% 50%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,1,2,3,4,5,9,10,14,15,19,20,21,22,23,24","SpawnOrder":"Random","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,2,2,2,3,2,2,4,3,3],"Min":"2","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri6_LinHor_Seq_20_5cnt","Shape":"22","TransformOrigins":"50% 100%, 0% 100%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"5,6,7,8,9","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,3,3,2,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri6_LinVer_Seq_20_6cnt","Shape":"22","TransformOrigins":"50% 0%, 0% 100%, 50% 100%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"16,11,6,8,13,18","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,2,2,4,3,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri6_Cor4_Seq_20_7cnt","Shape":"22","TransformOrigins":"50% 0%, 0% 100%, 50% 100%, 50% 50%, 50% 50%, 50% 50%","SpriteSpawnOffset":"2500","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Fixed","FixedSpawnNodes":"0,4,20,24,0,4,20","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.05,.64,.61,.99)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,2,2,1,4,3,2,4],"Min":"1","Max":"4"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Min":"1","Max":"4","Values":[1,3,1,4,1,2,1,2]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri6_Cor2_Rdm_20_7cnt","Shape":"22","TransformOrigins":"50% 100%, 0% 100%","SpriteSpawnOffset":"2500","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Fixed","FixedSpawnNodes":"20,4,20,4,20,4,20","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.05,.64,.61,.99)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[2,1,2,1,2,2,3,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"LocalRandomHelper":{"Enabled":true,"Min":"0","Max":"1","Values":[0,0,0,0,0,0,0,0]}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"50","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"5","Values":[2,4,4,4,4,1,4,1]}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"1","Max":"4","Values":[1,1,1,4,3,3,2,4]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri6_Mig_Seq_20_6cnt","Shape":"22","TransformOrigins":"0% 100%, 50% 100%","SpriteSpawnOffset":"2000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"8,12,16","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.05,.64,.61,.99)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.5","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,2,2,1,2,2,2,2,1,1],"Min":"1","Max":"2"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri6_Pnt2_Seq_20_7cnt","Shape":"22","TransformOrigins":"50% 100%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Fixed","FixedSpawnNodes":"21,3,21,3,21,3,21","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[1,3,2,3,2,1,1],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":true,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri6_Pnt2b_Seq_20_7cnt","Shape":"22","TransformOrigins":"0% 100%, 50% 50%, 50% 50%","SpriteSpawnOffset":"2850","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"7","SpawnOriginType":"Fixed","FixedSpawnNodes":"10,14,10,14,10,14,10,14","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[4,2,4,4,3,4,2,4],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":true,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri6_Migb_Seq_20_6cnt","Shape":"22","TransformOrigins":"50% 50%, 0% 100%, 50% 100%","SpriteSpawnOffset":"2000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"16,12,8","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.05,.64,.61,.99)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,1,2,3,2,1,2,1,2,3],"Min":"1","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"undefined","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Min":"undefined","Values":[3,2,1,3,1,3,2],"Max":"undefined"}}]}},{"Name":"Tri6_LinHorb_Seq_20_5cnt","Shape":"22","TransformOrigins":"50% 100%, 0% 100%","SpriteSpawnOffset":"4000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"5","SpawnOriginType":"Fixed","FixedSpawnNodes":"19,18,17,16,15","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":"1","Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"1","ApplyRandom":true,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":true,"Values":[3,3,3,2,3],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":true,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":true,"ApplyIteration":false}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri6_Echo3b_Seq_20_6cnt","Shape":"22","TransformOrigins":"50% 55%, 50% 55%, 50% 55%, 50% 55%, 50% 55%, 50% 55%, 50% 100%","SpriteSpawnOffset":"3000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"16,16,18,18,7,7","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"2","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[4,4,4,2,3,4],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri6_Echo3c_Seq_20_6cnt","Shape":"22","TransformOrigins":"50% 55%, 50% 55%, 50% 55%, 50% 55%, 50% 55%, 50% 55%, 50% 0%","SpriteSpawnOffset":"3333","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Random","FixedSpawnNodes":"","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(.09,.18,.27,.34)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"2.5","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[2,3,3,3,2,2],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}},{"Name":"Tri6_Echo3a_Seq_20_6cnt","Shape":"22","TransformOrigins":"50% 100%","SpriteSpawnOffset":"3000","InitOpacity":"0.0","TargetOpacity":"1.0","OpacityDuration":20000,"ItemCount":"6","SpawnOriginType":"Fixed","FixedSpawnNodes":"6,6,8,8,17,17","SpawnOrder":"Sequential","TransformDuration":"20000","TransformEasing":"cubic-bezier(0, 0.35, 0.14, 0.43)","TransformSettings":{"RandomHelper":{"Min":0,"Max":4,"MinDelta":0,"Enabled":true},"Transforms":[{"Type":"ScaleTransform","Enabled":true,"From":{"Value":"0.25","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"2","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[4,4,4,2,3,4],"Min":"2","Max":"3"}},{"Type":"RotateTransform","Enabled":false,"From":{"Value":"180","ApplyRandom":false,"ApplyIteration":true},"To":{"Value":"180","ApplyRandom":false,"ApplyIteration":true}},{"Type":"TranslateXTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"TranslateYTransform","Enabled":false,"From":{"Value":"0","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"0","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransform","Enabled":false,"From":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"90","ApplyRandom":false,"ApplyIteration":false},"LocalRandomHelper":{"Enabled":false,"Values":[null,null,null,null,null,null,null,null,null,null]}},{"Type":"RotateAndFlipTransformVert","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"RotateAndFlipTransformHorz","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectVerticalTransform","Enabled":true,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}},{"Type":"ReflectHorizontalTransform","Enabled":false,"From":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false},"To":{"Value":"N/A","ApplyRandom":false,"ApplyIteration":false}}]}}]');
                    this._updateCurrentShape()
                }
                catch(ex) {
                    MS.Entertainment.UI.Controls.fail("Failed to parse JSON shape animation definitions.");
                    this._stopAllAnims()
                }
            }, _updateCurrentShape: function _updateCurrentShape() {
                var success = false;
                var newIndex = -1;
                while (!success) {
                    newIndex = this._generateRandom(0, this._shapes.length);
                    if (newIndex !== this._currentShapeIndex && (this._shapes[newIndex].Animations) && (this._shapes[newIndex].Animations.length) && (this._shapes[newIndex].Animations.length > 0))
                        success = true
                }
                this._savedAnims = this._shapes[newIndex].Animations;
                this._currentShapeIndex = newIndex;
                this._lastAnimationIndex = newIndex;
                this._updateCurrentAnimation();
                MS.Entertainment.UI.Controls.MusicVisualization.currentShape = this._shapes[this._currentShapeIndex].type
            }, _updateCurrentAnimation: function _updateCurrentAnimation() {
                if (this._savedAnims.length === 0)
                    return;
                var newAnimationIndex = this._generateRandom(0, this._savedAnims.length);
                while (newAnimationIndex === this._lastAnimationIndex)
                    newAnimationIndex = this._generateRandom(0, this._savedAnims.length);
                this._savedAnims[newAnimationIndex].ItemCount = 10;
                this.currentAnimation = this._savedAnims[newAnimationIndex];
                this._lastAnimationIndex = newAnimationIndex
            }, _onSpawnInterval: function _onSpawnInterval() {
                if (this._frozen || this._unloaded || !this._animationEnabled)
                    return;
                if (this._queuedSprites.length > 0) {
                    var spriteSettings = this._queuedSprites.pop();
                    var bitmapSize = ((spriteSettings.sprite.style.width.replace("px", "")) * (spriteSettings.sprite.style.height.replace("px", "")));
                    if ((this._allocatedPixels + bitmapSize) > this._MAX_PIXEL_ALLOCATION);
                    else {
                        this._allocatedPixels += bitmapSize;
                        this._activeSprites++;
                        this._shapeContainer.appendChild(spriteSettings.sprite);
                        WinJS.Promise.timeout(this._PROCESS_SPRITE_QUEUE_DELAY).then(function() {
                            spriteSettings.sprite.style.transform = spriteSettings.transform;
                            spriteSettings.sprite.style.opacity = spriteSettings.opacity
                        }.bind(this))
                    }
                }
            }, _createShapeContainer: function _createShapeContainer() {
                var div = document.createElement("div");
                div.className = "shapeContainer";
                return div
            }, _createSprite: function _createSprite(shapeObj, color, animation, iteration) {
                var transformDiv = document.createElement("div");
                transformDiv.style.width = shapeObj.width + "px";
                transformDiv.style.height = shapeObj.height + "px";
                transformDiv.className = "sprite";
                transformDiv.setAttribute("focusable", "false");
                var spriteSettings = this._buildTransform(transformDiv, animation, iteration, shapeObj);
                if (spriteSettings.scaleSettings) {
                    this._updateShapeSize(shapeObj, Math.min(this._calculateMaxScale(shapeObj.baseWidth, shapeObj.baseHeight), spriteSettings.scaleSettings.ScaleFactor * this._scaleSettings.scaleFactor));
                    transformDiv.style.width = shapeObj.width + "px";
                    transformDiv.style.height = shapeObj.height + "px"
                }
                var svgNS = "http://www.w3.org/2000/svg";
                var svgEl = document.createElementNS(svgNS, "svg");
                for (var i = 0; i < shapeObj.params.length; i++)
                    svgEl.setAttribute(shapeObj.params[i].name, shapeObj.params[i].value);
                svgEl.setAttribute("focusable", "false");
                var svgShape = document.createElementNS(svgNS, shapeObj.element);
                for (var i = 0; i < shapeObj.elementParams.length; i++)
                    if (shapeObj.elementParams[i].name === "fill")
                        svgShape.setAttribute(shapeObj.elementParams[i].name, color);
                    else
                        svgShape.setAttribute(shapeObj.elementParams[i].name, shapeObj.elementParams[i].value);
                svgShape.setAttribute("focusable", "false");
                svgEl.appendChild(svgShape);
                transformDiv.appendChild(svgEl);
                return spriteSettings
            }, _buildTransform: function _buildTransform(sprite, animation, iteration, shapeObj) {
                if (animation.TransformSettings.RandomHelper.Enabled && iteration === 0) {
                    animation.TransformSettings.RandomHelper.Values = [];
                    this._generateRandoms(animation.TransformSettings.RandomHelper.Min, animation.TransformSettings.RandomHelper.Max, animation.TransformSettings.RandomHelper.Values, animation.TransformSettings.RandomHelper.MinDelta, animation.ItemCount)
                }
                var transformString = String.empty;
                var destinationString = String.empty;
                var targetOpacity = animation.TargetOpacity;
                var transforms = animation.TransformSettings.Transforms;
                var fromScaleMultiplier = 1;
                var toScaleMultiplier = 1;
                var scaleSettings;
                var scaleTransform;
                var found = transforms.some(function where(transform) {
                        if (transform.Type === "ScaleTransform") {
                            scaleTransform = transform;
                            return transform
                        }
                    });
                if (found) {
                    var scaleRandoms = animation.TransformSettings.RandomHelper.Values;
                    if (scaleTransform.Enabled) {
                        if (scaleTransform.LocalRandomHelper && scaleTransform.LocalRandomHelper.Enabled && iteration === 0) {
                            scaleTransform.LocalRandomHelper.Values = [];
                            this._generateRandoms(scaleTransform.LocalRandomHelper.Min, scaleTransform.LocalRandomHelper.Max, scaleTransform.LocalRandomHelper.Values, scaleTransform.LocalRandomHelper.MinDelta, animation.ItemCount)
                        }
                        if (scaleTransform.LocalRandomHelper && scaleTransform.LocalRandomHelper.Enabled)
                            scaleRandoms = scaleTransform.LocalRandomHelper.Values;
                        if (scaleTransform.From.ApplyRandom)
                            fromScaleMultiplier = scaleTransform.From.Value * scaleRandoms[iteration];
                        else
                            fromScaleMultiplier = scaleTransform.From.Value;
                        if (scaleTransform.To.ApplyRandom)
                            toScaleMultiplier = (scaleTransform.To.Value * scaleRandoms[iteration]);
                        else
                            toScaleMultiplier = scaleTransform.To.Value;
                        scaleSettings = {ScaleFactor: toScaleMultiplier};
                        fromScaleMultiplier = fromScaleMultiplier / toScaleMultiplier;
                        toScaleMultiplier = 1
                    }
                }
                for (var i = 0; i < transforms.length; i++) {
                    var randomValues = animation.TransformSettings.RandomHelper.Values;
                    if (transforms[i].Enabled) {
                        if (transforms[i].LocalRandomHelper && transforms[i].LocalRandomHelper.Enabled && iteration === 0) {
                            transforms[i].LocalRandomHelper.Values = [];
                            this._generateRandoms(transforms[i].LocalRandomHelper.Min, transforms[i].LocalRandomHelper.Max, transforms[i].LocalRandomHelper.Values, transforms[i].LocalRandomHelper.MinDelta, animation.ItemCount)
                        }
                        if (transforms[i].LocalRandomHelper && transforms[i].LocalRandomHelper.Enabled)
                            randomValues = transforms[i].LocalRandomHelper.Values;
                        switch (transforms[i].Type) {
                            case"ScaleTransform":
                                if (fromScaleMultiplier !== 1)
                                    transformString += " scale(" + fromScaleMultiplier + ")";
                                destinationString += " scale(" + toScaleMultiplier + ")";
                                break;
                            case"RotateTransform":
                                transformString += " rotate(" + transforms[i].From.Value * (transforms[i].From.ApplyIteration ? iteration : 1) * (transforms[i].From.ApplyRandom ? randomValues[iteration] : 1) + "deg)";
                                destinationString += " rotate(" + transforms[i].To.Value * (transforms[i].To.ApplyIteration ? iteration : 1) * (transforms[i].To.ApplyRandom ? randomValues[iteration] : 1) + "deg)";
                                break;
                            case"TranslateXTransform":
                                if (transforms[i].From.ApplyRandom)
                                    transformString += " translateX(" + (transforms[i].From.Value * randomValues[iteration]) + "px)";
                                else
                                    transformString += " translateX(" + transforms[i].From.Value + "px)";
                                if (transforms[i].To.ApplyRandom)
                                    destinationString += " translateX(" + (transforms[i].To.Value * randomValues[iteration]) + "px)";
                                else
                                    destinationString += " translateX(" + transforms[i].To.Value + "px)";
                                break;
                            case"TranslateYTransform":
                                if (transforms[i].From.ApplyRandom)
                                    transformString += " translateY(" + (transforms[i].From.Value * randomValues[iteration]) + "px)";
                                else
                                    transformString += " translateY(" + transforms[i].From.Value + "px)";
                                if (transforms[i].To.ApplyRandom)
                                    destinationString += " translateY(" + (transforms[i].To.Value * randomValues[iteration]) + "px)";
                                else
                                    destinationString += " translateY(" + transforms[i].To.Value + "px)";
                                break;
                            case"RotateAndFlipTransform":
                                switch (transforms[i].From.ApplyRandom ? (this._generateRandom(0, 100) % 8) : (iteration % 8)) {
                                    case 0:
                                        break;
                                    case 1:
                                        transformString += " scaleY(-1.0)";
                                        destinationString += " scaleY(-1.0)";
                                        break;
                                    case 2:
                                        transformString += " rotate(90deg)";
                                        destinationString += " rotate(90deg)";
                                        break;
                                    case 3:
                                        transformString += " scaleX(-1.0) rotate(90deg)";
                                        destinationString += " scaleX(-1.0) rotate(90deg)";
                                        break;
                                    case 4:
                                        transformString += " rotate(180deg)";
                                        destinationString += " rotate(180deg)";
                                        break;
                                    case 5:
                                        transformString += " rotate(180deg) scaleY(-1.0)";
                                        destinationString += " rotate(180deg) scaleY(-1.0)";
                                        break;
                                    case 6:
                                        transformString += " rotate(270deg)";
                                        destinationString += " rotate(270deg)";
                                        break;
                                    case 7:
                                        transformString += " scaleX(-1.0) rotate(270deg)";
                                        destinationString += " scaleX(-1.0) rotate(270deg)";
                                        break
                                }
                                break;
                            case"RotateAndFlipTransformVert":
                                switch (transforms[i].From.ApplyRandom ? (this._generateRandom(0, 100) % 4) : (iteration % 4)) {
                                    case 0:
                                        break;
                                    case 1:
                                        transformString += " scaleY(-1.0)";
                                        destinationString += " scaleY(-1.0)";
                                        break;
                                    case 2:
                                        transformString += " rotate(180deg)";
                                        destinationString += " rotate(180deg)";
                                        break;
                                    case 3:
                                        transformString += " rotate(180deg) scaleY(-1.0)";
                                        destinationString += " rotate(180deg) scaleY(-1.0)";
                                        break
                                }
                                break;
                            case"RotateAndFlipTransformHorz":
                                switch (transforms[i].From.ApplyRandom ? (this._generateRandom(0, 100) % 4) : (iteration % 4)) {
                                    case 0:
                                        transformString += " rotate(90deg)";
                                        destinationString += " rotate(90deg)";
                                        break;
                                    case 1:
                                        transformString += " scaleX(-1.0) rotate(90deg)";
                                        destinationString += " scaleX(-1.0) rotate(90deg)";
                                        break;
                                    case 2:
                                        transformString += " rotate(270deg)";
                                        destinationString += " rotate(270deg)";
                                        break;
                                    case 3:
                                        transformString += " scaleX(-1.0) rotate(270deg)";
                                        destinationString += " scaleX(-1.0) rotate(270deg)";
                                        break
                                }
                                break;
                            case"ReflectVerticalTransform":
                                var directionY = this._generateRandom(0, 100);
                                if (directionY % 2 === 1) {
                                    transformString += " scaleY(-1)";
                                    destinationString += " scaleY(-1)"
                                }
                                break;
                            case"ReflectHorizontalTransform":
                                var directionX = this._generateRandom(0, 100);
                                if (directionX % 2 === 1) {
                                    transformString += " scaleX(-1)";
                                    destinationString += " scaleX(-1)"
                                }
                                break;
                            default:
                                break
                        }
                    }
                }
                sprite.style.transform = transformString;
                if (animation.InitOpacity !== 1)
                    sprite.style.opacity = animation.InitOpacity;
                sprite.style.transitionTimingFunction = animation.TransformEasing;
                return {
                        sprite: sprite, transform: destinationString, opacity: targetOpacity, shapeObj: shapeObj, animationName: animation.Name, iteration: iteration, scaleSettings: scaleSettings, shapeWidth: shapeObj.width, shapeHeight: shapeObj.height
                    }
            }, _clearCanvas: function _clearCanvas(shapeContainers) {
                if (this._shapeContainer)
                    MS.Entertainment.Utilities.empty(this._shapeContainer);
                this._queuedSprites = [];
                this._allocatedPixels = 0;
                this._activeSprites = 0
            }, _calculateMaxScale: function _calculateMaxScale(shapeWidth, shapeHeight) {
                return (Math.sqrt((this._MAX_PIXEL_ALLOCATION / this._ITEMS_ON_SCREEN) / shapeWidth / shapeHeight))
            }, _generateRandoms: function _generateRandoms(min, max, arrValues, minDelta, count) {
                for (var i = 0; i < count; i++)
                    this._generateRandom(min, max, arrValues, minDelta)
            }, _generateRandom: function _generateRandom(min, max, arrValues, minDelta) {
                var rand = Math.floor((Math.random() * max) + min * 1);
                if (arrValues) {
                    var success = false;
                    while (!success) {
                        rand = Math.floor((Math.random() * max) + (min * 1));
                        success = true;
                        for (var i = 0; i < arrValues.length; i++)
                            if (Math.abs(arrValues[i] - rand) < minDelta)
                                success = false
                    }
                    arrValues.push(rand)
                }
                return rand
            }, _hasCollision: function hasCollision(pointIndex, indexArray) {
                var hasCollision = false;
                for (var i = 0; i < indexArray.length; i++)
                    if (this._spawnPoints[pointIndex].x === this._spawnPoints[indexArray[i]].x || this._spawnPoints[pointIndex].y === this._spawnPoints[indexArray[i]].y) {
                        hasCollision = true;
                        break
                    }
                return hasCollision
            }, _generateSpawnPoints: function _generateSpawnPoints() {
                this._spawnPoints = [];
                var rows = 4;
                var columns = 4;
                if (this._uiStateService.isSnapped) {
                    columns = 2;
                    rows = 8
                }
                var cellWidth = (1366 * this._scaleSettings.scaleFactor) / columns;
                var cellHeight = cellWidth / this._scaleSettings.baseAspectRatio;
                var xPos = 0;
                var yPos = 0;
                for (var i = 0; i <= rows; i++) {
                    xPos = 0;
                    for (var j = 0; j <= columns; j++) {
                        this._spawnPoints.push({
                            x: xPos, y: yPos
                        });
                        xPos += cellWidth
                    }
                    yPos += cellHeight
                }
            }, _updateShapeSizes: function _updateShapeSizes() {
                for (var i = 0; i < this._shapes.length; i++)
                    this._updateShapeSize(this._shapes[i], this._scaleSettings.scaleFactor)
            }, _updateShapeSize: function _updateShapeSize(shapeObj, scaleFactor) {
                shapeObj.width = shapeObj.baseWidth * scaleFactor;
                shapeObj.height = shapeObj.baseHeight * scaleFactor;
                shapeObj.params[2].value = shapeObj.width + "px";
                shapeObj.params[3].value = shapeObj.height + "px"
            }, _initShapeLibrary: function _initShapeLibrary() {
                this._shapes = [{
                        name: "circlePiece1", uri: "Images/NowPlaying/Shapes/Circle-Piece-1.svg", width: "342", height: "385", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Circle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "342px"
                                }, {
                                    name: "height", value: "385px"
                                }, {
                                    name: "viewBox", value: "0 -0.94 342 385"
                                }, {
                                    name: "enable-background", value: "new 0 -0.94 342 385"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M0,0v384h341.499C341.468,185.722,192.239,22.318,0,0z"
                                }]
                    }, {
                        name: "circlePiece2", uri: "Images/NowPlaying/Shapes/Circle-Piece-2.svg", width: "342", height: "576", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Circle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "342px"
                                }, {
                                    name: "height", value: "576px"
                                }, {
                                    name: "viewBox", value: "0 0 342 576"
                                }, {
                                    name: "enable-background", value: "new 0 0 342 576"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M0,576h341.499V0h-2.113C137.048,112.033,0.013,327.977,0,576z"
                                }]
                    }, {
                        name: "circlePiece3", uri: "Images/NowPlaying/Shapes/Circle-Piece-3.svg", width: "1025", height: "192", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Circle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "1025px"
                                }, {
                                    name: "height", value: "192px"
                                }, {
                                    name: "viewBox", value: "0 0 1025 192"
                                }, {
                                    name: "enable-background", value: "new 0 0 1025 192"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M1024.5,192C706.939,68.066,361.417,0.021,0,0v192H1024.5z"
                                }]
                    }, {
                        name: "diamond", uri: "Images/NowPlaying/Shapes/Diamond-1.svg", width: "683", height: "384", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Diamond, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "683px"
                                }, {
                                    name: "height", value: "384px"
                                }, {
                                    name: "viewBox", value: "0 0 683 384"
                                }, {
                                    name: "enable-background", value: "new 0 0 683 384"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M341.5,0 683,192 341.5,384 0,192 "
                                }]
                    }, {
                        name: "halfCircle", uri: "Images/NowPlaying/Shapes/Half-Circle-1-Rot.svg", width: "683", height: "341", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Circle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "683px"
                                }, {
                                    name: "height", value: "341px"
                                }, {
                                    name: "viewBox", value: "0 0 683 341"
                                }, {
                                    name: "enable-background", value: "new 0 0 683 341"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M 341,0 C 152.895,0 0,152.672 0,341 H 683 C 683,152.672 530.105,0 341,0 z"
                                }]
                    }, {
                        name: "parallelogram1", uri: "Images/NowPlaying/Shapes/Parallelogram-1.svg", width: "683", height: "384", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Parallelogram, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "683px"
                                }, {
                                    name: "height", value: "384px"
                                }, {
                                    name: "viewBox", value: "0 0 683 384"
                                }, {
                                    name: "enable-background", value: "new 0 0 683 384"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M341.5,384 0,384 341.5,0 683,0"
                                }]
                    }, {
                        name: "parallelogram2", uri: "Images/NowPlaying/Shapes/Parallelogram-2.svg", width: "1025", height: "192", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Parallelogram, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "1024px"
                                }, {
                                    name: "height", value: "192px"
                                }, {
                                    name: "viewBox", value: "0 0 1024 192"
                                }, {
                                    name: "enable-background", value: "new 0 0 1024 192"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M682.667,192 0,192 341.333,0 1024,0"
                                }]
                    }, {
                        name: "parallelogram3", uri: "Images/NowPlaying/Shapes/Parallelogram-3-Rot.svg", width: "900", height: "300", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Parallelogram, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "900px"
                                }, {
                                    name: "height", value: "300px"
                                }, {
                                    name: "viewBox", value: "0 0 900 300"
                                }, {
                                    name: "enable-background", value: "new 0 0 900 300"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M300,0 0,300 600,300 900,0"
                                }]
                    }, {
                        name: "quarterCircle", uri: "Images/NowPlaying/Shapes/Quarter-Circle-1-Rot.svg", width: "384", height: "384", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Circle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "384px"
                                }, {
                                    name: "height", value: "384px"
                                }, {
                                    name: "viewBox", value: "0 0 384 384"
                                }, {
                                    name: "enable-background", value: "new 0 0 384 384"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M384,384C384,171.924,212.079,0,0,0v384H384z"
                                }]
                    }, {
                        name: "rectangle1", uri: "Images/NowPlaying/Shapes/Rectangle-1.svg", width: "683", height: "192", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Rectangle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "683px"
                                }, {
                                    name: "height", value: "192px"
                                }, {
                                    name: "viewBox", value: "0 0 683 192"
                                }, {
                                    name: "enable-background", value: "new 0 0 683 192"
                                }], element: "rect", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "width", value: "683"
                                }, {
                                    name: "height", value: "192"
                                }]
                    }, {
                        name: "rectangle2", uri: "Images/NowPlaying/Shapes/Rectangle-2.svg", width: "342", height: "384", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Rectangle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "342px"
                                }, {
                                    name: "height", value: "384px"
                                }, {
                                    name: "viewBox", value: "0 0 342 384"
                                }, {
                                    name: "enable-background", value: "new 0 0 342 384"
                                }], element: "rect", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "width", value: "341.5"
                                }, {
                                    name: "height", value: "384"
                                }]
                    }, {
                        name: "semiCircle", uri: "Images/NowPlaying/Shapes/SemiCircle.svg", width: "383", height: "192", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Circle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "viewBox", value: "0 0 383 192"
                                }, {
                                    name: "enable-background", value: "new 0 0 1000 1000"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M0,192h383.831C383.658,86,297.804,0.399,191.915,0.399C86.027,0.399,0.173,86,0,192z"
                                }]
                    }, {
                        name: "square1", uri: "Images/NowPlaying/Shapes/Square-1-Rot.svg", width: "384", height: "384", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Square, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "384px"
                                }, {
                                    name: "height", value: "384px"
                                }, {
                                    name: "viewBox", value: "0 0 384 384"
                                }, {
                                    name: "enable-background", value: "new 0 0 384 384"
                                }], element: "rect", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "width", value: "384"
                                }, {
                                    name: "height", value: "383.999"
                                }]
                    }, {
                        name: "square2", uri: "Images/NowPlaying/Shapes/Square-2-Rot.svg", width: "543", height: "543", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Square, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "543px"
                                }, {
                                    name: "height", value: "543px"
                                }, {
                                    name: "viewBox", value: "0 0 543 543"
                                }, {
                                    name: "enable-background", value: "new 0 0 543 543"
                                }], element: "rect", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "x", value: "79.52"
                                }, {
                                    name: "y", value: "79.52"
                                }, {
                                    name: "transform", value: "matrix(0.7071 0.7071 -0.7071 0.7071 271.4997 -112.459)"
                                }, {
                                    name: "width", value: "383.959"
                                }, {
                                    name: "height", value: "383.959"
                                }]
                    }, {
                        name: "trapezoid1", uri: "Images/NowPlaying/Shapes/Trapezoid-1.svg", width: "1025", height: "192", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Trapezoid, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "1024px"
                                }, {
                                    name: "height", value: "192px"
                                }, {
                                    name: "viewBox", value: "0 0 1024 192"
                                }, {
                                    name: "enable-background", value: "new 0 0 1024 192"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M0,192 342,0 682,0 1024,192 "
                                }]
                    }, {
                        name: "trapezoid2", uri: "Images/NowPlaying/Shapes/Trapezoid-2.svg", width: "341", height: "576", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Trapezoid, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "341px"
                                }, {
                                    name: "height", value: "576px"
                                }, {
                                    name: "viewBox", value: "0 0 341 576"
                                }, {
                                    name: "enable-background", value: "new 0 0 341 576"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M341,576 0,384 0,192 341,0"
                                }]
                    }, {
                        name: "trapezoid3", uri: "Images/NowPlaying/Shapes/Trapezoid-3-Rot.svg", width: "900", height: "300", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Trapezoid, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "900px"
                                }, {
                                    name: "height", value: "300px"
                                }, {
                                    name: "viewBox", value: "0 0 900 300"
                                }, {
                                    name: "enable-background", value: "new 0 0 900 300"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M600,0 300,0 0,300 900,300"
                                }]
                    }, {
                        name: "triangle1", uri: "Images/NowPlaying/Shapes/Triangle-1.svg", width: "683", height: "384", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Triangle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "683px"
                                }, {
                                    name: "height", value: "384px"
                                }, {
                                    name: "viewBox", value: "0 0 683 384"
                                }, {
                                    name: "enable-background", value: "new 0 0 683 384"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M683,384 0,384 341.502,0"
                                }]
                    }, {
                        name: "triangle2", uri: "Images/NowPlaying/Shapes/Triangle-2.svg", width: "683", height: "384", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Triangle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "683px"
                                }, {
                                    name: "height", value: "384px"
                                }, {
                                    name: "viewBox", value: "0 0 683 384"
                                }, {
                                    name: "enable-background", value: "new 0 0 683 384"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M683,384 0,384 682.999,0"
                                }]
                    }, {
                        name: "triangle3", uri: "Images/NowPlaying/Shapes/Triangle-3.svg", width: "684", height: "384", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Triangle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "684px"
                                }, {
                                    name: "height", value: "384px"
                                }, {
                                    name: "viewBox", value: "0 0 684 384"
                                }, {
                                    name: "enable-background", value: "new 0 0 684 384"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M684,192 0,384 0,0"
                                }]
                    }, {
                        name: "triangle4", uri: "Images/NowPlaying/Shapes/Triangle-4.svg", width: "1366", height: "192", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Triangle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "1366px"
                                }, {
                                    name: "height", value: "192px"
                                }, {
                                    name: "viewBox", value: "0 0 1366 192"
                                }, {
                                    name: "enable-background", value: "new 0 0 1366 192"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M1366,192 0,192 1366,0"
                                }]
                    }, {
                        name: "triangle5", uri: "Images/NowPlaying/Shapes/Triangle-5.svg", width: "341", height: "768", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Triangle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "341px"
                                }, {
                                    name: "height", value: "768px"
                                }, {
                                    name: "viewBox", value: "0 0 341 768"
                                }, {
                                    name: "enable-background", value: "new 0 0 341 768"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M341,768 0,768 0,0 "
                                }]
                    }, {
                        name: "triangle6Rot", uri: "Images/NowPlaying/Shapes/Triangle-6-Rot.svg", width: "683", height: "341", type: MS.Entertainment.UI.Controls.ShapeVisualization.ShapeType.Triangle, params: [{
                                    name: "x", value: "0px"
                                }, {
                                    name: "y", value: "0px"
                                }, {
                                    name: "width", value: "683px"
                                }, {
                                    name: "height", value: "341px"
                                }, {
                                    name: "viewBox", value: "0 0 683 341"
                                }, {
                                    name: "enable-background", value: "new 0 0 683 341"
                                }], element: "path", elementParams: [{
                                    name: "fill", value: "#32B44A"
                                }, {
                                    name: "d", value: "M0,341 341.5,0 683,341"
                                }]
                    }];
                for (var i = 0; i < this._shapes.length; i++) {
                    this._shapes[i].baseWidth = this._shapes[i].width;
                    this._shapes[i].baseHeight = this._shapes[i].height
                }
            }
    }, {currentAnimation: null})})
